@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *	 it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -25,6 +24,10 @@
 #include <stdlib.h>
 #include <sys/time.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <pthread.h>
 
 #include "brighton.h"
@@ -79,6 +82,9 @@ extern brightonApp bmApp;
 extern brightonApp sidApp;
 extern brightonApp sid2App;
 
+extern int brightonCLIcheck(guimain *);
+extern int brightonCLIinit(guimain *, int *);
+
 #define BRIGHTON_BIT99APP (BRISTOL_SYNTHCOUNT + 1)
 
 char *bristolhome = NULL;
@@ -86,13 +92,20 @@ char *bristolhome = NULL;
 guimain global;
 
 brightonApp *synthesisers[BRISTOL_SYNTHCOUNT];
+#include "brightonreadme.h"
+
+static char *defname = "bristol";
 
 void *eventMgr();
 
+int readMe = 0;
+int gmc = 0;
 int opacity = 60;
 int gs = 1;
 int nar = 0;
+int azoom = 0;
 int quality = 6;
+int deswidth = -1;
 float scale = 1.0;
 float antialias = 0.0;
 int aliastype = 0;
@@ -103,10 +116,20 @@ int sampleCount = 256;
 int mwt = 50000;
 int activeSense = 2000;
 int activeSensePeriod = 15000;
+int bwflags = BRIGHTON_POST_WINDOW;
+int rlflags = BRIGHTON_SET_RAISE|BRIGHTON_SET_LOWER;
+int nrpcc = BRIGHTON_NRP_COUNT;
+int cli = 0;
+int cli_fd[2];
 
 static char *BRISTOL_DEF_HOST = "localhost";
 
+extern int vuInterval;
+void printBrightonHelp(int);
+void printBrightonReadme();
+
 extern int brightonMidiInput(bristolMidiMsg *);
+extern void brightonControlKeyInput(brightonWindow *, int, int);
 
 /*
  * Some of the emulations really benefit from a bit of detune, this is the
@@ -114,39 +137,148 @@ extern int brightonMidiInput(bristolMidiMsg *);
  */
 #define NON_DEF_DETUNE 100
 
+static int asc, emStart = 0, midiHandle = -1;
+
+volatile sig_atomic_t ladiRequest = 0;
+
+static void
+savehandler()
+{
+	if (global.synths->flags & LADI_ENABLE && global.synths->ladimode)
+		ladiRequest = 1;
+	else
+		ladiRequest = 0;
+}
+
+static void
+loadhandler()
+{
+	if (global.synths->flags & LADI_ENABLE && global.synths->ladimode)
+		ladiRequest = 2;
+	else
+		ladiRequest = 0;
+}
+
+static void
+brightonFindEmulation(int argc, char **argv)
+{
+	int argCount, i, found = -1;
+
+	for (argCount = 1; argCount < argc; argCount++)
+	{
+		if (argv[argCount][0] != '-')
+			continue;
+
+		for (i = 0 ; i < BRISTOL_SYNTHCOUNT; i++)
+		{
+			if (synthesisers[i] == NULL)
+				continue;
+
+			if ((strcmp("-emulate", argv[argCount]) == 0)
+				&& (argCount < (argc - 1)))
+			{
+				if (strcmp(synthesisers[i]->name, argv[argCount + 1]) == 0)
+					found = global.synths->synthtype = i;
+			} else {
+				if (strcmp(synthesisers[i]->name, &argv[argCount][1]) == 0)
+					found = global.synths->synthtype = i;
+			}
+		}
+	}
+
+	/*
+	 * Global.synths->synthtype will now either be the last emulator found or
+	 * will be the usual default (b3). Stuff this back into i, that is nicely
+	 * confusing but makes the lines shorter. Hm.
+	 */
+	i = global.synths->synthtype;
+
+	if (found > 0)
+		printf("%s emulation defaults:\n", synthesisers[i]->name);
+	else
+		printf("emulation defaults:\n");
+
+	if ((global.synths->voices = synthesisers[i]->emulate.voices)
+		< 0)
+	{
+		printf("    -voices  %i\n", BRISTOL_VOICECOUNT);
+		global.synths->voices = BRISTOL_VOICECOUNT;
+	} else
+		printf("    -voices  %i\n",synthesisers[i]->emulate.voices);
+
+	if (global.synths->voices == 1)
+	{
+		printf("    -retrig\n");
+		printf("    -lvel\n");
+		printf("    -hnp\n");
+		printf("    -wwf\n");
+		global.synths->lwf = 2;
+		global.synths->legatovelocity = 1;
+		global.synths->notepref = BRIGHTON_HNP;
+		global.synths->notetrig = 1;
+	}
+
+	if (i == BRISTOL_2600)
+		printf("    -multi   4\n");
+	printf("    -detune  %i\n", synthesisers[i]->emulate.detune);
+	printf("    -gain    %i\n",	synthesisers[i]->emulate.gain);
+	printf("    -pwd     %i\n",	synthesisers[i]->emulate.pwd);
+	printf("    -glide   %i\n",	synthesisers[i]->emulate.glide);
+	printf("    -curve   %i\n",	synthesisers[i]->emulate.velocity);
+
+	global.synths->detune = synthesisers[i]->emulate.detune;
+	global.synths->gain = synthesisers[i]->emulate.gain * 16;
+	global.synths->pwd = synthesisers[i]->emulate.pwd;
+	global.synths->glide = synthesisers[i]->emulate.glide;
+	global.synths->velocity = synthesisers[i]->emulate.velocity;
+	if (synthesisers[i]->emulate.opacity != 0)
+		printf("    -opacity %i\n",
+			opacity = synthesisers[i]->emulate.opacity);
+}
+
 /*
  * Need to make this multithreaded?
  */
 int
 main(int argc, char **argv)
 {
-	int argCount = 1, midiHandle = -1, cFD = -1, midiFD = -1, i, j, asc = 0;
-	pthread_t thread;
-	bristolMidiMsg msg;
+	int argCount = 1, i, j, logtype = BRISTOL_LOG_BRIGHTON;
+	pthread_t emgrThread, logthread;
+	char appname[65], *devname = defname;
 
 	signal(SIGINT, cleanupBristol);
 	signal(SIGPIPE, cleanupBristolQuietly);
 	signal(SIGHUP, cleanupBristol);
+	signal(SIGUSR1, savehandler);
+	signal(SIGUSR2, loadhandler);
 
 	global.home = getenv("BRISTOL");
 
 	global.synths = (guiSynth *) brightonmalloc(sizeof(guiSynth));
 
+	global.controlfd = -1;
 	global.voices = BRISTOL_VOICECOUNT;
 	global.synths->voices = BRISTOL_VOICECOUNT;
 	global.synths->pwd = 2;
 	global.synths->gain = 32;
+	global.synths->ladimem = 1024;
 	global.synths->synthtype = BRISTOL_HAMMONDB3;
 	global.synths->velocity = 510;  /* linear tracking */
 	global.synths->glide = 5;  /* maximum glide delay - up to 30s */
 	global.synths->detune = 40;
-	global.synths->lwf = 0;
+	global.synths->lowkey = 0;
+	global.synths->highkey = 127;
+	global.synths->lwf = 0; //3;
 	global.synths->mbi = 0;
-	global.synths->notepref = 0;
+	global.synths->notepref = BRIGHTON_POLYPHONIC;
 	global.synths->notetrig = 0;
-	global.port = -1;
+	global.synths->win = NULL;
+	global.synths->flags = REQ_FORWARD|REQ_LOCAL_FORWARD|REQ_REMOTE_FORWARD;
+	global.port = 5028;
 	global.host = BRISTOL_DEF_HOST;
 
+	memset(synthesisers, 0, sizeof(synthesisers));
+
 	synthesisers[BRISTOL_MINI] = &miniApp;
 	synthesisers[BRISTOL_PROPHET] = &prophetApp;
 	synthesisers[BRISTOL_PRO52] = &prophet52App;
@@ -190,7 +322,34 @@ main(int argc, char **argv)
 	synthesisers[BRISTOL_SID_M1] = &sidApp;
 	synthesisers[BRISTOL_SID_M2] = &sid2App;
 
-	global.synths->location = 0;
+	readme[BRISTOL_PRO52] = readme[BRISTOL_PROPHET];
+	readme[BRISTOL_VOYAGER] = readme[BRISTOL_EXPLORER];
+
+	if (argc == 1)
+	{
+		printf("You probably prefer to use the startBristol script.\n");
+		exit(0);
+	}
+
+	if (strcmp(argv[1], "-console") == 0)
+		logtype = BRISTOL_LOG_CONSOLE;
+
+	if (strcmp(argv[1], "-readme") == 0)
+	{
+		readMe = 1;
+		logtype = BRISTOL_LOG_CONSOLE;
+		if (argc == 2)
+		{
+			printBrightonReadme();
+			exit(0);
+		}
+	}
+
+	if ((strcmp(argv[argCount], "-V") == 0) && (strlen(argv[argCount]) == 2))
+	{
+		printf("Version %s\n", VERSION);
+		exit(0);
+	}
 
 	if ((argc > 1) && (strcmp(argv[1], "-summary") == 0))
 	{
@@ -198,47 +357,73 @@ main(int argc, char **argv)
 		exit(0);
 	}
 
-
 	if (argc < 2) {
 		printf("You probably prefer to use the startBristol script.\n");
 		exit(1);
 	}
 
-	printf("bristol version %s\n", VERSION);
+	if ((argc == 2) 
+		&& ((strcmp(argv[argCount], "--help") == 0)
+		|| (strcmp(argv[argCount], "-help") == 0)
+		|| (strcmp(argv[argCount], "-h") == 0)
+		|| (strcmp(argv[argCount], "--h") == 0)))
+	{
+#ifdef BRISTOL_BUILD
+		if (BRISTOL_BUILD == 0)
+			printf("bristol %i.%i.%i: ",
+				BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH);
+		else
+			printf("bristol %i.%i.%i-%i: ",
+				BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH, BRISTOL_BUILD);
+#else
+			printf("bristol %s\n", VERSION);
+#endif
+		printf("%s", helptext);
+		exit(0);
+	}
+
+	if (readMe == 0)
+		printf("%s", gplnotice);
+
+	global.synths->location = 0;
+
+	if ((getenv("BRISTOL_LOG_CONSOLE") != NULL)
+		&& (strcmp(getenv("BRISTOL_LOG_CONSOLE"), "true") == 0))
+		logtype = BRISTOL_LOG_CONSOLE;
+
+	cli_fd[0] = 0;//dup(0);
+	cli_fd[1] = 1;//dup(1);
+
+	logthread = bristolOpenStdio(logtype);
+
+	/* Set up a default SYSEX ID, may get changed later */
+	bristolMidiOption(0, BRISTOL_NRP_SYSID_H, 0x534C);
+	bristolMidiOption(0, BRISTOL_NRP_SYSID_L, 0x6162);
+
+	brightonFindEmulation(argc, argv);
 
 	/*
 	 * close our standard input, create a pipe which will never be used.
 	 */
 	while (argc > argCount)
 	{
-		if ((strcmp(argv[argCount], "--help") == 0)
-			|| (strcmp(argv[argCount], "-help") == 0)
-			|| (strcmp(argv[argCount], "-h") == 0)
-			|| (strcmp(argv[argCount], "--h") == 0))
-		{
-#ifdef BRISTOL_BUILD
-			if (BRISTOL_BUILD == 0)
-				printf("bristol %i.%i.%i: ",
-					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH);
-			else
-				printf("bristol %i.%i.%i-%i: ",
-					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH, BRISTOL_BUILD);
-#else
-				printf("bristol %s\n", VERSION);
-#endif
-			printf("%s", helptext);
-			exit(0);
-		}
+		if ((strcmp(argv[argCount], "-log") == 0)
+			&& (logtype != BRISTOL_LOG_CONSOLE))
+			bristolOpenStdio(BRISTOL_LOG_DAEMON);
+
+		if ((strcmp(argv[argCount], "-syslog") == 0)
+			&& (logtype != BRISTOL_LOG_CONSOLE))
+			bristolOpenStdio(BRISTOL_LOG_SYSLOG);
 
 		if ((strcmp(argv[argCount], "-V") == 0)
 			&& (strlen(argv[argCount]) == 2))
 		{
 #ifdef BRISTOL_BUILD
 			if (BRISTOL_BUILD == 0)
-				printf("bristol version %i.%i.%i\n",
+				printf("brighton version %i.%i.%i\n",
 					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH);
 			else
-				printf("bristol version %i.%i.%i-%i\n",
+				printf("brighton version %i.%i.%i-%i\n",
 					BRISTOL_MAJOR, BRISTOL_MINOR, BRISTOL_PATCH, BRISTOL_BUILD);
 #endif
 			exit(0);
@@ -250,8 +435,7 @@ main(int argc, char **argv)
 		if (strcmp(argv[argCount], "-pixmap") == 0)
 			library = 0;
 
-		if (((strcmp(argv[argCount], "-emulate") == 0)
-			||(strcmp(argv[argCount], "-synth") == 0))
+		if ((strcmp(argv[argCount], "-synth") == 0)
 			&& (argCount < argc))
 		{
 			int i;
@@ -286,7 +470,9 @@ main(int argc, char **argv)
 				printf("	-glide:  %i\n",	synthesisers[i]->emulate.glide);
 				printf("	-curve:  %i\n",	synthesisers[i]->emulate.velocity);
 
-				global.synths->voices = synthesisers[i]->emulate.voices;
+				if ((global.synths->voices = synthesisers[i]->emulate.voices)
+					< 0)
+					global.synths->voices = BRISTOL_VOICECOUNT;
 				global.synths->detune = synthesisers[i]->emulate.detune;
 				global.synths->gain = synthesisers[i]->emulate.gain * 128;
 				global.synths->pwd = synthesisers[i]->emulate.pwd;
@@ -295,11 +481,17 @@ main(int argc, char **argv)
 				if (synthesisers[i]->emulate.opacity != 0)
 					printf("    -opacity:%i\n",
 						opacity = synthesisers[i]->emulate.opacity);
+				if (global.synths->voices == 1)
+					global.synths->notepref = BRIGHTON_HNP;
+				devname = argv[argCount + 1];
 			}
 			argCount++;
 			continue;
 		}
 
+		if (strcmp(argv[argCount], "-iconify") == 0)
+			bwflags &= ~BRIGHTON_POST_WINDOW;
+
 		if ((strcmp(argv[argCount], "-quality") == 0) && (argCount < argc))
 		{
 			if ((quality = atoi(argv[argCount + 1])) < 2)
@@ -361,6 +553,73 @@ main(int argc, char **argv)
 			continue;
 		}
 
+		if ((strcmp(argv[argCount], "-jsmuuid") == 0) && (argCount < argc))
+		{
+			global.synths->ladimem += atoi(argv[argCount + 1]);
+			argCount++;
+			continue;
+		}
+
+		/*
+		 * LADI level 1 options:
+		 */
+		if ((strcmp(argv[argCount], "-ladi") == 0) && (argCount < argc))
+		{
+			global.synths->flags |= LADI_ENABLE;
+
+			if (strcmp(argv[argCount + 1], "both") == 0)
+				global.synths->ladimode = 1;
+			else if (strcmp(argv[argCount + 1], "bristol") == 0)
+				global.synths->ladimode = 0;
+			else if (strcmp(argv[argCount + 1], "brighton") == 0)
+				global.synths->ladimode = 1;
+			else {
+				if ((global.synths->ladimem = atoi(argv[argCount + 1])) <= 0)
+				{
+					global.synths->ladimem = 1024;
+					global.synths->ladiStateFile = argv[argCount + 1];
+				}
+			}
+
+			/*
+			 * We also need to force the startup to open this memory location
+			 * as well, that can get overridden if there is a -load option 
+			 * anywhere later on the command line.
+			 */
+			global.synths->location = global.synths->ladimem;
+
+			global.synths->mbi = (global.synths->location / 1000) * 1000;
+			global.synths->location = global.synths->location % 1000;
+
+			argCount++;
+			continue;
+		}
+
+		if ((strcmp(argv[argCount], "-audiodev") == 0) && (argCount < argc))
+		{
+			argCount++;
+			devname = argv[argCount];
+		}
+		if ((strcmp(argv[argCount], "-register") == 0)
+			&& (argCount < argc)
+			&& (argv[argCount + 1][0] != '-'))
+		{
+			argCount++;
+			devname = argv[argCount];
+		}
+
+		if ((strcmp(argv[argCount], "-lowkey") == 0) && (argCount < argc))
+		{
+			argCount++;
+			global.synths->lowkey = atoi(argv[argCount]);
+		}
+
+		if ((strcmp(argv[argCount], "-highkey") == 0) && (argCount < argc))
+		{
+			argCount++;
+			global.synths->highkey = atoi(argv[argCount]);
+		}
+
 		if (strcmp(argv[argCount], "-rate") == 0)
 		{
 			if ((sampleRate = atoi(argv[argCount + 1])) <= 0)
@@ -401,7 +660,6 @@ main(int argc, char **argv)
 			if ((activeSensePeriod = activeSense * 3) > 16000)
 				activeSensePeriod = 16000;
 
-printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			argCount++;
 		}
 
@@ -423,10 +681,12 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			activeSensePeriod = activeSensePeriod * sampleRate / 1000;
 			 */
 
-printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			argCount++;
 		}
 
+		if (strcmp(argv[argCount], "-gmc") == 0)
+			gmc = 1;
+
 		if ((strcmp(argv[argCount], "-mct") == 0)
 			&& (argCount < argc))
 		{
@@ -454,6 +714,12 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			continue;
 		}
 
+		if ((strcmp(argv[argCount], "-width") == 0) && (argCount < argc))
+		{
+			deswidth = atoi(argv[argCount + 1]);
+			argCount++;
+		}
+
 		if ((strcmp(argv[argCount], "-scale") == 0) && (argCount < argc))
 		{
 			if ((strcmp(argv[argCount + 1], "fs") == 0)
@@ -476,6 +742,22 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			scale = 1.0;
 		}
 
+		if (strcmp(argv[argCount], "-raise") == 0)
+			rlflags &= ~BRIGHTON_SET_RAISE;
+		if (strcmp(argv[argCount], "-lower") == 0)
+			rlflags &= ~BRIGHTON_SET_LOWER;
+
+		if ((strcmp(argv[argCount], "-rotaryUD") == 0)
+			|| (strcmp(argv[argCount], "-rud") == 0))
+			rlflags |= BRIGHTON_ROTARY_UD;
+
+		if (strcmp(argv[argCount], "-autozoom") == 0)
+		{
+			azoom = 1;
+			if (deswidth < 1)
+				deswidth = 100;
+		}
+
 		if ((strcmp(argv[argCount], "-ar") == 0) ||
 			(strcmp(argv[argCount], "-nar") == 0) ||
 			(strcmp(argv[argCount], "-aspect") == 0))
@@ -515,12 +797,36 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		{
 			if (argCount < argc)
 				global.synths->location = atoi(argv[argCount + 1]);
-			if (global.synths->location > 999)
-			{
-				global.synths->mbi = (global.synths->location / 1000) * 1000;
-				global.synths->location = global.synths->location % 1000;
-			}
 
+			global.synths->mbi = (global.synths->location / 1000) * 1000;
+			global.synths->location = global.synths->location % 1000;
+
+			argCount++;
+		}
+
+		/* SYSEX System ID */
+		if ((strcmp(argv[argCount], "-sysid") == 0) && (argCount < argc))
+		{
+			unsigned int sysid = 0x534C6162, o = argCount + 1;
+
+			/*
+			 * We will get 0x534C6162 for example
+			 */
+			if ((argv[o][0] == '0') || (argv[o][1] == 'x')
+				|| (strlen(argv[o]) == 10))
+			{
+				sscanf(argv[o], "0x%x", &sysid);
+				sysid &= 0x7f7f7f7f;
+				bristolMidiOption(0, BRISTOL_NRP_SYSID_H, sysid >> 16);
+				bristolMidiOption(0, BRISTOL_NRP_SYSID_L, sysid & 0x0000ffff);
+				printf("fixing sysex system id at 0x%x\n", sysid);
+			}
+		}
+
+		if ((strcmp(argv[argCount], "-nrpcc") == 0) && (argCount < argc))
+		{
+			nrpcc = atoi(argv[argCount + 1]);
+			printf("Reserving %i NRP controllers\n", nrpcc);
 			argCount++;
 		}
 
@@ -554,13 +860,25 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 
 		if ((strcmp(argv[argCount], "-voices") == 0) && (argCount < argc))
 		{
-			if (argCount < argc)
-				global.synths->voices = atoi(argv[argCount + 1]);
+			if (((global.synths->voices = atoi(argv[argCount + 1])) == 1)
+				&& (global.synths->notepref == BRIGHTON_POLYPHONIC))
+				global.synths->notepref = BRIGHTON_HNP;
 			argCount++;
 		}
 
 		if (strcmp(argv[argCount], "-mono") == 0)
+		{
 			global.synths->voices = 1;
+			global.synths->notepref = BRIGHTON_HNP;
+		}
+
+		if (strcmp(argv[argCount], "-localforward") == 0)
+			global.synths->flags &= ~REQ_LOCAL_FORWARD;
+		if (strcmp(argv[argCount], "-remoteforward") == 0)
+			global.synths->flags &= ~REQ_REMOTE_FORWARD;
+		if (strcmp(argv[argCount], "-forward") == 0)
+			global.synths->flags
+				&= ~(REQ_FORWARD|REQ_LOCAL_FORWARD|REQ_REMOTE_FORWARD);
 
 		if (strcmp(argv[argCount], "-mididbg2") == 0)
 			global.synths->flags |= REQ_MIDI_DEBUG2;
@@ -568,9 +886,39 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			|| (strcmp(argv[argCount], "-mididbg1") == 0))
 			global.synths->flags |= REQ_MIDI_DEBUG;
 
-		if ((strcmp(argv[argCount], "-midinrp") == 0)
-			|| (strcmp(argv[argCount], "-nrp") == 0))
-			global.synths->flags |= MIDI_NRP;
+		if (strcmp(argv[argCount], "-debug") == 0)
+		{
+			if ((argCount < argc) && (argv[argCount + 1][0] != '-'))
+			{
+				int i;
+
+				if ((i = atoi(argv[argCount + 1])) > 15)
+					global.synths->flags |= REQ_DEBUG_MASK;
+				else {
+					global.synths->flags &= ~REQ_DEBUG_MASK;
+					global.synths->flags |= (i << 12);
+				}
+			} else {
+				/*
+				 * If we only have -debug and no value then increment the debug 
+				 * level.
+				 */
+				global.synths->flags = (global.synths->flags & ~REQ_DEBUG_MASK)
+					| (((global.synths->flags & REQ_DEBUG_MASK) + REQ_DEBUG_1)
+						& REQ_DEBUG_MASK);
+			}
+
+			if (global.synths->flags & REQ_DEBUG_MASK)
+				printf("debuging level set to %i\n",
+					(global.synths->flags&REQ_DEBUG_MASK)>>12);
+		}
+
+		if (strcmp(argv[argCount], "-gnrp") == 0)
+			global.synths->flags ^= GUI_NRP;
+		if (strcmp(argv[argCount], "-enrp") == 0)
+			global.synths->flags ^= MIDI_NRP;
+		if (strcmp(argv[argCount], "-nrp") == 0)
+			global.synths->flags |= MIDI_NRP|GUI_NRP;
 
 		if ((strcmp(argv[argCount], "-channel") == 0) && (argCount < argc))
 		{
@@ -595,31 +943,47 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			argCount++;
 		}
 
-		if (strcmp(argv[argCount], "-engine") == 0)
-			global.flags |= BRIGHTON_NOENGINE;
-
 		if (strcmp(argv[argCount], "-port") == 0)
 		{
-			if (argCount < argc)
-				global.port = atoi(argv[++argCount]);
+			if ((argCount < (argc - 1)) && (argv[argCount + 1] != NULL))
+				global.port = atoi(argv[argCount + 1]);
+			argCount++;
 		}
 
 		if (strcmp(argv[argCount], "-host") == 0)
 			global.host = argv[++argCount];
 
+		if (strcmp(argv[argCount], "-window") == 0)
+			synthesisers[global.synths->synthtype]->flags ^= BRIGHTON_WINDOW;
+
+		if (strcmp(argv[argCount], "-cli") == 0)
+		{
+			cli = 1;
+			brightonCLIinit(&global, cli_fd);
+			//synthesisers[global.synths->synthtype]->flags ^= BRIGHTON_WINDOW;
+		}
+
+		if (strcmp(argv[argCount], "-nwf") == 0)
+			global.synths->lwf = 0;
 		if (strcmp(argv[argCount], "-lwf") == 0)
 			global.synths->lwf = 1;
+		if (strcmp(argv[argCount], "-lwf2") == 0)
+			global.synths->lwf = 2;
+		if (strcmp(argv[argCount], "-wwf") == 0)
+			global.synths->lwf = 2;
+		if (strcmp(argv[argCount], "-hwf") == 0)
+			global.synths->lwf = 3;
 
 		if (strcmp(argv[argCount], "-lnp") == 0)
-			global.synths->notepref = 1;
+			global.synths->notepref = BRIGHTON_LNP;
 		if (strcmp(argv[argCount], "-hnp") == 0)
-			global.synths->notepref = 2;
+			global.synths->notepref = BRIGHTON_HNP;
 		if (strcmp(argv[argCount], "-nnp") == 0)
-			global.synths->notepref = 0;
+			global.synths->notepref = BRIGHTON_POLYPHONIC;
 		if (strcmp(argv[argCount], "-retrig") == 0)
-			global.synths->notetrig = 1;
+			global.synths->notetrig = 1 - global.synths->notetrig;
 		if (strcmp(argv[argCount], "-lvel") == 0)
-			global.synths->legatovelocity = 1;
+			global.synths->legatovelocity = 1 - global.synths->legatovelocity;
 
 		/*
 		 * And finally all the different synths
@@ -724,6 +1088,7 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			global.synths->synthtype = BRISTOL_VOX;
 
 		if ((strcmp(argv[argCount], "-voxm2") == 0)
+			|| (strcmp(argv[argCount], "-voxM2") == 0)
 			|| (strcmp(argv[argCount], "-vox300") == 0))
 			global.synths->synthtype = BRISTOL_VOXM2;
 
@@ -782,7 +1147,8 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		if (strcmp(argv[argCount], "-bme700") == 0)
 		{
 			global.synths->synthtype = BRISTOL_BME700;
-			global.synths->voices = 8;
+			global.synths->voices = 1;
+			global.synths->notepref = BRIGHTON_HNP;
 		}
 
 		if ((strcmp(argv[argCount], "-bassmaker") == 0)
@@ -797,7 +1163,7 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		{
 			global.synths->synthtype = BRISTOL_SID_M1;
 			global.synths->voices = 1;
-			global.synths->notepref = 2;
+			global.synths->notepref = BRIGHTON_LNP;
 			global.synths->notetrig = 1;
 			antialias = 0.5;
 			aliastype = 1;
@@ -814,7 +1180,7 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		{
 			global.synths->synthtype = BRISTOL_SID_M2;
 			global.synths->voices = 1;
-			global.synths->notepref = 2;
+			global.synths->notepref = BRIGHTON_HNP;
 			global.synths->notetrig = 1;
 			antialias = 0.5;
 			aliastype = 1;
@@ -979,11 +1345,56 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		argCount++;
 	}
 
+#ifndef BRIGHTON_HAS_X11
+	/* Reverse the window logic */
+	synthesisers[global.synths->synthtype]->flags ^= BRIGHTON_WINDOW;
+#endif
+
+	{
+		char statfile[1024];
+		struct stat statres;
+
+		sprintf(statfile, "%s/%s.gz", getenv("BRISTOL"),
+			synthesisers[global.synths->synthtype]->resources[0].image);
+
+		if ((synthesisers[global.synths->synthtype]->resources[0].image != 0) &&
+			(stat(statfile, &statres) != 0))
+		{
+			sprintf(statfile, "%s/%s", getenv("BRISTOL"),
+				synthesisers[global.synths->synthtype]->resources[0].image);
+
+			if (stat(statfile, &statres) != 0)
+			{
+				printf("unknown emulator\n");
+				sleep(1);
+				exit(-1);
+			}
+		}
+	}
+
+	if (readMe) {
+		printBrightonHelp(global.synths->synthtype);
+		exit(0);
+	}
+
+	printf("brighton version %s\n", VERSION);
+
+	sleep(1);
+
+	printf("  %s", argv[0]);
+	for (i = 1; i < argc; i++) {
+		if (argv[i][0] == '-')
+			printf("\n    %s", argv[i]);
+		else
+			printf(" %s", argv[i]);
+	}
+	printf("\n");
+
 	/*
-	 * Hm, this is a joke really. The Tandy, Realistic, whatever is an old
-	 * synth, and this mimics random slider changes. Should not really do it
-	 * but its fun. The original used to lose its white caps.
-	 * Once the gui is created its more work to change the image.
+	 * Hm, this is a joke really. The Tandy, Realistic, whatever, is an old
+	 * synth and this mimics random slider changes. Should not really do it
+	 * but its fun: the original used to lose its white caps. Once the gui is
+	 * created its more work to change the image.
 	 */
 	if (global.synths->synthtype == BRISTOL_REALISTIC)
 	{
@@ -1010,32 +1421,130 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		exit(0);
 
 	global.synths->resources = synthesisers[global.synths->synthtype];
+
 	synthesisers[global.synths->synthtype]->width *= scale;
 	synthesisers[global.synths->synthtype]->height *= scale;
 
-	/*
-	 * win is actually set by the configuration routines, but we use it 
-	 * here anyway. The configuration options are a structure, they were a
-	 * list of parameters however that became unwieldy.
-	 */
-	global.synths->win =
-		brightonInterface(synthesisers[global.synths->synthtype],
-			quality, library, aliastype, antialias, gs);
-
-	global.synths->win->dcTimeout = dcTimeout;
-
-	if (nar)
-		global.synths->win->flags |= BRIGHTON_NO_ASPECT;
-
 	/*
 	 * Finally go and let the event manager handle our interface. Going to 
 	 * create a separate GUI thread, which will allow us to handle things like
 	 * MIDI events from the engine, timed operations, etc, from there.
 	 */
-	if (pthread_create(&thread, NULL, eventMgr, NULL) != 0)
+	if (pthread_create(&emgrThread, NULL, eventMgr, &global) != 0)
 		printf("Could not create GUI thread\n");
+
+	/*
+	 * We should not detach the thread, we should wait for it later before exit
 	if (pthread_detach(thread) != 0)
 		printf("Could not detach GUI thread\n");
+	 */
+
+	/*
+	 * win is actually set by the configuration routines, but we use it 
+	 * here anyway. The configuration options are a structure, they were a
+	 * list of parameters however that became unwieldy.
+	 */
+	if ((global.synths->flags & REQ_DEBUG_MASK) >= (REQ_DEBUG_4|REQ_DEBUG_1))
+		aliastype |= BRIGHTON_LIB_DEBUG;
+	synthesisers[global.synths->synthtype]->flags |= bwflags;
+
+	/*
+	 * We play around with the names here so that the window title gets filled
+	 * with more information than just the emulator name. This enhancement was
+	 * added for JSM but could be any Jack installation. If multiple instances
+	 * of the same emulator are started then it is not clear which is which:
+	 * both have the same title bar but different names in Jack. What we do here
+	 * is add the Jack registration name to the title bar.
+	 */
+	snprintf(appname, 64, "%s (%s)",
+		synthesisers[global.synths->synthtype]->name, devname);
+	devname = synthesisers[global.synths->synthtype]->name;
+	synthesisers[global.synths->synthtype]->name = appname;
+	if ((global.synths->win =
+		brightonInterface(synthesisers[global.synths->synthtype],
+			quality, library, aliastype, antialias, gs)) == NULL)
+	{
+		printf("could not create window\n");
+		exit(-1);
+	}
+	synthesisers[global.synths->synthtype]->name = devname;
+	global.synths->resources->name = devname;
+
+	/*
+	 * We now need to decide which memory to load. There are a few choices.
+	 * If we had a LADI declaration it will have been copied to both ladimem
+	 * and to the emulator memory. If they are still the same then use it.
+	 *
+	 * If the emulator location has change then we should look and see if the
+	 * LADI memory actually exists, if so, use it (overwrite the emulator
+	 * memory location again). If it does not exist we will then continue to 
+	 * access the emulator memory and it becomes the template for the eventual
+	 * LADI state information.
+	 */
+	if ((global.synths->flags & LADI_ENABLE) &&
+		(global.synths->ladimem
+			!= (global.synths->location + global.synths->mbi)))
+	{
+		int memHold = global.synths->location + global.synths->mbi;
+
+		global.synths->mbi = 0;
+		global.synths->location = global.synths->ladimem;
+
+		/* Location has changed. Stat the ladimem */
+		if (loadMemory(global.synths, global.synths->resources->name, 0,
+			global.synths->ladimem,
+			global.synths->mem.active, 0, BRISTOL_STAT) < 0)
+		{
+			/* No LADI mem */
+			global.synths->location = memHold;
+
+			global.synths->mbi = (global.synths->location / 1000) * 1000;
+			global.synths->location = global.synths->location % 1000;
+
+			printf("did not find LADI memory %i, using %i\n",
+				global.synths->ladimem, memHold);
+		} else
+			printf("using LADI state memory %i\n", global.synths->ladimem);
+	}
+
+	if (global.flags & REQ_EXIT)
+	{
+		printf("early termination logging thread (%i)\n", global.controlfd);
+
+		bristolOpenStdio(BRISTOL_LOG_TERMINATE);
+
+		if ((logthread != 0) && (pthread_join(logthread, NULL) != 0))
+			return(1);
+
+		exit(global.controlfd < 0? 1: 0);
+	}
+
+	global.synths->win->dcTimeout = dcTimeout;
+
+	global.synths->win->flags |= rlflags;
+
+	if (azoom) {
+		global.synths->win->flags |= BRIGHTON_AUTOZOOM;
+
+		if (deswidth >= 100)
+		{
+			global.synths->win->minw = deswidth;
+			global.synths->win->minh =
+				deswidth * global.synths->win->template->height
+				/ global.synths->win->template->width;
+			global.synths->win->maxw =
+				synthesisers[global.synths->synthtype]->width * scale;
+			global.synths->win->maxh =
+				synthesisers[global.synths->synthtype]->height * scale;
+		} else {
+			global.synths->win->minw 
+				= global.synths->win->maxw 
+				= global.synths->win->minh 
+				= global.synths->win->maxh = 0;
+		}
+	}
+	if (nar)
+		global.synths->win->flags |= BRIGHTON_NO_ASPECT;
 
 	brightonOpacity(global.synths->win, ((float) opacity) / 100.0f);
 
@@ -1052,33 +1561,192 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 
 	printf("user r %i/%i, e %i/%i\n", getuid(), getgid(), geteuid(), getegid());
 
-	/*
-	 * Here we should open an ALSA SEQ interface. We want to get an index back
-	 * and pass that to the event handler for selection. This should not really
-	 * be a SEQ interface, it should depend on the flags given to the GUI.
-	 */
-	if (global.libtest != 1)
-	{
-		if ((midiHandle = bristolMidiOpen("brighton",
-			BRISTOL_CONN_SEQ|BRISTOL_DUPLEX,
-			-1, BRISTOL_REQ_NSX, brightonMidiInput, &global)) < 0)
-			printf("Error opening midi device %s\n", "0.0");
-
-		midiFD = bristolGetMidiFD(midiHandle);
-//		cFD = bristolMidiDupFD(global.controlfd, BRISTOL_REQ_NSX,
-//			 brightonMidiInput, &global);
-//		cFD = bristolGetMidiFD(cFD);
-
-		printf("opened GUI midi handles: %i, %i\n", midiFD, cFD);
-	}
+	global.synths->win->nrpcount = nrpcc;
+	global.synths->win->nrpcontrol =
+		malloc(sizeof(brightonNRPcontrol) * nrpcc);
+	memset(global.synths->win->nrpcontrol, 0,
+		sizeof(brightonNRPcontrol) * nrpcc);
 
 	brightonReadConfiguration(global.synths->win,
 		synthesisers[global.synths->synthtype],
 		global.synths->midichannel,
 		synthesisers[global.synths->synthtype]->name);
 
-	while (1) {
-		i = brightonEventMgr();
+	if (bristolMidiSendMsg(global.controlfd, global.synths->sid,
+		BRISTOL_ACTIVE_SENSE, 0, activeSense == 0? 0:16383) != 0)
+	{
+		printf("Active sense transmit failure\n");
+		sleep(1);
+		exit(1);
+	} if (global.synths->flags & REQ_MIDI_DEBUG)
+		printf("Sent Active sense\n");
+
+	if ((global.synths->sid2 > 0) &&
+		(bristolMidiSendMsg(global.manualfd, global.synths->sid2,
+			BRISTOL_ACTIVE_SENSE, 0, activeSense == 0? 0:16383) != 0))
+	{
+		printf("Active sense transmit failure\n");
+		sleep(1);
+		exit(1);
+	} if (global.synths->flags & REQ_MIDI_DEBUG)
+		printf("Sent Active sense on second channel\n");
+
+	emStart = 1;
+
+	if (global.libtest != 1)
+	{
+		if (global.synths->flags & REQ_FORWARD)
+			bristolMidiOption(0, BRISTOL_NRP_FORWARD, 1);
+
+		if (global.synths->flags & NO_KEYTRACK)
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 0);
+		else
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 1);
+
+		/*
+		 * The Hammond needs a note event on the upper manual to kick the
+		 * gearbox into action.
+		 */
+		if (global.synths->synthtype == BRISTOL_HAMMONDB3)
+		{
+			sleep(1);
+			bristolMidiSendMsg(global.controlfd, global.synths->midichannel,
+				BRISTOL_EVENT_KEYON, 0, 10 + global.synths->transpose);
+			usleep(50000);
+			bristolMidiSendMsg(global.controlfd, global.synths->midichannel,
+				BRISTOL_EVENT_KEYOFF, 0, 10 + global.synths->transpose);
+		}
+	}
+
+	/* if we are CLI, somebody needs to force this to happen */
+	if (synthesisers[global.synths->synthtype]->flags & BRIGHTON_WINDOW)
+		brightonWorldChanged(global.synths->win, 500, 500);
+
+	/*
+	if (cli == 0)
+	{
+		close(cli_fd[0]);
+		close(cli_fd[1]);
+	}
+	*/
+
+	while (~global.flags & REQ_EXIT)
+	{
+		/*
+		 * Do whatever we want. Will turn into a wait routine on the MIDI
+		 * channel. This could be merged with the UI heartbeat code.
+		 */
+		if (cli) {
+			if (brightonCLIcheck(&global) < 0)
+				cleanupBristol();
+		}
+		/*
+		 * This was never operational anyway:
+		if (vuInterval != 0) {
+			usleep(vuInterval);
+			if (global.synths->flags & OPERATIONAL)
+				doAlarm();
+		} else if (cli == 0)
+		 */
+		else
+			sleep(1);
+	}
+
+	if (pthread_join(emgrThread, NULL) != 0)
+		printf("brighton event thread exit error\n");
+	else
+		printf("brighton event thread exited\n");
+
+	/* Now that the event manager thread has exit, put key repeat back on */
+	BAutoRepeat(global.synths->win->display, 1);
+
+	printf("terminating logging thread (%i)\n", global.controlfd);
+
+	if (global.libtest == 0)
+		sleep(1);
+
+	bristolOpenStdio(BRISTOL_LOG_TERMINATE);
+
+	if ((logthread != 0) && (pthread_join(logthread, NULL) != 0))
+		return(1);
+
+	exit(global.controlfd < 0? 1: 0);
+}
+
+void *
+eventMgr()
+{
+	bristolMidiMsg msg;
+	int i = 50, r;
+	int midiFD, cFD;
+
+	/* This will send activesense as soon as the interface initialises */
+	asc = activeSense * 4;
+
+	printf("starting event management thread\n");
+
+	if (global.libtest == 0)
+	{
+		while (global.controlfd < 0)
+		{
+			usleep(100000);
+
+			if (i-- < 0)
+			{
+				global.flags |= REQ_EXIT;
+				pthread_exit(0);
+			}
+		}
+
+		/*
+		 * Here we should open an ALSA SEQ interface. We want to get an index
+		 * back and pass that to the event handler for selection. This should
+		 * not really be a SEQ interface, it should depend on the flags given
+		 * to the GUI.
+		 */
+		if (gmc)
+		{
+			if ((midiHandle = bristolMidiOpen("brighton",
+				BRISTOL_CONN_SEQ|BRISTOL_RDONLY,
+				-1, -1, brightonMidiInput, &global)) < 0)
+				printf("Error opening midi device %s\n", "0.0");
+		}
+	}
+
+	while (global.synths->win == NULL)
+	{
+		usleep(100000);
+		printf("waiting for window creation\n");
+	}
+
+	midiFD = bristolGetMidiFD(midiHandle);
+	cFD = bristolGetMidiFD(global.controlfd);
+
+	printf("opened GUI midi handles: %i, %i\n", midiFD, cFD);
+
+/*
+	if (global.libtest != 1)
+	{
+		if (global.synths->flags & REQ_FORWARD)
+			bristolMidiOption(0, BRISTOL_NRP_FORWARD, 1);
+
+		if (global.synths->flags & NO_KEYTRACK)
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 0);
+		else
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 1);
+
+		bristolMidiSendMsg(global.controlfd, global.synths->midichannel,
+			BRISTOL_EVENT_KEYON, 0, 10 + global.synths->transpose);
+		bristolMidiSendMsg(global.controlfd, global.synths->midichannel,
+			BRISTOL_EVENT_KEYOFF, 0, 10 + global.synths->transpose);
+	}
+*/
+
+	while ((global.flags & REQ_EXIT) == 0) {
+		if (emStart)
+			i = brightonEventMgr();
+		else
+			i = 0;
 
 		/*
 		 * This will now become a select on the ALSA SEQ socket looking for
@@ -1110,9 +1778,39 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		 * for a select operation. We should look at some point in putting
 		 * this into a separate thread and using a semaphore since the 'busy'
 		 * waiting is ugly.
+		 *
+		 * We should also look into separating the MIDI and GUI threads more,
+		 * it would require a bit of internal signalling to prevent both wanting
+		 * X Server access at the same time.
 		 */
-//		bristolMidiDevRead(cFD, &msg);
-		bristolMidiDevRead(midiFD, &msg);
+		 /*
+		  * CHANGE TO READ ON EVERY OPENED HANDLE
+		bristolMidiDevRead(cFD, &msg);
+		bristolMidiDevRead(2, &msg);
+		  */
+		while ((r = bristolMidiDevRead(midiFD, &msg)) > 0)
+			;
+
+		if (r == BRISTOL_MIDI_CHANNEL)
+		{
+			if ((global.synths->flags & REQ_DEBUG_MASK) >= REQ_DEBUG_4)
+				printf("Read failed on Midi FD\n");
+			global.flags |= REQ_EXIT;
+
+			pthread_exit(0);
+		}
+
+		while ((r = bristolMidiTCPRead(&msg)) > 0)
+			;
+
+		if ((r < 0) && (global.libtest == 0))
+		{
+			if ((global.synths->flags & REQ_DEBUG_MASK) >= REQ_DEBUG_4)
+				printf("Read failed on TCP fd (no devices?)\n");
+			global.flags |= REQ_EXIT;
+
+			pthread_exit(0);
+		}
 
 		if (i == 0)
 			usleep(mwt);
@@ -1129,7 +1827,7 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 		 */
 		brightonFastTimer(0, 0, 0, BRIGHTON_FT_CLOCK, mwt / 1000);
 
-		if ((activeSense > 0) && ((asc -= mwt / 1000) < 0))
+		if ((activeSense > 0) && ((asc -= mwt / 1000) < 0) && (emStart))
 		{
 			asc = activeSense;
 
@@ -1145,16 +1843,44 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 				BRISTOL_ACTIVE_SENSE, 0, activeSensePeriod) != 0)
 			{
 				printf("Active sense transmit failure\n");
+				sleep(1);
 				exit(1);
 			}
 			if ((global.synths->sid2 > 0) &&
-				(bristolMidiSendMsg(global.controlfd, global.synths->sid2,
+				(bristolMidiSendMsg(global.manualfd, global.synths->sid2,
 					BRISTOL_ACTIVE_SENSE, 0, activeSensePeriod) != 0))
 			{
 				printf("Active sense transmit failure\n");
+				sleep(1);
 				exit(1);
 			}
 
+			if ((ladiRequest == 1) && global.synths->flags & LADI_ENABLE) {
+				int memHold = global.synths->cmem;
+
+				ladiRequest = 0;
+
+				printf("LADI save state signalled\n");
+
+				global.synths->cmem = global.synths->ladimem;
+				brightonControlKeyInput(global.synths->win, 's', 0);
+
+				global.synths->cmem = memHold;
+			}
+
+			if ((ladiRequest == 2) && global.synths->flags & LADI_ENABLE) {
+				int memHold = global.synths->cmem;
+
+				ladiRequest = 0;
+
+				printf("LADI load state signalled\n");
+
+				global.synths->cmem = global.synths->ladimem;
+				brightonControlKeyInput(global.synths->win, 'l', 0);
+
+				global.synths->cmem = memHold;
+			}
+
 			/*
 			 * We are going to piggyback the slow event timer onto this code.
 			 * It means the slow events will be related to the activeSensing
@@ -1164,28 +1890,12 @@ printf("as %i, asp %i\n", activeSense, activeSensePeriod);
 			brightonSlowTimer(0, 0, BRIGHTON_ST_CLOCK);
 		}
 	}
-}
 
-extern int vuInterval;
+	printf("brighton event manager thread exiting\n");
 
-void *
-eventMgr()
-{
-	while (1)
-	{
-		/*
-		 * Do whatever we want. Will turn into a wait routine on the MIDI
-		 * channel. This could be merged with the UI heartbeat code.
-		 */
-		if (vuInterval != 0)
-		{
-			usleep(vuInterval);
-			if (global.synths->flags & OPERATIONAL)
-				doAlarm();
-		}
-		else
-			sleep(1);
-	}
+	global.flags |= REQ_EXIT;
+
+	pthread_exit(0);
 }
 
 void
@@ -1200,6 +1910,39 @@ cleanout(void *id)
 void
 clearout(int result)
 {
+	/*
+	 * This is not the right way to do things. We should request an exit and
+	 * let all the processes do their work. The issue here is that this can
+	 * get called very early in the window creation process which causes other
+	 * issues if we try to close down nicely. A better option here would be
+	 * to assert the result is '0'. That way we get a nice core memory dump
+	 * to debug under other circumstances.
+	 */
+	printf("clearing out via early exit\n");
 	exit(result);
 }
 
+void
+printBrightonHelp(int synth)
+{
+	if ((synth >= BRISTOL_SYNTHCOUNT) || (readme[synth] == NULL))
+		return;
+
+	printf("\n%s\n", readme[synth]);
+}
+
+void
+printBrightonReadme()
+{
+	int i;
+
+	printf("\n%s\n\n\n", readmeheader);
+	for (i = 0; i < BRISTOL_SYNTHCOUNT; i++)
+	{
+		if ((synthesisers[i] != NULL) && (readme[i] != NULL))
+			printf("%s\n\n\n", readme[i]);
+	}
+	printf("\n%s\n", readmetrailer);
+	printf("%s", helptext);
+}
+
