@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -178,106 +177,106 @@ static struct {
  * This example is for a arp2600Bristol type synth interface.
  */
 static brightonLocations locations[DEVICE_COUNT] = {
-	{"", 1, C0 - 2, R0, W2, L1, 0, 1, 0,
+	{"EnvFollow Gain", 1, C0 - 2, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl2.xpm", 0, 0},
-	{"", 1, C2 - 2, R0, W2, L1, 0, 1, 0,
+	{"RM in1 level", 1, C2 - 2, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl2.xpm", 0, 0},
-	{"", 1, C3 - 1, R0, W1, L1, 0, 1, 0,
+	{"RM in2 level", 1, C3 - 1, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl2.xpm", 0, 0},
 
 	/* Osc-1 - 3 */
-	{"", 1, C4 - 5, R5, 7, 80, 0, 4, 0,
+	{"VCO1 Transpose", 1, C4 - 5, R5, 7, 80, 0, 4, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
 /*		"bitmaps/knobs/sliderblackl2.xpm", 0, 0}, */
 /*	{"", 1, C4 - 1, R0, W1, L1, 0, 1, 0, */
 /*		"bitmaps/knobs/sliderblackl2.xpm", 0, 0}, */
-	{"", 2, C4 - 6, R6 - 8, 9, 45, 0, 2, 0,
+	{"VCO1 Tracking", 2, C4 - 6, R6 - 8, 9, 45, 0, 2, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, BRIGHTON_THREEWAY},
-	{"", 1, C5, R0, W1, L1, 0, 1, 0,
+	{"VCO1 FM Mod1 lvl", 1, C5, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl.xpm", 0, 0},
-	{"", 1, C6, R0, W1, L1, 0, 1, 0,
+	{"VCO1 FM Mod2 lvl", 1, C6, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl.xpm", 0, 0},
-	{"", 1, C7, R0, W1, L1, 0, 1, 0,
+	{"VCO1 FM Mod3 lvl", 1, C7, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl.xpm", 0, 0},
-	{"", 2, C5 + 17, R5 - 15, 16, 14, 0, 1, 0,
+	{"VCO1 Sync 2->1", 2, C5 + 17, R5 - 15, 16, 14, 0, 1, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, 0},
-	{"", 1, 138, 77, 104, 30, 0, 1, 0,
+	{"VCO1 Tune Coarse", 1, 138, 77, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpointL.xpm", 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 1, 138, 120, 104, 30, 0, 1, 0,
+	{"VCO1 Tune Fine", 1, 138, 120, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpointL.xpm", 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
 
 	/* Osc-2 - 11 */
-	{"", 1, C8 - 6, R5, 7, 80, 0, 4, 0,
+	{"VCO2 Transpose", 1, C8 - 6, R5, 7, 80, 0, 4, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
 /*		"bitmaps/knobs/sliderblackl.xpm", 0, 0}, */
-/*	{"", 1, C8, R0, W1, L1, 0, 1, 0, */
+/*	{"VCO2-Transpose", 1, C8, R0, W1, L1, 0, 1, 0, */
 /*		"bitmaps/knobs/sliderblackl.xpm", 0, 0}, */
-	{"", 2, C8 - 7, R6 - 8, 9, 45, 0, 2, 0,
+	{"VCO2 Tracking", 2, C8 - 7, R6 - 8, 9, 45, 0, 2, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, BRIGHTON_THREEWAY},
-	{"", 1, C9, R0, W0, L1, 0, 1, 0,
+	{"VCO2 FM Mod1 lvl", 1, C9, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl.xpm", 0, 0},
-	{"", 1, C10, R0, W0, L1, 0, 1, 0,
+	{"VCO2 FM Mod2 lvl", 1, C10, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C11, R0, W0, L1, 0, 1, 0,
+	{"VCO2 FM Mod2 lvl", 1, C11, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C12, R0, W0, L1, 0, 1, 0,
+	{"VCO2 PWM", 1, C12, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, 250, 77, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"VCO2 Tune Coarse", 1, 250, 77, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 1, 250, 120, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"VCO2 Tune Fine", 1, 250, 120, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 1, 250, 165, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"VCO2 PW", 1, 250, 165, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 	
 	/* Osc-3 - 20 */
-	{"", 1, C13 - 6, R5, 7, 80, 0, 4, 0,
+	{"VCO3 Transpose", 1, C13 - 6, R5, 7, 80, 0, 4, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
 /*		"bitmaps/knobs/sliderblack2.xpm", 0, 0}, */
-/*	{"", 1, C13, R0, W1, L1, 0, 1, 0, */
+/*	{"VCO3 Transpose", 1, C13, R0, W1, L1, 0, 1, 0, */
 /*		"bitmaps/knobs/sliderblack2.xpm", 0, 0}, */
-	{"", 2, C13 - 7, R6 - 8, 9, 45, 0, 2, 0,
+	{"VCO3 Tracking", 2, C13 - 7, R6 - 8, 9, 45, 0, 2, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, BRIGHTON_THREEWAY},
-	{"", 1, C14, R0, W0, L1, 0, 1, 0,
+	{"VCO3 FM Mod1 lvl", 1, C14, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C15, R0, W0, L1, 0, 1, 0,
+	{"VCO3 FM Mod2 lvl", 1, C15, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C16, R0, W0, L1, 0, 1, 0,
+	{"VCO3 FM Mod3 lvl", 1, C16, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C17, R0, W0, L1, 0, 1, 0,
+	{"VCO3 PWM", 1, C17, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, 376, 77, 104, 30, 0, 1, 0,
+	{"VCO3 Tune Coarse", 1, 376, 77, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpoint.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 1, 376, 120, 104, 30, 0, 1, 0,
+	{"VCO3 Tune Fine", 1, 376, 120, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpoint.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 1, 376, 165, 104, 30, 0, 1, 0,
+	{"VCO3 PW", 1, 376, 165, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpoint.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 
 	/* VCF - 29 */
 /*	{"", 1, C23 + 11, R1 + 14, 8, 78, 0, 4, 0, */
 /*		"bitmaps/knobs/sliderblack2.xpm", 0, 0}, */
-	{"", 1, C18, R0, W0, L1, 0, 1, 0,
+	{"VCF Mix1 lvl", 1, C18, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C19, R0, W0, L1, 0, 1, 0,
+	{"VCF Mix2 lvl", 1, C19, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C20, R0, W0, L1, 0, 1, 0,
+	{"VCF Mix3 lvl", 1, C20, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C21, R0, W0, L1, 0, 1, 0,
+	{"VCF Mix4 lvl", 1, C21, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C22, R0, W0, L1, 0, 1, 0,
+	{"VCF Mix5 lvl", 1, C22, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C23, R0, W0, L1, 0, 1, 0,
+	{"VCF Mod1 lvl", 1, C23, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C24, R0, W0, L1, 0, 1, 0,
+	{"VCF Mod2 lvl", 1, C24, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, C25, R0, W0, L1, 0, 1, 0,
+	{"VCF Mod3 lvl", 1, C25, R0, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, 534, 77, 104, 30, 0, 1, 0,
+	{"VCF Cutoff", 1, 534, 77, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpoint.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
-	{"", 1, 534, 120, 104, 30, 0, 1, 0,
+	{"VCF Resonance", 1, 534, 120, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpoint.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 /*	{"", 1, 500, 165, 104, 30, 0, 1, 0, */
 /*		"bitmaps/knobs/sliderpoint.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE}, */
@@ -285,108 +284,108 @@ static brightonLocations locations[DEVICE_COUNT] = {
 /*		"bitmaps/knobs/sliderpoint.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE}, */
 
 	/* ADSR/AR - 39 */
-	{"", 1, C26, R1, W1, L1, 0, 1, 0,
+	{"Attack", 1, C26, R1, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C27, R1, W1, L1, 0, 1, 0,
+	{"Decay", 1, C27, R1, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C28, R1, W1, L1, 0, 1, 0,
+	{"Sustain", 1, C28, R1, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C29, R1, W1, L1, 0, 1, 0,
+	{"Release", 1, C29, R1, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C26, R0, W1, L1, 0, 1, 0,
+	{"AR Attack", 1, C26, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C29, R0, W1, L1, 0, 1, 0,
+	{"AR Release", 1, C29, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 2, C27 - 4, 474, 8, 25, 0, 1, 0,
+	{"AR Gate", 2, C27 - 4, 474, 8, 25, 0, 1, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 
 	/* ?? - 46 */
-	{"", 1, C30, R0, W1, L1, 0, 1, 0,
+	{"VCA Mix1 lvl", 1, C30, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C31, R0, W1, L1, 0, 1, 0,
+	{"VCA Mix2 lvl", 1, C31, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C32, R0, W1, L1, 0, 1, 0,
+	{"VCA Mod1 lvl", 1, C32, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, C33, R0, W2, L1, 0, 1, 0,
+	{"VCA Mod2 lvl", 1, C33, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
 	/* Mixer */
-	{"", 1, C34 + 1, R0, W2, L1, 0, 1, 0,
+	{"Mix lvl1", 1, C34 + 1, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
-	{"", 1, C35 + 1, R0, W2, L1, 0, 1, 0,
+	{"Mix lvl2", 1, C35 + 1, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
 
 	/* Effects return - 52 */
-	{"", 1, C36 + 2, R2, W2, L1, 0, 1, 0,
+	{"FX Return Left", 1, C36 + 2, R2, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
-	{"", 1, C37 + 2, R2, W2, L1, 0, 1, 0,
+	{"FX Return Right", 1, C37 + 2, R2, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
 
 	/* - 54 */
-	{"", 1, Cb0, R3, W0, L1, 0, 1, 0,
+	{"Chorus D1", 1, Cb0, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, Cb1, R3, W0, L1, 0, 1, 0,
+	{"Chorus D2", 1, Cb1, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, Cb2, R3, W0, L1, 0, 1, 0,
+	{"Chorus D3", 1, Cb2, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, Cb3, R3, W0, L1, 0, 1, 0,
+	{"Chorus D4", 1, Cb3, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
 
 	/* - 58 */
-	{"", 1, Cb4, R3, W0, L1, 0, 1, 0,
+	{"Reverb D1", 1, Cb4, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, Cb5, R3, W0, L1, 0, 1, 0,
+	{"Reverb D2", 1, Cb5, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr.xpm", 0, 0},
-	{"", 1, Cb6, R3, W1, L1, 0, 1, 0,
+	{"Reverb D3", 1, Cb6, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
-	{"", 1, Cb7, R3, W1, L1, 0, 1, 0,
+	{"Reverb D4", 1, Cb7, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackr2.xpm", 0, 0},
 	/* Manual start */
 	{"", 2, 712, 382, 12, 15, 0, 1, 0, "bitmaps/buttons/patchlow.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_NOSHADOW},
 
 	/* - 63 */
-	{"", 1, Cb9, R3, W1, L1, 0, 1, 0,
+	{"Noise White/Pink", 1, Cb9, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl2.xpm", 0, 0},
-	{"", 1, Cb10, R3, W1, L1, 0, 1, 0,
+	{"Noise Level", 1, Cb10, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblackl2.xpm", 0, 0},
 
 	/*; - 65 */
-	{"", 1, Cb11, R3, W0, L1, 0, 1, 0,
+	{"LFO Level", 1, Cb11, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
-	{"", 1, Cb12, R3, W0, L1, 0, 1, 0,
+	{"LFO Rate", 1, Cb12, R3, W0, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack2.xpm", 0, 0},
 
 	/* - 67 */
-	{"", 2, Cb12 + 5, 920, 16, 14, 0, 1, 0,
+	{"LFO Sync", 2, Cb12 + 5, 920, 16, 14, 0, 1, 0,
 		"bitmaps/buttons/klunk2.xpm", 0, 0},
 
 	/* Global (now program) Volume - 68 */
-	{"", 1, 786, 120, 104, 30, 0, 1, 0,
+	{"Volume Program", 1, 786, 120, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpointR.xpm", 0,BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
-	{"", 1, 888, 120, 104, 30, 0, 1, 0,
+	{"Volume Pan", 1, 888, 120, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpointR.xpm", 0,BRIGHTON_VERTICAL|BRIGHTON_REVERSE|
 		BRIGHTON_NOTCH},
 
 	/* Input gain and glide */
-	{"", 0, 47, 78, 50, 50, 0, 1, 0,
+	{"Input Gain", 0, 47, 78, 50, 50, 0, 1, 0,
 		"bitmaps/knobs/knobgreynew.xpm",
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, 101, 78, 50, 50, 0, 1, 0,
+	{"Glide", 0, 101, 78, 50, 50, 0, 1, 0,
 		"bitmaps/knobs/knobgreynew.xpm",
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 	/* Voltage processors */
-	{"", 1, 243, 711, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"Mix1 Gain1", 1, 243, 711, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
-	{"", 1, 243, 767, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"Mix1 Gain2", 1, 243, 767, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
-	{"", 1, 243, 821, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"Mix2 Gain2", 1, 243, 821, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
-	{"", 1, 243, 881, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
+	{"Mix3 LagRate", 1, 243, 881, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 
 	/* MARK used one dummy for VCA initial volume */
-	{"", 1, 786, 163, 104, 30, 0, 1, 0,
+	{"VCA Initial Vol", 1, 786, 163, 104, 30, 0, 1, 0,
 		"bitmaps/knobs/sliderpointR.xpm", 0,BRIGHTON_VERTICAL|BRIGHTON_REVERSE},
 	/* DUMMIES */
 	{"", 1, 0, 0, 104, 30, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
@@ -405,74 +404,116 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	 * First the outputs, then the inputs.
 	 */
 	/* 82 */
-	{"", 2, 110, 213, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"Output Preamp", 2, 110, 213, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 110, 280, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 110, 280, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 46, 439, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 46, 439, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* VCO1 */
-	{"", 2, 212, 249, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 212, 249, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 212, 316, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 212, 316, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* VCO2 */
-	{"", 2, 313, 249, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 313, 249, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 313, 316, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 313, 316, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 337, 249, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 337, 249, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 337, 316, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 337, 316, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* VCO3 */
-	{"", 2, 438, 249, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 438, 249, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 438, 316, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 438, 316, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 462, 249, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 462, 249, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 462, 316, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 462, 316, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* VCF */
-	{"", 2, 660, 287, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 660, 287, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* ADSR */
-	{"", 2, 760, 287, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 760, 287, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 735, 382, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 735, 382, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 864, 287, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 864, 287, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* Mix out */
-	{"", 2, 902, 235, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 902, 235, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* Noise/SH/Switch */
-	{"", 2, 146, 797, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 146, 797, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* SH/Switch - 324 */
-	{"", 2, 463, 797, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 463, 797, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 560, 666, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 560, 666, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 560, 906, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 560, 906, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 891, 339, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 891, 339, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 915, 339, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 915, 339, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* Voltage Processor Outputs */
-	{"", 2, 411, 734, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 411, 734, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 411, 792, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 411, 792, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 411, 832, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 411, 832, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
-	{"", 2, 411, 886, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 411, 886, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", 0},
 	/* Reverb R out */
-	{"", 2, 961, 505, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
+	{"", 2, 961, 505, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
+	/* CV/Audio inputs */
+	{"", 2, 30, 716, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
+		"bitmaps/buttons/patchonl.xpm", 0},
+	{"", 2, 30, 772, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
+		"bitmaps/buttons/patchonl.xpm", 0},
+	{"", 2, 30, 826, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
+		"bitmaps/buttons/patchonl.xpm", 0},
+	{"", 2, 30, 886, 12, 15, 0, 1, 0,
+		"bitmaps/buttons/patchoff.xpm",
+		"bitmaps/buttons/patchonl.xpm", 0},
 	/* DUMMIES */
 	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
@@ -488,22 +529,13 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
 		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
-	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
-		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
-	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
-		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
-	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
-		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
-	{"", 2, 0, 0, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoff.xpm",
-		"bitmaps/buttons/patchonl.xpm", BRIGHTON_WITHDRAWN},
 	/*
 	 * Inputs. From the envelope follower onwards, these offset start from '1'
 	 * rather than from zero, this corrected later in the code.
 	 */
-	/* Dummy */
 	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
-	{"", 2, 24, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
+	{"Input EnvFollow", 2, 24, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 67, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
@@ -511,7 +543,7 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	/* VCO1 - the KDB CV input is disabled - FFS in engine. */
 	{"", 2, 140, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 164, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 188, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
@@ -520,7 +552,7 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	/* VCO2 */
 	{"", 2, 242, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 266, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 289, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
@@ -531,7 +563,7 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	/* VCO3 12 */
 	{"", 2, 367, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 391, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 415, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
@@ -552,7 +584,7 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 588, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 611, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 635, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 659, 552, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
@@ -618,6 +650,15 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	{"", 2, 362, 802, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
+	/* CV/Audio inputs */
+	{"", 2, 950, 716, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
+	{"", 2, 950, 772, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
+	{"", 2, 950, 826, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
+	{"", 2, 950, 886, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
+		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON},
 	/* DUMMIES */
 	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
@@ -627,14 +668,6 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
 	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
 		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
-	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
-	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
-	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
-	{"", 2, 900, 900, 12, 15, 0, 1, 0, "bitmaps/buttons/patchoffb.xpm",
-		"bitmaps/buttons/patchon.xpm", BRIGHTON_CHECKBUTTON|BRIGHTON_WITHDRAWN},
 
 #define XS 808
 #define XD 30
@@ -938,7 +971,6 @@ arp2600Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 arp2600Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -974,8 +1006,6 @@ arp2600Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		 */
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -1030,33 +1060,6 @@ arp2600Callback(brightonWindow * win, int panel, int index, float value)
 	return(0);
 }
 
-static void
-arp2600FilterMix(guiSynth *synth, int fd, int chan, int c, int o, int v)
-{
-	/*
-	 * The filter will start to denormal if all the gains are set to zero. We
-	 * are going to check for this state and drive the noise signal up a couple
-	 * of decibels. It will actually help the filter 'self' oscillate at high
-	 * resonance anyway and was a target mod I wanted to add in.
-	 *
-	 * Firstly send the parameter, then see if we need to inject noise.
-	 *
-	 * This was moved into the emulator and the routine is now redundant.
-	 */
-	bristolMidiSendMsg(global.controlfd, synth->sid, c, o, v);
-
-	if ((synth->mem.param[29] + synth->mem.param[30] + synth->mem.param[31]
-		+ synth->mem.param[32] + synth->mem.param[33])
-			< ((float) (2.0/16383.0)))
-		bristolMidiSendMsg(global.controlfd, synth->sid, 103, 21, 1);
-	else
-		/*
-		 * Return noise to whatever value it should have had.
-		 */
-		bristolMidiSendMsg(global.controlfd, synth->sid, 103, 21,
-			 (int) (synth->mem.param[33] * C_RANGE_MIN_1));
-}
-
 static int
 arp2600Volume(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
@@ -1226,7 +1229,7 @@ arp2600Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the arp2600 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the arp2600 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1244,7 +1247,7 @@ arp2600Init(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
@@ -1351,8 +1354,8 @@ arp2600Init(brightonWindow *win)
 	*/
 
 	/* VCF mods */
-	dispatch[34].controller = 3;
-	dispatch[34].operator = 3;
+	dispatch[34].controller = 103;
+	dispatch[34].operator = 22;
 	dispatch[35].controller = 103;
 	dispatch[35].operator = 23;
 	dispatch[36].controller = 103;
@@ -1500,7 +1503,9 @@ arp2600Init(brightonWindow *win)
 	 * Need to specify env gain fixed, filter mod on, osc waveform.
 	 */
 	bristolMidiSendMsg(global.controlfd, synth->sid, 102, 0, 0); /* clear patch */
+	/* No filter kbd tracking - we emulate it */
 	bristolMidiSendMsg(global.controlfd, synth->sid, 3, 2, 16383);
+	bristolMidiSendMsg(global.controlfd, synth->sid, 3, 3, 0);
 	bristolMidiSendMsg(global.controlfd, synth->sid, 3, 4, 4);
 	bristolMidiSendMsg(global.controlfd, synth->sid, 3, 7, 4096);
 	bristolMidiSendMsg(global.controlfd, synth->sid, 3, 8, 100);
@@ -1576,6 +1581,8 @@ printf("going operational\n");
 /*		BRISTOL_EVENT_KEYOFF, 0, 10 + synth->transpose); */
 	configureGlobals(synth);
 
+	synth->loadMemory = (loadRoutine) arp2600LoadMemory;
+
 	return(0);
 }
 
