@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -130,85 +129,85 @@ extern guimain global;
  */
 static brightonLocations locations[DEVICE_COUNT] = {
 	/* Glide/Tranpose - 0 */
-	{"", 1, C0, R0, W1, L1, 0, 1, 0,
+	{"Glide", 1, C0, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 2, C1, R2 + 30, 15, L1/2 - 60, 0, 2, 0, 0, 0,
+	{"Transpose", 2, C1, R2 + 30, 15, L1/2 - 60, 0, 2, 0, 0, 0,
 		BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
 
 	/* VCO - 2 */
-	{"", 1, C2, R0, W1, L1, 0, 1, 0,
+	{"VCO LFO Sine Mod", 1, C2, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C3, R0, W1, L1, 0, 1, 0,
+	{"VCO LFO Square Mod", 1, C3, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C4, R0, W1, L1, 0, 1, 0,
+	{"VCO S/H Mod", 1, C4, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C5, R0, W1, L1, 0, 1, 0,
+	{"VCO ADSR Mod", 1, C5, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C6, R0, W1, L1, 0, 1, 0,
+	{"VCO PW", 1, C6, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C7, R0, W1, L1, 0, 1, 0,
+	{"VCO PWM LFO", 1, C7, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C8, R0, W1, L1, 0, 1, 0,
+	{"VCO PWM ADSR", 1, C8, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* LFO - 9 */
-	{"", 1, C9, R0, W1, L1, 0, 1, 0,
+	{"LFO Freq", 1, C9, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* Mixer - 10 */
-	{"", 1, C10, R0, W1, L1, 0, 1, 0,
+	{"Mix Noise", 1, C10, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C11, R0, W1, L1, 0, 1, 0,
+	{"Mix VCO Ramp", 1, C11, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C12, R0, W1, L1, 0, 1, 0,
+	{"Mix VCO Pulse", 1, C12, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* VCF - 13 */
-	{"", 1, C13, R0, W1, L1, 0, 1, 0,
+	{"VCF Cutoff", 1, C13, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C14, R0, W1, L1, 0, 1, 0,
+	{"VCO Resonance", 1, C14, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C15, R0, W1, L1, 0, 1, 0,
+	{"VCO KBD", 1, C15, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C16, R0, W1, L1, 0, 1, 0,
+	{"VCF LFO Sine", 1, C16, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C17, R0, W1, L1, 0, 1, 0,
+	{"VCF ADSR", 1, C17, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* VCA - 18 */
-	{"", 1, C18, R0, W1, L1, 0, 1, 0,
+	{"VCA Gain", 1, C18, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C19, R0, W1, L1, 0, 1, 0,
+	{"VCA ADSR", 1, C19, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* ADSR - 20 */
-	{"", 1, C20, R0, W1, L1, 0, 1, 0,
+	{"Attack", 1, C20, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C21, R0, W1, L1, 0, 1, 0,
+	{"Decay", 1, C21, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C22, R0, W1, L1, 0, 1, 0,
+	{"Sustain", 1, C22, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, C23, R0, W1, L1, 0, 1, 0,
+	{"Release", 1, C23, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* Pink/White noise - 24 */
-	{"", 2, C1, 220, 14, 100, 0, 1, 0,
+	{"Pink/White", 2, C1, 220, 14, 100, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* LFO mode - 25 */
-	{"", 2, C22, 220, 14, 100, 0, 1, 0,
+	{"LFO S/M", 2, C22, 220, 14, 100, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* LFO trigger - gate/auto - 26 */
-	{"", 2, C20, 220, 14, 100, 0, 1, 0,
+	{"LFO Trig", 2, C20, 220, 14, 100, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* Pitch Control - 27 */
-	{"", 2, 25, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch Flat", 2, 25, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
-	{"", 2, 53, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch Trill", 2, 53, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
-	{"", 2, 80, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch Sharp", 2, 80, R2, 30, 80, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
 
 	/* Memory Load, mem, Save - 30 */
@@ -275,7 +274,7 @@ brightonApp axxeApp = {
 	axxeConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{8, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	600, 350,
 	4, /* Panels */
 	{
@@ -342,12 +341,16 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 		case MIDI_PROGRAM:
 			printf("midi program: %x, %i\n", controller, value);
 			synth->location = value;
-			loadMemory(synth, "axxe", 0, synth->bank + synth->location,
-				synth->mem.active, 0, 0);
+			loadMemory(synth, synth->resources->name, 0,
+				synth->location, synth->mem.active, 0, 0);
 			break;
 		case MIDI_BANK_SELECT:
 			printf("midi banksel: %x, %i\n", controller, value);
-			synth->bank = value;
+			synth->bank = value * 10;
+
+			synth->location = (synth->location % 10) + value * 10;
+			loadMemory(synth, synth->resources->name, 0,
+				synth->location, synth->mem.active, 0, 0);
 			break;
 	}
 	return(0);
@@ -445,7 +448,6 @@ axxeMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 axxeMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -467,8 +469,8 @@ axxeMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	if (global.libtest == 0)
 	{
-#warning if we do not check for ack then socket might hang on exit
 		/*
+#warning if we do not check for ack then socket might hang on exit
 		 * To overcome that we should consider checking a sequence number in
 		 * the message library? That is non trivial since it requires that
 		 * our midi messges have a 'ack' flag included - we cannot check for
@@ -478,8 +480,6 @@ axxeMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		 */
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -609,7 +609,7 @@ axxeInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the axxe link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the axxe link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -627,7 +627,7 @@ axxeInit(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
