@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -136,88 +135,88 @@ static int shade_id;
 static
 brightonLocations locations[DEVICE_COUNT] = {
 	/* 0 */
-	{"", 2, C1b, R1b, 10, 60, 0, 1.01, 0,
+	{"Mod1 Tri/Square", 2, C1b, R1b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 0, C2, R1, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C3, R1, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C4b, R1b, 10, 60, 0, 1.01, 0,
+	{"Mod1 Rate", 0, C2, R1, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Mod2 Rate", 0, C3, R1, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Mod2 Tri/Square", 2, C4b, R1b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 4 */
-	{"", 2, C1b + 35, R2b, 10, 60, 0, 1.01, 0,
+	{"Mod1/Env", 2, C1b + 35, R2b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 0, C2b, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C4, R2b, 10, 60, 0, 1.01, 0,
+	{"ModMix", 0, C2b, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Single/Double", 2, C4, R2b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C4b + 12, R2b, 10, 60, 0, 1.01, 0,
+	{"Tri/Square", 2, C4b + 12, R2b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 8 */
-	{"", 0, C1, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C2, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C3, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C3, R4, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C1b, R4b, 10, 60, 0, 1.01, 0,
+	{"VCO Glide", 0, C1, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCO PW", 0, C2, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCO Vibrato", 0, C3, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCO Shape", 0, C3, R4, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Glide On/Off", 2, C1b, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C2 - 1, R4b, 10, 60, 0, 1.01, 0,
+	{"PW Man/Auto", 2, C2 - 1, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C2b, R4b, 10, 60, 0, 1.01, 0,
+	{"PW Env/Mod", 2, C2b, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C4, R4b, 10, 60, 0, 1.01, 0,
+	{"VCO 8'", 2, C4, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C4b + 12, R4b, 10, 60, 0, 1.01, 0,
+	{"VCO 16'/4'", 2, C4b + 12, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 17 */
-	{"", 0, C10, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Mix Noise/VCO", 0, C10, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
 
-	{"", 2, C10b, R3b, 10, 60, 0, 1.01, 0,
+	{"Res #/b", 2, C10b, R3b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 19 */
-	{"", 2, C10b - 60, R4b, 10, 60, 0, 1.01, 0,
+	{"Res 1", 2, C10b - 60, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C10b - 30, R4b, 10, 60, 0, 1.01, 0,
+	{"Res 2", 2, C10b - 30, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C10b, R4b, 10, 60, 0, 1.01, 0,
+	{"Res 4", 2, C10b, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C10b + 30, R4b, 10, 60, 0, 1.01, 0,
+	{"Res 8", 2, C10b + 30, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C10b + 60, R4b, 10, 60, 0, 1.01, 0,
+	{"Res 16", 2, C10b + 60, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 24 */
-	{"", 2, C20b, R1b, 10, 60, 0, 1.01, 0,
+	{"Env1 ASR/AR", 2, C20b, R1b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 0, C21, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C22, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C30b, R1b, 10, 60, 0, 1.01, 0,
+	{"Attack", 0, C21, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Decay", 0, C22, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Env2 ASR/AR", 2, C30b, R1b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 0, C31, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C32, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Attack", 0, C31, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Decay", 0, C32, R1, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
 
 	/* 30 */
-	{"", 0, C21, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C31, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCF Env Mix", 0, C21, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCA Env Mix", 0, C31, R2, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
 
 	/* 32 */
-	{"", 0, C20, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C21, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C22, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C22, R4, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C20b, R4b, 10, 60, 0, 1.01, 0,
+	{"Filt Resonace", 0, C20, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Filt Cutoff", 0, C21, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Filt Env", 0, C22, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Filt ModMix", 0, C22, R4, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"F-Mod Tri/Sqr", 2, C20b, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C21 - 7, R4b, 10, 60, 0, 1.01, 0,
+	{"F-Mod1/Mod2", 2, C21 - 7, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C21b + 12, R4b, 10, 60, 0, 1.01, 0,
+	{"F-KBD/Mod", 2, C21b + 12, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 39 */
-	{"", 0, C30, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 0, C32, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
-	{"", 2, C30b + 36, R4b, 10, 60, 0, 1.01, 0,
+	{"Mix VCF/Res", 0, C30, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCA ModMix", 0, C32, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"VCA Mod Mod1/Mod2", 2, C30b + 36, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C30b + 101, R4b, 10, 60, 0, 1.01, 0,
+	{"VCA Mod Tri/Sqr", 2, C30b + 101, R4b, 10, 60, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* 43 - 32 dummies masking opts */
@@ -255,10 +254,10 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 0, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* 59 Tuning, moved out of active devs */
-	{"", 0, C4, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 
+	{"Tuning", 0, C4, R3, SPW, SPH, 0, 1, 0, "bitmaps/knobs/yellowknob.xpm", 0, 
 		BRIGHTON_NOTCH},
 	/* 60 Global volume, moved out of active devs */
-	{"", 0, C31, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
+	{"Volume", 0, C31, R3, SPW, SPH, 0, 1, 0,"bitmaps/knobs/yellowknob.xpm", 0, 0},
 	/* Done */
 };
 
@@ -376,7 +375,7 @@ brightonApp bme700App = {
 	BME700Configure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{6, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	800, 417,
 	7,
 	{
@@ -558,12 +557,16 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 			 */
 			printf("midi program: %x, %i\n", controller, value);
 			synth->location = value;
-			loadMemory(synth, "realistic", 0, 0, ACTIVE_DEVS, 0, BRISTOL_FORCE);
+			loadMemory(synth, SYNTH_NAME, 0, synth->bank + synth->location,
+				OPTS_START, 0, BRISTOL_FORCE);
 			optsShim(synth);
 			break;
 		case MIDI_BANK_SELECT:
 			printf("midi banksel: %x, %i\n", controller, value);
 			synth->bank = value;
+			synth->location = value;
+			loadMemory(synth, SYNTH_NAME, 0, synth->bank + synth->location,
+				OPTS_START, 0, BRISTOL_FORCE);
 			break;
 	}
 	return(0);
@@ -926,7 +929,7 @@ BME700Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the BME700 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the BME700 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -944,7 +947,7 @@ BME700Init(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
