@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -995,7 +994,7 @@ bmCallback(brightonWindow * win, int panel, int index, float value)
 	guiSynth *synth = findSynth(global.synths, win);
 	int sendvalue;
 
-//printf("bmCallback(%i, %i, %f) %x\n", panel, index, value, (size_t) synth);
+//printf("bmCallback(%i, %i, %f) %x\n", panel, index, value, synth);
 
 	if (synth == 0)
 		return(0);
@@ -1034,7 +1033,7 @@ bmCallback(brightonWindow * win, int panel, int index, float value)
 			sendvalue);
 /*
 //	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -1107,6 +1106,10 @@ bmSendControl(guiSynth *synth, bmMem *bm, int c, int o)
 	 */
 	switch ((int) bm->control.ctype) {
 		case BM_C_CONTROL:
+			/*
+			 * Because of the way control.cc is implemented, this is actually
+			 * the second MIDI channel: this gives access to the first 15 CC
+			 */
 			bristolMidiSendMsg(global.controlfd, synth->midichannel, 0,
 				bm->control.cc, (int)
 					(bm->page[c].step[o].cont * C_RANGE_MIN_1));
@@ -1117,17 +1120,19 @@ bmSendControl(guiSynth *synth, bmMem *bm, int c, int o)
 
 			/*
 			 * Convert the setting into 12 steps, take cc to be midi channel
+			 * Channel is the 'cc' setting here.
 			 */
-			chan = bm->control.cc;
+			if ((chan = bm->control.cc) > 15)
+				chan = 15;
 			note = (int) (bm->page[c].step[o].cont * 12);
 			velocity = (int) (bm->page[c].step[o].velocity * 127);
 
 			note += synth->transpose
 				+ bm->page[c].step[o].transp * 12;
 
-			bristolMidiSendKeyMsg(global.controlfd, chan + 1,
+			bristolMidiSendKeyMsg(global.controlfd, chan,
 				BRISTOL_EVENT_KEYOFF, bm->lastcnote, velocity);
-			bristolMidiSendKeyMsg(global.controlfd, chan + 1,
+			bristolMidiSendKeyMsg(global.controlfd, chan,
 				BRISTOL_EVENT_KEYON, note, velocity);
 
 			bm->lastcnote = note;
@@ -1957,7 +1962,7 @@ bmInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the bm link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the bm link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(sizeof(bmMem));
 	synth->mem.count = sizeof(bmMem) / sizeof(float);
