@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -728,15 +727,12 @@ static int
 cs80DestroySynth(brightonWindow *win)
 {
 	guiSynth *synth = findSynth(global.synths, win);
-	bristolMidiMsg msg;
 
 	/*
 	 * Since we registered two synths, we now need to remove the upper
 	 * manual.
 	 */
 	bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0, BRISTOL_EXIT_ALGO);
-	if (bristolMidiRead(global.controlfd, &msg) < 0)
-		printf("socket closed\n");
 	return(0);
 }
 
@@ -1441,7 +1437,6 @@ printf("cs80Memory(%i, %i, %i)\n", c, o, v);
 static void
 cs80Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -1463,12 +1458,8 @@ cs80Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid2,
 			127, 0, (BRISTOL_MIDICHANNEL|newchan) + 1);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -1680,7 +1671,7 @@ cs80Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the cs80 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the cs80 link to bristol: %p\n", synth->win);
 
 	scratch = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
@@ -1698,7 +1689,7 @@ cs80Init(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 printf("	The SID are %i and %i\n", synth->sid, synth->sid2);
 
