@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -793,7 +792,6 @@ dxMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 dxMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -814,12 +812,8 @@ dxMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	}
 
 	if (global.libtest == 0)
-	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
-	}
 
 	synth->midichannel = newchan;
 
@@ -953,7 +947,7 @@ dxTune(guiSynth *synth)
 {
 	brightonEvent event;
 
-	printf("dxTune(%x, %i, %i)\n", (size_t) synth->win, OP_PANEL, OP1_START);
+	printf("dxTune(%p, %i, %i)\n", synth->win, OP_PANEL, OP1_START);
 
 	event.value = 0.5;
 	brightonParamChange(synth->win, 0, 1, &event);
@@ -999,7 +993,7 @@ dxInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the dx link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the dx link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1016,7 +1010,7 @@ dxInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = dxMidiSendMsg;
@@ -1097,14 +1091,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP1_START + 11].controller = 126;
 	dispatch[OP1_START + 11].operator = 3;
 	dispatch[OP1_START + 12].controller = 0;
-	dispatch[OP1_START + 12].operator = 9;
+	dispatch[OP1_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP1_START + 13].controller = 0;
-	dispatch[OP1_START + 13].operator = 10;
+	dispatch[OP1_START + 13].operator = 9;
 	dispatch[OP1_START + 14].controller = 0;
-	dispatch[OP1_START + 14].operator = 11;
+	dispatch[OP1_START + 14].operator = 10;
 	dispatch[OP1_START + 15].controller = 0;
-	dispatch[OP1_START + 15].operator = 12;
+	dispatch[OP1_START + 15].operator = 11;
 
 	/* operator = 2 */
 	dispatch[OP2_START].controller = 126;
@@ -1133,14 +1127,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP2_START + 11].controller = 126;
 	dispatch[OP2_START + 11].operator = 13;
 	dispatch[OP2_START + 12].controller = 1;
-	dispatch[OP2_START + 12].operator = 9;
+	dispatch[OP2_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP2_START + 13].controller = 1;
-	dispatch[OP2_START + 13].operator = 10;
+	dispatch[OP2_START + 13].operator = 9;
 	dispatch[OP2_START + 14].controller = 1;
-	dispatch[OP2_START + 14].operator = 11;
+	dispatch[OP2_START + 14].operator = 10;
 	dispatch[OP2_START + 15].controller = 1;
-	dispatch[OP2_START + 15].operator = 12;
+	dispatch[OP2_START + 15].operator = 11;
 
 	/* operator = 3 */
 	dispatch[OP3_START].controller = 126;
@@ -1169,14 +1163,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP3_START + 11].controller = 126;
 	dispatch[OP3_START + 11].operator = 23;
 	dispatch[OP3_START + 12].controller = 2;
-	dispatch[OP3_START + 12].operator = 9;
+	dispatch[OP3_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP3_START + 13].controller = 2;
-	dispatch[OP3_START + 13].operator = 10;
+	dispatch[OP3_START + 13].operator = 9;
 	dispatch[OP3_START + 14].controller = 2;
-	dispatch[OP3_START + 14].operator = 11;
+	dispatch[OP3_START + 14].operator = 10;
 	dispatch[OP3_START + 15].controller = 2;
-	dispatch[OP3_START + 15].operator = 12;
+	dispatch[OP3_START + 15].operator = 11;
 
 	/* operator = 4 */
 	dispatch[OP4_START].controller = 126;
@@ -1205,14 +1199,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP4_START + 11].controller = 126;
 	dispatch[OP4_START + 11].operator = 33;
 	dispatch[OP4_START + 12].controller = 3;
-	dispatch[OP4_START + 12].operator = 9;
+	dispatch[OP4_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP4_START + 13].controller = 3;
-	dispatch[OP4_START + 13].operator = 10;
+	dispatch[OP4_START + 13].operator = 9;
 	dispatch[OP4_START + 14].controller = 3;
-	dispatch[OP4_START + 14].operator = 11;
+	dispatch[OP4_START + 14].operator = 10;
 	dispatch[OP4_START + 15].controller = 3;
-	dispatch[OP4_START + 15].operator = 12;
+	dispatch[OP4_START + 15].operator = 11;
 
 	/* operator = 5 */
 	dispatch[OP5_START].controller = 126;
@@ -1241,14 +1235,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP5_START + 11].controller = 126;
 	dispatch[OP5_START + 11].operator = 43;
 	dispatch[OP5_START + 12].controller = 4;
-	dispatch[OP5_START + 12].operator = 9;
+	dispatch[OP5_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP5_START + 13].controller = 4;
-	dispatch[OP5_START + 13].operator = 10;
+	dispatch[OP5_START + 13].operator = 9;
 	dispatch[OP5_START + 14].controller = 4;
-	dispatch[OP5_START + 14].operator = 11;
+	dispatch[OP5_START + 14].operator = 10;
 	dispatch[OP5_START + 15].controller = 4;
-	dispatch[OP5_START + 15].operator = 12;
+	dispatch[OP5_START + 15].operator = 11;
 
 	/* operator = 6 */
 	dispatch[OP6_START].controller = 126;
@@ -1277,14 +1271,14 @@ dxInit(brightonWindow *win)
 	dispatch[OP6_START + 11].controller = 126;
 	dispatch[OP6_START + 11].operator = 53;
 	dispatch[OP6_START + 12].controller = 5;
-	dispatch[OP6_START + 12].operator = 9;
+	dispatch[OP6_START + 12].operator = 8;
 	/* L1-A-L2 */
 	dispatch[OP6_START + 13].controller = 5;
-	dispatch[OP6_START + 13].operator = 10;
+	dispatch[OP6_START + 13].operator = 9;
 	dispatch[OP6_START + 14].controller = 5;
-	dispatch[OP6_START + 14].operator = 11;
+	dispatch[OP6_START + 14].operator = 10;
 	dispatch[OP6_START + 15].controller = 5;
-	dispatch[OP6_START + 15].operator = 12;
+	dispatch[OP6_START + 15].operator = 11;
 
 	/* memories */
 	dispatch[MEM_START].operator = 0;
@@ -1361,7 +1355,7 @@ dxConfigure(brightonWindow *win)
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
-printf("going operational %x %x\n", (size_t) synth, (size_t) synth->win);
+	printf("going operational %p %p\n", synth, synth->win);
 
 	synth->flags |= OPERATIONAL;
 	synth->keypanel = 8;
@@ -1370,8 +1364,11 @@ printf("going operational %x %x\n", (size_t) synth, (size_t) synth->win);
 	synth->dispatch[ALGO_START].other2 = 0;
 	synth->dispatch[ALGO_START].other1 = -1;
 
-	dxLoadMem(synth, "dx", 0, synth->location, synth->mem.active,
-		FIRST_DEV, 0);
+	dxLoadMem(synth, "dx", 0, 0, synth->mem.active,
+		FIRST_DEV, BRISTOL_FORCE);
+
+	brightonPut(win,
+		"bitmaps/blueprints/dxshade.xpm", 0, 0, win->width, win->height);
 
 	/*
 	 * Hm. This is a hack for a few bits of bad rendering of a keyboard. Only
