@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -447,7 +446,6 @@ printf("bank %i, mem %i\n", synth->bank, synth->location);
 
 		synth->dispatch[MEM_START + 1].other1 = index;
 	} else {
-		bristolMidiMsg msg;
 		int newchan;
 
 		/*
@@ -473,8 +471,6 @@ printf("bank %i, mem %i\n", synth->bank, synth->location);
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 				break;
@@ -497,8 +493,6 @@ printf("bank %i, mem %i\n", synth->bank, synth->location);
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 				break;
@@ -607,6 +601,7 @@ granularMods(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	return(0);
 }
 
+/*
 static int
 granularOscWaveform(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
@@ -618,9 +613,6 @@ granularOscWaveform(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		return(0);
 	}
 
-	/*
-	 * This is a numeric. We need to force exclusion.
-	 */
 	if (synth->dispatch[53].other1 != -1)
 	{
 		event.command = BRIGHTON_PARAMCHANGE;
@@ -643,13 +635,7 @@ granularOscWaveform(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	return(0);
 }
-
-static int
-granularMidiNull(void *synth, int fd, int chan, int c, int o, int v)
-{
-/*	printf("%i, %i, %i\n", c, o, v); */
-	return(0);
-}
+*/
 
 static int
 granularMidiSendMsg(void *synth, int fd, int chan, int c, int o, int v)
@@ -696,7 +682,7 @@ granularCallback(brightonWindow *win, int panel, int index, float value)
 			sendvalue);
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -732,7 +718,7 @@ granularInit(brightonWindow* win)
 
 	synth->win = win;
 
-	printf("Initialise the granular link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the granular link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -749,7 +735,7 @@ granularInit(brightonWindow* win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 	{
