@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -329,7 +328,7 @@ brightonApp hammondApp = {
 	hammondConfigure, /* 3 callbacks, unused? */
 	0,
 	destroySynth,
-	{16, 0, 2, 2, 5, 520, 0, 0},
+	{32, 0, 2, 2, 5, 520, 0, 0},
 	808, 259,
 	5, /* four panels */
 	{
@@ -485,7 +484,6 @@ hammondMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static void
 hammondMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -506,12 +504,8 @@ hammondMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	}
 
 	if (global.libtest == 0)
-	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
-	}
 
 	synth->midichannel = newchan;
 
@@ -579,7 +573,7 @@ hammondCallback(brightonWindow * win, int panel, int index, float value)
 			sendvalue);
 #define DEBUG
 #ifdef DEBUG
-	printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+	printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 		global.controlfd, synth->sid,
 		synth->dispatch[index].controller,
 		synth->dispatch[index].operator,
@@ -1007,7 +1001,7 @@ hammondInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the hammond link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the hammond link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1024,7 +1018,7 @@ hammondInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = hammondNull; /*hammondMidiSendMsg; */
