@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -148,75 +147,74 @@ static guimain manual;
  * lossy.
  */
 static brightonLocations locations[C_COUNT] = {
-	{"", BRIGHTON_HAMMOND, C1, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondbrown.xpm", 
-		"bitmaps/knobs/hammondbar.xpm", BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C2, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondbrown.xpm", 
-		"bitmaps/knobs/hammondbar.xpm", BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C3, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C4, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C5, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C6, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C7, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C8, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C9, R1, W1, L1, 0, 8, 0,
-		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C11, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 16", BRIGHTON_HAMMOND, C1, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C12, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 5 1/3", BRIGHTON_HAMMOND, C2, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C13, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 8", BRIGHTON_HAMMOND, C3, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C14, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 4", BRIGHTON_HAMMOND, C4, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C15, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 2 2/3", BRIGHTON_HAMMOND, C5, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C16, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 2", BRIGHTON_HAMMOND, C6, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C17, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 1 3/5", BRIGHTON_HAMMOND, C7, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C18, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 1 1/3", BRIGHTON_HAMMOND, C8, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, C19, R1, W1, L1, 0, 8, 0,
+	{"Lower Drawbar 1", BRIGHTON_HAMMOND, C9, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+
+	{"Upper Drawbar 16", BRIGHTON_HAMMOND, C11, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 5 1/3", BRIGHTON_HAMMOND, C12, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 8", BRIGHTON_HAMMOND, C13, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 4", BRIGHTON_HAMMOND, C14, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 2 2/3", BRIGHTON_HAMMOND, C15, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 2", BRIGHTON_HAMMOND, C16, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 1 3/5", BRIGHTON_HAMMOND, C17, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 1 1/3", BRIGHTON_HAMMOND, C18, R1, W1, L1, 0, 8, 0,
+		"bitmaps/knobs/hammondblack.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
+	{"Upper Drawbar 1", BRIGHTON_HAMMOND, C19, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
 
 	/* 18 - switches, etc */
-	{"", 2, MC1, MR1, MW1, MH1, 0, 1, 0,
+	{"Leslie", 2, MC1, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
-	{"", 2, MC2, MR1, MW1, MH1, 0, 1, 0,
+	{"Reverb", 2, MC2, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
-	{"", 0, MC2 + 57, MR1 + 70, MW1 + 10, MH1, 0, 6, 0, 0, 0, 0},
-	{"", 2, MC3, MR1, MW1, MH1, 0, 1, 0,
+	{"VibraChorus", 0, MC2 + 57, MR1 + 70, MW1 + 10, MH1, 0, 6, 0, 0, 0, 0},
+	{"Bright", 2, MC3, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
 
 	/* 22 - perc switches, etc */
-	{"", 2, MC4, MR1, MW1, MH1, 0, 1, 0,
+	{"Perc 4", 2, MC4, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
-	{"", 2, MC5, MR1, MW1, MH1, 0, 1, 0,
+	{"Perc 2 2/3", 2, MC5, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
-	{"", 2, MC6, MR1, MW1, MH1, 0, 1, 0,
+	{"Perc Slow", 2, MC6, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
-	{"", 2, MC7, MR1, MW1, MH1, 0, 1, 0,
+	{"Perc Soft", 2, MC7, MR1, MW1, MH1, 0, 1, 0,
 		"bitmaps/buttons/rockersmooth.xpm",
 		"bitmaps/buttons/rockersmoothd.xpm", 0},
 
 	/* 26 - Bass drawbars */
-	{"", BRIGHTON_HAMMOND, Cm1, R1, W1, L1, 0, 8, 0,
+	{"Bass Drawbar 16", BRIGHTON_HAMMOND, Cm1, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", BRIGHTON_HAMMOND, Cm2, R1, W1, L1, 0, 8, 0,
+	{"Bass Drawbar 8", BRIGHTON_HAMMOND, Cm2, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
 
 	/*
@@ -241,11 +239,11 @@ static brightonLocations locations[C_COUNT] = {
 };
 
 static brightonLocations volume[1] = {
-	{"", 0, 125, 125, 750, 750, 0, 1, 0, 0, 0, 0},
+	{"MasterVolume", 0, 125, 125, 750, 750, 0, 1, 0, 0, 0, 0},
 };
 
 static brightonLocations leslie[2] = {
-	{"", 2, 100, 600, 800, 200, 0, 1, 0,
+	{"Leslie On/Off", 2, 100, 600, 800, 200, 0, 1, 0,
 		"bitmaps/buttons/sw3.xpm",
 		"bitmaps/buttons/sw5.xpm", BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW},
 	{"", 2, 100, 100, 800, 200, 0, 1, 0,
@@ -353,85 +351,102 @@ static brightonLocations mem[MEM_COUNT] = {
 #define OS3 150
 static brightonLocations opts[OPTS_COUNT] = {
 	/* Leslie type - 26 */
-	{"", 2, OC1, OR1, OS1, OS2, 0, 1, 0,
+	{"Leslie Separate", 2, OC1, OR1, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm",
 		"bitmaps/buttons/presson.xpm", 0},
-	{"", 2, OC1, OR2, OS1, OS2, 0, 1, 0,
+	{"Leslie Sync", 2, OC1, OR2, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm",
 		"bitmaps/buttons/presson.xpm", 0},
-	{"", 2, OC1, OR3, OS1, OS2, 0, 1, 0,
+	{"Leslie NoBass", 2, OC1, OR3, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm",
 		"bitmaps/buttons/presson.xpm", 0},
-	{"", 2, OC1, OR4, OS1, OS2, 0, 1, 0,
+	{"Leslie Break", 2, OC1, OR4, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoffg.xpm",
 		"bitmaps/buttons/pressong.xpm", 0},
 	/* Leslie global - 30 */
-	{"", 0, OC2, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Leslie X-Over", 0, OC2, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC3, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Leslie Inertia", 0, OC3, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Leslie HS - 32 */
-	{"", 0, OC5, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
+	{"Leslie HighFreq", 0, OC5, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC6, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
+	{"Leslie HighDepth", 0, OC6, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC7, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
+	{"Leslie HighPhase", 0, OC7, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreennew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Leslie global - 35 */
-	{"", 0, OC4, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobrednew.xpm",
+	{"Leslie Overdrive", 0, OC4, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobrednew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Leslie LS - 37 */
-	{"", 0, OC8, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Leslie LowFreq", 0, OC8, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC9, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Leslie LowDepth", 0, OC9, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC10, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Leslie LowPhase", 0, OC10, OR1, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* VC Params - 40 */
-	{"", 0, OC2, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"VibraChorus Phase", 0, OC2, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC3, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"VibraChorus LC", 0, OC3, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC4, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"VibraChorus Depth", 0, OC4, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Envelopes 43 */
-	{"", 0, OC2, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Perc Fast Decay", 0, OC2, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC3, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Perc Slow Decay", 0, OC3, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC5, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Perc Fast Attack", 0, OC5, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC6, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"Perc Slow Attack", 0, OC6, OR6, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Diverse - 47 */
-	{"", 2, OC1, OR7 - 10, OS1, OS2, 0, 1, 0,
+	{"Preacher", 2, OC1, OR7 - 10, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm",
 		"bitmaps/buttons/presson.xpm", 0},
-	{"", 2, OC1, OR7 + OD2 - 10, OS1, OS2, 0, 1, 0,
+	{"Compression", 2, OC1, OR7 + OD2 - 10, OS1, OS2, 0, 1, 0,
 		"bitmaps/buttons/pressoffg.xpm",
 		"bitmaps/buttons/pressong.xpm", 0},
-	{"", 0, OC2, OR7, OS3, OS3, 0, 7, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Bright", 0, OC2, OR7, OS3, OS3, 0, 7, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC3, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobrednew.xpm",
+	{"Click", 0, OC3, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobrednew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC10, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Leslie T Phase", 0, OC10, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC4, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
+	{"Damping", 0, OC4, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobyellownew.xpm",
 		0, BRIGHTON_NOSHADOW},
 	/* Hammond chorus phase delay 53 */
-	{"", 0, OC5, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
+	{"VibraChorus PD", 0, OC5, OR5, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobgreynew.xpm",
 		0, BRIGHTON_NOSHADOW},
 /* Five reverb controls from 27 */
-	{"", 0, OC6, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"Reverb Decay", 0, OC6, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC7, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"Reverb Drive", 0, OC7, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC8, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"Reverb Crossover", 0, OC8, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
-	{"", 0, OC9, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+	{"Reverb Feedback", 0, OC9, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
 		0, BRIGHTON_NOSHADOW},
+	{"Reverb Wet", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_NOSHADOW},
+
 	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
-		0, BRIGHTON_NOSHADOW},
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
+	{"", 0, OC10, OR7, OS3, OS3, 0, 1, 0, "bitmaps/knobs/knobbluenew.xpm",
+		0, BRIGHTON_WITHDRAWN},
 };
 
 static int commonPreset = -1;
@@ -488,6 +503,9 @@ umCallback(brightonWindow * win, int panel, int index, float value)
 	if (global.libtest)
 		return(0);
 
+	if (synth->flags & REQ_MIDI_DEBUG)
+		printf("umCallback(%i, %i, %f)\n", global.libtest, index, value);
+
 	/*
 	 * If the code from 0 to 11 then this will become the preset section and
 	 * be used to load the first 24 memories.
@@ -521,6 +539,9 @@ lmCallback(brightonWindow * win, int panel, int index, float value)
 	if (global.libtest)
 		return(0);
 
+	if (synth->flags & REQ_MIDI_DEBUG)
+		printf("lmCallback(%i, %i, %f)\n", panel, index, value);
+
 	/*
 	 * If the code from 0 to 11 then this will become the preset section and
 	 * be used to load the first 24 memories.
@@ -551,21 +572,17 @@ static int
 hammondB3destroy(brightonWindow *win)
 {
 	guiSynth *synth = findSynth(global.synths, win);
-	bristolMidiMsg msg;
 
-printf("hammondB3destroy(%i): %i\n", (size_t) win, synth->midichannel);
+	printf("hammondB3destroy(%p): %i\n", win, synth->midichannel);
+
 	/*
 	 * Since we registered two synths, we now need to remove the upper
 	 * manual.
 	 */
 	bristolMidiSendMsg(manual.controlfd, synth->sid2, 127, 0,
 		BRISTOL_EXIT_ALGO);
-	if (bristolMidiRead(manual.controlfd, &msg) < 0)
-		printf("socket closed\n");
 	bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
 		BRISTOL_EXIT_ALGO);
-	if (bristolMidiRead(global.controlfd, &msg) < 0)
-		printf("socket closed\n");
 	return(0);
 }
 
@@ -603,7 +620,7 @@ brightonApp hammondB3App = {
 	hammondB3Configure, /* 3 callbacks, unused? */
 	hammondB3MidiCallback,
 	hammondB3destroy,
-	{16, 0, 2, 1, 5, 520, 0, 0},
+	{-1, 0, 2, 1, 5, 520, 0, 0},
 	765, 400,
 	15,
 	{
@@ -928,7 +945,6 @@ hammondB3Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static void
 hammondB3Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 /*	printf("hammondB3Midi(%i %i %i %i %i)\n", fd, chan, c, o, v); */
 
@@ -957,13 +973,9 @@ hammondB3Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	 */
 	bristolMidiSendMsg(manual.controlfd, synth->sid2,
 		127, 0, (BRISTOL_MIDICHANNEL|newchan) + 1);
-	if (bristolMidiRead(manual.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 
 	bristolMidiSendMsg(global.controlfd, synth->sid,
 		127, 0, BRISTOL_MIDICHANNEL|newchan);
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 
 	synth->midichannel = newchan;
 
@@ -1517,7 +1529,7 @@ hammondB3Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the hammondB3 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the hammondB3 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1531,10 +1543,11 @@ hammondB3Init(brightonWindow *win)
 		if (synth->synthtype == BRISTOL_HAMMONDB3)
 		{
 			bcopy(&global, &manual, sizeof(guimain));
+			manual.home = 0;
 
 			synth->synthtype = BRISTOL_HAMMOND;
 			if ((synth->sid = initConnection(&global, synth)) < 0)
-				exit(0);
+				return(-1);
 
 			/*
 			 * send a hello, with a voice count, then request starting of the
@@ -1543,15 +1556,19 @@ hammondB3Init(brightonWindow *win)
 			 */
 			manual.flags |= BRISTOL_CONN_FORCE|BRIGHTON_NOENGINE;
 			manual.port = global.port;
+			manual.host = global.host;
 
 			synth->midichannel++;
 			synth->synthtype = BRISTOL_HAMMONDB3;
 			if ((synth->sid2 = initConnection(&manual, synth)) < 0)
-				exit(0);
+				return(-1);
 			synth->midichannel--;
+			global.manualfd = manual.controlfd;
+			global.manual = &manual;
+			manual.manual = &global;
 		} else {
 			if ((synth->sid2 = initConnection(&manual, synth)) < 0)
-				exit(0);
+				return(-1);
 		}
 	}
 
@@ -1841,7 +1858,7 @@ hammondB3Configure(brightonWindow *win)
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
-printf("going operational: %x, %x\n", (size_t) synth, (size_t) win);
+	printf("going operational: %p, %p\n", synth, win);
 
 	synth->flags |= OPERATIONAL;
 	synth->keypanel = 6;
@@ -1855,23 +1872,24 @@ printf("going operational: %x, %x\n", (size_t) synth, (size_t) win);
 	loadMemory(synth, "hammondB3", 0, 0, synth->mem.active, FIRST_DEV, 0);
 	hammondB3Switch(synth, 0, 0, 0, 0, 0);
 
-	bristolMidiSendMsg(global.controlfd, synth->midichannel,
-		BRISTOL_EVENT_KEYON, 0, 10 + synth->transpose);
-	bristolMidiSendMsg(global.controlfd, synth->midichannel,
-		BRISTOL_EVENT_KEYOFF, 0, 10 + synth->transpose);
 	/*
 	 * We also want to configure the mod wheel so that it is half way up, this
 	 * affects the speed of the leslie.
 	 */
-	bristolMidiControl(global.controlfd, synth->midichannel,
-		0, 1, C_RANGE_MIN_1 >> 1);
-	/*
-	 * And the expression (foot) pedal to full on as it governs the gain
-	 */
-	bristolMidiControl(global.controlfd, synth->midichannel,
-		0, 4, C_RANGE_MIN_1);
-	bristolMidiControl(global.controlfd, synth->midichannel + 1,
-		0, 4, C_RANGE_MIN_1);
+	if (global.libtest == 0)
+	{
+		bristolMidiControl(global.controlfd, synth->midichannel,
+			0, 1, C_RANGE_MIN_1 >> 1);
+		bristolMidiControl(global.controlfd, synth->midichannel + 1,
+			0, 1, C_RANGE_MIN_1 >> 1);
+		/*
+		 * And the expression (foot) pedal to full on as it governs the gain
+		 */
+		bristolMidiControl(global.controlfd, synth->midichannel,
+			0, 4, C_RANGE_MIN_1);
+		bristolMidiControl(global.controlfd, synth->midichannel + 1,
+			0, 4, C_RANGE_MIN_1);
+	}
 
 	synth->mem.param[LESLIE_ONOFF] = 1;
 
@@ -1886,6 +1904,11 @@ printf("going operational: %x, %x\n", (size_t) synth, (size_t) win);
 
 	dc = brightonGetDCTimer(win->dcTimeout);
 
+	bristolMidiSendMsg(global.controlfd, synth->midichannel,
+		BRISTOL_EVENT_KEYON, 0, 10 + synth->transpose);
+	bristolMidiSendMsg(global.controlfd, synth->midichannel,
+		BRISTOL_EVENT_KEYOFF, 0, 10 + synth->transpose);
+
 	return(0);
 }
 
