@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -147,104 +146,105 @@ extern guimain global;
 
 static brightonLocations locations[DEVICE_COUNT] = {
 	/* Power, etc */
-	{"", 2, C1, BR2, 18, BS5, 0, 1.01, 0,
+	{"On/Off", 2, C1, BR2, 18, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchoff.xpm",
 		"bitmaps/buttons/touch.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C2, R2, W1, L1, 0, 1, 0,
+	{"DCO Mod", 1, C2, R2, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack4.xpm", 0, BRIGHTON_HALFSHADOW},
-	{"", 1, C3, R2, W1, L1, 0, 1, 0,
+	{"VCF Mod", 1, C3, R2, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack4.xpm", 0, BRIGHTON_HALFSHADOW},
-	{"", 0, C4, R2, S2, S2, 0, 1, 0, 
+	{"Tuning", 0, C4, R2, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob1.xpm", 0, BRIGHTON_NOTCH},
-	{"", 0, C4, R3 + 20, S2, S2, 0, 1, 0, 
+	{"Glide", 0, C4, R3 + 20, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob1.xpm", 0, 0},
-	{"", 2, C4 + 3, R4, BS1, BS2 + 40, 0, 1, 0,
+	{"LFO Manual", 2, C4 + 3, R4, BS1, BS2 + 40, 0, 1, 0,
 		"bitmaps/buttons/touchoff.xpm",
 		"bitmaps/buttons/touch.xpm", BRIGHTON_NOSHADOW},
 	/* Transpose - 6 */
-	{"", 2, C5 + 1, BR2 + 30, 12, 180, 0, 2, 0,
+	{"Transpose", 2, C5 + 1, BR2 + 30, 12, 180, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 2, C6 + 2, BR2, BS1, BS5, 0, 1, 0,
+	{"Hold", 2, C6 + 2, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchoy.xpm",
 		"bitmaps/buttons/touchy.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C7, R2, W1, L1, 0, 1, 0,
+	{"LFO Rate", 1, C7, R2, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack4.xpm", 0, BRIGHTON_HALFSHADOW},
-	{"", 1, C8, R2, W1, L1, 0, 1, 0,
+	{"LFO Delay", 1, C8, R2, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderblack4.xpm", 0, BRIGHTON_HALFSHADOW},
-	{"", 2, C9, BR2 + 70, BS4, BS3, 0, 1, 0,
+	{"Man/Auto", 2, C9, BR2 + 70, BS4, BS3, 0, 1, 0,
 		"bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 	/* DCO - 11 */
-	{"", 1, C10, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"DCO-LFO Mod", 1, C10, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C11, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"PWM", 1, C11, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C12 + 1, BR2 + 30, 12, 180, 0, 2, 0,
+	{"PWM Source", 2, C12 + 1, BR2 + 30, 12, 180, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 2, C13 - 7, BR2, BS1, BS5, 0, 1, 0,
+	{"Pulse", 2, C13 - 7, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchow.xpm",
 		"bitmaps/buttons/touchw.xpm", BRIGHTON_NOSHADOW},
-	{"", 2, C14 - 3, BR2, BS1, BS5, 0, 8192, 0,
+	{"Ramp", 2, C14 - 3, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchoy.xpm",
 		"bitmaps/buttons/touchy.xpm", BRIGHTON_NOSHADOW},
-	{"", 2, C15 + 2, BR2, BS1, BS5, 0, 1, 0,
+	{"Square", 2, C15 + 2, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchoo.xpm",
 		"bitmaps/buttons/toucho.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C16, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"SubOsc", 1, C16, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C17, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Noise", 1, C17, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/* hpf - 19 */
-	{"", 1, C18, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"HPF", 1, C18, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/* filter - 20 */
-	{"", 1, C19, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Filter Freq", 1, C19, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C20, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Resonance", 1, C20, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C21, BR2 + 70, BS4, BS3, 0, 1, 0,
+	{"Env +/-", 2, C21, BR2 + 70, BS4, BS3, 0, 1, 0,
 		"bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C22, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"FilterEnvMod", 1, C22, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C23, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"FilterLFOMod", 1, C23, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C24, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Filter KBD", 1, C24, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/* VCA - 26  */
-	{"", 2, C25 - 10, BR2 + 70, BS4, BS3, 0, 1, 0,
+	{"Amp Env/Gate", 2, C25 - 10, BR2 + 70, BS4, BS3, 0, 1, 0,
 		"bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C26, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Amp Level", 1, C26, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/* adsr - 28 */
-	{"", 1, C27, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Attack", 1, C27, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C28, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Decay", 1, C28, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C29, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Sustain", 1, C29, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C30, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Release", 1, C30, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/* chorus - 32 */
-	{"", 2, C31 - 9, BR2, BS1, BS5, 0, 1, 0,
+	{"Chorus 1", 2, C31 - 9, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchow.xpm",
 		"bitmaps/buttons/touchw.xpm", BRIGHTON_NOSHADOW},
-	{"", 2, C32 - 4, BR2, BS1, BS5, 0, 2, 0,
+	{"Chorus 2", 2, C32 - 4, BR2, BS1, BS5, 0, 2, 0,
 		"bitmaps/buttons/touchoy.xpm",
 		"bitmaps/buttons/touchy.xpm", BRIGHTON_NOSHADOW},
-	{"", 2, C33, BR2, BS1, BS5, 0, 4, 0,
+	{"Chorus 3", 2, C33, BR2, BS1, BS5, 0, 4, 0,
 		"bitmaps/buttons/touchoo.xpm",
 		"bitmaps/buttons/toucho.xpm", BRIGHTON_NOSHADOW},
 	/* Arpeggiator - 35 */
-	{"", 2, C6_1, BR2, BS1, BS5, 0, 1, 0,
+	{"Arpeg On", 2, C6_1, BR2, BS1, BS5, 0, 1, 0,
 		"bitmaps/buttons/touchoo.xpm",
 		"bitmaps/buttons/toucho.xpm", BRIGHTON_NOSHADOW},
-	{"", 2, C6_2, BR2 + 30, 12, 180, 0, 2, 0,
+	{"Arpeg UpDown", 2, C6_2, BR2 + 30, 12, 180, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 2, C6_3, BR2 + 30, 12, 180, 0, 2, 0,
+	{"Arpeg Range", 2, C6_3, BR2 + 30, 12, 180, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 1, C6_4, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Arpeg Rate", 1, C6_4, R2, W1, L1, 0, 1, 0,
+		"bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW|BRIGHTON_REVERSE},
 /* memory tablet */
 	{"", 2, C34, MBR2, BS1, BS2, 0, 1, 0,
@@ -315,7 +315,7 @@ brightonApp junoApp = {
 	junoConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{6, 100, 2, 2, 5, 520, 0, 0},
+	{6, 100, 1, 2, 5, 520, 0, 0},
 	820, 250,
 	5, /* one panel only */
 	{
@@ -396,12 +396,21 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 		case MIDI_PROGRAM:
 			printf("midi program: %x, %i\n", controller, value);
 			synth->location = value;
-			loadMemory(synth, "juno", 0, synth->bank + synth->location,
-				synth->mem.active, FIRST_DEV, 0);
+			if (loadMemory(synth, "juno", 0, synth->bank + synth->location,
+				synth->mem.active, FIRST_DEV, 0) < 0)
+				displayText(synth, "FRE", synth->location, DISPLAY_DEV);
+			else
+				displayText(synth, "PRG", synth->location, DISPLAY_DEV);
 			break;
 		case MIDI_BANK_SELECT:
 			printf("midi banksel: %x, %i\n", controller, value);
 			synth->bank = value;
+			synth->location = (synth->location % 10) + value * 10;
+			if (loadMemory(synth, "juno", 0, synth->bank + synth->location,
+				synth->mem.active, FIRST_DEV, 0) < 0)
+				displayText(synth, "FRE", synth->location, DISPLAY_DEV);
+			else
+				displayText(synth, "PRG", synth->location, DISPLAY_DEV);
 			break;
 	}
 	return(0);
@@ -474,7 +483,6 @@ junoMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static void
 junoMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -498,8 +506,6 @@ junoMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -544,7 +550,7 @@ junoCallback(brightonWindow * win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -588,18 +594,14 @@ junoSublevel(guiSynth *synth)
 static void
 junoTranspose(guiSynth *synth)
 {
-	bristolMidiMsg msg;
-
 	if ((synth->flags & OPERATIONAL) == 0)
 		return;
 
-	printf("junoTranspose %f\n", synth->mem.param[KEY_TRANSPOSE]);
+	//printf("junoTranspose %f\n", synth->mem.param[KEY_TRANSPOSE]);
 
 	bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
 		BRISTOL_TRANSPOSE + (int) (synth->mem.param[KEY_TRANSPOSE] * 12) - 12);
 
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 /*
 	if (synth->mem.param[KEY_TRANSPOSE] == 0)
 		synth->transpose = 24;
@@ -613,8 +615,6 @@ junoTranspose(guiSynth *synth)
 static void
 junoHold(guiSynth *synth)
 {
-	bristolMidiMsg msg;
-
 	if ((synth->flags & OPERATIONAL) == 0)
 		return;
 
@@ -627,8 +627,6 @@ junoHold(guiSynth *synth)
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_HOLD|0);
 
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 }
 
 /*
@@ -657,7 +655,7 @@ junoInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the juno link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the juno link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -674,7 +672,7 @@ junoInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = junoMidiSendMsg;
