@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -265,113 +264,113 @@ static int dc1, mw = 0;
 /* 42 lines controls, 43rd in button line, then 41 buttons plus display */
 static brightonLocations locations[DEVICE_COUNT] = {
 	/* 0 - LFO */
-	{"", 1, C5, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"LFO-Rate", 1, C5, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"LFO-Delay", 1, C6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 0, C7 - 4, R1, JR1, JR1, 0, 3, 0, "bitmaps/knobs/knob1.xpm", 0, 
+	{"LFO-Waveform", 0, C7 - 4, R1, JR1, JR1, 0, 3, 0, "bitmaps/knobs/knob1.xpm", 0, 
 		BRIGHTON_STEPPED},
 	/* 3 - mods */
-	{"", 1, C8, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Mod-LFO-lvl", 1, C8, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C9, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Mod-Env-lvl", 1, C9, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C10, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
+	{"Mod-Osc1/2", 2, C10, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
 		0, BRIGHTON_NOSHADOW|BRIGHTON_THREEWAY},
 
-	{"", 1, C11, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Mod-PWM", 1, C11, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 
 	/* PWM */
-	{"", 2, C12, R1, 10, JR2, 0, 1.01, 0, "bitmaps/buttons/sw1.xpm",
+	{"ModPWM-EnvLFO", 2, C12, R1, 10, JR2, 0, 1.01, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C13, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Mod-XMod", 1, C13, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	/*
 	 * Transpose - replace with 4 buttons as multiselectors, this one and
 	 * three more in the 'free' section - 9
 	 */
-	{"", 2, C14 + 2, RB1, 11, JR3, 0, 1, 0,
+	{"VCO1-2'", 2, C14 + 2, RB1, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C14 + 2, RB2, 11, JR3, 0, 1, 0,
+	{"VCA1-4'", 2, C14 + 2, RB2, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C14 + 2, RB3, 11, JR3, 0, 1, 0,
+	{"VCA1-8'", 2, C14 + 2, RB3, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C14 + 2, RB4, 11, JR3, 0, 1, 0,
+	{"VCA1-16'", 2, C14 + 2, RB4, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
 	/*
 	 * Waveform buttons - 13
 	 */
-	{"", 2, C15 + 4, RB1, 11, JR3, 0, 1, 0,
+	{"VCO1-Tri", 2, C15 + 4, RB1, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C15 + 4, RB2, 11, JR3, 0, 1, 0,
+	{"VCO1-Ramp", 2, C15 + 4, RB2, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C15 + 4, RB3, 11, JR3, 0, 1, 0,
+	{"VCO1-Pulse", 2, C15 + 4, RB3, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C15 + 4, RB4, 11, JR3, 0, 1, 0,
+	{"VCO1-Square", 2, C15 + 4, RB4, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
 	/* Sync, sub - 17 */
-	{"", 2, C16, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
+	{"VCO-Sync", 2, C16, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
 		0, BRIGHTON_NOSHADOW|BRIGHTON_THREEWAY},
-	{"", 2, C17, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
+	{"VCO2-Sub", 2, C17, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 
 	/*
 	 * 4 transpose buttons for OSC-2. They were 5 however we already have one
 	 * button for sub wave - 19.
 	 */
-	{"", 2, C18 + 1, RB1, 11, JR3, 0, 1, 0,
+	{"VCO2-2'", 2, C18 + 1, RB1, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C18 + 1, RB2, 11, JR3, 0, 1, 0,
+	{"VCO2-4'", 2, C18 + 1, RB2, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C18 + 1, RB3, 11, JR3, 0, 1, 0,
+	{"VCO2-8'", 2, C18 + 1, RB3, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C18 + 1, RB4, 11, JR3, 0, 1, 0,
+	{"VCO2-16'", 2, C18 + 1, RB4, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
-	{"", 0, C19, R1, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
+	{"VCO2-Tune", 0, C19, R1, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
 		BRIGHTON_NOTCH},
 	
 	/*
 	 * Waveform buttons - 24
 	 */
-	{"", 2, C20 + 0, RB1, 11, JR3, 0, 1, 0,
+	{"VCO2-Tri", 2, C20 + 0, RB1, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C20 + 0, RB2, 11, JR3, 0, 1, 0,
+	{"VCO2-Ramp", 2, C20 + 0, RB2, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C20 + 0, RB3, 11, JR3, 0, 1, 0,
+	{"VCO2-Pulse", 2, C20 + 0, RB3, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C20 + 0, RB4, 11, JR3, 0, 1, 0,
+	{"VCO2-Noise", 2, C20 + 0, RB4, 11, JR3, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
 	/* Mix, HPF - 28 */
-	{"", 0, C21, R1, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
+	{"Mix Osc1/2", 0, C21, R1, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 1, C22, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"HPF", 1, C22, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 	
 	/* Filter - 30 */
-	{"", 1, C23, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCF-Cutoff", 1, C23, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C24, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCF-Resonance", 1, C24, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C25, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
+	{"VCF-Mode", 2, C25, R1 - 20, 10, JR2 + 35, 0, 2, 0, 0,
 		0, BRIGHTON_NOSHADOW|BRIGHTON_THREEWAY},
-	{"", 1, C26, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCF-EnvLvl", 1, C26, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C27, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
+	{"VCF-EnvSel", 2, C27, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
-	{"", 1, C28, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCF-LFO", 1, C28, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C29, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCF-KBD", 1, C29, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 
 	/* AMP - 37 */
-	{"", 1, C30, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCA-Env", 1, C30, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C31 - 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"VCA-LFO", 1, C31 - 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 /*
 	{"", 1, C31 + 1, R1 - 38, 5, JR2 + 75, 0, 3, 0,
@@ -379,33 +378,33 @@ static brightonLocations locations[DEVICE_COUNT] = {
 */
 
 	/* ADSR1 - 39 */
-	{"", 1, C32 + 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR1-Attack", 1, C32 + 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C33, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR1-Decay", 1, C33, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C34, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR1-Sustain", 1, C34, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C35, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR1-Release", 1, C35, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C36 - 6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR1-Kbd", 1, C36 - 6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 /*
 	{"", 2, C36, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 */
-	{"", 2, C37 + 1, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
+	{"ADSR1-+/-", 2, C37 + 1, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 
 	/* ADSR1 - 45 */
-	{"", 1, C38 + 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR2-Attack", 1, C38 + 1, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C39, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR2-Decay", 1, C39, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C40, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR2-Sustain", 1, C40, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C41, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR2-Release", 1, C41, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 1, C42 - 6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"ADSR2-Kdb", 1, C42 - 6, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
 /*
 	{"", 2, C42, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
@@ -413,53 +412,53 @@ static brightonLocations locations[DEVICE_COUNT] = {
 */
 
 	/* PWM, XMOD env - 50 */
-	{"", 1, C10a, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Mod-Osc-PW", 1, C10a, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C13a, R1, 10, JR2, 0, 1.01, 0, "bitmaps/buttons/sw1.xpm",
+	{"Mod-XMod-Lvl", 2, C13a, R1, 10, JR2, 0, 1.01, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 	/* Layer Pan */
-	{"", 0, C2, R2 + 50, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
+	{"Layer Pan", 0, C2, R2 + 50, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
 		BRIGHTON_NOTCH},
 
 	/* Arpeg range - 53 */
-	{"", 2, JBC2, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg 1 Oct", 2, JBC2, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbo.xpm",
 		"bitmaps/buttons/jboo.xpm", 0},
-	{"", 2, JBC3, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg 2 Oct", 2, JBC3, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbo.xpm",
 		"bitmaps/buttons/jboo.xpm", 0},
-	{"", 2, JBC4, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg 3 Oct", 2, JBC4, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbo.xpm",
 		"bitmaps/buttons/jboo.xpm", 0},
-	{"", 2, JBC5, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg 4 Oct", 2, JBC5, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbo.xpm",
 		"bitmaps/buttons/jboo.xpm", 0},
 
 	/* Arpeg mode - 57 */
-	{"", 2, JBC6, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg Mode Up", 2, JBC6, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jby.xpm",
 		"bitmaps/buttons/jbyo.xpm", 0},
-	{"", 2, JBC7, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg Mode Down", 2, JBC7, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jby.xpm",
 		"bitmaps/buttons/jbyo.xpm", 0},
-	{"", 2, JBC8, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg Mode UpDn", 2, JBC8, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jby.xpm",
 		"bitmaps/buttons/jbyo.xpm", 0},
-	{"", 2, JBC9, 800, JBS, JBH, 0, 4, 0,
+	{"Arpeg Mode Rand", 2, JBC9, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jby.xpm",
 		"bitmaps/buttons/jbyo.xpm", 0},
 
 	/* Key assign mode - 61 */
-	{"", 2, JBC10, 800, JBS, JBH, 0, 4, 0,
+	{"Mode Solo", 2, JBC10, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbw.xpm",
 		"bitmaps/buttons/jbwo.xpm", 0},
-	{"", 2, JBC11, 800, JBS, JBH, 0, 4, 0,
+	{"Mode Uni", 2, JBC11, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbw.xpm",
 		"bitmaps/buttons/jbwo.xpm", 0},
-	{"", 2, JBC12, 800, JBS, JBH, 0, 4, 0,
+	{"Mode Poly1", 2, JBC12, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbw.xpm",
 		"bitmaps/buttons/jbwo.xpm", 0},
-	{"", 2, JBC13, 800, JBS, JBH, 0, 4, 0,
+	{"Mode Poly2", 2, JBC13, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbw.xpm",
 		"bitmaps/buttons/jbwo.xpm", 0},
 
@@ -512,29 +511,29 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	/*
 	 * 92 - Second row Hold up/down
 	 */
-	{"", 2, JBC14, 800, JBS, JBH, 0, 4, 0,
+	{"Hold Lower", 2, JBC14, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbg.xpm",
 		"bitmaps/buttons/jbgo.xpm", 0},
-	{"", 2, JBC15, 800, JBS, JBH, 0, 4, 0,
+	{"Hold Upper", 2, JBC15, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbg.xpm",
 		"bitmaps/buttons/jbgo.xpm", 0},
 
 	/* layer mod enable - 94 */
-	{"", 2, JBC19, 800, JBS, JBH, 0, 1, 0,
+	{"Mod Layer 1", 2, JBC19, 800, JBS, JBH, 0, 1, 0,
 		"bitmaps/buttons/jbb2.xpm",
 		"bitmaps/buttons/jbb2o.xpm", 0},
-	{"", 2, JBC20, 800, JBS, JBH, 0, 1, 0,
+	{"Mod Layer 2", 2, JBC20, 800, JBS, JBH, 0, 1, 0,
 		"bitmaps/buttons/jbb2.xpm",
 		"bitmaps/buttons/jbb2o.xpm", 0},
 
 	/* Dual/Split/Layer - 96 */
-	{"", 2, JBC16, 800, JBS, JBH, 0, 4, 0,
+	{"Dual", 2, JBC16, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbb1.xpm",
 		"bitmaps/buttons/jbb1o.xpm", 0},
-	{"", 2, JBC17, 800, JBS, JBH, 0, 4, 0,
+	{"Split", 2, JBC17, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbb1.xpm",
 		"bitmaps/buttons/jbb1o.xpm", 0},
-	{"", 2, JBC18, 800, JBS, JBH, 0, 4, 0,
+	{"All", 2, JBC18, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbb1.xpm",
 		"bitmaps/buttons/jbb1o.xpm", 0},
 
@@ -549,24 +548,24 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	 *
 	 * It could be worthwhile having the arpeggiator clock within the memory?
 	 */
-	{"", 1, C3, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
+	{"Arpeg Rate", 1, C3, R2, W1, L1, 0, 1, 0, "bitmaps/knobs/sliderblack4.xpm", 0, 
 		BRIGHTON_HALFSHADOW},
-	{"", 2, C4, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
+	{"Arpeg Int/Ext", 2, C4, R1, 10, JR2, 0, 1, 0, "bitmaps/buttons/sw1.xpm",
 		"bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
 	/* Volume and Balance 101 and 102 */
-	{"", 0, C1, R2 + 50, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 0},
-	{"", 0, C1 +18, R1 +30, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0,
+	{"MasterVolume", 0, C1, R2 + 50, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 0},
+	{"LayerBalance", 0, C1 +18, R1 +30, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0,
 		BRIGHTON_NOTCH},
 
 	/* Second row, tuning - 103 */
-	{"", 0, 15, 800, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
+	{"MasterTuning", 0, 15, 800, JR1, JR1, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 2, JBC1, 800, JBS, JBH, 0, 4, 0,
+	{"Tune", 2, JBC1, 800, JBS, JBH, 0, 4, 0,
 		"bitmaps/buttons/jbr.xpm",
 		"bitmaps/buttons/jbro.xpm", BRIGHTON_CHECKBUTTON},
 
 	/* Panel switch - 105 */
-	{"", 2, JBC20_2, 800, JBS, JBH, 0, 1, 0,
+	{"LayerSelect", 2, JBC20_2, 800, JBS, JBH, 0, 1, 0,
 		"bitmaps/buttons/jbb2.xpm",
 		"bitmaps/buttons/jbb2o.xpm", 0},
 
@@ -699,7 +698,6 @@ static int
 jupiterDestroySynth(brightonWindow *win)
 {
 	guiSynth *synth = findSynth(global.synths, win);
-	bristolMidiMsg msg;
 
 	/*
 	 * Since we registered two synths, we now need to remove the upper
@@ -707,12 +705,8 @@ jupiterDestroySynth(brightonWindow *win)
 	 */
 	bristolMidiSendMsg(manual.controlfd, synth->sid2, 127, 0,
 		BRISTOL_EXIT_ALGO);
-	if (bristolMidiRead(manual.controlfd, &msg) < 0)
-		printf("socket closed\n");
 	bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
 		BRISTOL_EXIT_ALGO);
-	if (bristolMidiRead(global.controlfd, &msg) < 0)
-		printf("socket closed\n");
 	return(0);
 }
 
@@ -729,7 +723,7 @@ brightonApp jupiterApp = {
 	jupiterConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	jupiterDestroySynth,
-	{8, 100, 8, 2, 5, 520, 0, 0},
+	{8, 50, 32, 2, 5, 520, 0, 0},
 	900, 350,
 	5,
 	{
@@ -902,8 +896,6 @@ midiRelease(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	if (!global.libtest)
 	{
-		bristolMidiMsg msg;
-
 		/*
 		 * Midi release is ALL notes, ALL synths. If this behaviour (in the
 		 * engine) changes we may need to put in an all_notes_off on the
@@ -911,9 +903,6 @@ midiRelease(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		 */
 		bristolMidiSendMsg(fd, synth->sid, 127, 0,
 			BRISTOL_ALL_NOTES_OFF);
-
-		if (bristolMidiRead(fd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 }
 
@@ -946,14 +935,17 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 			if (jupiterDebug(synth, 2))
 				printf("midi program: %x, %i\n", controller, value);
 			synth->location = value;
-			loadMemory(synth, "jupiter8", 0, synth->bank + synth->location + mw,
+			loadMemory(synth, "jupiter8", 0, synth->bank + synth->location,
 				synth->mem.active, 0, 0);
 			modPanelFix(win, synth);
 			break;
 		case MIDI_BANK_SELECT:
 			if (jupiterDebug(synth, 2))
 				printf("midi banksel: %x, %i\n", controller, value);
-			synth->bank = value;
+			synth->bank = value + 10;
+			loadMemory(synth, "jupiter8", 0, synth->bank + synth->location,
+				synth->mem.active, 0, 0);
+			modPanelFix(win, synth);
 			break;
 	}
 	return(0);
@@ -1639,6 +1631,7 @@ jupiterMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 			= ((int) synth->mem.param[MY_MEM]) % 10;
 	}
 
+/*
 printf("am showing %1.0f %1.0f, %1.0f %1.0f from %1.0f %1.0f (%1.0f)\n",
 synth->mem.param[DISPLAY_DEV + 0],
 synth->mem.param[DISPLAY_DEV + 1],
@@ -1647,6 +1640,7 @@ synth->mem.param[DISPLAY_DEV + 3],
 synth->mem.param[MY_MEM],
 synth->mem.param[MEM_LINK],
 synth->mem.param[PANEL_SELECT]);
+*/
 
 	/*
 	 * Load will set the displays and nothing else, they will alsways point to
@@ -1661,7 +1655,6 @@ synth->mem.param[PANEL_SELECT]);
 static void
 jupiterMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -1683,12 +1676,8 @@ jupiterMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid2,
 			127, 0, (BRISTOL_MIDICHANNEL|newchan) + 1);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -2146,7 +2135,6 @@ static void
 jupiterSetSplit(guiSynth *synth)
 {
 	int lsp, usp;
-	bristolMidiMsg msg;
 
 	if (jupiterDebug(synth, 1))
 		printf("setSplit: %i\n", (int) synth->mem.param[SPLIT_POINT]);
@@ -2170,12 +2158,8 @@ jupiterSetSplit(guiSynth *synth)
 	if (global.libtest == 0) {
 		bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
 			BRISTOL_HIGHKEY|lsp);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 		bristolMidiSendMsg(global.controlfd, synth->sid2, 127, 0,
 			BRISTOL_LOWKEY|usp);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	setsplit = 0;
@@ -2188,7 +2172,6 @@ static void
 jupiterKeyMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
 	brightonEvent event;
-	bristolMidiMsg msg;
 
 	if (jupiterDebug(synth, 1))
 		printf("jupiterKeyMode(%i, %i)\n", c, v);
@@ -2246,12 +2229,8 @@ jupiterKeyMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 			if (global.libtest == 0) {
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_HIGHKEY|127);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 				bristolMidiSendMsg(global.controlfd, synth->sid2,
 					127, 0, BRISTOL_LOWKEY|0);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 			}
 			jupiterSetVoices(synth);
 			break;
@@ -2295,12 +2274,8 @@ jupiterKeyMode(guiSynth *synth, int fd, int chan, int c, int o, int v)
 			if (global.libtest == 0) {
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_HIGHKEY|127);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 				bristolMidiSendMsg(global.controlfd, synth->sid2,
 					127, 0, BRISTOL_LOWKEY|127);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 			}
 			jupiterSetVoices(synth);
 			break;
@@ -2667,7 +2642,7 @@ jupiterInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the jupiter link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the jupiter link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -2704,8 +2679,10 @@ jupiterInit(brightonWindow *win)
 	{
 		int v = synth->voices;
 
+		bcopy(&global, &manual, sizeof(guimain));
+
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 		/*
 		 * We have to tweak the voicecount as we use the same synth definition
@@ -2714,13 +2691,17 @@ jupiterInit(brightonWindow *win)
 		synth->voices = synth->voices >> 1;
 		manual.flags |= BRISTOL_CONN_FORCE|BRIGHTON_NOENGINE;
 		manual.port = global.port;
+		manual.host = global.host;
 
 		if ((synth->sid2 = initConnection(&manual, synth)) < 0)
-			exit(0);
+			return(-1);
+
+		global.manualfd = manual.controlfd;
+		global.manual = &manual;
+		manual.manual = &global;
 
 		synth->voices = v;
 	}
-printf("	The SID are %i and %i\n", synth->sid, synth->sid2);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 	{
