@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -600,7 +599,7 @@ ms20Callback(brightonWindow * win, int panel, int index, float value)
 			synth->dispatch[index].operator,
 			sendvalue);
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -612,7 +611,6 @@ ms20Callback(brightonWindow * win, int panel, int index, float value)
 static void
 ms20Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	printf("ms20Midi(%i %i %i %i %i)\n", fd, chan, c, o, v);
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -640,8 +638,6 @@ ms20Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		{
 			bristolMidiSendMsg(global.controlfd, synth->sid,
 				127, 0, BRISTOL_MIDICHANNEL|newchan);
-			if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-				cleanupBristol();
 		}
 
 		synth->midichannel = newchan;
@@ -892,7 +888,7 @@ ms20Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the ms20 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the ms20 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -909,7 +905,7 @@ ms20Init(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 /*	for (i = 0; i < DEVICE_COUNT; i++) */
 /*		synth->dispatch[i].routine = ms20MidiSendMsg; */
