@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -474,7 +473,6 @@ masterMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 masterMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -498,8 +496,6 @@ masterMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -565,7 +561,7 @@ masterCallback(brightonWindow * win, int panel, int index, float value)
 		sendvalue);
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -634,7 +630,7 @@ masterTune(guiSynth *synth)
 {
 	brightonEvent event;
 
-	printf("masterTune(%x, %i, %i)\n", (size_t) synth->win, OP_PANEL, OP1_START);
+	printf("masterTune(%p, %i, %i)\n", synth->win, OP_PANEL, OP1_START);
 
 	event.value = 0.5;
 	brightonParamChange(synth->win, 0, 1, &event);
@@ -680,7 +676,7 @@ masterInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the master link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the master link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -697,7 +693,7 @@ masterInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = masterMidiSendMsg;
@@ -1018,7 +1014,7 @@ masterConfigure(brightonWindow *win)
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
-printf("going operational %x %x\n", (size_t) synth, (size_t) synth->win);
+printf("going operational %p %p\n", synth, synth->win);
 
 	synth->flags |= OPERATIONAL;
 	synth->keypanel = KEY_PANEL;
