@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -93,95 +92,95 @@ extern guimain global;
  */
 static brightonLocations locations[DEVICE_COUNT] = {
 /* CONTROL */
-	{"", 0, 47, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm",
+	{"Tune", 0, 47, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm",
 		0, BRIGHTON_NOTCH},
-	{"", 0, 18, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Glide", 0, 18, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, 80, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Mod", 0, 80, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*3 */
 /* OSCILATORS */
-	{"", 0, C1, R1, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc1 Transpose", 0, C1, R1, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C1, R3, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc2 Transpose", 0, C1, R3, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C1, R5, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc3 Transpose", 0, C1, R5, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
 /*	{"", 0, C2, 382, S2, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 0, 0}, */
-	{"", 0, C2, 382, S2, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm",
+	{"Osc2 Tuning", 0, C2, 382, S2, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm",
 		0, BRIGHTON_NOTCH},
-	{"", 0, C2, 660, S2, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc3 Tuning", 0, C2, 660, S2, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, BRIGHTON_NOTCH},
-	{"", 0, C3, R1, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc1 Waveform", 0, C3, R1, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C3, R3, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc2 Waveform", 0, C3, R3, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C3, R5, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc3 Waveform", 0, C3, R5, S1, S1, 0, 5, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*11 */
 /* MIXER */
-	{"", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc1 Mix lvl", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C6, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Ext Mix lvl", 0, C6, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C4, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc2 Mix lvl", 0, C4, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C6, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Noise Mix lvl", 0, C6, R4, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C4, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc3 Mix lvl", 0, C4, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*16 */
 /* FILTER */
-	{"", 0, C7, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Freq", 0, C7, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C8, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Emph", 0, C8, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C9, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Contour", 0, C9, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*19 */
 /* ADSR */
-	{"", 0, C7, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Attack", 0, C7, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C8, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Decay", 0, C8, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C9, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter Release", 0, C9, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*22 */
 /* another ADSR */
-	{"", 0, C7, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Attack", 0, C7, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C8, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Decay", 0, C8, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 0, C9, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Release", 0, C9, R5, S1, S1, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0}, /*25 */
 /* MAIN OUT */
-	{"", 0, C10, R1, S3, S3, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"MasterVolume", 0, C10, R1, S3, S3, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		0, 0},
-	{"", 2, C10 + 7, R1 - 92, B1, B1, 0, 1, 0,
+	{"On/Off", 2, C10 + 7, R1 - 92, B1, B1, 0, 1, 0,
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C10 + 7, R2 + 75, B1, B1, 0, 1, 0, 
+	{"A-440", 2, C10 + 7, R2 + 75, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0}, /*28 */
 /* MIX BUTTONS */
-	{"", 2, C5, R1 + 45, B1, B1, 0, 1, 0, 
+	{"Mix Osc1", 2, C5, R1 + 45, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C5, R2 + 45, B1, B1, 0, 1, 0, 
+	{"Mix Ext", 2, C5, R2 + 45, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C5, R3 + 45, B1, B1, 0, 1, 0, 
+	{"Mix Osc2", 2, C5, R3 + 45, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C5, R4 + 45, B1, B1, 0, 1, 0, 
+	{"Mix Noise", 2, C5, R4 + 45, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C5, R5 + 45, B1, B1, 0, 1, 0, 
+	{"Mix Osc3", 2, C5, R5 + 45, B1, B1, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0}, /*33 */
 /* NOISE BUTTON */
-	{"", 2, C11 - 6, C6, B2, B3, 0, 1, 0, 
+	{"White/Pink", 2, C11 - 6, C6, B2, B3, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, BRIGHTON_VERTICAL}, /*34 */
 /* CONTROL BUTTONS */
-	{"", 2, C12 - 22, R2 - 13, B1, B1, 0, 1, 0, 0, 0, 0},
-	{"", 2, C12 - 22, R2 + 97, B1, B1, 0, 1, 0, 0, 0, 0},
-	{"", 2, C12 - 9, R5 + 15, B2, B3, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
-	{"", 2, 78, 765, B1, B1, 0, 1, 0,
+	{"Osc Mod 1", 2, C12 - 22, R2 - 13, B1, B1, 0, 1, 0, 0, 0, 0},
+	{"Osc Mod 2", 2, C12 - 22, R2 + 97, B1, B1, 0, 1, 0, 0, 0, 0},
+	{"Osc 3 LFO", 2, C12 - 9, R5 + 15, B2, B3, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
+	{"Release", 2, 78, 765, B1, B1, 0, 1, 0,
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0},
-	{"", 2, 78, 825, B1, B1, 0, 1, 0,
+	{"Multitrig", 2, 78, 825, B1, B1, 0, 1, 0,
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0}, /*39 */
 /* FILTER BUTTONS */
-	{"", 2, C11 - 5, R1 + 30, B1, B1, 0, 1, 0, 0, 0, 0},
-	{"", 2, C11 - 5, R1 + 170, B1, B1, 0, 1, 0, 0, 0, 0},
-	{"", 2, C11 - 5, R3, B1, B1, 0, 1, 0, 0, 0}, /*42 */
+	{"Filter Mod1", 2, C11 - 5, R1 + 30, B1, B1, 0, 1, 0, 0, 0, 0},
+	{"Filter Velocity", 2, C11 - 5, R1 + 170, B1, B1, 0, 1, 0, 0, 0, 0},
+	{"Filter KBD", 2, C11 - 5, R3, B1, B1, 0, 1, 0, 0, 0}, /*42 */
 /* Memory tablet */
 	{"", 2, R7, R6 - S5 * 3, S4, S5, 0, 1, 0,
 		"bitmaps/digits/L.xpm", 0, BRIGHTON_CHECKBUTTON},
@@ -364,13 +363,22 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 	{
 		case MIDI_PROGRAM:
 			printf("midi program: %x, %i\n", controller, value);
-			synth->location = value;
-			loadMemory(synth, "mini", 0, synth->bank + synth->location,
-				synth->mem.active, FIRST_DEV, 0);
+			synth->location = value + synth->bank * 10;
+			if (loadMemory(synth, synth->resources->name, 0,
+				synth->location, synth->mem.active, FIRST_DEV, 0) < 0)
+				displayText(synth, "FRE", synth->location, FIRST_DEV + 59);
+			else
+				displayText(synth, "PRG", synth->location, FIRST_DEV + 59);
 			break;
 		case MIDI_BANK_SELECT:
 			printf("midi banksel: %x, %i\n", controller, value);
 			synth->bank = value;
+			synth->location = (synth->location % 10) + value * 10;
+			if (loadMemory(synth, synth->resources->name, 0,
+				synth->location, synth->mem.active, FIRST_DEV, 0) < 0)
+				displayText(synth, "FRE", synth->location, FIRST_DEV + 59);
+			else
+				displayText(synth, "PRG", synth->location, FIRST_DEV + 59);
 			break;
 	}
 	return(0);
@@ -466,7 +474,6 @@ miniMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static void
 miniMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 /*	printf("miniMidi(%x, %i %i %i %i %i)\n", synth, fd, chan, c, o, v); */
@@ -492,13 +499,10 @@ miniMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
 
-printf("M: going to display: %x, %x\n", (size_t) synth, (size_t) synth->win);
 	displayText(synth, "MIDI", newchan + 1, FIRST_DEV + 58);
 }
 
@@ -577,7 +581,7 @@ miniInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the mini link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the mini link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -594,11 +598,12 @@ miniInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = miniMidiSendMsg;
 
+	/* Tune, Glide, Modmix */
 	dispatch[FIRST_DEV + 0].controller = 10;
 	dispatch[FIRST_DEV + 0].operator = 0;
 	dispatch[FIRST_DEV + 1].controller = 9;
@@ -606,6 +611,7 @@ miniInit(brightonWindow *win)
 	dispatch[FIRST_DEV + 2].controller = 11;
 	dispatch[FIRST_DEV + 2].operator = 1;
 
+	/* Osc */
 	dispatch[FIRST_DEV + 3].controller = 0;
 	dispatch[FIRST_DEV + 3].operator = 1;
 	dispatch[FIRST_DEV + 4].controller = 1;
@@ -623,6 +629,7 @@ miniInit(brightonWindow *win)
 	dispatch[FIRST_DEV + 10].controller = 2;
 	dispatch[FIRST_DEV + 10].operator = 0;
 
+	/* MIX */
 	dispatch[FIRST_DEV + 11].controller = 0;
 	dispatch[FIRST_DEV + 11].operator = 3;
 	dispatch[FIRST_DEV + 12].controller = 8;
@@ -748,7 +755,40 @@ miniInit(brightonWindow *win)
 int
 configureGlobals(guiSynth *synth)
 {
-	if (!global.libtest) {
+	if (!global.libtest)
+	{
+		if (synth->flags & REQ_MIDI_DEBUG)
+		{
+			bristolMidiSendNRP(global.controlfd, synth->midichannel,
+				BRISTOL_NRP_DEBUG, 1);
+			if (synth->sid2 != 0)
+				bristolMidiSendNRP(global.manualfd, synth->midichannel+1,
+					BRISTOL_NRP_DEBUG, 1);
+		}
+		if (synth->flags & REQ_MIDI_DEBUG2)
+		{
+			bristolMidiSendNRP(global.controlfd, synth->midichannel,
+				BRISTOL_NRP_DEBUG, 2);
+			if (synth->sid2 != 0)
+				bristolMidiSendNRP(global.manualfd, synth->midichannel+1,
+					BRISTOL_NRP_DEBUG, 2);
+		}
+
+		/* Send debug level request to engine and also setup local debugging */
+		bristolMidiSendMsg(global.controlfd, synth->sid,
+			BRISTOL_SYSTEM, 0,
+				BRISTOL_REQ_DEBUG|((synth->flags&REQ_DEBUG_MASK)>>12));
+		if (synth->sid2 != 0)
+			bristolMidiSendMsg(global.manualfd, synth->sid2,
+				BRISTOL_SYSTEM, 0,
+					BRISTOL_REQ_DEBUG|((synth->flags&REQ_DEBUG_MASK)>>12));
+
+		bristolMidiOption(global.controlfd, BRISTOL_NRP_DEBUG,
+			(synth->flags&(REQ_DEBUG_3|REQ_DEBUG_4))>>14);
+		if (synth->sid2 != 0)
+			bristolMidiOption(global.manualfd, BRISTOL_NRP_DEBUG,
+				(synth->flags&(REQ_DEBUG_3|REQ_DEBUG_4))>>14);
+
 		bristolMidiSendNRP(global.controlfd, synth->midichannel,
 			BRISTOL_NRP_GLIDE, synth->glide * C_RANGE_MIN_1 / 30);
 		bristolMidiSendNRP(global.controlfd, synth->midichannel,
@@ -768,12 +808,12 @@ configureGlobals(guiSynth *synth)
 		bristolMidiSendNRP(global.controlfd, synth->midichannel,
 			BRISTOL_NRP_MNL_VELOC, synth->legatovelocity);
 
-		if (synth->flags & REQ_MIDI_DEBUG)
-			bristolMidiSendNRP(global.controlfd, synth->midichannel,
-				BRISTOL_NRP_DEBUG, 1);
-		if (synth->flags & REQ_MIDI_DEBUG2)
-			bristolMidiSendNRP(global.controlfd, synth->midichannel,
-				BRISTOL_NRP_DEBUG, 2);
+		/* Global message forwarding on/off */
+		bristolMidiSendNRP(global.controlfd, synth->midichannel,
+			BRISTOL_NRP_FORWARD, synth->flags & REQ_FORWARD? 1:0);
+		bristolMidiOption(global.controlfd, BRISTOL_NRP_FORWARD,
+			synth->flags & REQ_FORWARD? 1:0);
+
 		if (synth->flags & MIDI_NRP)
 			bristolMidiSendNRP(global.controlfd, synth->midichannel,
 				BRISTOL_NRP_ENABLE_NRP, 1);
@@ -781,6 +821,11 @@ configureGlobals(guiSynth *synth)
 			bristolMidiSendNRP(global.controlfd, synth->midichannel,
 				BRISTOL_NRP_ENABLE_NRP, 0);
 
+		bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
+			BRISTOL_LOWKEY|synth->lowkey);
+		bristolMidiSendMsg(global.controlfd, synth->sid, 127, 0,
+			BRISTOL_HIGHKEY|synth->highkey);
+
 		if (synth->sid2 != 0) {
 			bristolMidiSendNRP(global.controlfd, synth->midichannel+1,
 				BRISTOL_NRP_GLIDE, synth->glide * C_RANGE_MIN_1 / 30);
@@ -800,8 +845,49 @@ configureGlobals(guiSynth *synth)
 			else
 				bristolMidiSendNRP(global.controlfd, synth->midichannel+1,
 					BRISTOL_NRP_ENABLE_NRP, 0);
+
+			bristolMidiSendMsg(global.controlfd, synth->sid2, 127, 0,
+				BRISTOL_LOWKEY|synth->lowkey);
+			bristolMidiSendMsg(global.controlfd, synth->sid2, 127, 0,
+				BRISTOL_HIGHKEY|synth->highkey);
 		}
+
+		if (synth->flags & REQ_LOCAL_FORWARD)
+		{
+			bristolMidiOption(global.controlfd, BRISTOL_NRP_FORWARD, 1);
+			bristolMidiOption(global.controlfd, BRISTOL_NRP_MIDI_GO, 1);
+			bristolMidiOption(global.controlfd, BRISTOL_NRP_REQ_FORWARD, 1);
+		}
+
+		/* Forwarding options for this emulator */
+		if (synth->flags & REQ_REMOTE_FORWARD)
+			bristolMidiSendMsg(global.controlfd, synth->sid,
+				BRISTOL_SYSTEM, 0, BRISTOL_REQ_FORWARD|1);
+
+		/* Forwarding options disabled for second manual, local and remote */
+		if (global.manualfd != 0)
+		{
+			bristolMidiOption(global.manualfd, BRISTOL_NRP_REQ_SYSEX, 1);
+			bristolMidiSendMsg(global.manualfd, synth->sid2,
+				BRISTOL_SYSTEM, 0, BRISTOL_REQ_FORWARD);
+		}
+
+		bristolMidiSendNRP(global.controlfd, synth->midichannel,
+			BRISTOL_NRP_MIDI_GO, 1024);
 	}
+	/*
+	 * We should consider seeding the synth with a short note blip
+	if ((global.synths->notepref == BRIGHTON_HNP)
+		|| (global.synths->notepref == BRIGHTON_LNP))
+	{
+		sleep(1);
+		bristolMidiSendMsg(global.controlfd,
+			synth->midichannel, BRISTOL_EVENT_KEYON, 0, 5);
+		sleep(1);
+		bristolMidiSendMsg(global.controlfd,
+			synth->midichannel, BRISTOL_EVENT_KEYOFF, 0, 5);
+	}
+	 */
 	return(0);
 }
 
@@ -814,6 +900,8 @@ miniConfigure(brightonWindow *win)
 	guiSynth *synth = findSynth(global.synths, win);
 	brightonEvent event;
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 	if (synth == 0)
 	{
 		printf("problems going operational\n");
@@ -823,17 +911,17 @@ miniConfigure(brightonWindow *win)
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
-printf("going operational: %x, %x\n", (size_t) synth, (size_t) win);
+	printf("going operational: %p, %p\n", synth, win);
 
 	synth->flags |= OPERATIONAL;
 	synth->transpose = 36;
 	synth->keypanel = KEY_PANEL;
-	synth->keypanel2 = KEY_PANEL;
+	synth->keypanel2 = -1;
 	loadMemory(synth, "mini", 0, synth->location, synth->mem.active,
 		FIRST_DEV, 0);
 
-	brightonPut(win,
-		"bitmaps/blueprints/minishade.xpm", 0, 0, win->width, win->height);
+	brightonPut(win, "bitmaps/blueprints/minishade.xpm",
+		0, 0, win->width, win->height);
 
 	/*
 	 * Hm. This is a hack for a few bits of bad rendering of a keyboard. Only
