@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -816,8 +815,6 @@ printf("mixMemory()\n");
 static void
 mixMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
-
 	if ((synth->flags & OPERATIONAL) == 0)
 		return;
 
@@ -831,8 +828,6 @@ mixMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	bristolMidiSendMsg(global.controlfd, synth->sid,
 		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 
 	displayPanelText(synth, "MIDI", synth->midichannel + 1,
 		DISPLAY_PANEL, DISPLAY_DEV);
@@ -1312,7 +1307,7 @@ mixInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the mix link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the mix link to bristol: %p\n", synth->win);
 
 /* Alloc device_count. This is wrong, should be 'abs max'. */
 	synth->mem.param = (float *) initMixerMemory(16);
@@ -1330,7 +1325,7 @@ mixInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 /*	for (i = 0; i < DEVICE_COUNT; i++) */
 /*		synth->dispatch[i].routine = (synthRoutine) mixSendMsg; */
@@ -1356,7 +1351,7 @@ mixConfigure(brightonWindow *win)
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
-printf("going operational %x %x\n", (size_t) synth, (size_t) synth->win);
+	printf("going operational %p %p\n", synth, synth->win);
 
 	synth->flags |= OPERATIONAL;
 	synth->keypanel = -1;
