@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -43,8 +42,8 @@ extern guimain global;
 
 #include "brightonKeys.h"
 
-#define DEVICE_COUNT 60
-#define ACTIVE_DEVS 60 /* Actually 57 - should not sve pitch control. */
+#define DEVICE_COUNT 61
+#define ACTIVE_DEVS 61 /* Actually 57 - should not sve pitch control. */
 #define MEM_START 0
 #define MIDI_START (MEM_START + 18)
 
@@ -113,145 +112,149 @@ extern guimain global;
 
 static brightonLocations locations[DEVICE_COUNT] = {
 	/* Glide/Tranpose - 0 */
-	{"", 1, C0 - 4, R0, W2, L1, 0, 1, 0,
+	{"Glide", 1, C0 - 4, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"", 2, C1, R2 + 20, 15, L1/2 - 40, 0, 2, 0, 0, 0,
+	{"Transpose", 2, C1, R2 + 20, 15, L1/2 - 40, 0, 2, 0, 0, 0,
 		BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
 
 	/* VCO - 2 */
-	{"", 1, C2, R0, W2, L1, 0, 1, 0,
+	{"VCO1-FM-1", 1, C2, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlue.xpm", 0, 0},
-	{"", 1, C3, R0, W2, L1, 0, 1, 0,
+	{"VCO1-FM-2", 1, C3, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C4, R0, W2, L1, 0, 1, 0,
+	{"VCO1-PW", 1, C4, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"", 1, C5, R0, W2, L1, 0, 1, 0,
+	{"VCO1-PWM", 1, C5, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpYellow.xpm", 0, 0},
-	{"", 1, C2, R4, W2, L1, 0, 1, 0,
+	{"VCO1-Coarse", 1, C2, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlue.xpm", 0, BRIGHTON_NOTCH},
-	{"", 1, C3, R4, W2, L1, 0, 1, 0,
+	{"VCO1-Fine", 1, C3, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, BRIGHTON_NOTCH},
 
-	{"", 2, C2, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO1-Mod1-SinSqr", 2, C2, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C3, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO1-Mod2-SH/Env", 2, C3, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C5, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO1-PWM-Sine/Env", 2, C5, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C4, R4, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO1-KBD/LFO", 2, C4, R4, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* VCO2 - 12 */
-	{"", 1, C6, R0, W2, L1, 0, 1, 0,
+	{"VCO2-FM-1", 1, C6, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlue.xpm", 0, 0},
-	{"", 1, C7, R0, W2, L1, 0, 1, 0,
+	{"VCO2-FM-2", 1, C7, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C8, R0, W2, L1, 0, 1, 0,
+	{"VCO2-PW", 1, C8, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"", 1, C9, R0, W2, L1, 0, 1, 0,
+	{"VCO2-PWM", 1, C9, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpYellow.xpm", 0, 0},
-	{"", 1, C6, R4, W2, L1, 0, 1, 0,
+	{"VCO2-Coarse", 1, C6, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlue.xpm", 0, BRIGHTON_NOTCH},
-	{"", 1, C7, R4, W2, L1, 0, 1, 0,
+	{"VCO2-Fine", 1, C7, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, BRIGHTON_NOTCH},
 
-	{"", 2, C6, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO2-Mod1-SinS/H", 2, C6, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C7, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO2-Mod2-SH/Env", 2, C7, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C9, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO2-PWM-Sine/Env", 2, C9, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C8, R4, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO2-Sync", 2, C8, R4, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* Mods - 22 */
-	{"", 1, C10, R0, W2, L1, 0, 1, 0,
+	{"S/H-Mix-1", 1, C10, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpWhite.xpm", 0, 0},
-	{"", 1, C11, R0, W2, L1, 0, 1, 0,
+	{"S/H-Mix-2", 1, C11, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpWhite.xpm", 0, 0},
-	{"", 1, C12, R0, W2, L1, 0, 1, 0,
+	{"S/H-Gain", 1, C12, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"", 1, C12, R4, W2, L1, 0, 1, 0,
+	{"LFO-Freq", 1, C12, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
 
-	{"", 2, C10, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"S/H-Mix1-RampSqr", 2, C10, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C11, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"S/H-Mix2-NoiseVCO2", 2, C11, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C12 - CDIFF, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"S/H-Trig-KBD/LFO", 2, C12 - CDIFF, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* Filter, etc - 29 */
-	{"", 1, C13, R0, W2, L1, 0, 1, 0,
+	{"Mix-1-Lvl", 1, C13, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpYellow.xpm", 0, 0},
-	{"", 1, C14, R0, W2, L1, 0, 1, 0,
+	{"Mix-2-Lvl", 1, C14, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlue.xpm", 0, 0},
-	{"", 1, C15, R0, W2, L1, 0, 1, 0,
+	{"Mix-3-Lvl", 1, C15, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C16, R0, W2, L1, 0, 1, 0,
+	{"VCF-Mod-1", 1, C16, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpGreen.xpm", 0, 0},
-	{"", 1, C17, R0, W2, L1, 0, 1, 0,
+	{"VCF-Mod-2", 1, C17, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpGreen.xpm", 0, 0},
-	{"", 1, C18, R0, W2, L1, 0, 1, 0,
+	{"VCF-Mod-3", 1, C18, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpGreen.xpm", 0, 0},
-	{"", 1, C19, R0, W2, L1, 0, 1, 0,
+	{"VCA-EnvLvl", 1, C19, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C13, R4, W2, L1, 0, 1, 0,
+	{"VCF-Cutoff", 1, C13, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpWhite.xpm", 0, 0},
-	{"", 1, C14, R4, W2, L1, 0, 1, 0,
+	{"VCF-Res", 1, C14, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpWhite.xpm", 0, 0},
-	{"", 1, C18, R4, W2, L1, 0, 1, 0,
+	{"HPF-Cutoff", 1, C18, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpGreen.xpm", 0, 0},
-	{"", 1, C19, R4, W2, L1, 0, 1, 0,
+	{"VCA-Gain", 1, C19, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
 	/* And its routing - 40 */
-	{"", 2, C13, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"MIX-Noise/RM", 2, C13, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C14, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"MIX-VCO1-RampSqr", 2, C14, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C15, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"MIX-VCO2-RampSqr", 2, C15, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C16, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO-Mod1-KBD/SH", 2, C16, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C17, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO-Mod2-SH/LFO", 2, C17, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C18, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCO-Mod3-ADSR/SR", 2, C18, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C19, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"VCA-ADSR/AR", 2, C19, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* ADSR - 47 */
-	{"", 1, C20, R0, W2, L1, 0, 1, 0,
+	{"Attack", 1, C20, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C21, R0, W2, L1, 0, 1, 0,
+	{"Decay", 1, C21, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C22, R0, W2, L1, 0, 1, 0,
+	{"Sustain", 1, C22, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C23, R0, W2, L1, 0, 1, 0,
+	{"Release", 1, C23, R0, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C20, R4, W2, L1, 0, 1, 0,
+	{"AR-Attack", 1, C20, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
-	{"", 1, C21, R4, W2, L1, 0, 1, 0,
+	{"AR-Release", 1, C21, R4, W2, L1, 0, 1, 0,
 		"bitmaps/knobs/sliderArpRed.xpm", 0, 0},
 
-	{"", 2, C20, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"ADSR-KBD/REPEAT", 2, C20, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C22, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"ADSR-KBD/AUTO", 2, C22, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, C23, R5, W1 * 3/2, 60, 0, 1, 0,
+	{"AR-KBD/REPEAT", 2, C23, R5, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* Pink/White noise - 56 */
-	{"", 2, C1, R4, W1 * 3/2, 60, 0, 1, 0,
+	{"Noise-White/Pink", 2, C1, R4, W1 * 3/2, 60, 0, 1, 0,
 		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 
 	/* Pitch Control - 57 */
-	{"", 2, 25, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch-Flat", 2, 25, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
-	{"", 2, 53, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch-Trill", 2, 53, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
-	{"", 2, 80, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
+	{"Pitch-Sharp", 2, 80, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
+
+	/* Dual osc frequencies */
+	{"VCO2-Dual", 2, C8, R4 + 160, W1 * 3/2, 60, 0, 1, 0,
+		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 };
 
 static brightonLocations locations2[DEVICE_COUNT] = {
@@ -395,6 +398,10 @@ static brightonLocations locations2[DEVICE_COUNT] = {
 		"bitmaps/buttons/touchnlw.xpm", 0},
 	{"", 2, 80, R5, 30, 60, 0, 1, 0, "bitmaps/buttons/touchnlw.xpm",
 		"bitmaps/buttons/touchnlw.xpm", 0},
+
+	/* Dual osc frequencies */
+	{"VCO2-Dual", 2, C8, R4 + 160, W1 * 3/2, 60, 0, 1, 0,
+		"bitmaps/buttons/klunk.xpm", 0, BRIGHTON_VERTICAL},
 };
 
 #define BDIFF (CDIFF)
@@ -496,7 +503,7 @@ brightonApp odysseyApp = {
 	odysseyConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{4, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	600, 450,
 	6, /* Panels */
 	{
@@ -708,7 +715,6 @@ odysseyMemory(brightonWindow * win, int panel, int index, float value)
 static int
 odysseyMidi(guiSynth *synth, int c)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -740,8 +746,6 @@ odysseyMidi(guiSynth *synth, int c)
 		 */
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -830,7 +834,7 @@ odysseyCallback(brightonWindow * win, int panel, int index, float value)
 	} else {
 		brightonEvent event;
 		/*
-		 * We were in panel 0, we need to adjust the paramter in the next
+		 * We were in panel 0, we need to adjust the parameter in the next
 		 * layer
 		 */
 		if (calling)
@@ -1001,7 +1005,7 @@ odysseyInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the odyssey link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the odyssey link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	bs = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
@@ -1020,7 +1024,7 @@ odysseyInit(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
@@ -1188,6 +1192,8 @@ odysseyInit(brightonWindow *win)
 	dispatch[58].operator = 58;
 	dispatch[59].controller = 126;
 	dispatch[59].operator = 59;
+	dispatch[60].controller = 126;
+	dispatch[60].operator = 60;
 
 	/*
 	 * Need to specify env gain fixed, filter mod on, osc waveform.
