@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -119,63 +118,63 @@ extern guimain global;
 #define S4 10
 
 static brightonLocations locations[DEVICE_COUNT] = {
-	{"", 0, C0, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C1, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, BRIGHTON_NOTCH},
-	{"", 0, C1, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"MasterVolume", 0, C0, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"MasterTuning", 0, C1, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, BRIGHTON_NOTCH},
+	{"BendDepth", 0, C1, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 
-	/* VCO 3 */
-	{"", 0, C2, R0, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
+	/* VCO 1 */
+	{"OSC-Transpose", 0, C2, R0, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
 		BRIGHTON_STEPPED},
-	{"", 0, C3, R0, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
+	{"OSC-WaveForm", 0, C3, R0, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
 		BRIGHTON_STEPPED},
-	{"", 0, C4, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C5, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 2, C6 - 5, R0 + 10, S4, S3, 0, 2, 0, 0, 0,
+	{"OSC-PWM", 0, C4, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Osc-Tuning", 0, C5, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Osc-Subwave", 2, C6 - 5, R0 + 10, S4, S3, 0, 2, 0, 0, 0,
 		BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
 
 	/* Noise - Mods 8 */
-	{"", 0, C2, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C3, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C5, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 2, C6 - 5, R1 + 20, S4, S3, 0, 2, 0, 0, 0,
+	{"Noise", 0, C2, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"LFO-Freq", 0, C3, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"LFO-Delay", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"LFO-Level", 0, C5, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"LFO-Dest", 2, C6 - 5, R1 + 20, S4, S3, 0, 2, 0, 0, 0,
 		BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
 
 	/* Filter 13 */
-	{"", 0, C7, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C8, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C9, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C10, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"VCF-Freq", 0, C7, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"VCF-Res", 0, C8, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"VCF-Env", 0, C9, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"VCF-KBD", 0, C10, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 
 	/* Mode 17 */
-	{"", 2, C8 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
+	{"MODE Hold", 2, C8 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
 		"bitmaps/buttons/pressonw.xpm", BRIGHTON_RADIOBUTTON},
-	{"", 2, C9 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
+	{"MODE Mono", 2, C9 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
 		"bitmaps/buttons/pressonw.xpm", BRIGHTON_RADIOBUTTON},
-	{"", 2, C10 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
+	{"MODE Mono", 2, C10 + 8, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffw.xpm",
 		"bitmaps/buttons/pressonw.xpm", BRIGHTON_RADIOBUTTON},
 
 	/* ENV 20 */
-	{"", 0, C11, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C12, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C13, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C14, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"FiltEnv-Attack", 0, C11, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"FiltEnv-Decay", 0, C12, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"FiltEnv-Sustain", 0, C13, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"FiltEnv-Relase", 0, C14, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 
 	/* ENV 24 */
-	{"", 0, C11, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C12, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C13, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C14, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Env-Attack", 0, C11, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Env-Decay", 0, C12, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Env-Sustain", 0, C13, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Env-Release", 0, C14, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 
 	/* Amp - Effects 28 */
-	{"", 2, C15, R0 + 20, S4, S1 - 20, 0, 1, 0,
+	{"Amp-Env/Gate", 2, C15, R0 + 20, S4, S1 - 20, 0, 1, 0,
 		"bitmaps/buttons/sw1.xpm", "bitmaps/buttons/sw2.xpm", BRIGHTON_NOSHADOW},
-	{"", 0, C16, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
-	{"", 0, C15, R1, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
+	{"Amp-Gain", 0, C16, R0, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Chorus-Mode", 0, C15, R1, S1, S1, 0, 3, 0, "bitmaps/knobs/knob6.xpm", 0, 
 		BRIGHTON_STEPPED},
-	{"", 0, C16, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Chorus-Depth", 0, C16, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 	/* Glide */
-	{"", 0, C0, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
+	{"Glide", 0, C0, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob6.xpm", 0, 0},
 
 	/* Memory - MIDI 33 */
 	{"", 2, C17, R1, S2, S1, 0, 1, 0, "bitmaps/buttons/pressoffo.xpm",
@@ -373,8 +372,6 @@ poly6MidiSendMsg(void *synth, int fd, int chan, int c, int o, int v)
 static void
 poly6Hold(guiSynth *synth)
 {
-	bristolMidiMsg msg;
-
 	if (((synth->flags & OPERATIONAL) == 0) || (global.libtest))
 		return;
 
@@ -384,9 +381,6 @@ poly6Hold(guiSynth *synth)
 	else
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_HOLD|0);
-
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 }
 
 static void
@@ -404,6 +398,23 @@ fixModes(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	poly6Hold(synth);
 }
 
+int
+poly6loadMemory(guiSynth *synth, char *algo, char *name, int location,
+int active, int skip, int flags)
+{
+	brightonEvent event;
+
+	loadMemory(synth, "poly", 0, location, synth->mem.active, 0, 0);
+	fixModes(synth, synth->sid, synth->midichannel, 0, 0, 0);
+
+	event.type = BRISTOL_FLOAT;
+	event.value = 1;
+	brightonParamChange(synth->win, 0, MEM_START + location / 10, &event);
+	brightonParamChange(synth->win, 0, MEM_START+8+(location % 10), &event);
+
+	return(0);
+}
+
 static void
 poly6Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
@@ -557,7 +568,6 @@ poly6Waveform(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 poly6Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -581,8 +591,6 @@ poly6Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -627,7 +635,7 @@ poly6Callback(brightonWindow * win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -755,7 +763,7 @@ poly6Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the poly6 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the poly6 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -775,7 +783,7 @@ poly6Init(brightonWindow *win)
 		synth->voices = 6;
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = poly6MidiSendMsg;
@@ -1007,6 +1015,9 @@ printf("going operational\n");
 	event.value = 0.1;
 	brightonParamChange(synth->win, 2, 1, &event);
 	configureGlobals(synth);
+
+	synth->loadMemory = (loadRoutine) poly6loadMemory;
+
 	return(0);
 }
 
