@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -343,29 +342,29 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 
-	{"", 9, C1, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 Octave", 9, C1, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C2, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 Wave", 9, C2, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C3, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 16'", 9, C3, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C4, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 8'", 9, C4, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C5, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 4'", 9, C5, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C6, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 2'", 9, C6, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C7, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO1 Level", 9, C7, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	/* Noise 18 */
-	{"", 9, C8, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO Mode", 9, C8, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 
@@ -373,39 +372,39 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 
 	/* DCO 21 */
-	{"", 9, C9, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 Octave", 9, C9, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C10, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 Wave", 9, C10, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C11, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 16'", 9, C11, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C12, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 8'", 9, C12, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C13, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 4'", 9, C13, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C14, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 2'", 9, C14, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C15, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 Level", 9, C15, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* DCO 2 31 */
-	{"", 9, C16, PR1, P8W + 2, P8H, 0, 1, 0, 
+	{"DCO2 Interval", 9, C16, PR1, P8W + 2, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C17, PR1, P8W, P8H, 0, 1, 0, 
+	{"DCO2 Detune", 9, C17, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	/* Noise 33 */	
-	{"", 9, C18, PR1, P8W, P8H, 0, 1, 0, 
+	{"Noise", 9, C18, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
@@ -416,47 +415,47 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* VCF 41 */
-	{"", 9, C19, PR1, P8W, P8H, 0, 1, 0, 
+	{"VCF Cutoff", 9, C19, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C20, PR1, P8W, P8H, 0, 1, 0, 
+	{"VCF Resonance", 9, C20, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C21, PR1, P8W, P8H, 0, 1, 0, 
+	{"VCF KBD", 9, C21, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C22, PR1, P8W + 2, P8H, 0, 1, 0, 
+	{"VCF EnvPolarity", 9, C22, PR1, P8W + 2, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C23, PR1, P8W, P8H, 0, 1, 0, 
+	{"VCF EngTrack", 9, C23, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, C24, PR1, P8W, P8H, 0, 1, 0, 
+	{"VCF Env Trigger", 9, C24, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
-	{"", 9, C25, PR1, P8W, P8H, 0, 1, 0, 
+	{"On/Off", 9, C25, PR1, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* DEG 1 51 */
-	{"", 9, LC1, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Attack", 9, LC1, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC2, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Decay", 9, LC2, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC3, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Break", 9, LC3, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC4, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Slope", 9, LC4, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC5, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Sustain", 9, LC5, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC6, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG1 Release", 9, LC6, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
@@ -464,22 +463,22 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* DEG 2 61 */
-	{"", 9, LC7, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Attack", 9, LC7, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC8, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Decay", 9, LC8, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC9, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Break", 9, LC9, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC10, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Slope", 9, LC10, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC11, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Sustain", 9, LC11, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC12, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG2 Release", 9, LC12, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
@@ -487,22 +486,22 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* DEG 3 71 */
-	{"", 9, LC13, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Attack", 9, LC13, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC14, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Decay", 9, LC14, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC15, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Break", 9, LC15, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC16, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Slope", 9, LC16, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC17, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Sustain", 9, LC17, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC18, PR2, P8W, P8H, 0, 1, 0, 
+	{"DEG3 Release", 9, LC18, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
@@ -510,30 +509,127 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
 	/* Mod 81 */
-	{"", 9, LC19, PR2, P8W, P8H, 0, 1, 0, 
+	{"LFO Freq", 9, LC19, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC20, PR2, P8W, P8H, 0, 1, 0, 
+	{"LFO Delay", 9, LC20, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC21, PR2, P8W, P8H, 0, 1, 0, 
+	{"LFO DCO", 9, LC21, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC22, PR2, P8W, P8H, 0, 1, 0, 
+	{"LFO VCF", 9, LC22, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	{"", 2, 0, 0, 10, 10, 0, 1, 0, 0, 0, BRIGHTON_WITHDRAWN},
-	{"", 9, LC23, PR2, P8W, P8H, 0, 1, 0, 
+	{"MidiChannel", 9, LC23, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC24, PR2, P8W, P8H, 0, 1, 0, 
+	{"ProgEnable", 9, LC24, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
-	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+	{"OmniOnOff", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
 		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_CHECKBUTTON|BRIGHTON_TRACKING},
 	/* Done */
 
+	{"", 9, 5, 5, 5, 5, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+	{"", 9, LC25, PR2, P8W, P8H, 0, 1, 0, 
+		"bitmaps/buttons/blue.xpm", "bitmaps/buttons/green.xpm",
+		BRIGHTON_WITHDRAWN},
+
 };
 
 #define PMW 40
@@ -565,7 +661,7 @@ brightonLocations p800mods[MODS_COUNT] = {
 	{"", 2, 740, 460, 50, 40, 0, 1, 0, "bitmaps/buttons/polywhiteH.xpm",
 		"bitmaps/buttons/polyredV.xpm", BRIGHTON_CHECKBUTTON},
 
-	{"", 0, 210, 645, 130, 180, 0, 1, 0, "bitmaps/knobs/line.xpm", 0,
+	{"MasterVolume", 0, 210, 645, 130, 180, 0, 1, 0, "bitmaps/knobs/line.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_NOSHADOW},
 	{"", 1, 190, 500, 150, 35, 0, 1, 0, "bitmaps/buttons/polywhiteV.xpm", 0, 
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOTCH|BRIGHTON_NOSHADOW},
@@ -586,7 +682,7 @@ brightonLocations p800mods[MODS_COUNT] = {
 		"bitmaps/buttons/polyredV.xpm", BRIGHTON_CHECKBUTTON},
 
 	/* Entry pot 18 */
-	{"", 0, 835, 750, 100, 140, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 0},
+	{"DataEntry", 0, 835, 750, 100, 140, 0, 1, 0, "bitmaps/knobs/knob1.xpm", 0, 0},
 
 	/* Midi is now in the memory */
 	{"", 2, 0, 0, 25, 140, 0, 1, 0, "bitmaps/buttons/polywhiteV.xpm",
@@ -636,7 +732,7 @@ brightonApp poly800App = {
 	poly800Configure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{6, 100, 2, 2, 5, 520, 0, 0},
+	{8, 100, 2, 2, 5, 520, 0, 0},
 	1016, 370,
 	3, /* one panel only */
 	{
@@ -652,20 +748,6 @@ brightonApp poly800App = {
 			DEVICE_COUNT,
 			locations
 		},
-/*
-		{
-			"Keyboard",
-			0,
-			"bitmaps/keys/vkbg.xpm",
-			0x020|BRIGHTON_STRETCH,
-			0,
-			0,
-			poly800KeyCallback,
-			35, 550, 960, 430,
-			VKEY_COUNT,
-			vkeys
-		},
-*/
 		{
 			"Keyboard",
 			0,
@@ -868,6 +950,14 @@ loadMemoryShim(guiSynth *synth, int from)
 	brightonParamChange(synth->win, MODS_PANEL, ENTRY_POT, &event);
 }
 
+int
+p800loadMemory(guiSynth *synth, char *algo, char *name, int location,
+int active, int skip, int flags)
+{
+	loadMemoryShim(synth, location);
+	return(0);
+}
+
 static int
 midiCallback(brightonWindow *win, int controller, int value, float n)
 {
@@ -1504,8 +1594,6 @@ static int pmc = -1;
 static void
 p800midichannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
-
 	if (global.libtest)
 		return;
 
@@ -1519,8 +1607,6 @@ p800midichannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	bristolMidiSendMsg(global.controlfd, synth->sid,
 		127, 0, BRISTOL_MIDICHANNEL|v);
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 
 	pmc = v;
 }
@@ -1623,7 +1709,7 @@ poly800Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the poly800 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the poly800 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1640,8 +1726,9 @@ poly800Init(brightonWindow *win)
 	 */
 	if (!global.libtest)
 	{
+		bcopy(&global, &manual, sizeof(guimain));
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 		manual.flags |= BRISTOL_CONN_FORCE|BRIGHTON_NOENGINE;
 		manual.port = global.port;
@@ -1812,6 +1899,8 @@ poly800Configure(brightonWindow *win)
 
 	memSave = 0;
 
+	synth->loadMemory = (loadRoutine) p800loadMemory;
+
 	return(0);
 }
 
