@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -128,31 +127,31 @@ extern guimain global;
  */
 static brightonLocations locations[DEVICE_COUNT] = {
 /* mod */
-	{"FiltEnv", 0, C1, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"ModFiltEnv", 0, C1, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"FiltEnv", 2, C2, R1S, S2, S3, 0, 1.01, 0,
+	{"ModFiltEnv", 2, C2, R1S, S2, S3, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"OscBMod", 0, C1, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"ModOscBMod", 0, C1, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"OscBRt", 2, C2, R2S, S2, S3, 0, 1.01, 0,
+	{"ModOscBRt", 2, C2, R2S, S2, S3, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
-	{"LfoMod", 0, C1, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"ModLfo", 0, C1, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"LfoRt", 2, C2, R3S, S2, S3, 0, 1.01, 0,
+	{"ModLfoRt", 2, C2, R3S, S2, S3, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 
-	{"OSCAFr", 1, C3, RA, S4, S5, 0, 2, 0,
+	{"ModOSCAFr", 1, C3, RA, S4, S5, 0, 2, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
-	{"OSCAPWM", 1, C3, RB, S4, S5, 0, 2, 0,
+	{"ModOSCAPWM", 1, C3, RB, S4, S5, 0, 2, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
-	{"OSCBFr", 1, C3, RC, S4, S5, 0, 2, 0,
+	{"ModOSCBFr", 1, C3, RC, S4, S5, 0, 2, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
-	{"OSCBPWM", 1, C3, RD, S4, S5, 0, 2, 0,
+	{"ModOSCBPWM", 1, C3, RD, S4, S5, 0, 2, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
-	{"Filter", 1, C3, RE, S4, S5, 0, 2, 0,
+	{"ModFilter", 1, C3, RE, S4, S5, 0, 2, 0,
 		"bitmaps/buttons/klunk3.xpm", 0, 0},
 /* Osc-A */
-	{"OSCAFine", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"OscAFine", 0, C4, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_NOTCH},
 	{"OscAOct", 0, C5, R1, S1, S1, 0, 3, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_STEPPED},
@@ -165,7 +164,7 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	{"OscASync", 2, C9, R1S, S2, S3, 0, 1.01, 0,
 		"bitmaps/buttons/klunk4.xpm", 0, BRIGHTON_VERTICAL},
 /* Osc-B */
-	{"OSCBFine", 0, C4, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"OscBFine", 0, C4, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_NOTCH},
 	{"OscBOct", 0, C5, R2, S1, S1, 0, 3, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_STEPPED},
@@ -220,20 +219,20 @@ static brightonLocations locations[DEVICE_COUNT] = {
 /* Filter */
 	{"Cutoff", 0, C13, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"Emph", 0, C14, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"Emphasis", 0, C14, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
 	{"EnvAmt", 0, C15, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
 	{"KeyAmt", 0, C16, R1, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
 /* ENV */
-	{"Attack", 0, C13, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"Filt Attack", 0, C13, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"Decay", 0, C14, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"Filt Decay", 0, C14, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"Sustain", 0, C15, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"Filt Sustain", 0, C15, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
-	{"Release", 0, C16, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"Filt Release", 0, C16, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW},
 /* ENV */
 	{"Attack", 0, C13, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
@@ -245,9 +244,9 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	{"Release", 0, C16, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_HALFSHADOW},
 /* Tune Vol */
-	{"Tuning", 0, C17, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"VolumeTuning", 0, C17, R2, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_NOTCH|BRIGHTON_REDRAW},
-	{"Master", 0, C17, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
+	{"MasterVolume", 0, C17, R3, S1, S1, 0, 1, 0, "bitmaps/knobs/knob9.xpm", 0,
 		BRIGHTON_REDRAW|BRIGHTON_HALFSHADOW},
 /* memories */
 	{"", 2, C10 + 10, R4, S6, S7, 0, 1, 0, "bitmaps/buttons/pressoffg.xpm",
@@ -317,7 +316,7 @@ brightonApp pro1App = {
 	pro1Configure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{5, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	895, 500,
 	5,
 	{
@@ -727,7 +726,6 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 static int
 pro1Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -751,8 +749,6 @@ pro1Midi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -830,7 +826,7 @@ pro1Callback(brightonWindow * win, int panel, int index, float value)
 
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -843,8 +839,6 @@ pro1Callback(brightonWindow * win, int panel, int index, float value)
 static void
 pro1Hold(guiSynth *synth)
 {
-	bristolMidiMsg msg;
-
 	if ((synth->flags & OPERATIONAL) == 0)
 		return;
 
@@ -854,9 +848,6 @@ pro1Hold(guiSynth *synth)
 	else
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_HOLD|0);
-
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 }
 
 /*
@@ -888,7 +879,7 @@ pro1Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the pro1 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the pro1 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -906,7 +897,7 @@ pro1Init(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
