@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -105,79 +104,79 @@ extern brightonLocations mem[];
 
 static brightonLocations locations[MOD_COUNT] = {
 	/* Tune 0 */
-	{"", 0, C1 - 10, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 
+	{"SynthTuning", 0, C1 - 10, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 0, C3 - 15, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 
+	{"PolyTuning", 0, C3 - 15, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 
 		BRIGHTON_NOTCH},
 
 	/* Mods 2 */
-	{"", 1, C1, R3, W1, L1, 0, 1, 0,
+	{"Mod-Tone", 1, C1, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C2, R3, W1, L1, 0, 1, 0,
+	{"Mod-Filter", 1, C2, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C3, R3, W1, L1, 0, 1, 0,
+	{"Mod-Glide", 1, C3, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C4, R3, W1, L1, 0, 1, 0,
+	{"Mod-LFO-Rate", 1, C4, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 2, C4 - D2, R0, W2, L2, 0, 2, 0, 0, 0,
+	{"Mod-Shape", 2, C4 - D2, R0, W2, L2, 0, 2, 0, 0, 0,
 		BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
-	{"", 2, C4 - D2, R2 + 50, W2, L2, 0, 1, 0, "bitmaps/buttons/sw3.xpm",
+	{"Mod-EnvTrig", 2, C4 - D2, R2 + 50, W2, L2, 0, 1, 0, "bitmaps/buttons/sw3.xpm",
 		"bitmaps/buttons/sw5.xpm", BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
 
 	/* Osc-1 8 */
-	{"", 2, C5, R1, W2, L2, 0, 1, 0,
+	{"Osc1-Sync", 2, C5, R1, W2, L2, 0, 1, 0,
 		"bitmaps/buttons/sw3.xpm", "bitmaps/buttons/sw5.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
-	{"", 2, C6 + 20, R0 + 40, W4, L3, 0, 2, 0,
+	{"Osc1-Transpose", 2, C6 + 20, R0 + 40, W4, L3, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 2, C7 + 15, R1, W2, L2, 0, 1, 0,
+	{"Osc1-Waveform", 2, C7 + 15, R1, W2, L2, 0, 1, 0,
 		"bitmaps/buttons/sw3.xpm", "bitmaps/buttons/sw5.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
 
 	/* Osc-2 11 */
-	{"", 1, C5, R3, W1, L1, 0, 1, 0,
+	{"Osc2-Detune", 1, C5, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, BRIGHTON_NOTCH},
-	{"", 2, C6 + 20, R4 + 40, W4, L3, 0, 2, 0,
+	{"Osc2-Transpose", 2, C6 + 20, R4 + 40, W4, L3, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW},
-	{"", 2, C7 + 15, R5, W2, L2, 0, 1, 0,
+	{"Osc2-Waveform", 2, C7 + 15, R5, W2, L2, 0, 1, 0,
 		"bitmaps/buttons/sw3.xpm", "bitmaps/buttons/sw5.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
 
 	/* Contour 14 */
-	{"", 2, C8 + 6, R0, W2, L2, 0, 2, 0,
+	{"Env-AR/ASR", 2, C8 + 6, R0, W2, L2, 0, 2, 0,
 		"bitmaps/buttons/sw3.xpm", "bitmaps/buttons/sw5.xpm",
 		BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
-	{"", 2, C8 + 6, R2, W2, L2, 0, 2, 0,
+	{"Env-Trigger", 2, C8 + 6, R2, W2, L2, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
-	{"", 1, C9, R0, W1, L1, 0, 1, 0,
+	{"Env-Attack", 1, C9, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C10, R0, W1, L1, 0, 1, 0,
+	{"Env-Release", 1, C10, R0, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
 
 	/* Filter 18 */
-	{"", 2, C8 + 6, R5, W2, L2, 0, 2, 0,
+	{"VCF-KeyTrack", 2, C8 + 6, R5, W2, L2, 0, 2, 0,
 		0, 0, BRIGHTON_THREEWAY|BRIGHTON_NOSHADOW|BRIGHTON_VERTICAL},
-	{"", 1, C9 - 25, R3, W1, L1, 0, 1, 0,
+	{"VCF-Cutoff", 1, C9 - 25, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C9 + 25, R3, W1, L1, 0, 1, 0,
+	{"VCF-Emphasis", 1, C9 + 25, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C10 + 20, R3, W1, L1, 0, 1, 0,
+	{"VCF-Env", 1, C10 + 20, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
 
 	/* Mixer 22 */
-	{"", 1, C11, R3, W1, L1, 0, 1, 0,
+	{"Mix-Osc1", 1, C11, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C12, R3, W1, L1, 0, 1, 0,
+	{"Mix-Osc2", 1, C12, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C13, R3, W1, L1, 0, 1, 0,
+	{"Mix-Noise", 1, C13, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C14, R3, W1, L1, 0, 1, 0,
+	{"Mix-Bell", 1, C14, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
-	{"", 1, C15, R3, W1, L1, 0, 1, 0,
+	{"Mix-Poly", 1, C15, R3, W1, L1, 0, 1, 0,
 		"bitmaps/knobs/knob7.xpm", 0, 0},
 
 	/* Volume 27 */
-	{"", 0, C13 - 15, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 0},
+	{"MasterVolume", 0, C13 - 15, R0 + 50, W3, W3, 0, 1, 0, "bitmaps/knobs/knob3.xpm", 0, 0},
 
 	/* Midi Up/Down, save */
 	{"", 2, C15, R0, W1, 100, 0, 1, 0, "bitmaps/buttons/pressoff.xpm",
@@ -203,7 +202,8 @@ brightonApp realisticApp = {
 	realisticConfigure, /* 3 callbacks */
 	realisticMidiCallback,
 	0,
-	{1, 100, 2, 2, 5, 520, 0, 0},
+	/* This looks like 32 voices however the synth is single voice in preops */
+	{32, 100, 2, 2, 5, 520, 0, 0},
 	600, 320,
 	2, /* panel count */
 	{
@@ -287,7 +287,7 @@ realisticCallback(brightonWindow * win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -502,7 +502,7 @@ realisticInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the Realistic link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the Realistic link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -519,7 +519,7 @@ realisticInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = realisticMidiSendMsg;
@@ -665,6 +665,9 @@ realisticConfigure(brightonWindow *win)
 		return(-1);
 	}
 
+	if (synth->flags & OPERATIONAL)
+		return(0);
+
 printf("going operational\n");
 
 	synth->flags |= OPERATIONAL;
