@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -93,11 +92,11 @@ extern brightonLocations mem[];
 #define C6 850
 
 static brightonLocations locations[MOD_COUNT] = {
-	{"", 0, 90, 100, 300, 600, 0, 1, 0, 0, 0, 0},
-	{"", 0, 123, 100, 300, 600, 0, 1, 0, 0, 0, 0},
+	{"Bass", 0, 90, 100, 300, 600, 0, 1, 0, 0, 0, 0},
+	{"Volume", 0, 123, 100, 300, 600, 0, 1, 0, 0, 0, 0},
 	{"", 2, 50, 200, 30, 400, 0, 1, 0,
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0},
-	{"", 0, 156, 100, 300, 600, 0, 1, 0, 0, 0, 0},
+	{"Chorus", 0, 156, 100, 300, 600, 0, 1, 0, 0, 0, 0},
 };
 
 #define S1 200
@@ -143,7 +142,7 @@ brightonApp rhodesApp = {
 	rhodesConfigure, /* 3 callbacks */
 	rhodesMidiCallback,
 	0,
-	{16, 0, 2, 2, 5, 520, 0, 0},
+	{16, 0, 2, 2, 5, 520, 0, 1},
 	750, 250,
 	7, /* panel count */
 	{
@@ -292,7 +291,7 @@ rhodesCallback(brightonWindow * win, int panel, int index, float value)
 			sendvalue);
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -541,7 +540,7 @@ rhodesInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the rhodes link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the rhodes link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -564,7 +563,7 @@ rhodesInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = rhodesMidiSendMsg;
@@ -620,6 +619,7 @@ rhodesConfigure(brightonWindow *win)
 		printf("problems going operational\n");
 		return(-1);
 	}
+
 	if (synth->flags & OPERATIONAL)
 		return(0);
 
