@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -93,8 +92,8 @@ extern brightonLocations mem[];
 #define C6 850
 
 static brightonLocations locations[MOD_COUNT] = {
-	{"", 0, 50, 100, 400, 700, 0, 1, 0, 0, 0, 0},
-	{"", 0, 125, 100, 400, 700, 0, 1, 0, 0, 0, 0},
+	{"Bass", 0, 50, 100, 400, 700, 0, 1, 0, 0, 0, 0},
+	{"Volume", 0, 125, 100, 400, 700, 0, 1, 0, 0, 0, 0},
 /*	{"", 2, 50, 200, 30, 400, 0, 1, 0, */
 /*		"bitmaps/buttons/rockerwhite.xpm", 0, 0}, */
 };
@@ -134,7 +133,7 @@ brightonApp rhodesBassApp = {
 	rBassConfigure, /* 3 callbacks */
 	rBassMidiCallback,
 	0,
-	{32, 0, 2, 2, 5, 520, 0, 0},
+	{32, 0, 2, 2, 5, 520, 0, 1},
 	290, 200,
 	7, /* panel count */
 	{
@@ -283,7 +282,7 @@ rBassCallback(brightonWindow * win, int panel, int index, float value)
 			sendvalue);
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -517,7 +516,7 @@ rBassInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the Rhodes Bass link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the Rhodes Bass link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -539,7 +538,7 @@ rBassInit(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 	{
@@ -586,12 +585,15 @@ rBassConfigure(brightonWindow *win)
 		return(-1);
 	}
 
-printf("going operational\n");
+	if (synth->flags & OPERATIONAL)
+		return(0);
+
+	printf("going operational\n");
 
 	synth->flags |= OPERATIONAL;
 	synth->keypanel = 3;
 	synth->keypanel2 = -1;
-	synth->transpose = 36;
+	synth->transpose = 48;
 /*	loadMemory(synth, "rhodes", 0, synth->location, synth->mem.active, */
 /*		FIRST_DEV, 0); */
 
