@@ -9,12 +9,12 @@ Check out
 	touch response
 
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -23,8 +23,7 @@ Check out
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -91,13 +90,13 @@ static int dc, shade_id;
 #define L1 600
 
 static brightonLocations modspanel[MODS_COUNT] = {
-	{"", 1, C1, R1, W1, L1, 0, 1, 0, 0, 0,
+	{"Tune", 1, C1, R1, W1, L1, 0, 1, 0, 0, 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOSHADOW|BRIGHTON_NOTCH},
-	{"", 1, C2, R1, W1, L1, 0, 1, 0, 0, 0,
+	{"LFO Frequency", 1, C2, R1, W1, L1, 0, 1, 0, 0, 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOSHADOW},
-	{"", 1, C3, R1, W1, L1, 0, 1, 0, 0, 0,
+	{"Vibrato", 1, C3, R1, W1, L1, 0, 1, 0, 0, 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOSHADOW},
-	{"", 1, C4, R1, W1, L1, 0, 1, 0, 0, 0,
+	{"Volume", 1, C4, R1, W1, L1, 0, 1, 0, 0, 0,
 		BRIGHTON_VERTICAL|BRIGHTON_REVERSE|BRIGHTON_NOSHADOW},
 };
 
@@ -145,46 +144,46 @@ static brightonLocations modspanel[MODS_COUNT] = {
 
 static brightonLocations options[OPTS_COUNT] = {
 	/* Osc parameters */
-	{"", 0, oC1, oR1, oW1, oL1, 0, 1, 0, 0, 0, BRIGHTON_REVERSE|BRIGHTON_NOTCH},
-	{"", 0, oC2, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC3, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Detune", 0, oC1, oR1, oW1, oL1, 0, 1, 0, 0, 0, BRIGHTON_REVERSE|BRIGHTON_NOTCH},
+	{"PulseWidth", 0, oC2, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"PulseWidthMod", 0, oC3, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
 	/* waveforms */
-	{"", 2, oC50, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
+	{"Bass 1", 2, oC50, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
 		"bitmaps/buttons/rockersmoothBBd.xpm", 0},
-	{"", 2, oC51, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
+	{"Bass 2", 2, oC51, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
 		"bitmaps/buttons/rockersmoothBBd.xpm", 0},
-	{"", 2, oC52, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
+	{"Mid 1", 2, oC52, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
 		"bitmaps/buttons/rockersmoothBBLd.xpm", 0},
-	{"", 2, oC53, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
+	{"Mid 2", 2, oC53, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
 		"bitmaps/buttons/rockersmoothBBLd.xpm", 0},
-	{"", 2, oC54, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
+	{"Treble 1", 2, oC54, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
 		"bitmaps/buttons/rockersmoothBCd.xpm", 0},
-	{"", 2, oC55, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
+	{"Treble 2", 2, oC55, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
 		"bitmaps/buttons/rockersmoothBCd.xpm", BRIGHTON_NOSHADOW},
 
 	/* The remaining envelope parameters */
-	{"", 0, oC4, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC5, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 2, oC6, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBW.xpm",
+	{"Decay", 0, oC4, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Release", 0, oC5, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"TouchSense", 2, oC6, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBW.xpm",
 		"bitmaps/buttons/rockersmoothBWd.xpm", 0},
 
 	/* Tremelo */
-	{"", 0, oC16, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Tremelo", 0, oC16, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
 
 	/* Chorus */
-	{"", 0, oC7, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC8, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC9, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC10, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Choris Wet", 0, oC7, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"ChorusD1", 0, oC8, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"ChorusD2", 0, oC9, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"ChorusSpeed", 0, oC10, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
 
 	/* Reverb */
-	{"", 0, oC11, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC12, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC13, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
-	{"", 0, oC14, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Reberb Wet", 0, oC11, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Reverb Cross", 0, oC12, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Reverb Feed", 0, oC13, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
+	{"Reverg Length", 0, oC14, oR1, oW1, oL1, 0, 1, 0, 0, 0, 0},
 
 	/* Single/Multi LFO */
-	{"", 2, oC56, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBGR.xpm",
+	{"LFO Multi", 2, oC56, 100, 27, 600, 0, 1, 0, "bitmaps/buttons/rockersmoothBGR.xpm",
 		"bitmaps/buttons/rockersmoothBGRd.xpm", 0},
 };
 
@@ -219,21 +218,21 @@ static brightonLocations options[OPTS_COUNT] = {
 
 static
 brightonLocations mem[MEM_COUNT] = {
-	{"", 0, mC2 -  70, mR2, 260, 260, 0, 1, 0, 0, 0, 0},
-	{"", 0, mC4 - 100, mR2, 260, 260, 0, 1, 0, 0, 0, 0},
+	{"Bass", 0, mC2 -  70, mR2, 260, 260, 0, 1, 0, 0, 0, 0},
+	{"Treble", 0, mC4 - 100, mR2, 260, 260, 0, 1, 0, 0, 0, 0},
 
 	/* memories */
-	{"", 2, mC0, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
+	{"Mem-0", 2, mC0, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBB.xpm",
 		"bitmaps/buttons/rockersmoothBBd.xpm", 0}, /* -127 65 */
-	{"", 2, mC1, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBG.xpm",
+	{"Mem-1", 2, mC1, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBG.xpm",
 		"bitmaps/buttons/rockersmoothBGd.xpm", 0},
-	{"", 2, mC2, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
+	{"Mem-2", 2, mC2, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBBL.xpm",
 		"bitmaps/buttons/rockersmoothBBLd.xpm", 0}, /* 190 31 -3 */
-	{"", 2, mC3, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBGR.xpm",
+	{"Mem-3", 2, mC3, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBGR.xpm",
 		"bitmaps/buttons/rockersmoothBGRd.xpm", 0},
-	{"", 2, mC4, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
+	{"Mem-4", 2, mC4, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBC.xpm",
 		"bitmaps/buttons/rockersmoothBCd.xpm", 0},
-	{"", 2, mC4s, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBW.xpm",
+	{"Mem-5", 2, mC4s, mR1, 125, 400, 0, 1, 0, "bitmaps/buttons/rockersmoothBW.xpm",
 		"bitmaps/buttons/rockersmoothBWd.xpm", 0},
 
 	{"", 1, 0, 0, 50, 50, 0, 1, 0, "bitmaps/knobs/sliderpointL.xpm", 0,
@@ -363,7 +362,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 
 		synth->dispatch[MEM_START].other1 = index;
 	} else {
-		bristolMidiMsg msg;
 		int newchan;
 
 		/*
@@ -389,8 +387,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 				break;
@@ -413,8 +409,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 				break;
@@ -439,7 +433,7 @@ brightonApp roadrunnerApp = {
 	roadrunnerConfigure, /* 3 callbacks */
 	midiCallback,
 	destroySynth,
-	{32, 0, 2, 2, 5, 520, 0, 0},
+	{-1, 0, 2, 2, 5, 520, 0, 0},
 	660, 138,
 	7, /* panel count */
 	{
@@ -792,7 +786,7 @@ roadrunnerCallback(brightonWindow *win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -828,8 +822,7 @@ roadrunnerInit(brightonWindow* win)
 
 	synth->win = win;
 
-	printf("Initialise the roadrunner link to bristol: %x\n",
-		(size_t) synth->win);
+	printf("Initialise the roadrunner link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -846,7 +839,7 @@ roadrunnerInit(brightonWindow* win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = roadrunnerMidiNull;
@@ -991,7 +984,6 @@ printf("going operational\n");
 	synth->keypanel = KEY_PANEL;
 	synth->keypanel2 = -1;
 	synth->transpose = 36;
-	loadMemory(synth, "roadrunner", 0, synth->location, synth->mem.active, 0, 0);
 
 	/*
 	 * Hm. This is a hack for a few bits of bad rendering of a keyboard. Only
@@ -1031,6 +1023,8 @@ printf("going operational\n");
 	brightonParamChange(synth->win, MEM_PANEL, 2, &event);
 	panelSwitch(synth, 0, 0, 0, 0, 0);
 
+	loadMemory(synth, "roadrunner", 0, 0, synth->mem.active-6, 0, 0);
+
 	synth->dispatch[MEM_START].other1 = 2;
 	synth->dispatch[MEM_START].other2 = 0;
 
