@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -33,11 +32,13 @@
 #include <unistd.h>
 
 #include "bristol.h"
+#include "brightoninternals.h"
 #include "brightonMini.h"
 
 extern guimain global;
 
 extern void brightonWriteConfiguration(brightonWindow *, char *, int);
+extern void brightonMidiInput(bristolMidiMsg *, guimain *);
 
 struct sockaddr address;
 
@@ -61,7 +62,7 @@ cleanupBristolQuietly()
 	cleanupBristol();
 	 */
 	signal(SIGPIPE, SIG_DFL);
-printf("cleared sigpipe handler\n");
+	signal(SIGINT, SIG_DFL);
 	cleanupBristol();
 }
 
@@ -69,56 +70,69 @@ void
 cleanupBristol()
 {
 	bristolMidiMsg msg;
-	guiSynth *synth;
-	int result;
 
-#ifdef DEBUG
-	printf("cleanupBrighton()\n");
-#endif
-	printf("cleanupBrighton(%i)\n", global.controlfd);
+	close(0);
 
+	if (global.synths->win != 0)
+		global.synths->win->flags |= BRIGHTON_EXITING;
 
-	BAutoRepeat(global.synths->win->display, 1);
+	if (global.synths->flags & REQ_DEBUG_MASK)
+		printf("cleanupBrighton()\n");
 
-	if (global.controlfd >= 0)
+	if (global.libtest)
 	{
-		for (synth = global.synths; synth != 0; synth = synth->next)
-		{
-			if (synth->sid2 != 0)
-			{
-				bristolMidiSendMsg(global.controlfd, synth->sid2,
-					127, 0, BRISTOL_EXIT_ALGO);
-				if ((result = bristolMidiRead(global.controlfd, &msg)) < 0)
-					printf("socket closed\n");
-			}
-			bristolMidiSendMsg(global.controlfd, synth->sid,
-				127, 0, BRISTOL_EXIT_ALGO);
+		global.flags |= REQ_EXIT;
+		return;
+	}
 
-			if ((result = bristolMidiRead(global.controlfd, &msg)) < 0)
-				printf("socket closed\n");
-
-			printf("request acked: %i\n", result);
-
-			if (result >= 0)
-				sleep(2);
-		}
+	if (global.controlfd < 0)
+	{
+		global.flags |= REQ_EXIT;
+		return;
 	}
 
 	/*
-	 * I do not like killing the engine with a signal, I would prefer to send
-	 * it a message requesting it terminate itself. If there are no more 
-	 * bristolAudio's active, the engine will exit gracefully.
-	if (global.enginePID > 0)
-		kill(guimain.enginePID, SIGINT);
+	 * We don't need to clean out every synth, we need to cleanout the second
+	 * manual if it is configured, then the controlfd
 	 */
+	bristolMidiSendNRP(global.controlfd, global.synths->sid,
+		BRISTOL_NRP_MIDI_GO, 0);
 
-	/*
-	 * This is not actually an FD, but a libbristolmidi handle.
-	 */
-	if (global.controlfd > 0)
-		bristolMidiClose(global.controlfd);
+	if ((global.manualfd != 0) && (global.manualfd != global.controlfd))
+	{
+		bristolMidiSendMsg(global.manualfd, global.synths->sid2,
+			127, 0, BRISTOL_EXIT_ALGO);
+		if (global.synths->flags & REQ_DEBUG_MASK)
+			printf("Secondary layer removed\n");
+		/* OK, so sleep is lax, we used to wait for a message */
+		bristolMidiRead(global.controlfd, &msg);
+	}
 
-	exit(0);
+	/* The engine has to restart MIDI processing after the exit */
+	bristolMidiSendNRP(global.controlfd, global.synths->sid,
+		BRISTOL_NRP_MIDI_GO, 0);
+
+	bristolMidiSendMsg(global.controlfd, global.synths->sid,
+		127, 0, BRISTOL_EXIT_ALGO);
+	if (global.synths->flags & REQ_DEBUG_MASK)
+		printf("Primary layer removed\n");
+	/* OK, so sleep is lax, we used to wait for a message */
+	bristolMidiRead(global.controlfd, &msg);
+
+	usleep(250000);
+
+	if ((global.manualfd != 0) && (global.manualfd != global.controlfd))
+		bristolMidiClose(global.manualfd);
+	bristolMidiClose(global.controlfd);
+
+	printf("brighton exiting\n");
+
+	//BAutoRepeat(global.synths->win->display, 1);
+
+	//sleep(1);
+	global.flags |= REQ_EXIT;
+
+	//exit(0);
 }
 
 int
@@ -127,7 +141,8 @@ destroySynth(brightonWindow* win)
 	guiSynth *synth = findSynth(global.synths, win);
 	bristolMidiMsg msg;
 
-printf("destroySynth(%i): %i\n", (size_t) win, (size_t) synth->sid);
+	printf("destroySynth(%p): %i\n", win, synth->sid);
+
 	/*
 	 * Since we registered two synths, we now need to remove the upper
 	 * manual.
@@ -147,6 +162,8 @@ displayPanel(guiSynth *synth, char *text, int value, int panel, int index)
 	brightonEvent event;
 	char display[32];
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 	sprintf(display, "%s", text);
 	event.type = BRIGHTON_MEM;
 	event.m = (void *) &display[0];
@@ -159,6 +176,8 @@ displayPanelText(guiSynth *synth, char *text, int value, int panel, int index)
 	brightonEvent event;
 	char display[32];
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 	sprintf(display, "%s: %i", text, value);
 	event.type = BRIGHTON_MEM;
 	event.m = (void *) &display[0];
@@ -173,6 +192,8 @@ displayText(guiSynth *synth, char *text, int value, int index)
 	brightonEvent event;
 	char display[32];
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 	sprintf(display, "%s: %i", text, value);
 	event.type = BRIGHTON_MEM;
 	event.m = (void *) &display[0];
@@ -347,17 +368,83 @@ getMemoryLocation(char *algo, char *name, int location, int perms)
 
 	if ((fd = open(path, perms, 0600)) < 0)
 	{
-		// printf("%s\n", path);
+		//printf("%s\n", path);
 		sprintf(path, "%s/memory/%s/%s%i.%s",
-			global.home, algo, algo,location, ext);
+			global.home, algo, algo, location, ext);
 		return(open(path, perms, 0660));
 	}
 
-	// printf("%s\n", path);
+	//printf("%s\n", path);
 
 	return(fd);
 }
 
+#define RECOPY_MAX_FILE 1024 * 1024
+
+static void
+bsmCopy(char *src, char *dst)
+{
+	char buffer[RECOPY_MAX_FILE];
+	int srcfd, dstfd, count;
+
+	srcfd = open(src, O_RDONLY);
+
+	if ((count = read(srcfd, buffer, RECOPY_MAX_FILE)) > 0)
+	{
+		dstfd = open(dst, O_WRONLY|O_TRUNC|O_CREAT, 0644);
+		count = write(dstfd, buffer, count);
+		close(dstfd);
+	}
+
+	close(srcfd);
+}
+
+/*
+ * The next two will copy files into and out of the bristol cache. Read is only
+ * really used on system startup if a session file has been given by Jack SM but
+ * it could be more general. Write will copy cache files out to wherever they 
+ * have been requested.
+ */
+int
+bristolMemoryExport(int source, char *dst, char *algo)
+{
+	char srcpath[1024];
+	char seqpath[1024];
+
+	snprintf(srcpath, 1024, "%s/memory/%s/%s%i.mem",
+		getBristolCache(algo), algo, algo, source);
+	bsmCopy(srcpath, dst);
+
+	printf("Export %s to %s\n", srcpath, dst);
+
+	snprintf(srcpath, 1024, "%s/memory/%s/%s%i.seq",
+		getBristolCache(algo), algo, algo, source);
+	snprintf(seqpath, 1024, "%s.seq", dst);
+	bsmCopy(srcpath, seqpath);
+
+	return(0);
+}
+
+int
+bristolMemoryImport(int dest, char *src, char *algo)
+{
+	char dstpath[1024];
+	char seqpath[1024];
+
+	snprintf(dstpath, 1024, "%s/memory/%s/%s%i.mem",
+		getBristolCache(algo), algo, algo, dest);
+	bsmCopy(src, dstpath);
+
+	printf("Import %s to %s\n", src, dstpath);
+
+	snprintf(seqpath, 1024, "%s.seq", src);
+	snprintf(dstpath, 1024, "%s/memory/%s/%s%i.seq",
+		getBristolCache(algo), algo, algo, dest);
+	bsmCopy(seqpath, dstpath);
+
+	return(0);
+}
+
 static int
 checkSeqMem(arpeggiatorMemory *mem)
 {
@@ -387,15 +474,15 @@ fillSequencer(guiSynth *synth, arpeggiatorMemory *mem, int flags)
 
 	if (checkSeqMem(mem) < 0)
 	{
-		printf("Sequencer sizing problem: %i %i\n",
-			(int) mem->s_max, (int) mem->chord);
+		printf("Sequencer sizing problem: %f %p\n", mem->s_max, mem->chord);
 		return(-1);
 	}
 
 	if (flags & BRISTOL_SID2)
 		sid = synth->sid2;
 
-	printf("Sending Arpeggiation to engine: %i events\n", (int) mem->s_max);
+	if ((synth->flags & REQ_DEBUG_MASK) > REQ_MIDI_DEBUG2)
+		printf("Sending Arpeggiation to engine: %i events\n", (int) mem->s_max);
 
 	if (mem->s_max > 0)
 	{
@@ -413,7 +500,8 @@ fillSequencer(guiSynth *synth, arpeggiatorMemory *mem, int flags)
 	bristolMidiSendMsg(global.controlfd, sid,
 		BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 0);
 
-	printf("Sending Chord to engine: %i events\n", (int) mem->c_count);
+	if ((synth->flags & REQ_DEBUG_MASK) > REQ_MIDI_DEBUG2)
+		printf("Sending Chord to engine: %i events\n", (int) mem->c_count);
 
 	if (mem->c_count > 0)
 	{
@@ -483,7 +571,10 @@ loadSequence(memory *seq, char *algo, int location, int flags)
 
 	if (strcmp(&tmem.algo[0], "bristol sequencer") != 0)
 	{
-		printf("Attempt to read %s into sequencer\n", &tmem.algo[0]);
+		if (tmem.algo[0] == '\0')
+			printf("Attempt to read (noname) into sequencer\n");
+		else
+			printf("Attempt to read %s into sequencer\n", &tmem.algo[0]);
 
 		if ((flags & BRISTOL_SEQFORCE) == 0)
 		{
@@ -514,17 +605,22 @@ loadSequence(memory *seq, char *algo, int location, int flags)
 }
 
 int
-loadMemory(guiSynth *synth, char *algo, char *name, int location, int active,
-int skip, int flags)
+loadMemory(guiSynth *synth, char *algo, char *name, int location,
+int active, int skip, int flags)
 {
 	brightonEvent event;
 	int i, fd, panel = 0, index = 0;
 	memory tmem;
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 /*printf("loadmemory %i: %s %x\n", location, synth->resources->name, flags); */
 	if (active == 0)
 		active = synth->mem.active;
 
+	synth->lmem = synth->cmem;
+	synth->cmem = location;
+
 	if (flags & BRISTOL_SEQLOAD) {
 		if (BRISTOL_SID2) {
 			if (loadSequence(&synth->seq2, algo, location, flags) >= 0)
@@ -588,10 +684,13 @@ int skip, int flags)
 
 	if (strcmp(&tmem.algo[0], algo) != 0)
 	{
-		printf("Attempt to read %s into %s algorithm\n", &tmem.algo[0], algo);
-
 		if ((flags & BRISTOL_FORCE) == 0)
 		{
+			if (tmem.algo[0] == '\0')
+				printf("Attempt to read (noname) into %s algorithm\n", algo);
+			else
+				printf("Attempt to read %s into %s algo\n", &tmem.algo[0],algo);
+
 			close(fd);
 			return(-1);
 		}
@@ -603,7 +702,8 @@ int skip, int flags)
 	synth->mem.count = tmem.count;
 	synth->mem.vers = tmem.vers;
 
-	if (synth->mem.active != tmem.active)
+	if (((synth->flags & REQ_DEBUG_MASK) > REQ_MIDI_DEBUG2)
+		&& (synth->mem.active != tmem.active))
 		printf("Active count changed. Overriding\n");
 /*	synth->mem.active = tmem.active; */
 
@@ -733,45 +833,33 @@ saveMemory(guiSynth *synth, char *algo, char *name, int location, int skip)
 }
 
 static void
-forkengine(guimain *global)
+connectengine(guimain *global)
 {
 	int flags;
 
-	if ((global->flags & BRIGHTON_NOENGINE) == 0)
-	{
-		printf("starting engine: %i\n", global->port);
-		if ((global->enginePID = fork()) == 0)
-		{
-			execlp(BRISTOL_ENGINE, "engine", NULL);
-			printf("Could not find bristol engine\n");
-			_exit(-1);
-		}
-
-		sleep(1);
-	}
-
-	flags = BRISTOL_CONN_TCP|BRISTOL_DUPLEX|
+	flags = BRISTOL_CONN_TCP|BRISTOL_DUPLEX|BRISTOL_CONN_NBLOCK|
 		(global->flags &  BRISTOL_CONN_FORCE);
 
 	if ((global->controlfd = bristolMidiOpen(global->host,
-		flags, global->port, -1, NULL, NULL)) < 0)
+		flags, global->port, -1, brightonMidiInput, global)) < 0)
 	{
 		printf("opening link to engine: %i\n", global->port);
 		if ((global->controlfd = bristolMidiOpen(global->host,
-			flags, global->port, BRISTOL_REQ_SYSEX, NULL, NULL)) < 0)
+			flags, global->port, -1, brightonMidiInput, global)) < 0)
 			cleanupBristol();
 	} else
-		printf("%s already active\n", BRISTOL_ENGINE);
+		printf("%s already active (%i)\n", BRISTOL_ENGINE, global->controlfd);
 }
 
 int
 initConnection(guimain *global, guiSynth *synth)
 {
 	bristolMidiMsg msg;
-	int sid;
+	int sid = -1;
 
-	forkengine(global);
+	connectengine(global);
 
+	//printf("initConnection()\n");
 	/*
 	 * send a hello, with a voice count, then request starting of the
 	 * synthtype. All of these will require an ACK, and we should wait here
@@ -780,8 +868,18 @@ initConnection(guimain *global, guiSynth *synth)
 	bristolMidiSendMsg(global->controlfd, synth->midichannel,
 /*		127, 0, BRISTOL_HELLO|BRISTOL_VOICES|synth->voices); */
 		127, 0, BRISTOL_HELLO|synth->voices);
+
+	if (synth->sid > 0)
+		synth->sid2 = global->controlfd + 1;
+
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
+	{
+		printf("unacknowledged request on %i\n", global->controlfd);
+		bristolMidiPrint(&msg);
+		sleep(1);
+		global->flags |= REQ_EXIT;
+		return(-1);
+	}
 
 	/*
 	 * The returned value is the connection ID that we need to use for all
@@ -789,22 +887,51 @@ initConnection(guimain *global, guiSynth *synth)
 	 */
 	sid = (msg.params.bristol.valueMSB << 7) + msg.params.bristol.valueLSB;
 
-printf("	sid is %i\n", sid);
+	if (synth->flags & REQ_MIDI_DEBUG2)
+	{
+		printf("sid is %i (%i, %i)\n", sid,
+			msg.params.bristol.valueMSB, msg.params.bristol.valueLSB);
+
+		bristolMidiPrint(&msg);
+	}
+
 	bristolMidiSendMsg(global->controlfd, sid,
 		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
+
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
+	{
+		bristolMidiSendMsg(global->controlfd, sid,
+			127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
+		if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
+		{
+			printf("unacknowledged request on %i\n", sid);
+			bristolMidiPrint(&msg);
+			sleep(1);
+			global->flags |= REQ_EXIT;
+			return(-1);
+		}
+	}
 
 	bristolMidiSendMsg(global->controlfd, sid,
 		127, 0, BRISTOL_REQSTART);
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
+	{
+		printf("unacknowledged request on %i\n", sid);
+		sleep(1);
+		global->flags |= REQ_EXIT;
+		return(-1);
+	}
 
 /*printf("	initialising one connection of type %i\n", synth->synthtype); */
 	bristolMidiSendMsg(global->controlfd, sid,
 		127, 0, BRISTOL_INIT_ALGO|synth->synthtype);
 	if (bristolMidiRead(global->controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
+	{
+		printf("unacknowledged request on %i\n", sid);
+		sleep(1);
+		global->flags |= REQ_EXIT;
+		return(-1);
+	}
 
 	return(sid);
 }
