@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -289,34 +288,34 @@ brightonLocations locations[DEVICE_COUNT] = {
 	 * PW/Tune/transpose(/Glide?) - Pots
 	 * Att/Dec/Sust/Rel - Sliders
 	 */
-	{"", 2, V1C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Noise", 2, V1C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Tri", 2, V1C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Ramp", 2, V1C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Square", 2, V1C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 0, V1C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 0, V1C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
+	{"V1-PW", 0, V1C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"V1-Tune", 0, V1C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 0, V1C3b, V1R3, VBW3, VBH2, 0, 24,0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 0, V1C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 2, V1C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Transpose", 0, V1C3b, V1R3, VBW3, VBH2, 0, 24,0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"V1-Glide", 0, V1C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"V1-RM", 2, V1C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-3-1-Sync", 2, V1C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Mute", 2, V1C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V1C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V1-Filt", 2, V1C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 1, V1C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V1-Attack", 1, V1C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V1C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V1-Decay", 1, V1C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V1C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V1-Sustain", 1, V1C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V1C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V1-Release", 1, V1C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
 	/* Dummies */
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
@@ -329,34 +328,34 @@ brightonLocations locations[DEVICE_COUNT] = {
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
 	/* Voice 2 - 20 */
-	{"", 2, V2C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Noise", 2, V2C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Tri", 2, V2C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Ramp", 2, V2C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Square", 2, V2C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 0, V2C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0,0},
-	{"", 0, V2C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
+	{"V2-PW", 0, V2C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0,0},
+	{"V2-Tune", 0, V2C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 0, V2C3b, V1R3, VBW3, VBH2, 0, 24, 0, "bitmaps/knobs/knobred.xpm",0,0},
-	{"", 0, V2C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0,0},
-	{"", 2, V2C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Transpose", 0, V2C3b, V1R3, VBW3, VBH2, 0, 24, 0, "bitmaps/knobs/knobred.xpm",0,0},
+	{"V2-Glide", 0, V2C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0,0},
+	{"V2-RM-1-2", 2, V2C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Sync", 2, V2C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Mute", 2, V2C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V2C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V2-Filt", 2, V2C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 1, V2C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V2-Attack", 1, V2C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V2C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V2-Decay", 1, V2C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V2C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V2-Sustain", 1, V2C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V2C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V2-Release", 1, V2C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
 	/* Dummies */
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
@@ -369,34 +368,34 @@ brightonLocations locations[DEVICE_COUNT] = {
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
 	/* Voice 3 - 40 */
-	{"", 2, V3C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Noise", 2, V3C1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Tri", 2, V3C2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Ramp", 2, V3C3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Square", 2, V3C4, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 0, V3C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 0, V3C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
+	{"V3-PW", 0, V3C1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"V3-Tune", 0, V3C2b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 
 		BRIGHTON_NOTCH},
-	{"", 0, V3C3b, V1R3, VBW3, VBH2, 0, 24, 0, "bitmaps/knobs/knobred.xpm", 0,0},
-	{"", 0, V3C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 2, V3C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Transpose", 0, V3C3b, V1R3, VBW3, VBH2, 0, 24, 0, "bitmaps/knobs/knobred.xpm", 0,0},
+	{"V3-Glide", 0, V3C4b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"V3-RM-2-3", 2, V3C1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Sync", 2, V3C2, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Mute", 2, V3C3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, V3C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"V3-Filt", 2, V3C4, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 1, V3C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V3-Attack", 1, V3C5, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V3C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V3-Decay", 1, V3C6, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V3C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V3-Sustain", 1, V3C7, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, V3C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"V3-Release", 1, V3C8, V1R2, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
 	/* Dummies */
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
@@ -409,22 +408,22 @@ brightonLocations locations[DEVICE_COUNT] = {
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
 	/* Filter - 60 */
-	{"", 2, FC1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"VCF-HP", 2, FC1, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, FC2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"VCF-BP", 2, FC2, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, FC3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"VCF-LP", 2, FC3, V1R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 0, FC1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 0, FC2b, V1R3, VBW3, VBH2, 0, 15,0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 0, FC3b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
-	{"", 2, FC3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"VCF-Cutoff", 0, FC1b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"VCF-Resonance", 0, FC2b, V1R3, VBW3, VBH2, 0, 15,0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"VCF-Mix", 0, FC3b, V1R3, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobred.xpm", 0, 0},
+	{"VCF-LPF24", 2, FC3, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
-	/* DCO Multi and MasterVol */
-	{"", 2, FC1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	/* VCF Multi and MasterVol */
+	{"VCF-Multi", 2, FC1, V1R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 0,  S2C23+8, S2R0 - 50, VBW3, VBH2, 0, 15, 0,
+	{"MasterVolume", 0,  S2C23+8, S2R0 - 50, VBW3, VBH2, 0, 15, 0,
 		"bitmaps/knobs/knobyellow.xpm", 0, 0},
 
 	/*
@@ -438,50 +437,50 @@ brightonLocations locations[DEVICE_COUNT] = {
 	 *	wah
 	 */
 	/* Key Mode */
-	{"", 2, S2C0-3, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Mode Mono", 2, S2C0-3, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0-3, S2R0+60, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Mode Poly1", 2, S2C0-3, S2R0+60, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0-3, S2R0+120, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Mode Poly2", 2, S2C0-3, S2R0+120, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0-3, S2R0+180, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Mode Arpeg1", 2, S2C0-3, S2R0+180, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C0-3, S2R0+240, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Mode Arpeg2", 2, S2C0-3, S2R0+240, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
-	/* Mods */
-	{"", 0, S2C1b+2, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobgreen.xpm",
+	/* Mods - 76 */
+	{"LFO-Rate", 0, S2C1b+2, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobgreen.xpm",
 		0, 0},
-	{"", 0, S2C3b+2, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobgreen.xpm",
+	{"LFO-Depth", 0, S2C3b+2, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobgreen.xpm",
 		0, 0},
-	{"", 2, S2C1, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"???1", 2, S2C1, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C2, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"???2", 2, S2C2, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C3, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"???3", 2, S2C3, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C4, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"???4", 2, S2C4, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
 	/* 80 */
-	{"", 2, S2C5, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-PW-V1", 2, S2C5, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C5, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-PW-V2", 2, S2C5, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C5, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-PW-V3", 2, S2C5, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C6, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-FM-V1", 2, S2C6, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C6, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-FM-V2", 2, S2C6, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C6, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-FM-V3", 2, S2C6, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C6, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-FM-Filt", 2, S2C6, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 	/* Mod Touch control */
-	{"", 2, S2C1b, S2R0 - 70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-F-Touch", 2, S2C1b, S2R0 - 70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C3b, S2R0 - 70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"LFO-G-Touch", 2, S2C3b, S2R0 - 70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
@@ -497,33 +496,33 @@ brightonLocations locations[DEVICE_COUNT] = {
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
 	/* Env mods 95 */
-	{"", 2, S2C7, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-FM-V1", 2, S2C7, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C7, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-FM-V2", 2, S2C7, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C7, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-FM-V3", 2, S2C7, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C7, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-FM-Filt", 2, S2C7, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C8, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-PW-V1", 2, S2C8, S2R0, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C8, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-PW-V2", 2, S2C8, S2R2, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C8, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-PW-V3", 2, S2C8, S2R3, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
 	/* 102 - Mod Env */
-	{"", 1, S2C8b, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"ModEnv-Attack", 1, S2C8b, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, S2C9, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"ModEnv-Sustain", 1, S2C9, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, S2C10, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"ModEnv-Decay", 1, S2C10, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, S2C11, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"ModEnv-Release", 1, S2C11, S2R0, VBW2, VBH4, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 1, S2C12, S2R0, VBW2, VBH4, 0, 1, 0, "bitmaps/buttons/polywhiteV.xpm",
+	{"ModEnv-Gain", 1, S2C12, S2R0, VBW2, VBH4, 0, 1, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, 0},
-	{"", 2, S2C11+13, S2R0-70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"ModEnv-Touch", 2, S2C11+13, S2R0-70, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
@@ -548,10 +547,10 @@ brightonLocations locations[DEVICE_COUNT] = {
 	{"", 1, 0, 0, VBW2, VBH3, 0, 15, 0, "bitmaps/buttons/polywhiteV.xpm",
 		0, BRIGHTON_NOSHADOW|BRIGHTON_WITHDRAWN},
 
-	{"", 0, S2C14, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm",0,0},
-	{"", 2, S2C13, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Arpeg-Rate", 0, S2C14, S2R1, VBW3, VBH2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm",0,0},
+	{"Arpeg-Trig", 2, S2C13, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
-	{"", 2, S2C15, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
+	{"Arpeg-Scan", 2, S2C15, S2R4, VBW1, VBH1, 0, 1, 0, "bitmaps/buttons/sidb.xpm",
 		"bitmaps/buttons/sidbon.xpm", 0},
 
 	/* Memories 118 - first 8 digits */
@@ -619,7 +618,7 @@ brightonApp sidApp = {
 	sidConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{6, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	1000, 330,
 	3, /* panels */
 	{
@@ -789,6 +788,16 @@ loadMemoryMidiShim(guiSynth *synth, int from)
 	loadMemoryShim(synth);
 }
 
+int
+loadMemoryKeyShim(guiSynth *synth, char *algo, char *name, int location,
+int active, int skip, int flags)
+{
+	loadMemory(synth, SYNTH_NAME, 0, location,
+		synth->mem.active, 0, BRISTOL_FORCE);
+	loadMemoryShim(synth);
+	return(0);
+}
+
 static int
 midiCallback(brightonWindow *win, int controller, int value, float n)
 {
@@ -877,14 +886,6 @@ sidMidiShim(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	bristolMidiSendMsg(fd, synth->sid, c, o, v);
 }
 
-static int
-sidMidiSendMsg(guiSynth *synth, int fd, int chan, int o, int c, int v)
-{
-	bristolMidiSendMsg(fd, synth->sid, o, c, v);
-
-	return(0);
-}
-
 static void
 sidLFOWave(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
@@ -1107,9 +1108,11 @@ sidMemory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 			}
 			synth->flags &= ~BANK_SELECT;
 
-			/* Doubleclick on load will toggle debugging */
+			/*
+			 * Doubleclick on load will toggle debugging
 			if (brightonDoubleClick(dc) != 0)
 				bristolMidiSendMsg(fd, synth->sid, 126, 4, 1);
+			 */
 
 			break;
 		case 2:
@@ -1336,9 +1339,6 @@ sidCallback(brightonWindow *win, int panel, int index, float value)
 static void
 sidMidiChannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
-
-
 	if ((c == 0) && (++synth->midichannel > 15))
 		synth->midichannel = 15;
 	else if ((c == 1) && (--synth->midichannel < 0))
@@ -1351,8 +1351,6 @@ sidMidiChannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	bristolMidiSendMsg(global.controlfd, synth->sid,
 		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 }
 
 /*
@@ -1381,7 +1379,7 @@ sidInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the sid link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the sid link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1399,7 +1397,7 @@ sidInit(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
@@ -1826,6 +1824,8 @@ sidConfigure(brightonWindow *win)
 
 	dc = brightonGetDCTimer(1000000);
 
+	synth->loadMemory = (loadRoutine) loadMemoryKeyShim;
+
 	return(0);
 }
 
