@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -899,7 +898,7 @@ brightonApp sid2App = {
 	sid2Configure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{6, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	1000, 450,
 	3, /* panels */
 	{
@@ -1156,14 +1155,6 @@ sid2MidiShim(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	bristolMidiSendMsg(fd, synth->sid2, c, o, v);
 }
 
-static int
-sid2MidiSendMsg(guiSynth *synth, int fd, int chan, int o, int c, int v)
-{
-	bristolMidiSendMsg(fd, synth->sid2, o, c, v);
-
-	return(0);
-}
-
 static void
 sid2LFOWave(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
@@ -1615,9 +1606,6 @@ sid2Callback(brightonWindow *win, int panel, int index, float value)
 static void
 sid2MidiChannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
-
-
 	if ((c == 0) && (++synth->midichannel > 15))
 		synth->midichannel = 15;
 	else if ((c == 1) && (--synth->midichannel < 0))
@@ -1630,8 +1618,6 @@ sid2MidiChannel(guiSynth *synth, int fd, int chan, int c, int o, int v)
 
 	bristolMidiSendMsg(global.controlfd, synth->sid2,
 		127, 0, BRISTOL_MIDICHANNEL|synth->midichannel);
-	if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-		cleanupBristol();
 }
 
 /*
@@ -1660,7 +1646,7 @@ sid2Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the sid2 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the sid2 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1678,7 +1664,7 @@ sid2Init(brightonWindow *win)
 	if (!global.libtest)
 	{
 		if ((synth->sid2 = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 	}
 
 	for (i = 0; i < DEVICE_COUNT; i++)
