@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -123,37 +122,37 @@ extern int empty;
  */
 static brightonLocations locations[DEVICE_COUNT] = {
 /* LFO Master - 0*/
-	{"LFO-F", 1, 114, 490, 13, 180, 0, 1, 0,
+	{"LFO-MasterF", 1, 114, 490, 13, 180, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
 /* LFO 1 */
-	{"LFO-W", 0, 40, 253, S1, S1, 0, 3, 0, 
+	{"LFO-X-Wave", 0, 40, 253, S1, S1, 0, 3, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_STEPPED},
-	{"LFO-F", 1, 90, 230, 13, 180, 0, 1, 0,
+	{"LFO-X-Rate", 1, 90, 230, 13, 180, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"LFO-MOD", 2, 40, 370, S3, S4, 0, 2, 0,
+	{"LFO-Dest", 2, 40, 370, S3, S4, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_THREEWAY},
 /* LFO 2  - 4*/
-	{"LFO-F", 1, 135, 230, 13, 180, 0, 1, 0,
+	{"LFO-Y-Rate", 1, 135, 230, 13, 180, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"LFO-W", 0, 168, 253, S1, S1, 0, 3, 0, 
+	{"LFO-Wave", 0, 168, 253, S1, S1, 0, 3, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_STEPPED},
-	{"LFO-MOD", 2, 188, 370, S3, S4, 0, 2, 0,
+	{"LFO-Dest", 2, 188, 370, S3, S4, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_THREEWAY},
 /* LFO Mix */
 	{"LFO-MIX", 0, 106, 88, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
 
 /* Osc 1 - 8 */
-	{"OSCA-F", 0, 230, 260, S2, S2, 0, 1, 0, 
+	{"OscA-Tuning", 0, 230, 260, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
-	{"OSCA-F", 0, 230, 360, S2, S2, 0, 1, 0, 
+	{"OscA-PW", 0, 230, 360, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-MOD", 2, 278, 280, S5, S6, 0, 2, 0,
+	{"OscA-Transpose", 2, 278, 280, S5, S6, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_VERTICAL|BRIGHTON_THREEWAY},
 	/* This can replace the previous for a 5 octave span */
-//	{"OSCA-F", 0, 278, 260, S2, S2, 0, 4, 0, 
+//	{"OscA-F", 0, 278, 260, S2, S2, 0, 4, 0, 
 //		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_STEPPED},
-	{"OSCA-MOD", 2, 278, 380, S5, S6, 0, 2, 0,
+	{"OscA-Waveform", 2, 278, 380, S5, S6, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_VERTICAL|BRIGHTON_THREEWAY},
 
 	{"dummy", 0, 0, 0, S2, S2, 0, 1, 0, 
@@ -161,23 +160,23 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	{"dummy", 0, 0, 0, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_WITHDRAWN},
 
-	{"OSCA-F", 0, 230, 518, S2, S2, 0, 1, 0, 
+	{"OscA-FM-XY", 0, 230, 518, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-F", 0, 285, 518, S2, S2, 0, 1, 0, 
+	{"OscA-FM-ADSR", 0, 285, 518, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-MOD", 2, 260, 640, S5, S6, 0, 2, 0,
+	{"OscA-Mode", 2, 260, 640, S5, S6, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_VERTICAL|BRIGHTON_THREEWAY},
 /* Osc 2 - 17 */
-	{"OSCA-F", 0, 390, 260, S2, S2, 0, 1, 0, 
+	{"OscB-Tuning", 0, 390, 260, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
-	{"OSCA-F", 0, 390, 360, S2, S2, 0, 1, 0, 
+	{"OscB-PW", 0, 390, 360, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-MOD", 2, 337, 280, S5, S6, 0, 2, 0,
+	{"OscB-Transpose", 2, 337, 280, S5, S6, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_VERTICAL|BRIGHTON_THREEWAY},
 	/* This can replace the previous for a 5 octave span */
-//	{"OSCA-F", 0, 337, 260, S2, S2, 0, 4, 0, 
+//	{"OscA-F", 0, 337, 260, S2, S2, 0, 4, 0, 
 //		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_STEPPED},
-	{"OSCA-MOD", 2, 337, 380, S5, S6, 0, 2, 0,
+	{"OscB-Waveform", 2, 337, 380, S5, S6, 0, 2, 0,
 		"sonicrocker", 0, BRIGHTON_VERTICAL|BRIGHTON_THREEWAY},
 
 	{"dummy", 0, 0, 0, S2, S2, 0, 1, 0, 
@@ -185,37 +184,37 @@ static brightonLocations locations[DEVICE_COUNT] = {
 	{"dummy", 0, 0, 0, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_WITHDRAWN},
 
-	{"OSCA-F", 0, 335, 518, S2, S2, 0, 1, 0, 
+	{"OscB-FM-XY", 0, 335, 518, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-F", 0, 390, 518, S2, S2, 0, 1, 0, 
+	{"OscB-FM-OscA", 0, 390, 518, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OSCA-F", 0, 362, 616, S2, S2, 0, 1, 0, 
+	{"OscB-PWM", 0, 362, 616, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
 /* Osc Mix - 26 */
-	{"OSC-MIX", 0, 309, 88, S2, S2, 0, 1, 0, 
+	{"Osc-A/B-Mix", 0, 309, 88, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
 
 /* Ring Mod */
-	{"RINGMOD", 2, 460, 274, S5, S6, 0, 1, 0,
+	{"RM-Src1", 2, 460, 274, S5, S6, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", BRIGHTON_VERTICAL},
-	{"RINGMOD", 2, 460, 380, S5, S6, 0, 1, 0,
+	{"RM-Src2", 2, 460, 380, S5, S6, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", BRIGHTON_VERTICAL},
 
 /* Noise */
-	{"Noise", 2, 469, 580, S3, S4 + 10, 0, 1, 0,
+	{"WhitePink", 2, 469, 580, S3, S4 + 10, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker3.xpm",
 		"bitmaps/buttons/sonicrocker1.xpm", 0},
 
 /* Mixer - 30 */
-	{"MIX-OSC", 0, 548, 260, S2, S2, 0, 1, 0, 
+	{"Mix-OscA/B", 0, 548, 260, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"MIX-OSC", 0, 548, 370, S2, S2, 0, 1, 0, 
+	{"Mix-RM", 0, 548, 370, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"MIX-OSC", 0, 548, 480, S2, S2, 0, 1, 0, 
+	{"Mix-Ext", 0, 548, 480, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"MIX-OSC", 0, 548, 590, S2, S2, 0, 1, 0, 
+	{"Mix-Noise", 0, 548, 590, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
 
 /* Bypass */
@@ -240,58 +239,58 @@ static brightonLocations locations[DEVICE_COUNT] = {
 		"bitmaps/buttons/sonicrocker3.xpm", BRIGHTON_VERTICAL},
 
 /* Filter - 41 */
-	{"Cutoff", 1, 740, 260, 13, 270, 0, 1, 0,
+	{"VCF-Cutoff", 1, 740, 260, 13, 270, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
-	{"Res", 1, 784, 260, 13, 270, 0, 1, 0,
+	{"VCF-Res", 1, 784, 260, 13, 270, 0, 1, 0,
 		"bitmaps/knobs/sliderArpBlack.xpm", 0, 0},
 
 /* Direct output mixer */
-	{"OscA", 0, 845, 88, S2, S2, 0, 1, 0, 
+	{"Direct-OscA", 0, 845, 88, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"OscB", 0, 890, 88, S2, S2, 0, 1, 0, 
+	{"Direct-OscB", 0, 890, 88, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"Ringmod", 0, 935, 88, S2, S2, 0, 1, 0, 
+	{"Direct-Ringmod", 0, 935, 88, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
 
 /* Triggers */
-	{"Trigger", 2, 625, 612, S3, S4 + 10, 0, 1, 0,
+	{"TriggerKBD", 2, 625, 612, S3, S4 + 10, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
-	{"Trigger", 2, 655, 612, S3, S4 + 10, 0, 1, 0,
+	{"TriggerLFO-X", 2, 655, 612, S3, S4 + 10, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
-	{"Trigger", 2, 685, 612, S3, S4 + 10, 0, 1, 0,
+	{"TriggerLFO-Y", 2, 685, 612, S3, S4 + 10, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
 
 /* Pitch control - 49 */
-	{"OSC1-F", 0, 728, 616, S2, S2, 0, 1, 0, 
+	{"Osc1-FM-ADSR", 0, 728, 616, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"Trigger", 2, 762, 612, S3, S4 + 10, 0, 1, 0,
+	{"Osc1-FM-KBD", 2, 762, 612, S3, S4 + 10, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
-	{"OSC1-F", 0, 787, 616, S2, S2, 0, 1, 0, 
+	{"Osc1-FM-X/Y", 0, 787, 616, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
 
 /* Memories dummies - 52 */
 	/* Multi LFO-X */
-	{"LFO-XM", 2, 150, 884, S3, S4 + 4, 0, 1, 0,
+	{"LFO-X-Multi", 2, 150, 884, S3, S4 + 4, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
 	/* Multi LFO-X */
-	{"LFO-YM", 2, 180, 884, S3, S4 + 4, 0, 1, 0,
+	{"LFO-Y-Multi", 2, 180, 884, S3, S4 + 4, 0, 1, 0,
 		"bitmaps/buttons/sonicrocker1.xpm",
 		"bitmaps/buttons/sonicrocker3.xpm", 0},
 	{"is dummydummy", 0, 100, 880, S7, S7, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_WITHDRAWN},
 
-	{"dummy", 0, 807, 880, S7, S7, 0, 1, 0, 
+	{"FX-1", 0, 807, 880, S7, S7, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"dummy", 0, 842, 880, S7, S7, 0, 1, 0, 
+	{"FX-2", 0, 842, 880, S7, S7, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"dummy", 0, 877, 880, S7, S7, 0, 1, 0, 
+	{"FX-3", 0, 877, 880, S7, S7, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"dummy", 0, 912, 880, S7, S7, 0, 1, 0, 
+	{"FX-4", 0, 912, 880, S7, S7, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, 0},
 	{"dummy", 0, 0, 0, S2, S2, 0, 1, 0, 
 		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_WITHDRAWN},
@@ -399,7 +398,7 @@ brightonApp sonic6App = {
 	sonic6Configure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{8, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	900, 520,
 	3,
 	{
@@ -540,7 +539,7 @@ sonic6Callback(brightonWindow * win, int panel, int index, float value)
 			synth->dispatch[index].operator,
 			sendvalue);
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -551,7 +550,6 @@ sonic6Callback(brightonWindow * win, int panel, int index, float value)
 static int
 sonic6Midi(guiSynth *synth, int fd, int chan, int c, int o, int value)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -569,8 +567,6 @@ sonic6Midi(guiSynth *synth, int fd, int chan, int c, int o, int value)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -1002,7 +998,7 @@ sonic6Init(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the sonic6 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the sonic6 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1019,7 +1015,7 @@ sonic6Init(brightonWindow *win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++) {
 		synth->dispatch[i].controller = 126;
