@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -185,93 +184,93 @@ static guimain manual;
 
 static brightonLocations stratusLocations[DEVICE_COUNT] = {
 	/* Organ */
-	{"", 1, C1-10, 110, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-2'", 1, C1-10, 110, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW|BRIGHTON_REVERSE},
-	{"", 1, C1-10, 180, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-4'", 1, C1-10, 180, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW|BRIGHTON_REVERSE},
-	{"", 1, C1-10, 250, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-8'", 1, C1-10, 250, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW|BRIGHTON_REVERSE},
-	{"", 1, C1-10, 320, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-16'", 1, C1-10, 320, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW|BRIGHTON_REVERSE},
 
 	/* Filter - 4 */
-	{"", 0, C3, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"VCF-Env", 0, C3, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
-	{"", 0, C4, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"VCF-Cutoff", 0, C4, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"VCF-Res", 0, C5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C6, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"VCF-Pedal", 0, C6, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 8 */
-	{"", 0, C7, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Glide-Amount", 0, C7, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C8, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Glide-Speed", 0, C8, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C9, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Glide-Multi", 0, C9, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C10, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Glide-Dir", 0, C10, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 12 */
-	{"", 0, C11, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Osc1-Tuning", 0, C11, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
-	{"", 2, C11 + (D2*1) + 3, R1, S2b+4, S2-10, 0, 1, 0,
+	{"Osc1-Sync", 2, C11 + (D2*1) + 3, R1, S2b+4, S2-10, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C11 + (D2*2), R1, S2b+4, S2-10, 0, 12, 0,
+	{"Osc1-Octave", 2, C11 + (D2*2), R1, S2b+4, S2-10, 0, 12, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
 	/* - 15 */
-	{"", 2, C11 + (D2*1) + 3, R2, S2b+4, S2-10, 0, 1, 0,
+	{"Osc1-Trill", 2, C11 + (D2*1) + 3, R2, S2b+4, S2-10, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C11 + (D2*2), R2, S2b+4, S2-10, 0, 12, 0,
+	{"Osc2-Octave", 2, C11 + (D2*2), R2, S2b+4, S2-10, 0, 12, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 0, C11 + (D2*0), R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Osc2-Tuning", 0, C11 + (D2*0), R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
 
 	/* - 18 */
-	{"", 0, C13+10, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Routing", 0, C13+10, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED|BRIGHTON_NOTCH},
-	{"", 0, C14 + D1/2 + 3, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Multi", 0, C14 + D1/2 + 3, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C16-5, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Shape", 0, C16-5, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 21 */
-	{"", 0, C1-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-Synth", 0, C1-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C2-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-Organ", 0, C2-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C3-5, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-X", 0, C3-5, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* - 24 String section */
-	{"", 0, C4, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"X1", 0, C4, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, C5, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"X2", 0, C5, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, C6, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"X3", 0, C6, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, C7, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"X4", 0, C7, 0, 0, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* - 28 */
-	{"", 0, C7+10, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc-Waveform", 0, C7+10, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C8 + D1/2 + 3, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc-WaveAlt", 0, C8 + D1/2 + 3, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C10-5, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc-WaveLegato", 0, C10-5, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 31 */
-	{"", 0, C13, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Rate", 0, C13, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C14, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Attack", 0, C14, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C15, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Delay", 0, C15, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C16, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Depth", 0, C16, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 	/* Dummies 12 - 39 */
@@ -342,13 +341,13 @@ static brightonLocations stratusLocations[DEVICE_COUNT] = {
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* These are shadows for the envelope parameters from the mod panel - 66 */
-	{"", 0, C3, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Attack", 0, C3, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C4, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Decay", 0, C4, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Sustain", 0, C5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C6, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Release", 0, C6, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 /*
@@ -498,7 +497,7 @@ brightonApp stratusApp = {
 	stratusConfigure, /* 3 callbacks, unused? */
 	stratusMidiCallback,
 	destroySynth,
-	{5, 100, 2, 2, 5, 520, 0, 0},
+	{5, 100, 1, 2, 5, 520, 0, 0},
 	840, 385,
 	7, /* Panel count */
 	{
@@ -744,7 +743,7 @@ printf("stratusModCallback(%i, %i, %f) %x\n", panel, dev, value,
 				synth->dispatch[index].operator,
 				sendvalue);
 	} else
-		printf("dispatch2[%x,%i](%i, %i, %i, %i, %i)\n",(size_t) synth, index,
+		printf("dispatch2[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd,
 			synth->dispatch[index].other1, /* The SID is buried here */
 			synth->dispatch[index].controller,
@@ -1104,39 +1103,9 @@ stratusOrganEnv(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	}
 }
 
-static void
-stratusSync(guiSynth *synth, int fd, int chan, int c, int o, int v)
-{
-	bristolMidiSendMsg(global.controlfd, synth->sid, c, o, v);
-	return;
-	/*
-	 * When we sync the oscillators we have to clean up the signal on Osc-A
-	 * since otherwise the complexities of the bitone harmonic mixing and detune
-	 * make an apparant mess of Osc-B net output.
-	 */
-	if (v != 0) {
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 0, 0);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 1, 16383);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 2, 0);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 3, 0);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 4, 0);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 5, 0);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 6, 16383);
-		bristolMidiSendMsg(global.controlfd, synth->sid, 0, 12, 0);
-
-	} else {
-		/* Return the osc-a harmonics and all */
-		stratusOscHarmonics(synth, fd, chan, 0, 0,
-			(int) (synth->mem.param[52] * C_RANGE_MIN_1));
-		stratusWaveform(synth, fd, chan, 0, 0,
-			(int) (synth->mem.param[28] * C_RANGE_MIN_1));
-	}
-}
-
 static int
 stratusMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
-	bristolMidiMsg msg;
 	int newchan;
 
 	if ((synth->flags & OPERATIONAL) == 0)
@@ -1162,8 +1131,6 @@ stratusMidi(guiSynth *synth, int fd, int chan, int c, int o, int v)
 	{
 		bristolMidiSendMsg(global.controlfd, synth->sid,
 			127, 0, BRISTOL_MIDICHANNEL|newchan);
-		if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-			cleanupBristol();
 	}
 
 	synth->midichannel = newchan;
@@ -1227,7 +1194,7 @@ stratusCallback(brightonWindow *win, int panel, int index, float value)
 				synth->dispatch[index].operator,
 				sendvalue);
 	} else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n",(size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd,
 			synth->dispatch[index].other1, /* The SID is buried here */
 			synth->dispatch[index].controller,
@@ -1256,11 +1223,13 @@ panelSwitch(guiSynth *synth, int fd, int chan, int cont, int op, int value)
 		event.intvalue = 1;
 		brightonParamChange(synth->win, 0, -1, &event);
 
-		if (strcmp("stratus", synth->resources->name) == 0)
-			shade_id = brightonPut(synth->win, "bitmaps/blueprints/stratusshade.xpm",
+		if (strstr(synth->resources->name, "stratus") != 0)
+			shade_id = brightonPut(synth->win,
+				"bitmaps/blueprints/stratusshade.xpm",
 				0, 0, synth->win->width, synth->win->height);
-		else if (strcmp("trilogy", synth->resources->name) == 0)
-			shade_id = brightonPut(synth->win, "bitmaps/blueprints/trilogyshade.xpm",
+		else if (strstr(synth->resources->name, "trilogy") != 0)
+			shade_id = brightonPut(synth->win,
+				"bitmaps/blueprints/trilogyshade.xpm",
 				0, 0, synth->win->width, synth->win->height);
 	} else {
 		event.type = BRIGHTON_EXPOSE;
@@ -1301,7 +1270,7 @@ stratusInit(brightonWindow *win)
 
 	synth->win = win;
 
-	printf("Initialise the strilogy link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the strilogy link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -1334,21 +1303,27 @@ stratusInit(brightonWindow *win)
 		int voices = synth->voices;
 
 		printf("Initialise the strilogy synth circuits\n");
+		bcopy(&global, &manual, sizeof(guimain));
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 		/*
 		 * We have to tweak the voicecount as we use the same synth definition
 		 * to request the second layer
 		 */
 		manual.flags |= BRISTOL_CONN_FORCE|BRIGHTON_NOENGINE;
+		manual.host = global.host;
 		manual.port = global.port;
 
 		printf("Initialise the strilogy organ divider circuits\n");
 		synth->synthtype = BRISTOL_TRILOGY_ODC;
 		synth->voices = BRISTOL_VOICECOUNT;
 		if ((synth->sid2 = initConnection(&manual, synth)) < 0)
-			exit(0);
+			return(-1);
+
+		global.manualfd = manual.controlfd;
+		global.manual = &manual;
+		manual.manual = &global;
 		/*
 		 * Having played with the synth type to get the correct emulator we now
 		 * have to put it back to the actual model so that the GUI will load
@@ -1406,7 +1381,6 @@ stratusInit(brightonWindow *win)
 	dispatch[12].operator = 9;
 	dispatch[13].controller = 126;
 	dispatch[13].operator = 11; /* Sync */
-//	dispatch[13].routine = (synthRoutine) stratusSync;
 	dispatch[14].controller = 0;
 	dispatch[14].operator = 8;
 	/* VCO 2 - tune, sync-2, octave */
@@ -1680,10 +1654,10 @@ stratusConfigure(brightonWindow *win)
 	brightonParamChange(synth->win, synth->panel,
 		MEM_START + synth->location - 1, &event);
 
-	if (strcmp("stratus", synth->resources->name) == 0)
+	if (strstr(synth->resources->name, "stratus") != NULL)
 		shade_id = brightonPut(win, "bitmaps/blueprints/stratusshade.xpm",
 			0, 0, win->width, win->height);
-	else if (strcmp("trilogy", synth->resources->name) == 0)
+	else if (strstr(synth->resources->name, "trilogy") != NULL)
 		shade_id = brightonPut(win, "bitmaps/blueprints/trilogyshade.xpm",
 			0, 0, win->width, win->height);
 
@@ -1695,6 +1669,7 @@ stratusConfigure(brightonWindow *win)
 	event.type = BRIGHTON_EXPOSE;
 	event.intvalue = 1;
 	brightonParamChange(synth->win, KEY_PANEL, -1, &event);
+//	brightonParamChange(synth->win, 0, -1, &event);
 	configureGlobals(synth);
 
 	dc = brightonGetDCTimer(win->dcTimeout);
