@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -108,93 +107,93 @@ extern int stratusMidiCallback(brightonWindow *, int, int, float);
 
 static brightonLocations trilogyLocations[DEVICE_COUNT] = {
 	/* Organ */
-	{"", 1, C1-10, 110, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-2'", 1, C1-10, 110, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_REVERSE|BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW},
-	{"", 1, C1-10, 180, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-4'", 1, C1-10, 180, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_REVERSE|BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW},
-	{"", 1, C1-10, 250, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-8'", 1, C1-10, 250, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_REVERSE|BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW},
-	{"", 1, C1-10, 320, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
+	{"Organ-15'", 1, C1-10, 320, 90, 40, 0, 1, 0, "bitmaps/knobs/sliderblack.xpm", 0,
         BRIGHTON_REVERSE|BRIGHTON_VERTICAL|BRIGHTON_NOSHADOW},
 
 	/* Filter - 4 */
-	{"", 0, C3, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Filter-Env", 0, C3, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
-	{"", 0, C4, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Filter-Cutoff", 0, C4, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Filter-Res", 0, C5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C6, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Filter-Pedal", 0, C6, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 8 */
-	{"", 0, C7, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Glide-Amout", 0, C7, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C8, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Glide-Speed", 0, C8, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C9, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Glide-Multi", 0, C9, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C10, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Glide-Dir", 0, C10, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 12 */
-	{"", 0, C11, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Osc1-Tuning", 0, C11, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
-	{"", 2, C11 + (D2*1), R1, S2b+4, S2-10, 0, 1, 0,
+	{"Osc1-Sync", 2, C11 + (D2*1), R1, S2b+4, S2-10, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C11 + (D2*2), R1, S2b+4, S2-10, 0, 12, 0,
+	{"Osc1-Octave", 2, C11 + (D2*2), R1, S2b+4, S2-10, 0, 12, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
 
 	/* - 15 */
-	{"", 2, C11 + (D2*4) - 5, R1, S2b+4, S2-10, 0, 1, 0,
+	{"Osc2-Trill", 2, C11 + (D2*4) - 5, R1, S2b+4, S2-10, 0, 1, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 2, C11 + (D2*3) - 3, R1, S2b+4, S2-10, 0, 12, 0,
+	{"Osc2-Octave", 2, C11 + (D2*3) - 3, R1, S2b+4, S2-10, 0, 12, 0,
 		"bitmaps/buttons/pressoff.xpm", "bitmaps/buttons/presson.xpm", 0},
-	{"", 0, C15, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Osc2-Tuning", 0, C15, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_NOTCH},
 
 	/* - 18 */
-	{"", 0, C16+5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Routing", 0, C16+5, R1, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED|BRIGHTON_NOTCH},
-	{"", 0, C17 + D1 / 2, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Muli", 0, C17 + D1 / 2, R1, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C19-5, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
+	{"LFO-Waveform", 0, C19-5, R1, S1, S2, 0, 3, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 21 */
-	{"", 0, C1-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-Synth", 0, C1-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C2-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-Organ", 0, C2-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C3-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Mix-String", 0, C3-5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 	/* - 24 String section */
-	{"", 0, C4, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"String-footage", 0, C4, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"String-Timbre", 0, C5, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C6, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"String-Attack", 0, C6, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C7, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"String-Decay", 0, C7, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 	/* - 28 */
-	{"", 0, C8, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"Osc-Waveform", 0, C8, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C9, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc-WaveAlt", 0, C9, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
-	{"", 0, C10, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Osc-WaveLegato", 0, C10, R2, S1, S2, 0, 1.01, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_STEPPED},
 
 	/* - 31 */
-	{"", 0, C16, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Rate", 0, C16, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C17, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Attack", 0, C17, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C18, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Delay", 0, C18, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
-	{"", 0, C19, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
+	{"LFO-Depth", 0, C19, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knobblue.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", 0},
 
 	/* Dummies 12 - 35 */
@@ -264,13 +263,13 @@ static brightonLocations trilogyLocations[DEVICE_COUNT] = {
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* These are shadows for the envelope parameters from the mod panel - 66 */
-	{"", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Attack", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Decay", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Sustain", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
-	{"", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
+	{"Release", 0, 0, R2, S1, S2, 0, 1, 0, "bitmaps/knobs/knob4.xpm", 
 		"bitmaps/knobs/alpharotary.xpm", BRIGHTON_WITHDRAWN|BRIGHTON_NOSHADOW},
 
 	/* Memory - 40 */
@@ -374,13 +373,13 @@ static brightonLocations trilogyOpts[19] = {
 
 #define TRILOGY_MODCOUNT 7
 static brightonLocations trilogyMods[TRILOGY_MODCOUNT] = {
-	{"", 1, 440, 180, 80, 700, 0, 1, 0,
+	{"Attack", 1, 440, 180, 80, 700, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, 580, 180, 80, 700, 0, 1, 0,
+	{"Decay", 1, 580, 180, 80, 700, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, 720, 180, 80, 700, 0, 1, 0,
+	{"Sustain", 1, 720, 180, 80, 700, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
-	{"", 1, 860, 180, 80, 700, 0, 1, 0,
+	{"Release", 1, 860, 180, 80, 700, 0, 1, 0,
 		"bitmaps/knobs/sliderblack.xpm", 0, 0},
 
 	/* View Opts button */
@@ -408,7 +407,7 @@ brightonApp trilogyApp = {
 	stratusConfigure, /* 3 callbacks, unused? */
 	stratusMidiCallback,
 	destroySynth,
-	{5, 100, 2, 2, 5, 520, 0, 0},
+	{5, 100, 1, 2, 5, 520, 0, 0},
 	900, 385,
 	7, /* Panel count */
 	{
