@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -91,21 +90,21 @@ extern guimain global;
 #define C6 850
 
 static brightonLocations locations[DEVICE_COUNT] = {
-	{"", 1, C1, R1, W1, L1, 0, 8, 0,
+	{"Drawbar 16", 1, C1, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C2, R1, W1, L1, 0, 8, 0,
+	{"Drawbar 8", 1, C2, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C3, R1, W1, L1, 0, 8, 0,
+	{"Drawbar 4", 1, C3, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C4, R1, W1, L1, 0, 8, 0,
+	{"Drawbar IV", 1, C4, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C5, R1, W1, L1, 0, 8, 0,
+	{"Drawbar Sine", 1, C5, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C6, R1, W1, L1, 0, 8, 0,
+	{"Drawbar Ramp", 1, C6, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 2, 650, 775, 75, 200, 0, 1, 0,
+	{"Opts", 2, 650, 775, 75, 200, 0, 1, 0,
 		"bitmaps/buttons/rockerwhite.xpm", 0, BRIGHTON_VERTICAL},
-	{"", 2, 250, 775, 75, 200, 0, 1, 0,
+	{"Vibrato", 2, 250, 775, 75, 200, 0, 1, 0,
 		"bitmaps/buttons/rockerwhite.xpm", 0, BRIGHTON_VERTICAL},
 };
 
@@ -262,7 +261,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 			displayPanelText(synth, "PROG", synth->location, panel,
 				MEM_COUNT - 1);
 	} else {
-		bristolMidiMsg msg;
 		int newchan;
 
 		/*
@@ -281,8 +279,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 
@@ -301,8 +297,6 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					127, 0, BRISTOL_MIDICHANNEL|newchan);
-				if (bristolMidiRead(global.controlfd, &msg) != BRISTOL_OK)
-					cleanupBristol();
 
 				synth->midichannel = newchan;
 
@@ -383,7 +377,7 @@ brightonApp voxApp = {
 	voxConfigure, /* 3 callbacks */
 	midiCallback,
 	destroySynth,
-	{16, 0, 2, 2, 5, 520, 0, 0},
+	{-1, 0, 2, 2, 5, 520, 0, 1},
 	550, 250,
 	7, /* panel count */
 	{
@@ -558,7 +552,7 @@ voxCallback(brightonWindow *win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -653,7 +647,7 @@ voxInit(brightonWindow* win)
 
 	synth->win = win;
 
-	printf("Initialise the vox link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the vox link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -670,7 +664,7 @@ voxInit(brightonWindow* win)
 	 */
 	if (!global.libtest)
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 	for (i = 0; i < DEVICE_COUNT; i++)
 		synth->dispatch[i].routine = voxMidiSendMsg;
