@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -126,45 +125,45 @@ static int dc;
 
 static brightonLocations locations[MOD_COUNT] = {
 	/* 0 - Drawbars, lower, upper then bass */
-	{"", 1, C3, R1, W1, L1, 0, 8, 0,
+	{"Lower 8'", 1, C3, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C4, R1, W1, L1, 0, 8, 0,
+	{"Lower 4'", 1, C4, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C5, R1, W1, L1, 0, 8, 0,
+	{"Lower 2'", 1, C5, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C6, R1, W1, L1, 0, 8, 0,
+	{"Lower IV'", 1, C6, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C7, R1, W1, L1, 0, 8, 0,
+	{"Lower Flute", 1, C7, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C8, R1, W1, L1, 0, 8, 0,
+	{"Lower Reed", 1, C8, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C9, R1, W1, L1, 0, 8, 0,
+	{"Upper 16'", 1, C9, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C10, R1, W1, L1, 0, 8, 0,
+	{"Upper 8'", 1, C10, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C11, R1, W1, L1, 0, 8, 0,
+	{"Upper 4'", 1, C11, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C12, R1, W1, L1, 0, 8, 0,
+	{"Upper II'", 1, C12, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C13, R1, W1, L1, 0, 8, 0,
+	{"Upper III'", 1, C13, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondwhite.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C14, R1, W1, L1, 0, 8, 0,
+	{"Upper Flute", 1, C14, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C15, R1, W1, L1, 0, 8, 0,
+	{"Upper Reed", 1, C15, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C1, R1, W1, L1, 0, 8, 0,
+	{"Bass Flute", 1, C1, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
-	{"", 1, C2, R1, W1, L1, 0, 8, 0,
+	{"Bass Reed", 1, C2, R1, W1, L1, 0, 8, 0,
 		"bitmaps/knobs/hammondbrown.xpm", 0, BRIGHTON_REVERSE|BRIGHTON_HSCALE},
 
 	/* 15 - Percussive buttons, 4 of them */
-	{"", 2, C16, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushout.xpm",
+	{"Perc-I", 2, C16, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushout.xpm",
 		"bitmaps/buttons/pushin.xpm", 0},
-	{"", 2, C17, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushout.xpm",
+	{"Perc-II", 2, C17, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushout.xpm",
 		"bitmaps/buttons/pushin.xpm", 0},
-	{"", 2, C18, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushoutw.xpm",
+	{"Perc-L", 2, C18, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushoutw.xpm",
 		"bitmaps/buttons/pushinw.xpm", 0},
-	{"", 2, C19, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushoutw.xpm",
+	{"Perc-S", 2, C19, R1, W1, L2, 0, 1, 0, "bitmaps/buttons/pushoutw.xpm",
 		"bitmaps/buttons/pushinw.xpm", 0},
 
 	/* 19 - Dummies, 12 in total for opts and vars */
@@ -367,6 +366,18 @@ memCallback(brightonWindow* win, int panel, int index, float value)
 	return(0);
 }
 
+int
+voxloadMemory(guiSynth *synth, char *algo, char *name, int location,
+int active, int skip, int flags)
+{
+	loadMemory(synth, "voxM2", 0, location,
+		synth->mem.active, 0, BRISTOL_NOCALLS);
+
+	memFix(synth->win, synth);
+
+	return(0);
+}
+
 int
 voxPedalCallback(brightonWindow *win, int panel, int index, float value)
 {
@@ -432,7 +443,7 @@ brightonApp voxM2App = {
 	voxM2Configure, /* 3 callbacks */
 	midiCallback,
 	destroySynth,
-	{16, 0, 2, 2, 5, 520, 0, 0},
+	{-1, 0, 2, 2, 5, 520, 0, 0},
 	520, 310,
 	9, /* panel count */
 	{
@@ -676,7 +687,7 @@ voxM2Callback(brightonWindow *win, int panel, int index, float value)
 #define DEBUG
 #ifdef DEBUG
 	else
-		printf("dispatch[%x,%i](%i, %i, %i, %i, %i)\n", (size_t) synth, index,
+		printf("dispatch[%p,%i](%i, %i, %i, %i, %i)\n", synth, index,
 			global.controlfd, synth->sid,
 			synth->dispatch[index].controller,
 			synth->dispatch[index].operator,
@@ -912,7 +923,7 @@ voxM2Init(brightonWindow* win)
 
 	synth->win = win;
 
-	printf("Initialise the voxM2 link to bristol: %x\n", (size_t) synth->win);
+	printf("Initialise the voxM2 link to bristol: %p\n", synth->win);
 
 	synth->mem.param = (float *) brightonmalloc(DEVICE_COUNT * sizeof(float));
 	synth->mem.count = DEVICE_COUNT;
@@ -931,17 +942,24 @@ voxM2Init(brightonWindow* win)
 	{
 		if (synth->midichannel > 14)
 			synth->midichannel--;
+		bcopy(&global, &manual, sizeof(guimain));
 		synth->synthtype = BRISTOL_VOX;
 		if ((synth->sid = initConnection(&global, synth)) < 0)
-			exit(0);
+			return(-1);
 
 		/* Crank up another VOX for the upper manual */
 		manual.flags |= BRISTOL_CONN_FORCE|BRIGHTON_NOENGINE;
 		manual.port = global.port;
+		manual.host = global.host;
 		synth->synthtype = BRISTOL_VOXM2;
 		synth->midichannel++;
+
 		if ((synth->sid2 = initConnection(&manual, synth)) < 0)
-			exit(0);
+			return(-1);
+
+		global.manualfd = manual.controlfd;
+		global.manual = &manual;
+		manual.manual = &global;
 
 		synth->midichannel--;
 	}
@@ -1174,6 +1192,8 @@ printf("going operational\n");
 
 	dc = brightonGetDCTimer(win->dcTimeout);
 
+	synth->loadMemory = (loadRoutine) voxloadMemory;
+
 	return(0);
 }
 
