@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -109,121 +108,85 @@ extern int empty;
  * This example is for a voyagerBristol type synth interface.
  */
 static brightonLocations locations[DEVICE_COUNT] = {
+
 /* LFO and control */
-	{"LFO-F", 0, C1 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 2, C1 - 5, R24 + 25, B3, B4, 0, 3, 0,
+	{"LFO-Freq", 0, C1 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 0, 0, 0},
+	{"LFO-Sync", 2, C1 - 5, R24 + 25, B3, B4, 0, 3, 0,
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"TUNE", 0, C1, R34, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
-	{"GLD", 0, C1, R44, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
+	{"GLOBAL TUNE", 0, C1, R34, S1, S1, 0, 1, 0, 0, 0, BRIGHTON_NOTCH},
+	{"Glide", 0, C1, R44, S1, S1, 0, 1, 0, 0, 0, 0},
 /* Routing */
-	{"", 0, C2, R14, S1, S1, 0, 4, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C2, R24, S1, S1, 0, 3, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C2, R34, S1, S1, 0, 5, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"M1-V", 0, C2, R44, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C3, R14, S1, S1, 0, 4, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C3, R24, S1, S1, 0, 3, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C3, R34, S1, S1, 0, 5, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"M2-V", 0, C3, R44, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
+	{"Mod1-Source", 0, C2, R14, S1, S1, 0, 4, 0, 0, 0, 0},
+	{"Mod1-Shape", 0, C2, R24, S1, S1, 0, 3, 0, 0, 0, 0},
+	{"Mod1-Dest", 0, C2, R34, S1, S1, 0, 5, 0, 0, 0, 0},
+	{"Mod1-Gain", 0, C2, R44, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mod2-Source", 0, C3, R14, S1, S1, 0, 4, 0, 0, 0, 0},
+	{"Mod2-Shape", 0, C3, R24, S1, S1, 0, 3, 0, 0, 0, 0},
+	{"Mod2-Dest", 0, C3, R34, S1, S1, 0, 5, 0, 0, 0, 0},
+	{"Mod2-Gain", 0, C3, R44, S1, S1, 0, 1, 0, 0, 0, 0},
 /* Oscillators */
-	{"", 0, C4, R24, S1, S1, 0, 5, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C4, R34, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"O2-F", 0, C5 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
-	{"", 0, C5, R24, S1, S1, 0, 5, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C5, R34, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"O3-F", 0, C6 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, BRIGHTON_NOTCH},
-	{"", 0, C6, R24, S1, S1, 0, 5, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C6, R34, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
+	{"Osc1-Transpose", 0, C4, R24, S1, S1, 0, 5, 0, 0, 0, 0},
+	{"Osc1-Waveform", 0, C4, R34, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Osc2-Tuning", 0, C5 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 0, 0, BRIGHTON_NOTCH},
+	{"Osc2-Transpose", 0, C5, R24, S1, S1, 0, 5, 0, 0, 0, 0},
+	{"Osc2-Waveform", 0, C5, R34, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Osc3-Tuning", 0, C6 - 7, R14 - 20, S1 + 40, S1 + 40, 0, 1, 0, 0, 0, BRIGHTON_NOTCH},
+	{"Osc3-Transpose", 0, C6, R24, S1, S1, 0, 5, 0, 0, 0, 0},
+	{"Osc3-Waveform", 0, C6, R34, S1, S1, 0, 1, 0, 0, 0, 0},
 /* Osc mod switches */
-	{"", 2, C4, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
-	{"", 2, C4 + 40, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
-	{"", 2, C4 + 95, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
-	{"", 2, C4 + 135, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
+	{"Sync-1/2", 2, C4, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
+	{"FM-1/3", 2, C4 + 40, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
+	{"KBD-3", 2, C4 + 95, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
+	{"Freq-3", 2, C4 + 135, R44 + 10, B1, B2, 0, 1, 0, 0, 0, BRIGHTON_VERTICAL},
 /* Mixer */
-	{"EXT", 0, C7, R15, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"O1", 0, C7, R25, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"O2", 0, C7, R35, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"O3", 0, C7, R45, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"NSE", 0, C7, R55, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 2, C8, R15 + 25, B3, B4, 0, 1, 0, 
+	{"Mix-EXT", 0, C7, R15, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mix-O1", 0, C7, R25, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mix-O2", 0, C7, R35, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mix-O3", 0, C7, R45, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mix-NSE", 0, C7, R55, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"Mix-Ext-off", 2, C8, R15 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C8, R25 + 25, B3, B4, 0, 1, 0, 
+	{"Mix-O1-off", 2, C8, R25 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C8, R35 + 25, B3, B4, 0, 1, 0, 
+	{"Mix-O2-off", 2, C8, R35 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C8, R45 + 25, B3, B4, 0, 1, 0, 
+	{"Mix-O3-off", 2, C8, R45 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
-	{"", 2, C8, R55 + 25, B3, B4, 0, 1, 0, 
+	{"Mix-Nse-off", 2, C8, R55 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerblue.xpm", 0, 0},
 /* Filter */
-	{"COFF", 0, C9 - 7, R15 - 25, S1 + 40, S1 + 40, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"SPC", 0, C9, R25, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"RES", 0, C9, R35, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"KBD", 0, C9, R45, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 2, C9, R55 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
+	{"VCF-Cutoff", 0, C9 - 7, R15 - 25, S1 + 40, S1 + 40, 0, 1, 0, 0, 0, 0},
+	{"VCF-Space", 0, C9, R25, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-Res", 0, C9, R35, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-KBD", 0, C9, R45, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-Mode", 2, C9, R55 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
 /* Filter envelope */
-	{"F-A", 0, C10, R15, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"F-D", 0, C10, R25, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"F-S", 0, C10, R35, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"F-R", 0, C10, R45, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 0, C10, R55, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
+	{"VCF-Attack", 0, C10, R15, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-Decay", 0, C10, R25, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-Sustain", 0, C10, R35, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-Release", 0, C10, R45, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCF-EnvLevel", 0, C10, R55, S1, S1, 0, 1, 0, 0, 0, 0},
 /* Amp Envelope */
-	{"V-A", 0, C11, R15, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"V-D", 0, C11, R25, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"V-S", 0, C11, R35, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"V-R", 0, C11, R45, S1, S1, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"F-AMT", 2, C11, R55 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
+	{"VCA-Attack", 0, C11, R15, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCA-Decay", 0, C11, R25, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCA-Sustain", 0, C11, R35, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCA-Release", 0, C11, R45, S1, S1, 0, 1, 0, 0, 0, 0},
+	{"VCA-Veloc", 2, C11, R55 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
 /* Vol, on/off, glide and release */
-	{"VOL", 0, C12 - 7, R15 - 25 , S1 + 40, S1 + 40, 0, 1, 0, 
-		"bitmaps/knobs/knob4.xpm", 0, 0},
-	{"", 2, C12 - 4, R25 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
-	{"", 2, C12 - 4, R35 + 25, B3, B4, 0, 1, 0, 
+	{"Volume", 0, C12 - 7, R15 - 25 , S1 + 40, S1 + 40, 0, 1, 0, 0, 0, 0},
+	{"On/Off", 2, C12 - 4, R25 + 25, B3, B4, 0, 1, 0, 0, 0, 0},
+	{"LFO-Multi", 2, C12 - 4, R35 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0},
-	{"", 2, C12 - 4, R45 + 25, B3, B4, 0, 1, 0, 
+	{"Glide-On", 2, C12 - 4, R45 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0},
-	{"", 2, C12 - 4, R55 + 25, B3, B4, 0, 1, 0, 
+	{"Release-On", 2, C12 - 4, R55 + 25, B3, B4, 0, 1, 0, 
 		"bitmaps/buttons/rockerwhite.xpm", 0, 0},
-	{"", 5, 405, 410, 180, 518, 0, 1, 0, "bitmaps/buttons/pointer.xpm", 0, 0},
-	{"", 5, 405, 410, 180, 518, 0, 1, 0, "bitmaps/buttons/pointer.xpm", 0, 
+	{"Reserved", 5, 405, 410, 180, 518, 0, 1, 0, "bitmaps/buttons/pointer.xpm", 0, 0},
+	{"Reserved", 5, 405, 410, 180, 518, 0, 1, 0, "bitmaps/buttons/pointer.xpm", 0, 
 		BRIGHTON_WITHDRAWN},
 /* logo */
-	{"", 4, 762, 1037, 240, 140, 0, 1, 0, "bitmaps/images/voyager.xpm", 0, 0},
+	{"", 4, 762, 1037, 240, 140, 0, 1, 0, "bitmaps/images/explorer.xpm", 0, 0},
+
 /* memories */
 	{"", 2, 418, 225, 17, 78, 0, 1.01, 0,
 		"bitmaps/buttons/pressoff.xpm",
@@ -311,7 +274,7 @@ brightonApp voyagerApp = {
 	voyagerConfigure, /* 3 callbacks, unused? */
 	midiCallback,
 	destroySynth,
-	{8, 100, 2, 2, 5, 520, 0, 0},
+	{1, 100, 2, 2, 5, 520, 0, 0},
 	817, 472,
 	7, /* one panel only */
 	{
@@ -424,6 +387,8 @@ voyagerConfigure(brightonWindow *win)
 	guiSynth *synth = findSynth(global.synths, win);
 	brightonEvent event;
 
+	memset(&event, 0, sizeof(brightonEvent));
+
 	if (synth == 0)
 	{
 		printf("problems going operational\n");
