@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -39,14 +38,18 @@ bristolActiveSense(audioMain *audiomain, bristolMidiMsg *msg)
 	if (baudio == NULL)
 		return(0);
 
-	baudio->sensecount =
+	if ((baudio->sensecount =
 		((msg->params.bristol.valueMSB << 7) + msg->params.bristol.valueLSB)
-			* audiomain->samplerate / 1000;
+			* audiomain->samplerate / 1000) == 0)
+		baudio->mixflags &= ~BRISTOL_SENSE;
+	else
+		/*
+		 * Note that we are now receiving active sensing
+		 */
+		baudio->mixflags |= BRISTOL_SENSE;
 
-	/*
-	 * Note that we are now receiving active sensing
-	 */
-	baudio->mixflags |= BRISTOL_SENSE;
+	if (audiomain->debuglevel > 5)
+		printf("ActiveSense %p: %i\n", baudio, baudio->sensecount);
 
 	return(0);
 }
@@ -54,7 +57,11 @@ bristolActiveSense(audioMain *audiomain, bristolMidiMsg *msg)
 int
 bristolActiveSenseCheck(audioMain *audiomain, Baudio *baudio)
 {
-	if (((baudio->mixflags & BRISTOL_SENSE) == 0)
+	if (audiomain->debuglevel > 13)
+		printf("ActiveSenseCheck %p: %i\n", baudio, baudio->sensecount);
+
+	if ((~baudio->mixflags & BRISTOL_SENSE)
+		|| (baudio->sensecount == 0)
 		|| (baudio->mixflags & (BRISTOL_HOLDDOWN|BRISTOL_REMOVE)))
 		return(1);
 
