@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -364,7 +363,7 @@ static int operate(bristolOP *operator,
 		 */
 		gdelta = wtp - ((float) ((int) wtp));
 
-		if (wtp == AKSDCO_WAVE_SZE_M) {
+		if (wtp >= AKSDCO_WAVE_SZE_M) {
 			ob1[obp] = wt1[(int) wtp] + (wt1[0] - wt1[(int) wtp]) * gdelta;
 			ob2[obp] = wt2[(int) wtp] + (wt2[0] - wt2[(int) wtp]) * gdelta;
 		} else {
@@ -380,7 +379,7 @@ static int operate(bristolOP *operator,
 		 */
 		if ((wtp += ib[obp] * transp) >= AKSDCO_WAVE_SZE)
 		{
-			while (wtp > AKSDCO_WAVE_SZE)
+			while (wtp >= AKSDCO_WAVE_SZE)
 				wtp -= AKSDCO_WAVE_SZE;
 		}
 		/*
