@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -172,7 +171,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	bristolOPParams *param, void *lcl)
 {
 	register bristolAKSENVlocal *local = lcl;
-	register float cgain, attack, on, decay, off, *ob, gain, egain, oncount;
+	register float cgain, attack, on, decay, off, *ob, egain, oncount;
 	register int count, rampup = -1;
 	bristolAKSENV *specs;
 
@@ -206,9 +205,9 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	egain = local->egain;
 	oncount = local->oncount;
 
-	if (egain < gain)
+	if (egain < cgain)
 		rampup = 1;
-	else if (egain > gain)
+	else if (egain > cgain)
 		rampup = 0;
 
 	count = specs->spec.io[AKSENV_OUT_IND].samplecount;
@@ -219,6 +218,17 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		cgain = 0.0;
 		local->cstate = STATE_ATTACK;
 		voice->flags &= ~BRISTOL_KEYON;
+
+		if ((~voice->flags & BRISTOL_KEYREON) &&
+			(voice->offset > 0) && (voice->offset < count))
+		{
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				printf("envelope trigger offset %i frames\n", voice->offset);
+
+			memset(ob, 0.0f, voice->offset * sizeof(float));
+			ob += voice->offset;
+			count -= voice->offset;
+		}
 	}
 
 	/*
