@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -114,8 +113,11 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("arpreset(%x)\n", operator);
 #endif
 
-	param->param[0].mem = bristolmalloc0(sizeof(float) * ARPDCO_WAVE_SZE);
 	param->param[0].int_val = ARPDCO_WAVE_SZE / 2;
+	if (param->param[0].mem != NULL) bristolfree(param->param[0].mem);
+	param->param[0].mem = bristolmalloc0(sizeof(float) * ARPDCO_WAVE_SZE);
+	if (param->param[1].mem != NULL) bristolfree(param->param[1].mem);
+	param->param[1].mem = bristolmalloc0(sizeof(float) * ARPDCO_WAVE_SZE);
 	param->param[1].float_val = 1.0;
 	param->param[2].float_val = 1.0;
 	param->param[3].float_val = 1.0;
@@ -135,7 +137,8 @@ static int param(bristolOP *operator, bristolOPParams *param,
 	unsigned char index, float value)
 {
 #ifdef BRISTOL_DBG
-	printf("arpdcoparam(%x, %x, %i, %f)\n", operator, param, index, value);
+	printf("arpdcoparam(%i, %f/%f)\n",
+		index, value, (int) (value*CONTROLLER_RANGE));
 #endif
 
 	switch (index) {
@@ -378,7 +381,6 @@ static int operate(bristolOP *operator,
 	else
 		sb = 0;
 
-	wt = (float *) specs->wave[3];
 	width = param->param[0].float_val; /* Pulse width */
 	dosquare = param->param[6].int_val;
 /*	wtp = local->wtp; */
@@ -395,9 +397,23 @@ static int operate(bristolOP *operator,
 		* param->param[11].float_val;
 	wtp = local->wtp;
 	cpwm = local->cpwm;
-	ramp = specs->wave[3];
+
 	sinwave = specs->wave[0];
-	triwave = specs->wave[4];
+	if (bristolBLOcheck(voice->dFreq*transp))
+	{
+		/* Need to restuff tri, ramp */
+		memset(param->param[0].mem, 0, ARPDCO_WAVE_SZE * sizeof(float));
+		memset(param->param[1].mem, 0, ARPDCO_WAVE_SZE * sizeof(float));
+		ramp = param->param[0].mem;
+		wt = ramp;
+		triwave = param->param[1].mem;
+		generateBLOwaveformF(voice->cFreq*transp, ramp, BLO_SAW);
+		generateBLOwaveformF(voice->cFreq*transp, triwave, BLO_TRI);
+	} else {
+		wt = ramp = specs->wave[3];
+		triwave = specs->wave[4];
+	}
+
 
 	/*
 	 * Go jumping through the wavetable, with each jump defined by the value
@@ -765,7 +781,7 @@ fillWave(float *mem, int count, int type)
 			/* 
 			 * This is a square wave, with decaying plateaus.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -803,7 +819,7 @@ fillWave(float *mem, int count, int type)
 			mem[count - 1] = mem[1]
 				= BRISTOL_VPO;
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = bloramp[i];
@@ -816,7 +832,7 @@ fillWave(float *mem, int count, int type)
 			 * Triangular wave. From MIN point, ramp up at twice the rate of
 			 * the ramp wave, then ramp down at same rate.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blotriangle[i];
