@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,14 +15,14 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
 /*#define DEBUG */
 
 #include <math.h>
+#include <assert.h>
 
 #include "bristol.h"
 #include "bristolmm.h"
@@ -30,10 +30,13 @@
 #include "dco.h"
 
 extern tableEntry defaultTable[];
-extern void inthandler();
 extern void freeBristolAudio();
 static void a440();
 
+#ifndef BRISTOL_SEMAPHORE
+extern void rbMidiNote(audioMain *, bristolMidiMsg *);
+#endif
+
 /*
  * Used by the a440 sine generator.
  */
@@ -48,7 +51,7 @@ bristolUnlinkVoice(audioMain *audiomain, bristolVoice *v)
 	 * See if this is a monophonic voice with note preference
 	 */
 	if ((v->baudio != NULL) && (v->baudio->voicecount == 1)
-		&& (v->baudio->notemap.flags != 0))
+		&& (v->baudio->notemap.flags & (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 	{
 		v->flags &= ~BRISTOL_KEYDONE;
 		v = v->next;
@@ -104,6 +107,7 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	register int i;
 	register Baudio *thisaudio;
 	register float *extmult, *leftch, *rightch, gain;
+	bristolMidiMsg msg;
 
 	/*
 	 * Clear the output buffer at this point.
@@ -120,7 +124,26 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	 * postops if a voice is active.
 	 */
 	if ((thisaudio = audiomain->audiolist) == NULL)
+	{
+#ifndef BRISTOL_SEMAPHORE
+		/*
+		 * This is not threadsafe. We should flag the rb as inactive.
+		 */
+		jack_ringbuffer_stop(audiomain->rb);
+		jack_ringbuffer_reset(audiomain->rb);
+#endif
 		return(0);
+	}
+
+#ifndef BRISTOL_SEMAPHORE
+	while (jack_ringbuffer_read_space(audiomain->rb) >= sizeof(bristolMidiMsg))
+	{
+		jack_ringbuffer_read(audiomain->rb, (char *) &msg,
+			sizeof(bristolMidiMsg));
+
+		rbMidiNote(audiomain, &msg);
+	}
+#endif
 
 	while (thisaudio != NULL)
 	{
@@ -135,6 +158,10 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			Baudio *holder = thisaudio;
 			bristolVoice *vl;
 
+#ifdef BRISTOL_SEMAPHORE
+			sem_wait(audiomain->sem_long);
+#endif
+
 			thisaudio = thisaudio->next;
 
 			freeBristolAudio(audiomain, holder);
@@ -142,7 +169,11 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			if ((audiomain->audiolist == NULL)
 				&& (audiomain->flags & BRISTOL_MIDI_WAIT))
 			{
+				if (audiomain->debuglevel > 5)
+					printf("Terminate\n");
+
 				audiomain->atStatus = BRISTOL_TERM;
+				holder->mixflags &= ~BRISTOL_REMOVE;
 				return(0);
 			}
 
@@ -154,6 +185,16 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 				if (vl->baudio == holder)
 					vl->baudio = NULL;
 
+			for (vl = audiomain->newlist; vl != NULL; vl = vl->next)
+				if (vl->baudio == holder)
+					vl->baudio = NULL;
+
+			holder->mixflags &= ~BRISTOL_REMOVE;
+
+#ifdef BRISTOL_SEMAPHORE
+			sem_post(audiomain->sem_long);
+#endif
+
 			continue;
 		}
 
@@ -179,9 +220,15 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		a440(audiomain, outbuf, audiomain->samplecount);
 
 	/*
-	 * Get the long semaphore over the whole polyphonic process
+	 * Get the long semaphore over the whole polyphonic process. We need to 
+	 * replace this with some message passing algorithms using something that
+	 * is a bit less risky. Minimally we should just have a short_sem that
+	 * covers management of a secure messaging list. That can wait though, this
+	 * code is already pretty recent (0.40 stream).
 	 */
+#ifdef BRISTOL_SEMAPHORE
 	sem_wait(audiomain->sem_long);
+#endif
 
 	if (audiomain->newlist != NULL)
 	{
@@ -190,11 +237,13 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		/*
 		 * Get the short semaphore over the new voice management sequences
 		 */
+#ifdef BRISTOL_SEMAPHORE
 		sem_wait(audiomain->sem_short);
+#endif
 
 		/*
-		 * Move the new voices to the playlist. Then do a scan of the playlist
-		 * and see if there is anything to be moved to the freelist (KEYDONE).
+		 * Do a scan of the playlist and see if there is anything to be moved
+		 * to the freelist (KEYDONE).
 		 */
 		while (v != NULL)
 		{
@@ -202,11 +251,13 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			{
 //printf("remove %x->%x/%x (0x%04x)\n", (size_t) v, (size_t) v->last,
 //(size_t) v->next, v->flags);
+
 				/*
 				 * See if this is a monophonic voice with note preference
 				 */
 				if ((v->baudio != NULL) && (v->baudio->voicecount == 1)
-					&& (v->baudio->notemap.flags != 0))
+					&& (v->baudio->notemap.flags &
+						(BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 				{
 					v->flags &= ~BRISTOL_KEYDONE;
 					v = v->next;
@@ -259,8 +310,11 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			v->last = NULL;
 			audiomain->playlist = v;
 
-			v->flags &= ~(BRISTOL_KEYDONE|BRISTOL_KEYOFF|BRISTOL_KEYOFFING
+			v->flags &= ~(BRISTOL_KEYDONE
+				|BRISTOL_KEYOFF
+				|BRISTOL_KEYOFFING
 				|BRISTOL_KEYSUSTAIN);
+
 			if ((v->flags & BRISTOL_KEYREON) == 0)
 				v->flags |= BRISTOL_KEYON;
 		}
@@ -269,7 +323,9 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		/*
 		 * Free the short semaphore over the new voice management sequences
 		 */
+#ifdef BRISTOL_SEMAPHORE
 		sem_post(audiomain->sem_short);
+#endif
 	}
 
 	voice = audiomain->playlist;
@@ -298,8 +354,26 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 			continue;
 		}
 
-		if (!(voice->flags & BRISTOL_KEYDONE))
+		if (~voice->flags & BRISTOL_KEYDONE)
 		{
+			/*
+			 * If this audio is mono, and count is zero and SUSTAIN is not
+			 * active then signal KEY_OFF, clear extreme note;
+			 */
+			if ((voice->baudio->voicecount == 1)
+				&& (voice->baudio->notemap.flags
+					& (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP))
+				&& (voice->baudio->notemap.count == 0)
+				&& (voice->baudio->notemap.extreme >= 0)
+				&& (voice->baudio->contcontroller[BRISTOL_CC_HOLD1] < 0.5))
+			{
+				if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+					printf("engine cleared MNL note\n");
+				voice->baudio->notemap.extreme = -1;
+				voice->flags |= BRISTOL_KEYOFF;
+				voice->flags &= ~BRISTOL_KEYSUSTAIN;
+			}
+
 			if (voice->baudio->mixflags & BRISTOL_MUST_PRE)
 			{
 				if (voice->baudio->preops)
@@ -344,7 +418,7 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 
 			/*
 			 * If we are arpeggiating or sequencing we need to check on rolling
-			 * this voice along
+			 * this voice along, and need a couple of sanity checks.
 			 */
 			if (((voice->baudio->mixflags
 				& (BRISTOL_SEQUENCE|BRISTOL_ARPEGGIATE))
@@ -360,7 +434,8 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 				voice->baudio, voice, startbuf);
 
 			if ((voice->baudio->voicecount == 1)
-				&& (voice->baudio->notemap.flags != 0))
+				&& (voice->baudio->notemap.flags 
+					& (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 				voice->flags &= ~BRISTOL_KEYDONE;
 
 			/*
@@ -381,7 +456,8 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	{
 		if (thisaudio->postops != NULL)
 		{
-			if (thisaudio->mixflags & BRISTOL_HAVE_OPED)
+			if (((thisaudio->mixflags & (BRISTOL_HOLDDOWN|BRISTOL_REMOVE)) == 0)
+				&& (thisaudio->mixflags & BRISTOL_HAVE_OPED))
 				thisaudio->postops(audiomain,
 					thisaudio, thisaudio->firstVoice, startbuf);
 		}
@@ -392,13 +468,16 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 	while (voice != NULL)
 	{
 		voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
+		voice->offset = -1;
 		voice = voice->next;
 	}
 
 	/*
 	 * Free the long semaphore over the polyphonic process.
 	 */
+#ifdef BRISTOL_SEMAPHORE
 	sem_post(audiomain->sem_long);
+#endif
 
 	/*
 	 * At this point all the voices have put their output onto the leftbuf and
@@ -579,6 +658,9 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 }
 
 /*
+ * The routines fillFreqTable and fillFreqBuf need to be consolidated to one
+ * routine. Minimally cFreq/dFreq and dfreq/dfreq need to be maintained.
+ *
  * This fills the table with frequencies. The alternative fillFreqTable() will
  * insert sample steps through the 1K wavetable oscillators. This call is used
  * for the tendency tables which are frequency based.
@@ -588,33 +670,14 @@ fillFreqBuf(Baudio *baudio, register bristolVoice *voice, register float *buf,
 	register int size, int glide)
 {
 	register int i;
-	float freq;
-
-	/* freq = defaultTable[voice->key.key].rate; */
-	freq = voice->dfreq;
-
-/*
-	for (i = 0; i < size; i+=8)
-	{
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-		*buf++ = freq * baudio->midi_pitchfreq;
-	}
-
-	return(0);
-*/
 
 	/*
 	 * Take a buffer, and fill in some frequency value to make an oscillator
 	 * reproduce the correct note. Will cater for polyphonic portamento, and
 	 * eventually also for some generic mods.
 	 */
-	if ((voice->cfreq == voice->dfreq) || (glide == 0) || (voice->lastkey == 0))
+	if ((voice->cfreq == voice->dfreq) || (glide == 0) || (voice->lastkey == 0)
+		|| (baudio->glide == 0.0f))
 	{
 		/*
 		 * Note has no glide, just fill the table.
@@ -631,31 +694,44 @@ fillFreqBuf(Baudio *baudio, register bristolVoice *voice, register float *buf,
 			*buf++ = voice->dfreq;
 		}
 	} else {
-		if (voice->cfreq == voice->dfreq)
-		{
-			for (i = 0; i < size; i+=8)
-			{
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-				*buf++ = voice->dfreq;
-			}
-		} else if (voice->lastkey < voice->key.key) {
+		if (voice->cfreq < voice->dfreq) {
 			/*
-			 * This should not have looked at the keys, it should look at the
-			 * frequencies only
-			 *
 			 * Glide frequency up at some rate.
 			 */
+			if (voice->cfreqmult < 1.0f)
+				/*
+				 * Frequency inversion, probably due to pitchbend, we have to
+				 * recalculate cfreqmult. This does lead to some stretching of
+				 * the overall glide.
+				 */
+				voice->cfreqmult = powf(M_E, logf(voice->dfreq/voice->cfreq)
+					/ (baudio->glide * baudio->samplerate));
+
+			if ((voice->cfreqmult >=    0.999999f)
+				&& (voice->cfreqmult <= 1.000001f))
+			{
+				voice->cFreq = voice->dFreq;
+				voice->cfreq = voice->dfreq;
+				for (i = 0; i < size; i+=8)
+				{
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+				}
+				return(0);
+			}
+
 			for (i = 0; i < size; i++)
 			{
 				if (voice->cfreq > voice->dfreq)
 				{
 					voice->cfreq = voice->dfreq;
+					voice->cFreq = voice->dFreq;
 					for (; i < size; i++)
 						*buf++ = voice->cfreq;
 					break;
@@ -669,11 +745,40 @@ fillFreqBuf(Baudio *baudio, register bristolVoice *voice, register float *buf,
 			/*
 			 * Glide frequency down at some rate.
 			 */
+			if (voice->cfreqmult > 1.0f)
+				/*
+				 * Frequency inversion, probably due to pitchbend, we have to
+				 * recalculate cfreqmult. This does lead to some stretching of
+				 * the overall glide.
+				 */
+				voice->cfreqmult = powf(M_E, logf(voice->dfreq/voice->cfreq)
+					/ (baudio->glide * baudio->samplerate));
+
+			if ((voice->cfreqmult >=    0.999999f)
+				&& (voice->cfreqmult <= 1.000001f))
+			{
+				voice->cFreq = voice->dFreq;
+				voice->cfreq = voice->dfreq;
+				for (i = 0; i < size; i+=8)
+				{
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+					*buf++ = voice->dfreq;
+				}
+				return(0);
+			}
+
 			for (i = 0; i < size; i++)
 			{
 				if (voice->cfreq < voice->dfreq)
 				{
 					voice->cfreq = voice->dfreq;
+					voice->cFreq = voice->dFreq;
 					for (; i < size; i++)
 						*buf++ = voice->cfreq;
 					break;
@@ -683,6 +788,10 @@ fillFreqBuf(Baudio *baudio, register bristolVoice *voice, register float *buf,
 		}
 	}
 
+//	printf("freqbuf: %f %f %f %f %i\n",
+//		voice->cFreq, voice->dFreq, voice->cFreqmult,
+//		baudio->glide, baudio->samplerate);
+
 	return(0);
 }
 
@@ -697,12 +806,15 @@ fillFreqTable(Baudio *baudio, register bristolVoice *voice, register float *buf,
 	register int i;
 
 	/*
+printf("fft %f %f %1.12f\n", voice->cFreq, voice->dFreq, voice->cFreqmult);
 	 * Take a buffer, and fill in some frequency value to make an oscillator
 	 * reproduce the correct note. Will cater for polyphonic portamento, and
 	 * eventually also for some generic mods.
 	 */
-	if ((voice->cFreq == voice->dFreq) || (glide == 0) || (voice->lastkey == 0))
+	if ((voice->cFreq == voice->dFreq) || (glide == 0)
+		|| (voice->lastkey == 0) || (baudio->glide == 0.0f))
 	{
+//printf("fft %f %f %i\n", voice->cFreq, voice->dFreq, voice->lastkey);
 		/*
 		 * Note has no glide, just fill the table.
 		 */
@@ -718,46 +830,89 @@ fillFreqTable(Baudio *baudio, register bristolVoice *voice, register float *buf,
 			*buf++ = voice->dFreq;
 		}
 	} else {
-		if (voice->cFreq == voice->dFreq)
-		{
-			for (i = 0; i < size; i+=8)
-			{
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-				*buf++ = voice->dFreq;
-			}
-		} else if (voice->lastkey < voice->key.key) {
+		if (voice->cFreq < voice->dFreq) {
 			/*
 			 * Glide frequency up at some rate.
 			 */
+			if (voice->cFreqmult < 1.0f)
+				/*
+				 * Frequency inversion, probably due to pitchbend, we have to
+				 * recalculate cfreqmult. This does lead to some stretching of
+				 * the overall glide.
+				 */
+				voice->cFreqmult = powf(M_E, logf(voice->dFreq/voice->cFreq)
+					/ (baudio->glide * baudio->samplerate));
+
+//printf("1: inversion %f %i\n", voice->cFreqmult, voice->cFreqmult == 1.0);
+//limit 0.99999999
+			if ((voice->cFreqmult >=    0.999999f)
+				&& (voice->cFreqmult <= 1.000001f))
+			{
+				voice->cFreq = voice->dFreq;
+				for (i = 0; i < size; i+=8)
+				{
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+				}
+				return(0);
+			}
+
 			for (i = 0; i < size; i++)
 			{
-				if (voice->cFreq > voice->dFreq)
+				if (voice->cFreq >= voice->dFreq)
 				{
 					voice->cFreq = voice->dFreq;
+					voice->cfreq = voice->dfreq;
 					for (; i < size; i++)
 						*buf++ = voice->cFreq;
 					break;
 				}
-				/*
-				 * Hm, this is incorrect, it should be a multiplier.....
-				 */
 				*buf++ = (voice->cFreq *= voice->cFreqmult);
 			}
 		} else {
 			/*
 			 * Glide frequency down at some rate.
 			 */
+			if (voice->cFreqmult > 1.0f)
+				/*
+				 * Frequency inversion, probably due to pitchbend, we have to
+				 * recalculate cfreqmult. This does lead to some stretching of
+				 * the overall glide.
+				 */
+				voice->cFreqmult = powf(M_E, logf(voice->dFreq/voice->cFreq)
+					/ (baudio->glide * baudio->samplerate));
+
+//printf("2: inversion %f %i\n", voice->cFreqmult, voice->cFreqmult == 1.0);
+			if ((voice->cFreqmult >=    0.999999f)
+				&& (voice->cFreqmult <= 1.000001f))
+			{
+				voice->cFreq = voice->dFreq;
+				for (i = 0; i < size; i+=8)
+				{
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+					*buf++ = voice->dFreq;
+				}
+				return(0);
+			}
+
 			for (i = 0; i < size; i++)
 			{
 				if (voice->cFreq < voice->dFreq)
 				{
 					voice->cFreq = voice->dFreq;
+					voice->cfreq = voice->dfreq;
 					for (; i < size; i++)
 						*buf++ = voice->cFreq;
 					break;
@@ -767,6 +922,10 @@ fillFreqTable(Baudio *baudio, register bristolVoice *voice, register float *buf,
 		}
 	}
 
+//	printf("freqtab: %f %f %f %f %i\n",
+//		voice->cFreq, voice->dFreq, voice->cFreqmult,
+//		baudio->glide, baudio->samplerate);
+
 	return(0);
 }
 
