@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -110,14 +109,7 @@ audioThread(audioMain *audiomain)
 
 			audiomain->atStatus = BRISTOL_EXIT;
 
-			printf("\nCould not open the requested jack interface.\n\n");
-			printf("The server may already be active with different user ");
-			printf("permissions, try starting\nbristol as root. It may be ");
-			printf("that the server cannot be started because the audio\n");
-			printf("interface is in use by another application, that the ");
-			printf("server could not be found\nin the PATH variable, or that ");
-			printf("some parameter values did not match which should\nhave ");
-			printf("been reported above\n\n");
+			printf("jack audio interface returned\n");
 
 			_exit(0);
 		}
@@ -140,8 +132,20 @@ audioThread(audioMain *audiomain)
 		audiomain->samplecount, audiomain->flags|audiomain->preload))
 			< 0)
 	{
-		printf("Problem opening audio device %s, exiting audio thread\n",
-			device);
+		/*
+		 * If we have Jack support but have not request jack audio then this
+		 * failure is typically due to not giving the -jack option. Chekc for
+		 * it and report.
+		 */
+#ifdef _BRISTOL_JACK
+		if (~audiomain->flags & BRISTOL_JACK)
+		{
+			printf("Failed to open audio device %s\n", device);
+			printf("If jack is running then use 'startBristol -jack'\n");
+		} else
+#endif
+			printf("Problem opening audio device %s, exiting audio thread\n",
+				device);
 		audiomain->atStatus = BRISTOL_FAIL;
 		pthread_exit(&pthreadstatus);
 	}
@@ -165,6 +169,7 @@ audioThread(audioMain *audiomain)
 	 */
 	outbuf = (float *) bristolmalloc(audiomain->segmentsize * 2);
 
+	/* This will give 2 deinterleaved sample streams. */
 	startbuf = (float *) bristolmalloc(audiomain->segmentsize * 2);
 
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG1)
@@ -196,6 +201,16 @@ audioThread(audioMain *audiomain)
 		{
 			printf("Audio device read issue\n");
 
+#ifdef _BRISTOL_DRAIN
+			/*
+			 * This code avoids reopening of the audio interface which was
+			 * pretty ugly.
+			 */
+			bristolAudioStart();
+//			for (i = 1; i < audiomain->preload; i++)
+//				bristolAudioWrite(outbuf, audiomain->samplecount);
+			bristolAudioRead(startbuf, audiomain->samplecount);
+#else
 			bristolAudioClose();
 			if (bristolAudioOpen(device, audiomain->samplerate,
 				audiomain->samplecount,
@@ -212,8 +227,8 @@ audioThread(audioMain *audiomain)
 					bristolAudioWrite(outbuf, audiomain->samplecount);
 
 			bristolAudioStart();
-
 			bristolAudioRead(startbuf, audiomain->samplecount);
+#endif
 		}
 
 		/*
@@ -237,6 +252,15 @@ audioThread(audioMain *audiomain)
 			{
 				printf("Audio device write issue: restart pl %i\n",
 					audiomain->preload);
+#ifdef _BRISTOL_DRAIN
+				/*
+				 * This code avoids reopening of the audio interface which was
+				 * pretty ugly.
+				 */
+				bristolAudioStart();
+//				for (i = 1; i < audiomain->preload; i++)
+//					bristolAudioWrite(outbuf, audiomain->samplecount);
+#else
 				bristolAudioClose();
 				if (bristolAudioOpen(device, audiomain->samplerate,
 					audiomain->samplecount,
@@ -253,6 +277,7 @@ audioThread(audioMain *audiomain)
 						bristolAudioWrite(outbuf, audiomain->samplecount);
 
 				bristolAudioStart();
+#endif
 			}
 		}
 	}
@@ -264,8 +289,7 @@ audioThread(audioMain *audiomain)
 
 	bristolAudioClose();
 
-	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG1)
-		printf("audioThread exiting\n");
+	printf("audioThread exiting\n");
 
 	audiomain->atStatus = BRISTOL_EXIT;
 	pthread_exit(&pthreadstatus);
@@ -356,8 +380,25 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 			baudio->glidemax = value * 30;
 			return(0);
 		case BRISTOL_NRP_LWF:
-			if (value != 0)
-				baudio->mixflags |= BRISTOL_LW_FILTER;
+			if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
+				printf("LWF requested as %i\n", (int) (value * 16384.0f));
+			baudio->mixflags &= ~(BRISTOL_LW_FILTER|BRISTOL_LW_FILTER2);
+			switch ((int) (value * 16384.0f)) {
+				case 0: /* was heavyweight, now medium */
+					break;
+				case 1:
+					/* Chamberlains - not changed */
+					baudio->mixflags |= BRISTOL_LW_FILTER;
+					break;
+				case 2:
+					/* -lwf2 */
+					baudio->mixflags |= BRISTOL_LW_FILTER2;
+					break;
+				case 3:
+					/* Now -hwf heavyweight */
+					baudio->mixflags |= (BRISTOL_LW_FILTER|BRISTOL_LW_FILTER2);
+					break;
+			}
 			return(0);
 		case BRISTOL_NRP_MNL_PREF:
 			baudio->notemap.flags &= ~(BRISTOL_MNL_LNP|BRISTOL_MNL_HNP);
@@ -365,6 +406,7 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 				baudio->notemap.flags |= BRISTOL_MNL_LNP;
 			else if ((int) (value * CONTROLLER_RANGE) == 2)
 				baudio->notemap.flags |= BRISTOL_MNL_HNP;
+			baudio->notemap.extreme = -1;
 			return(0);
 		case BRISTOL_NRP_MNL_TRIG:
 			if (value != 0)
@@ -378,6 +420,14 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 			else
 				baudio->notemap.flags &= ~BRISTOL_MNL_VELOC;
 			return(0);
+		case BRISTOL_NRP_MIDI_GO:
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO,
+				(value > 0)? (int) 1: (int) 0);
+			return(0);
+		case BRISTOL_NRP_FORWARD:
+			bristolMidiOption(0, BRISTOL_NRP_FORWARD,
+				value > 0? (int) 1: (int) 0);
+			return(0);
 		case BRISTOL_NRP_DEBUG:
 			/*
 			 * This is a little unfortunte, we do not have direct access to
@@ -389,16 +439,17 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 			 * 052009 - integrate code to request the MIDI library include byte
 			 * debuging of interfaces.
 			 */
-			if (value == 0)
+			if ((value * C_RANGE_MIN_1) == 0)
 				baudio->midiflags &= ~(BRISTOL_MIDI_DEBUG1|BRISTOL_MIDI_DEBUG2);
-			else if (((int) (value * C_RANGE_MIN_1)) == 1) {
-				baudio->midiflags |= BRISTOL_MIDI_DEBUG1;
-				baudio->midiflags &= ~BRISTOL_MIDI_DEBUG2;
-			} else
-				baudio->midiflags |= (BRISTOL_MIDI_DEBUG2|BRISTOL_MIDI_DEBUG1);
+			else {
+				if ((value * C_RANGE_MIN_1) >= 1.99)
+					baudio->midiflags |= BRISTOL_MIDI_DEBUG2;
+				else
+					baudio->midiflags |= BRISTOL_MIDI_DEBUG1;
+			}
 
 			bristolMidiOption(0, BRISTOL_NRP_DEBUG,
-				(value * C_RANGE_MIN_1) > 1? (int) 1: (int) 0);
+				((int) (value * C_RANGE_MIN_1)));
 
 			return(0);
 		case BRISTOL_NRP_ENABLE_NRP:
@@ -407,6 +458,9 @@ bristolMidiController(Baudio *baudio, int NRP, float value)
 			else
 				baudio->midiflags &= ~BRISTOL_MIDI_NRP_ENABLE;
 			return(0);
+		case BRISTOL_NRP_REQ_SYSEX:
+			printf("		REQ SYSEX %f\n", value);
+			return(0);
 	}
 
 	/*
@@ -459,6 +513,15 @@ linearMap(float map[128])
  * should be extended to include curves as well.
  * Each parameter takes 8 value.
  */
+static void
+nonSensitiveMap(float map[128], int start, int end)
+{
+	int j;
+
+	for (j = 0; j < 128; j++)
+		map[j] = 0.71f;
+}
+
 static void
 linearScaledMap(float map[128], int start, int end)
 {
@@ -537,7 +600,9 @@ mapVelocityCurve(int velocity, float map[128])
 	 * The first 100 are linear scaled maps. The second 100 should be
 	 * exponentially scaled ones?
 	 */
-	if (v < 100) {
+	if (v == 0) {
+		nonSensitiveMap(map, 0, 127);
+	} else if (v < 100) {
 		linearScaledMap(map, v / 10, v % 10);
 	} else if (v < 200) {
 		/*
@@ -589,6 +654,11 @@ mapVelocityCurve(int velocity, float map[128])
 	if (r) invertMap(map);
 }
 
+/*
+ * This was initiallly a call made in the MIDI thread. That naturally led to 
+ * problems with timing and was eventually moved as a request from the MIDI
+ * thread to the audio thread.
+ */
 void
 initBristolAudio(audioMain *audiomain, Baudio *baudio)
 {
@@ -605,8 +675,8 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 		return;
 
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG3)
-		printf("using baudio: %x->%x\n", (size_t) audiomain->audiolist,
-			(size_t) baudio->next);
+		printf("using baudio: %p->%p\n", audiomain->audiolist,
+			baudio->next);
 
 	baudio->leftbuf = (float *) bristolmalloc0(audiomain->segmentsize);
 	baudio->rightbuf = (float *) bristolmalloc0(audiomain->segmentsize);
@@ -640,8 +710,9 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 	 */
 	if (bristolAlgos[(audiomain->midiflags & BRISTOL_PARAMMASK)].name != NULL)
 	{
-		char file[64];
+		char file[256];
 		float velocity;
+		int vint;
 
 		/*
 		 * Get hold of any local configuration, microtunings, etc.
@@ -664,8 +735,10 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 			 * have to support at least per key remapping so having a private
 			 * map will be a requirement.
 			 */
-			printf("No private microtonal mapping for %s\n",
-				bristolAlgos[(audiomain->midiflags & BRISTOL_PARAMMASK)].name);
+			if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG4)
+				printf("no private microtonal mapping for %s\n",
+					bristolAlgos[(audiomain->midiflags & BRISTOL_PARAMMASK)].name);
+			initMicrotonalTable(&baudio->microtonalmap[0]);
 		} else
 			printf("micro first %f last %f (%s)\n",
 				baudio->microtonalmap[0].step, baudio->microtonalmap[127].step,
@@ -676,8 +749,10 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 		 * could be moved into the library.....
 		 */
 		if (bristolGetMap(file, "velocityTable", &velocity, 1, 0) != 0)
-			mapVelocityCurve((int) &velocity, &baudio->velocitymap[0]);
-		else if (bristolGetMap(file, "velocityMap",
+		{
+			vint = (int) velocity;
+			mapVelocityCurve(vint, &baudio->velocitymap[0]);
+		} else if (bristolGetMap(file, "velocityMap",
 			&baudio->velocitymap[0], 128, 0) == 0)
 			/*
 			 * If we do not have some mapping we should default to a linear
@@ -685,8 +760,9 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 			 */
 			mapVelocityCurve(510, &baudio->velocitymap[0]);
 
-		printf("veloc first %1.2f last %1.2f\n",
-			baudio->velocitymap[0], baudio->velocitymap[127]);
+		if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG4)
+			printf("veloc first %1.2f last %1.2f\n",
+				baudio->velocitymap[0], baudio->velocitymap[127]);
 	}
 
 	/*
@@ -717,8 +793,8 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 				= (char *) bristolmalloc0(op->specs->localsize);
 		}
 
-		if ((baudio->effect != NULL) && (baudio->effect[1] != NULL)
-			&& (baudio->soundCount > 1))
+		if ((baudio->soundCount > 1)
+			&& (baudio->effect != NULL) && (baudio->effect[1] != NULL))
 		{
 			op = (audiomain->effects)[(*baudio->effect[1]).index];
 			baudio->FXlocals[i][1]
@@ -738,7 +814,7 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 				= (char *) bristolmalloc0(op->specs->localsize);
 
 			if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG4)
-				printf("malloc locals %x %i\n", (size_t) baudio->locals[i][j],
+				printf("malloc locals %p %i\n", baudio->locals[i][j],
 					op->specs->localsize);
 		}
 	}
@@ -811,10 +887,10 @@ freeBristolAudio(audioMain *audiomain, Baudio *baudio)
 	if (baudio == (Baudio *) NULL)
 		return;
 
-	baudio->mixflags = BRISTOL_HOLDDOWN;
+	baudio->mixflags |= BRISTOL_HOLDDOWN;
 
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG1)
-		printf("freeBristolAudio(%x, %x)\n", (size_t) audiomain, (size_t) baudio);
+		printf("freeBristolAudio(%p, %p)\n", audiomain, baudio);
 
 	freeSoundAlgo(baudio, 0, baudio->sound);
 	bristolfree(baudio->sound);
@@ -871,9 +947,16 @@ freeBristolAudio(audioMain *audiomain, Baudio *baudio)
 
 	while (voice != NULL)
 	{
-		if ((voice->baudio != NULL)
-			&& (voice->baudio->controlid == baudio->controlid))
-			voice->baudio = NULL;
+		if (voice->baudio != NULL)
+		{
+			if (voice->baudio->controlid == baudio->controlid)
+			{
+				voice->flags |= BRISTOL_KEYDONE;
+				voice->baudio = NULL;
+			}
+		} else
+			voice->flags |= BRISTOL_KEYDONE;
+
 		voice = voice->next;
 	}
 
@@ -883,7 +966,12 @@ freeBristolAudio(audioMain *audiomain, Baudio *baudio)
 	if (baudio->mixlocals != NULL)
 		bristolfree(baudio->mixlocals);
 
+	/*
+	 * This can only be freed once exit handling has been done by the audio
+	 * engine and MIDI event management code, refer to audioEngine.c and 
+	 * bristolsystem.c
 	bristolfree(baudio);
+	 */
 }
 
 void
@@ -907,11 +995,16 @@ freeMidiVoices(audioMain *audiomain, bristolVoice *voice)
 static void
 resetAudioThread(audioMain *audiomain)
 {
+	Baudio *holder;
+
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG1)
 		printf("resetAudioThread()\n");
 
-	while (audiomain->audiolist != NULL)
+	while (audiomain->audiolist != NULL) {
+		holder = audiomain->audiolist;
 		freeBristolAudio(audiomain, audiomain->audiolist);
+		bristolfree(holder);
+	}
 
 	/*
 	 * Free the array of voice pointers.
@@ -940,8 +1033,8 @@ initPalette(audioMain *audiomain, bristolOP *palette[])
 				audiomain->samplecount);
 
 		if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG3)
-			printf("Assigned operator %s to %i at %x\n",
-				palette[i]->specs->opname, i, (size_t) palette[i]);
+			printf("Assigned operator %s to %i at %p\n",
+				palette[i]->specs->opname, i, palette[i]);
 		if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG4)
 			bristolOPprint(palette[i]);
 	}
@@ -965,10 +1058,10 @@ freePalette(audioMain *audiomain, bristolOP *palette[])
 		if (palette[i] != (bristolOP *) NULL)
 		{
 			if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG3)
-				printf("Removed operator %s from %i at %x\n",
-					palette[i]->specs->opname, i, (size_t) palette[i]);
-printf("Removed operator %s from %i at %x\n",
-palette[i]->specs->opname, i, (size_t) palette[i]);
+				printf("Removed operator %s from %i at %p\n",
+					palette[i]->specs->opname, i, palette[i]);
+//printf("Removed operator %s from %i at %x\n",
+//palette[i]->specs->opname, i, (size_t) palette[i]);
 
 			palette[i]->destroy(palette[i]);
 
@@ -984,7 +1077,7 @@ initMidiVoices(audioMain *audiomain)
 	bristolVoice *voice;
 
 	if ((audiomain->debuglevel & BRISTOL_DEBUG_MASK) > BRISTOL_DEBUG1)
-		printf("initMidiVoices(%x, %i)\n", (size_t) audiomain, audiomain->voiceCount);
+		printf("initMidiVoices(%p, %i)\n", audiomain, audiomain->voiceCount);
 
 	audiomain->playlist = NULL;
 	audiomain->playlast = NULL;
@@ -1031,17 +1124,24 @@ extern bristolMidiHandler bristolMidiRoutines;
 void
 doNoteChanges(bristolVoice *voice)
 {
-	if ((voice->flags & BRISTOL_KEYDONE) == 0)
+	if (~voice->flags & BRISTOL_KEYDONE)
 	{
 		/*
 		 * First make sure we have no microtonal map.
 		 */
-		if ((voice->baudio->microtonalmap[voice->key.key].step <= 0)
-            && (bristolMidiRoutines.bmr[7].floatmap[voice->key.key] <= 0))
-			voice->cFreq = voice->dFreq
-				= voice->baudio->ctab[voice->key.key].step;
+		if (voice->baudio->microtonalmap[voice->key.key].step > 0.0) {
+			voice->dFreq = voice->baudio->microtonalmap[voice->key.key].step;
+			voice->dfreq = voice->baudio->microtonalmap[voice->key.key].freq;
+		} else if (bristolMidiRoutines.bmr[7].floatmap[voice->key.key] > 0.0) {
+			/*
+			 * This is the synth global microtonal map
+			 */
+			voice->dFreq = bristolMidiRoutines.freq[voice->key.key].step;
+			voice->dfreq = bristolMidiRoutines.freq[voice->key.key].freq;
+		} else {
+			voice->dFreq = voice->baudio->ctab[voice->key.key].step;
+			voice->dfreq = voice->baudio->ctab[voice->key.key].freq;
+		}
 	}
-/*	if (voice->next != NULL) */
-/*		doNoteChanges(voice->next); */
 }
 
