@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -189,9 +188,10 @@ static float
 tof(float *buf, int size, float lim, float start)
 {
 	int i;
+	float *buf2 = buf;
 
 	for (i = 0; i < size; i++)
-		*buf++ = (start += ((*buf - start) * lim));
+		*buf++ = (start += ((*buf2++ - start) * lim));
 
 	return(start);
 }
@@ -616,6 +616,8 @@ static int
 bristolAksDestroy(audioMain *audiomain, Baudio *baudio)
 {
 	printf("removing one aks\n");
+	baudio->mixlocals = NULL;
+	return(0);
 
 	bristolfree(freqbuf); /* replace with 'buf = NULL'; */
 	bristolfree(zerobuf); /* replace with 'buf = NULL'; */
