@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -37,7 +36,6 @@
  * audio threads, unless we ensure a single thread deals with any given algo
  * type, since then they are only used sequentially.
  */
-static float *freqbuf = (float *) NULL;
 static float *fmbuf = (float *) NULL;
 static float *filtbuf = (float *) NULL;
 static float *noisebuf = (float *) NULL;
@@ -95,7 +93,7 @@ arp2600AddPatch(int from, int to)
 {
 	int v;
 
-	printf("arp2600AddPatch(%i, %i)\n", from, to);
+	//printf("arp2600AddPatch(%i, %i)\n", from, to);
 
 	for (v = 0; v < abufs.voiceCount; v++)
 	{
@@ -110,7 +108,7 @@ arp2600ClearPatch(int from, int to)
 {
 	int v;
 
-	printf("arp2600ClearPatch(%i, %i)\n", from, to);
+	//printf("arp2600ClearPatch(%i, %i)\n", from, to);
 
 	for (v = 0; v < abufs.voiceCount; v++)
 	{
@@ -181,28 +179,16 @@ baudio->effect[0]->param->param[5].float_val);
 	 * as they are difficult to track between GUI and engine.
 	 */
 	if (operator == 102)
-	{
 		/*
 		 * Clear to default patch table.
 		 */
 		return(arp2600ClearPatchtable());
-	}
 
 	if (operator == 100)
-	{
-		/*
-		 * Clear to default patch table.
-		 */
 		return(arp2600AddPatch(controller, value * CONTROLLER_RANGE));
-	}
 
 	if (operator == 101)
-	{
-		/*
-		 * Clear to default patch table.
-		 */
 		return(arp2600ClearPatch(controller, value * CONTROLLER_RANGE));
-	}
 
 	if (operator == 103)
 	{
@@ -311,7 +297,7 @@ baudio->effect[0]->param->param[5].float_val);
 			((arp2600mods *) baudio->mixlocals)->fxRpan = value;
 			break;
 		case 10:
-			((arp2600mods *) baudio->mixlocals)->preamp = value * 8.0f;
+			((arp2600mods *) baudio->mixlocals)->preamp = value * 16.0f;
 			break;
 		case 11: /* This looks quite high, it has to normalise the signals */
 			((arp2600mods *) baudio->mixlocals)->volume = value * 256;
@@ -338,6 +324,7 @@ bristolVoice *voice, register float *startbuf)
 	 * once. We first copy it to the first voice, and worry about the rest 
 	 * when they come around in the poly routine below.
 	 */
+	memset(b, 0.0f, samplecount * sizeof(float));
 	for (i = 0; i < samplecount; i+=8)
 	{
 		*b++ = *s++ * gain;
@@ -349,6 +336,7 @@ bristolVoice *voice, register float *startbuf)
 		*b++ = *s++ * gain;
 		*b++ = *s++ * gain;
 	}
+
 	return(0);
 }
 
@@ -377,26 +365,52 @@ bristolVoice *voice, register float *startbuf)
 	arp2600mods *mods = (arp2600mods *) baudio->mixlocals;
 	int sc = audiomain->samplecount, kflags = 0;
 	int ss = audiomain->segmentsize, i;
-	float *b = abufs.outputs[voice->index][ARP_O_PREAMP];
+	float *b = abufs.outputs[voice->index][ARP_O_PREAMP], g;
 	float *s = abufs.outputs[0][ARP_O_PREAMP];
 
 /*printf("operateOneArp2600(%i, %x, %x)\n", voice->index, audiomain, baudio); */
 
+	/*
+	 * As of 0.50.4 we should be supporting 4 IO for CV or audio from Jack,
+	 * these buffers if available should be copied over to maintain consistency.
+	 */
+	if (audiomain->iocount > 0)
+	{
+		for (i = 0; i < audiomain->iocount; i++)
+		{
+			if (audiomain->io_i[i] == NULL)
+				continue;
+
+			bufmerge(
+				audiomain->io_i[i], 12.0f, //4.0 * audiomain->m_io_igc,
+				abufs.outputs[voice->index][ARP_O_IN_1 + i], 0.0,
+				sc);
+		}
+	}
+
+	bristolbzero(abufs.inputs[voice->index][ARP_I_OUT_1], sc);
+	bristolbzero(abufs.inputs[voice->index][ARP_I_OUT_2], sc);
+	bristolbzero(abufs.inputs[voice->index][ARP_I_OUT_3], sc);
+	bristolbzero(abufs.inputs[voice->index][ARP_I_OUT_4], sc);
+
 /* Preamp patching and envelope follower. */
 	/*
 	 * Since the input buffer is singular and we may have multiple voice, then
 	 * to get the correct routing we need to copy this to each voice buffers.
 	 */
-	for (i = 0; i < sc; i+=8)
+	if (voice->index != 0)
 	{
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
-		*b++ = *s++;
+		for (i = 0; i < sc; i+=8)
+		{
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+			*b++ = *s++;
+		}
 	}
 
 	/* Envelope follower. */
@@ -441,7 +455,7 @@ bristolVoice *voice, register float *startbuf)
 	/*
 	 * Apply a gain to the LFO for mods.
 	 */
-	bufmerge(freqbuf, 0.0, abufs.outputs[voice->index][ARP_O_CLOCK],
+	bufmerge(abufs.outputs[voice->index][ARP_O_KBD_CV], 0.0, abufs.outputs[voice->index][ARP_O_CLOCK],
 		4.0 * mods->lfogain, sc);
 /* */
 
@@ -520,20 +534,37 @@ bristolVoice *voice, register float *startbuf)
 /*printf("%i: osc1 to %x\n", voice->index, */
 /*abufs.outputs[voice->index][ARP_O_VCO1_SQR]); */
 
-	fillFreqTable(baudio, voice, freqbuf, sc, 1);
+	/*
+	 * We need to make this connection for CV KBD open again. If nothing is
+	 * patched in then we execute this code, or rather, this freq table goes
+	 * into a buffer which is the default input. If the control is repatched
+	 * then we need to configure a 1/Octave intepretter for the alternative
+	 * buffer. This leads to a few issues with the filter which I think takes
+	 * a linear 1.0f for frequency control - these differences are historical
+	 * and we should see about resolving them for this emulator, later the 
+	 * others.
+	 */
+	fillFreqTable(baudio, voice,
+		abufs.outputs[voice->index][ARP_O_KBD_CV], sc, 1);
 
 	if (mods->flags & ARP_OSC1_TRACKING)
-		bufmerge(freqbuf, 1.0, fmbuf, 0.0, sc);
+		bufmerge(abufs.inputs[voice->index][ARP_I_VCO1_KBD], 1.0,
+			fmbuf, 0.0, sc);
 	else {
 		register int i;
-		register float *bufptr = fmbuf, value = 5.0, mult = 1/4;
+		register float *bufptr = fmbuf, value = 5.0, mult = 0.25;
 
 		if (mods->flags & ARP_OSC1_LFO)
 		{
-			value = 0.02;
-			mult = 1/16;
+			value = 0.005;
+			mult = 0.40;
 		}
 
+		if (abufs.inputs[voice->index][ARP_I_VCO1_KBD] == 
+			abufs.outputs[voice->index][ARP_O_KBD_CV])
+			mult *= baudio->contcontroller[1];
+
+		mult += value;
 		/*
 		 * If we do not have keyboard tracking of Osc, then put in a LFO
 		 * frequency. This is about 0.1 Hz, although we should consider
@@ -545,14 +576,14 @@ bristolVoice *voice, register float *startbuf)
 		 */
 		for (i = 0; i < sc; i+=8)
 		{
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
 		}
 	}
 
@@ -601,17 +632,24 @@ bristolVoice *voice, register float *startbuf)
 	 * Fill our fmbuf scratch table again.
 	 */
 	if (mods->flags & ARP_OSC2_TRACKING)
-		bufmerge(freqbuf, 1.0, fmbuf, 0.0, sc);
+		bufmerge(abufs.inputs[voice->index][ARP_I_VCO2_KBD], 1.0,
+			fmbuf, 0.0, sc);
 	else {
 		register int i;
-		register float *bufptr = fmbuf, value = 5.0, mult = 1/4;
+		register float *bufptr = fmbuf, value = 5.0, mult = 0.25;
 
 		if (mods->flags & ARP_OSC2_LFO)
 		{
-			value = 0.02;
-			mult = 1/16;
+			value = 0.005;
+			mult = 0.40;
 		}
 
+		if (abufs.inputs[voice->index][ARP_I_VCO2_KBD] == 
+			abufs.outputs[voice->index][ARP_O_KBD_CV])
+			mult *= baudio->contcontroller[1];
+
+		mult += value;
+
 		/*
 		 * If we do not have keyboard tracking of Osc, then put in a LFO
 		 * frequency. This is about 0.1 Hz, although we should consider
@@ -623,14 +661,14 @@ bristolVoice *voice, register float *startbuf)
 		 */
 		for (i = 0; i < sc; i+=8)
 		{
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
 		}
 	}
 
@@ -678,17 +716,24 @@ bristolVoice *voice, register float *startbuf)
 	 * Fill our fmbuf scratch table again.
 	 */
 	if (mods->flags & ARP_OSC3_TRACKING)
-		bufmerge(freqbuf, 1.0, fmbuf, 0.0, sc);
+		bufmerge(abufs.inputs[voice->index][ARP_I_VCO3_KBD], 1.0,
+			fmbuf, 0.0, sc);
 	else {
 		register int i;
-		register float *bufptr = fmbuf, value = 5.0, mult = 1/4;
+		register float *bufptr = fmbuf, value = 5.0f, mult = 0.25f;
 
 		if (mods->flags & ARP_OSC3_LFO)
 		{
-			value = 0.02;
-			mult = 1/16;
+			value = 0.005;
+			mult = 0.40;
 		}
 
+		if (abufs.inputs[voice->index][ARP_I_VCO3_KBD] == 
+			abufs.outputs[voice->index][ARP_O_KBD_CV])
+			mult *= baudio->contcontroller[1];
+
+		mult += value;
+
 		/*
 		 * If we do not have keyboard tracking of Osc, then put in a LFO
 		 * frequency. This is about 0.1 Hz, although we should consider
@@ -700,14 +745,14 @@ bristolVoice *voice, register float *startbuf)
 		 */
 		for (i = 0; i < sc; i+=8)
 		{
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
-			*bufptr++ = value + baudio->contcontroller[1] * mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
+			*bufptr++ = mult;
 		}
 	}
 
@@ -759,10 +804,10 @@ bristolVoice *voice, register float *startbuf)
 	 * These are controllers 44 to 47.
 	 */
 	bufmerge(abufs.inputs[voice->index][ARP_I_MIX_1_1],
-		abufs.gains[ARP_I_MIX_1_1],
+		abufs.gains[ARP_I_MIX_1_1] * 2.0f,
 		abufs.outputs[voice->index][ARP_O_MIX_1], 0.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_MIX_1_2],
-		abufs.gains[ARP_I_MIX_1_2],
+		abufs.gains[ARP_I_MIX_1_2] * 2.0f,
 		abufs.outputs[voice->index][ARP_O_MIX_1], 1.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_MIX_1_3], 1.0,
 		abufs.outputs[voice->index][ARP_O_MIX_1], 1.0, sc);
@@ -774,7 +819,7 @@ bristolVoice *voice, register float *startbuf)
 		abufs.outputs[voice->index][ARP_O_MIX_1], sc);
 
 	bufmerge(abufs.inputs[voice->index][ARP_I_MIX_2_1],
-		abufs.gains[ARP_I_MIX_2_1],
+		abufs.gains[ARP_I_MIX_2_1] * 2.0f,
 		abufs.outputs[voice->index][ARP_O_MIX_2], 0.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_MIX_2_2], 1.0,
 		abufs.outputs[voice->index][ARP_O_MIX_2], 1.0, sc);
@@ -831,22 +876,8 @@ bristolVoice *voice, register float *startbuf)
 		abufs.gains[ARP_I_FILT_2SQR] * 96.0, mixbuf, 1.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_FILT_3TRI],
 		abufs.gains[ARP_I_FILT_3TRI] * 96.0, mixbuf, 1.0, sc);
-	/*
-	 * This is for denormal prevention, antialiasing, self oscillation,
-	 * whatever else you might consider it for, it injects a constant low
-	 * floor noise signal
-	if (abufs.inputs[voice->index][ARP_I_FILT_NSE] ==
-		abufs.inputs[voice->index][ARP_O_NOISE])
-		bufmerge(abufs.inputs[voice->index][ARP_I_FILT_NSE],
-			abufs.gains[ARP_I_FILT_NSE] * 96.0 + 0.01, mixbuf, 1.0, sc);
-	else {
-		bufmerge(abufs.inputs[voice->index][ARP_I_FILT_NSE],
-			abufs.gains[ARP_I_FILT_NSE] * 96.0, mixbuf, 1.0, sc);
-		bufmerge(abufs.inputs[voice->index][ARP_O_NOISE], 0.01, mixbuf, 1.0,sc);
-	}
-	 */
-		bufmerge(abufs.inputs[voice->index][ARP_I_FILT_NSE],
-			abufs.gains[ARP_I_FILT_NSE] * 96.0, mixbuf, 1.0, sc);
+	bufmerge(abufs.inputs[voice->index][ARP_I_FILT_NSE],
+		abufs.gains[ARP_I_FILT_NSE] * 48.0, mixbuf, 1.0, sc);
 /* */
 
 /* Filter */
@@ -855,12 +886,30 @@ bristolVoice *voice, register float *startbuf)
 	audiomain->palette[(*baudio->sound[3]).index]->specs->io[2].buf
 		= abufs.outputs[voice->index][ARP_O_VCF];
 
+	/*
+	 * Fill the KBD_CV now with a linear value related to the MIDI key number.
+	 */
+	b = abufs.outputs[voice->index][ARP_O_KBD_CV];
+	g = abufs.gains[ARP_I_FILT_KBD] * voice->cfreq * 1024 / baudio->samplerate;
+//printf("gain is %f %f %f\n", g, voice->cfreq, abufs.gains[ARP_I_FILT_KBD]);
+	for (i = 0; i < sc; i+=8)
+	{
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+		*b++ = g;
+	}
+
 	bufmerge(abufs.inputs[voice->index][ARP_I_FILT_KBD],
 		abufs.gains[ARP_I_FILT_KBD], scratch, 0.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_FILT_ADSR],
-		abufs.gains[ARP_I_FILT_ADSR], scratch, 1.0, sc);
+		abufs.gains[ARP_I_FILT_ADSR] * 0.25, scratch, 1.0, sc);
 	bufmerge(abufs.inputs[voice->index][ARP_I_FILT_2SIN],
-		abufs.gains[ARP_I_FILT_2SIN], scratch, 1.0, sc);
+		abufs.gains[ARP_I_FILT_2SIN] * 0.25, scratch, 1.0, sc);
 
 	bristolbzero(abufs.outputs[voice->index][ARP_O_VCF], ss);
 
@@ -971,6 +1020,73 @@ bristolVoice *voice, register float *startbuf)
 			baudio->leftbuf, 1.0, sc);
 	}
 /* */
+
+	/*
+	 * As of 0.50.4 we should be supporting 4 IO for CV or audio from Jack,
+	 * these buffers if available should be copied over to maintain consistency.
+	 */
+	if (audiomain->iocount > 0)
+	{
+		register int i;
+		register float *s, *d;
+
+		if (audiomain->io_o[0] != NULL)
+		{
+			//bristolbzero(audiomain->io_o[0], audiomain->segmentsize);
+			bufmerge(
+				abufs.inputs[voice->index][ARP_I_OUT_1], 0.0833f,
+				audiomain->io_o[0], 1.0f, //4.0 * audiomain->m_io_igc,
+				audiomain->samplecount);
+		}
+
+		if ((audiomain->io_o[1] != NULL) && (audiomain->iocount > 1))
+		{
+			//bristolbzero(audiomain->io_o[1], audiomain->segmentsize);
+			bufmerge(
+				abufs.inputs[voice->index][ARP_I_OUT_2], 0.0833f,
+				audiomain->io_o[1], 1.0f, //4.0 * audiomain->m_io_igc,
+				audiomain->samplecount);
+		}
+
+		if (((d = audiomain->io_o[2]) != NULL) && (audiomain->iocount > 2))
+		{
+			s = abufs.inputs[voice->index][ARP_I_OUT_3];
+			//bristolbzero(d, audiomain->segmentsize);
+
+			/* Third IO will be DC */
+			for (i = 0; i < sc; i+=8)
+			{
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+			}
+		}
+
+		if (((d = audiomain->io_o[3]) != NULL) && (audiomain->iocount > 3))
+		{
+			s = abufs.inputs[voice->index][ARP_I_OUT_4];
+			//bristolbzero(d, audiomain->segmentsize);
+
+			/* Fourth IO will be DC */
+			for (i = 0; i < sc; i+=8)
+			{
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+				if ((*d += *s * 0.04166667f + 0.5f) < 0) *d = 0.0f;s++;d++;
+			}
+		}
+	}
+
 	return(0);
 }
 
@@ -978,12 +1094,10 @@ int
 operateArp2600Postops(audioMain *audiomain, Baudio *baudio,
 bristolVoice *voice, register float *startbuf)
 {
+	/*
 	if (baudio->mixlocals == NULL)
-		/* ((void *) baudio->mixlocals) = voice->locals[voice->index]; */
 		baudio->mixlocals = (float *) voice->locals[voice->index];
-
-/*	bristolbzero(abufs.outputs[0][ARP_O_FX_L], audiomain->segmentsize); */
-/*	bristolbzero(abufs.outputs[0][ARP_O_FX_R], audiomain->segmentsize); */
+	*/
 
 /* Drive data through the effects: */
 	/*
@@ -1051,7 +1165,6 @@ static int bristolArp2600Destroy(audioMain *audiomain, Baudio *baudio)
 	 */
 	return(0);
 
-	bristolfree(freqbuf); /* replace with 'buf = NULL'; */
 	bristolfree(fmbuf);
 	bristolfree(filtbuf);
 /*	bristolfree(noisebuf); */
@@ -1077,8 +1190,6 @@ static int bristolArp2600Destroy(audioMain *audiomain, Baudio *baudio)
 int
 bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 {
-	arp2600mods *mods;
-
 	printf("initialising arp2600\n");
 
 	baudio->soundCount = 14; /* Number of operators in this voice */
@@ -1184,8 +1295,6 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 		scratch = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (mixbuf == NULL)
 		mixbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (freqbuf == NULL)
-		freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (fmbuf == NULL)
 		fmbuf = (float *) bristolmalloc0(audiomain->segmentsize);
 
@@ -1226,16 +1335,16 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 		abufs.defaults[0] = sc * ARP_O_PREAMP;
 		abufs.defaults[1] = sc * ARP_O_VCO1_SAW;
 		abufs.defaults[2] = sc * ARP_O_VCO2_SIN;
-		abufs.defaults[3] = sc * ARP_ZERO;
+		abufs.defaults[3] = sc * ARP_O_KBD_CV;
 		abufs.defaults[4] = sc * ARP_O_SH;
 		abufs.defaults[5] = sc * ARP_O_ADSR;
 		abufs.defaults[6] = sc * ARP_O_VCO2_SIN;
-		abufs.defaults[7] = sc * ARP_ZERO;
+		abufs.defaults[7] = sc * ARP_O_KBD_CV;
 		abufs.defaults[8] = sc * ARP_O_SH;
 		abufs.defaults[ARP_I_VCO2_ADSR] = sc * ARP_O_ADSR;
 		abufs.defaults[ARP_I_VCO2_VCO1] = sc * ARP_O_VCO1_SQR;
 		abufs.defaults[ARP_I_VCO2_NSE] = sc * ARP_O_NOISE;
-		abufs.defaults[12] = sc * ARP_ZERO;
+		abufs.defaults[12] = sc * ARP_O_KBD_CV;
 		abufs.defaults[13] = sc * ARP_O_NOISE;
 		abufs.defaults[14] = sc * ARP_O_ADSR;
 		abufs.defaults[15] = sc * ARP_O_VCO2_SIN;
@@ -1245,10 +1354,10 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 		abufs.defaults[ARP_I_FILT_2SQR] = sc * ARP_O_VCO2_SQR;
 		abufs.defaults[ARP_I_FILT_3TRI] = sc * ARP_O_VCO3_SAW;
 		abufs.defaults[ARP_I_FILT_NSE] = sc * ARP_O_NOISE;
-		abufs.defaults[22] = sc * ARP_ZERO;
+		abufs.defaults[ARP_I_FILT_KBD] = sc * ARP_O_KBD_CV;
 		abufs.defaults[23] = sc * ARP_O_ADSR;
 		abufs.defaults[24] = sc * ARP_O_VCO2_SIN;
-		abufs.defaults[25] = sc * ARP_O_VCF; /*Wrong - gate. ZERO */
+		abufs.defaults[25] = sc * ARP_O_VCF; /* Wrong - gate. ZERO */
 		abufs.defaults[26] = sc * ARP_O_CLOCK;
 		abufs.defaults[27] = sc * ARP_ZERO;
 		abufs.defaults[28] = sc * ARP_ZERO;
@@ -1274,6 +1383,10 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 		abufs.defaults[48] = sc * ARP_ZERO;
 		abufs.defaults[49] = sc * ARP_ZERO;
 		abufs.defaults[50] = sc * ARP_ZERO;
+		abufs.defaults[51] = sc * ARP_ZERO;
+		abufs.defaults[52] = sc * ARP_ZERO;
+		abufs.defaults[53] = sc * ARP_ZERO;
+		abufs.defaults[54] = sc * ARP_ZERO;
 
 		for (i = 0; i < audiomain->voiceCount; i++) {
 			abufs.inputs[i][0] = &abufs.buf[
@@ -1366,6 +1479,29 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 				abufs.defaults[42] + audiomain->samplecount * i * ARP_OUTCNT];
 			abufs.inputs[i][43] = &abufs.buf[
 				abufs.defaults[43] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][44] = &abufs.buf[
+				abufs.defaults[44] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][45] = &abufs.buf[
+				abufs.defaults[45] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][46] = &abufs.buf[
+				abufs.defaults[46] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][47] = &abufs.buf[
+				abufs.defaults[47] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][48] = &abufs.buf[
+				abufs.defaults[48] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][40] = &abufs.buf[
+				abufs.defaults[49] + audiomain->samplecount * i * ARP_OUTCNT];
+			abufs.inputs[i][50] = &abufs.buf[
+				abufs.defaults[50] + audiomain->samplecount * i * ARP_OUTCNT];
+			/* These are the CV I/O */
+			abufs.inputs[i][51] = &abufs.buf[
+				abufs.defaults[51] + audiomain->samplecount * 0 * ARP_OUTCNT];
+			abufs.inputs[i][52] = &abufs.buf[
+				abufs.defaults[52] + audiomain->samplecount * 0 * ARP_OUTCNT];
+			abufs.inputs[i][53] = &abufs.buf[
+				abufs.defaults[53] + audiomain->samplecount * 0 * ARP_OUTCNT];
+			abufs.inputs[i][54] = &abufs.buf[
+				abufs.defaults[54] + audiomain->samplecount * 0 * ARP_OUTCNT];
 		}
 
 		for (v = 0; v < ARP_OUTCNT; v++)
@@ -1374,8 +1510,6 @@ bristolArp2600Init(audioMain *audiomain, Baudio *baudio)
 
 	((arp2600mods *) baudio->mixlocals)->mod[ARP_O_VCF].o1gain = 0.125;
 
-	mods = (arp2600mods *) baudio->mixlocals;
-
 	baudio->mixflags |= BRISTOL_STEREO;
 
 	return(0);
