@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -63,13 +62,15 @@
 #define ARP_O_MIX_2		25
 #define ARP_O_MIX_2_2	26
 #define ARP_O_LAG		27
-#define ARP_O_FX_R		28 /* fix this, not used */
-#define ARP_O_FX_L		29 /* fix this, not used */
-#define ARP_O_FINAL		30
-#define ARP_O_PAN		31
-#define ARP_ZERO		32 /* Other outputs PRIOR to this one! */
-#define ARP_OUTCNT		33 /* END OF OUTPUTS */
-#define BRISTOL_ARP_BUFCNT 35
+#define ARP_O_PAN		28
+#define ARP_O_IN_1		29 /* These are input into the emulator, ie, outputs */
+#define ARP_O_IN_2		30 /* These are input into the emulator, ie, outputs */
+#define ARP_O_IN_3		31 /* These are input into the emulator, ie, outputs */
+#define ARP_O_IN_4		32 /* These are input into the emulator, ie, outputs */
+#define ARP_O_KBD_CV	33 /* Other outputs PRIOR to this one! */
+#define ARP_ZERO		34 /* Other outputs PRIOR to this one! */
+#define ARP_OUTCNT		35 /* END OF OUTPUTS */
+#define BRISTOL_ARP_BUFCNT 36
 
 #define ARP_I_ENVFOLLOW 0
 #define ARP_I_RM1		1
@@ -122,7 +123,11 @@
 #define ARP_I_MIX_1_3	48
 #define ARP_I_MIX_1_4	49
 #define ARP_I_MIX_2_2	50
-#define ARP_INCOUNT		51 /* END OF INPUTS */
+#define ARP_I_OUT_1		51
+#define ARP_I_OUT_2		52
+#define ARP_I_OUT_3		53
+#define ARP_I_OUT_4		54
+#define ARP_INCOUNT		55 /* END OF INPUTS */
 
 typedef struct arp2600Mod {
 	float *buf;   /* stuff with some buffer pointer by GUI requests */
@@ -155,7 +160,7 @@ typedef struct arp2600Mods {
 	float fxLpan;
 	float fxRpan;
 	float lfogain;
-	arp2600mod mod[20];
+	arp2600mod mod[ARP_INCOUNT];
 	float shkbd;
 } arp2600mods;
 
