@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -62,7 +61,7 @@ bme700Controller(Baudio *baudio, u_char operator, u_char controller, float value
 			alterAllNotes(baudio);
 			break;
 		case 2:
-			BME700LOCAL->volume = value * 0.015625; // 0.03125;
+			BME700LOCAL->volume = value * 0.025625; // 0.03125;
 			break;
 		case 3:
 			BME700LOCAL->oscmix = value * mixscaler;
@@ -271,28 +270,6 @@ bristolVoice *voice, register float *startbuf)
 	return(0);
 }
 
-/*
- * This looks a little specific, it is used here to apply the normalised LFO
- * to the frequency buffer - as a multiplier. That maintains frequency mod over
- * different notes. It could be made a little more general such as all freq 
- * mods including ADSR for the prophet, etc
- */
-static void
-mult2buf(register float *d, register float *s, register float v, register int c)
-{
-	for (; c > 0; c-=8)
-	{
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-		*d = (1.0 + *s++) * *d * v; d++;
-	}
-}
-
 int
 operateOneBme700(audioMain *audiomain, Baudio *baudio,
 bristolVoice *voice, register float *startbuf)
