@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define DEBUG */
@@ -706,6 +705,22 @@ bristolVoice *voice, register float *startbuf)
 int
 static bristolCs80Destroy(audioMain *audiomain, Baudio *baudio)
 {
+	/*
+	 * The following can be left up to the library. If we free this here then
+	 * we do need to NULL the pointer otherwise glibc may get picky.
+	 *
+	 * We do need to clean this up, though.
+	 */
+	bristolfree(((csmods *) baudio->mixlocals)->chanmods[CH_I].keybrill);
+	bristolfree(((csmods *) baudio->mixlocals)->chanmods[CH_II].keybrill);
+	bristolfree(((csmods *) baudio->mixlocals)->chanmods[CH_I].keylevel);
+	bristolfree(((csmods *) baudio->mixlocals)->chanmods[CH_II].keylevel);
+
+	bristolfree(((csmods *) baudio->mixlocals)->lout);
+	bristolfree(((csmods *) baudio->mixlocals)->rout);
+
+	return(0);
+
 	if (freqbuf != NULL)
 		bristolfree(freqbuf);
 	if (adsr1buf != NULL)
@@ -747,15 +762,6 @@ static bristolCs80Destroy(audioMain *audiomain, Baudio *baudio)
 	pwmbuf = NULL;
 	modbuf = NULL;
 	sinebuf = NULL;
-
-	/*
-	 * The following can be left up to the library. If we free this here then
-	 * we do need to NULL the pointer otherwise glibc may get picky.
-	 */
-	bristolfree(baudio->mixlocals);
-	baudio->mixlocals = NULL;
-	
-	return(0);
 }
 
 int
