@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -140,6 +139,7 @@ dxOpOne(int ind, float *kbuf, float *opbuf, float *obuf, int count,
 dxmix *mix, audioMain *am, Baudio *ba, bristolVoice *v)
 {
 	float igain = mix[ind].igain;
+	int flags = v->flags & BRISTOL_KEYDONE;
 
 	bufmerge(opbuf, 1.0, modbuf, 0.0, count);
 /*printf("dxOpOne(%i)\n", ind); */
@@ -170,6 +170,8 @@ dxmix *mix, audioMain *am, Baudio *ba, bristolVoice *v)
 		v,
 		(*ba->sound[ind]).param,
 		v->locals[v->index][ind]);
+
+	v->flags = v->flags & flags? v->flags: v->flags & ~BRISTOL_KEYDONE;
 }
 
 int
@@ -758,6 +760,7 @@ DXalgoNpostops(audioMain *am, Baudio *ba, bristolVoice *v, register float *s)
 int
 destroyOneDXVoice(audioMain *audiomain, Baudio *baudio)
 {
+	printf("destroy DX sound\n");
 	return(0);
 	bristolfree(lfobuf);
 	bristolfree(op0buf);
