@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define DEBUG */
@@ -31,7 +30,7 @@ static float *noisebuf = (float *) NULL;
 static float *filtbuf = (float *) NULL;
 static float *filtbuf2 = (float *) NULL;
 static float *lfotri = (float *) NULL;
-static float *lfosquare = (float *) NULL;
+static float *lforamp = (float *) NULL;
 static float *lfosh = (float *) NULL;
 static float *adsrbuf = (float *) NULL;
 static float *mbuf1 = (float *) NULL;
@@ -163,6 +162,9 @@ u_char operator, float value)
 				else
 					baudio->mixflags |= FILTER_MODE;
 				break;
+			case 13:
+				((bExp *) baudio->mixlocals)->noisegain = value;
+				break;
 		}
 	}
 
@@ -281,6 +283,7 @@ operateExpPreops(audioMain *audiomain, Baudio *baudio,
 bristolVoice *voice, register float *startbuf)
 {
 	bristolbzero(noisebuf, audiomain->segmentsize);
+
 	/*
 	 * Noise - we do this early for the same reason as oscillator 3
 	 */
@@ -300,8 +303,8 @@ bristolVoice *voice, register float *startbuf)
 			= noisebuf;
 		audiomain->palette[(*baudio->sound[8]).index]->specs->io[1].buf
 			= lfotri;
-		audiomain->palette[(*baudio->sound[8]).index]->specs->io[2].buf
-			= lfosquare;
+		audiomain->palette[(*baudio->sound[8]).index]->specs->io[6].buf
+			= lforamp;
 		audiomain->palette[(*baudio->sound[8]).index]->specs->io[3].buf
 			= lfosh;
 		audiomain->palette[(*baudio->sound[8]).index]->specs->io[4].buf
@@ -312,6 +315,7 @@ bristolVoice *voice, register float *startbuf)
 			(*baudio->sound[8]).param,
 			baudio->locals[voice->index][8]);
 	}
+
 	return(0);
 }
 
@@ -342,15 +346,15 @@ register int count, float c1)
 			break;
 		case SHAPE_KEY:
 			mod = onbuf;
-			gain = gain / 25;
+			gain = gain * 0.1f;
 			break;
 		case SHAPE_FILTENV:
 			mod = adsrbuf;
-			gain = gain / 25;
+			gain = gain * 0.1f;
 			break;
 		case SHAPE_ON:
 			mod = onbuf;
-			gain = gain / 25;
+			gain = gain * 0.1f;
 			break;
 	}
 
@@ -364,7 +368,7 @@ register int count, float c1)
 			source = lfotri;
 			break;
 		case 2:
-			source = lfosquare;
+			source = lforamp;
 			break;
 		case 4:
 			source = lfosh;
@@ -453,8 +457,8 @@ bristolVoice *voice, register float *startbuf)
 			= noisebuf;
 		audiomain->palette[(*baudio->sound[8]).index]->specs->io[1].buf
 			= lfotri;
-		audiomain->palette[(*baudio->sound[8]).index]->specs->io[2].buf
-			= lfosquare;
+		audiomain->palette[(*baudio->sound[8]).index]->specs->io[6].buf
+			= lforamp;
 		audiomain->palette[(*baudio->sound[8]).index]->specs->io[3].buf
 			= lfosh;
 		(*baudio->sound[8]).operate(
@@ -589,7 +593,7 @@ bristolVoice *voice, register float *startbuf)
 	 * any existing ext source signal).
 	 */
 	if (baudio->mixflags & MIX_NOISE)
-		bufmerge(noisebuf, 1.0, startbuf, 1.0, samplecount);
+		bufmerge(noisebuf, bexplorer->noisegain, startbuf, 1.0, samplecount);
 
 	/*
 	 * First oscillator
@@ -821,7 +825,9 @@ explorerPostops(audioMain *am, Baudio *ba, bristolVoice *v, register float *s)
 	/* Output conditioning, prevents filter self oscillation from overdrive */
 	bufmerge(ba->leftbuf, 0.0, ba->leftbuf, 0.2, am->samplecount);
 	bufmerge(ba->rightbuf, 0.0, ba->rightbuf, 0.2, am->samplecount);
-	
+
+	bristolbzero(s, am->segmentsize);
+
 	return(0);
 }
 
@@ -841,7 +847,7 @@ destroyOneExpVoice(audioMain *audiomain, Baudio *baudio)
 	bristolfree(filtbuf);
 	bristolfree(filtbuf2);
 	bristolfree(lfotri);
-	bristolfree(lfosquare);
+	bristolfree(lforamp);
 	bristolfree(lfosh);
 	bristolfree(adsrbuf);
 	bristolfree(mbuf1);
@@ -903,8 +909,8 @@ printf("initialising one bristolexplorer\n");
 		filtbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (lfotri == (float *) NULL)
 		lfotri = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (lfosquare == (float *) NULL)
-		lfosquare = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (lforamp == (float *) NULL)
+		lforamp = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (lfosh == (float *) NULL)
 		lfosh = (float *) bristolmalloc0(audiomain->segmentsize);
 	if (adsrbuf == (float *) NULL)
