@@ -1,11 +1,11 @@
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -364,6 +363,8 @@ bristolVoice *voice, register float *startbuf)
 int
 destroyOneGranularVoice(audioMain *audiomain, Baudio *baudio)
 {
+	baudio->mixlocals = NULL;
+	return(0);
 	if (freqbuf != NULL)
 		bristolfree(freqbuf);
 	freqbuf = NULL;
@@ -468,7 +469,8 @@ printf("initialising one granular sound\n");
 	busbuf[8] = bus9buf;
 
 	mods = (granularmods *) bristolmalloc0(sizeof(granularmods));
-printf("size is %i\n", sizeof(granularmods));
+
+	//printf("size is %i\n", sizeof(granularmods));
 
 	baudio->mixlocals = (float *) mods;
 	baudio->mixflags |= BRISTOL_STEREO;
