@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define DEBUG */
@@ -952,6 +951,15 @@ bristolVoice *voice, register float *startbuf)
 int
 static bristolJupiterDestroy(audioMain *audiomain, Baudio *baudio)
 {
+	int i;
+
+	printf("removing one jupiter\n");
+
+	for (i = 0; i < BRISTOL_VOICECOUNT; i++)
+		bristolfree(((bmods *) baudio->mixlocals)->dco2buf[i]);
+
+	bristolfree(((bmods *) baudio->mixlocals)->lout);
+	bristolfree(((bmods *) baudio->mixlocals)->rout);
 
 	/*
 	 * The following can be left up to the library. If we free this here then
