@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define DEBUG */
@@ -61,8 +60,9 @@ u_char operator, float value)
 		}
 	}
 
+#warning fill this out with a structure for osc3 and noise mxing, etc.
 	if (controller == 11)
-		*((float *) baudio->mixlocals) = value;
+		((mMods *) baudio->mixlocals)->mixmod = value;
 
 	if (controller == 12)
 	{
@@ -168,6 +168,12 @@ u_char operator, float value)
 				else
 					baudio->mixflags |= BRISTOL_MULTITRIG;
 				break;
+			case 16:
+				((mMods *) baudio->mixlocals)->mixosc3 = value;
+				break;
+			case 17:
+				((mMods *) baudio->mixlocals)->mixnoise = value;
+				break;
 		}
 	}
 
@@ -184,7 +190,7 @@ int
 operateOneMMVoiceFinal(audioMain *audiomain, Baudio *baudio,
 bristolVoice *voice, register float *startbuf)
 {
-	register int samplecount = audiomain->samplecount;
+	register int samplecount = audiomain->samplecount, i;
 	register float *adsrbuf, *filtbuf;
 
 	/*
@@ -239,7 +245,6 @@ bristolVoice *voice, register float *startbuf)
 	 */
 	if ((baudio->mixflags & OSC3_TRACKING) == 0)
 	{
-		register int i;
 		register float *bufptr = freqbuf;
 		/*
 		 * If we do not have keyboard tracking of Osc-3, then put in a LFO
@@ -277,6 +282,7 @@ bristolVoice *voice, register float *startbuf)
 	 * If we are mixing the osc3 source, put it into the scratch (along with
 	 * any existing ext source signal).
 	 */
+#warning correct the gain to bein the mix, not in the osc
 	if (baudio->mixflags & MIX_OSC3)
 		bufmerge(osc3buf, 1.0, scratch, 1.0, samplecount);
 
@@ -300,12 +306,38 @@ bristolVoice *voice, register float *startbuf)
 	/*
 	 * Mix osc3 and noise - these are both in the scratch if we have
 	 * them flagged to be mixed.
-	 */
+#warning convert noise mod to be S&H via OSC3
 	bufmerge(noisebuf,
-		*((float *) baudio->mixlocals) * baudio->contcontroller[1],
+		((mMods *) baudio->mixlocals)->mixmod * baudio->contcontroller[1],
 		modbuf, 0.0, samplecount);
+	 */
+	for (i = 0; i < samplecount; i+=8)
+	{
+		if ((((mMods *) baudio->mixlocals)->lsample < 0) && (osc3buf[i] >= 0))
+			((mMods *) baudio->mixlocals)->csample = noisebuf[i];
+
+		((mMods *) baudio->mixlocals)->lsample = osc3buf[i];
+
+		modbuf[i] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+1] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+2] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+3] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+4] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+5] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+6] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+		modbuf[i+7] = ((mMods *) baudio->mixlocals)->csample
+			* ((mMods *) baudio->mixlocals)->mixmod;
+	}
+
 	bufmerge( osc3buf,
-		(1 - *((float *) baudio->mixlocals)) * baudio->contcontroller[1],
+		(1 - ((mMods *) baudio->mixlocals)->mixmod) * baudio->contcontroller[1],
 		modbuf, 1.0, samplecount);
 
 	/*
@@ -412,6 +444,8 @@ bristolVoice *voice, register float *startbuf)
 		bufmerge(modbuf, 0.5, adsrbuf, 1.0, samplecount);
 	}
 
+	if (baudio->mixflags & MIX_EXT)
+		bufmerge(startbuf, baudio->extgain * 2, scratch, 1.0, samplecount);
 	bufmerge(scratch, 256.0, scratch, 1.0, samplecount);
 
 	/*
@@ -526,7 +560,9 @@ printf("initialising one bristolmini\n");
 	if (scratch == (float *) NULL)
 		scratch = (float *) bristolmalloc0(audiomain->segmentsize);
 
-	baudio->mixlocals = bristolmalloc0(sizeof(float));
+//	baudio->mixlocals = bristolmalloc0(sizeof(float));
+	baudio->mixlocals = (float *) bristolmalloc0(sizeof(mMods));
+
 	return(0);
 }
 
