@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -287,6 +286,12 @@ odysseyController(Baudio *baudio, u_char operator, u_char controller, float valu
 			else
 				((odysseymods *) baudio->mixlocals)->arAuto = 1;
 			break;
+		case 60:
+			if (ivalue)
+				baudio->mixflags &= ~A_VCO_DUAL;
+			else
+				baudio->mixflags |= A_VCO_DUAL;
+			break;
 	}
 	return(0);
 }
@@ -297,6 +302,59 @@ bristolVoice *voice, register float *startbuf)
 {
 	odysseymods *mods = (odysseymods *) baudio->mixlocals;
 	int sc = audiomain->samplecount;
+	int i;
+
+	/*
+	 * The following could be optimised by checking for 'Dual', then checking
+	 * for preference, then making a single search.
+	 */
+	if ((baudio->mixflags & A_VCO_DUAL) && (baudio->voicecount == 1))
+	{
+		if (baudio->notemap.flags & BRISTOL_MNL_HNP)
+		{
+			/*
+			 * Find the lowest note, the library gave us the highest
+			 */
+			mods->low = voice->key.key;
+			for (i = 0; i < 128; i++)
+			{
+				if ((baudio->notemap.key[i] != 0) && (i < mods->low))
+				{
+					mods->low = i;
+					break;
+				}
+			}
+
+			mods->dFreq = baudio->ctab[mods->low].step;
+			mods->dfreq = baudio->ctab[mods->low].freq;
+
+			if ((mods->low < mods->last)
+				&& (baudio->notemap.flags & BRISTOL_MNL_TRIG))
+				voice->flags |= BRISTOL_KEYREON;
+
+			mods->last = mods->low;
+		} else {
+			/* Find the highest note */
+			mods->high = voice->key.key;
+			for (i = 127; i > 0; i--)
+			{
+				if ((baudio->notemap.key[i] != 0) && (i < mods->high))
+				{
+					mods->high = i;
+					break;
+				}
+			}
+
+			mods->dFreq = baudio->ctab[mods->high].step;
+			mods->dfreq = baudio->ctab[mods->high].freq;
+
+			if ((mods->high > mods->last)
+				&& (baudio->notemap.flags & BRISTOL_MNL_TRIG))
+				voice->flags |= BRISTOL_KEYREON;
+
+			mods->last = mods->high;
+		}
+	}
 
 	if (((odysseymods *) baudio->mixlocals)->lfolocals == 0)
 	{
@@ -479,8 +537,30 @@ bristolVoice *voice, register float *startbuf)
 	 * not then the glide information gets damaged.
 	 */
 	fillFreqTable(baudio, voice, fmbuf, sc, 1);
-	//fillFreqTable(baudio, voice, freqbuf, sc, 1);
-	bufmerge(fmbuf, 1.0, freqbuf, 0.0, sc);
+
+	/*
+	 * If we are dual then put a different freq into the second osc. We will 
+	 * have to avoid issues of glide for now, find dfreq for this low key and
+	 * then apply that to freqbuf.
+	 */
+	if ((baudio->mixflags & A_VCO_DUAL) && (baudio->voicecount == 1))
+	{
+		float *buf = freqbuf;
+		int i;
+
+		for (i = 0; i < sc; i+=8)
+		{
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+			*buf++ = mods->dFreq;
+		}
+	} else
+		bufmerge(fmbuf, 1.0, freqbuf, 0.0, sc);
 
 	/*
 	 * Put in any desired mods. There are two, MG and Wheel mod. Wheel mod is
@@ -506,12 +586,9 @@ bristolVoice *voice, register float *startbuf)
 		voice->locals[voice->index][0]);
 
 	audiomain->palette[(*baudio->sound[1]).index]->specs->io[0].buf = freqbuf;
-/*	audiomain->palette[(*baudio->sound[1]).index]->specs->io[1].buf = scratch; */
 	audiomain->palette[(*baudio->sound[1]).index]->specs->io[2].buf = osc2ramp;
 	audiomain->palette[(*baudio->sound[1]).index]->specs->io[3].buf = osc1ramp;
 	audiomain->palette[(*baudio->sound[1]).index]->specs->io[4].buf = osc2puls;
-/*	audiomain->palette[(*baudio->sound[1]).index]->specs->io[5].buf = NULL; */
-/*	audiomain->palette[(*baudio->sound[1]).index]->specs->io[6].buf = NULL; */
 
 	if (baudio->mixflags & A_LFO_TRILL)
 		bufmerge(lfosine, 0.2, freqbuf, 1.0, sc);
@@ -582,7 +659,7 @@ bristolVoice *voice, register float *startbuf)
 /* HP Filter DONE */
 
 	// This bypasses the HPF that has been giving me noise
-	bufmerge(filtbuf, 1.0, baudio->rightbuf, 0.0, sc);
+	//bufmerge(filtbuf, 1.0, baudio->rightbuf, 0.0, sc);
 
 /* AMP */
 	bufmerge(shmix, mods->mod[8].gain, baudio->rightbuf, 1.0, sc);
