@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,13 +15,13 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#define A_LFO_MULTI 0x0001
-#define A_LFO_TRILL 0x0002
+#define A_LFO_MULTI	0x0001
+#define A_LFO_TRILL	0x0002
+#define A_VCO_DUAL	0x0004
 
 #define O_OSC1_M1 0
 
@@ -56,5 +56,10 @@ typedef struct odysseyMods {
 	float a_gain;
 	odysseymod mod[20];
 	float shkbd;
+	int low;
+	int high;
+	float dfreq;
+	float dFreq;
+	int last;
 } odysseymods;
 
