@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 #define DEBUG
@@ -530,7 +529,7 @@ bristolVoice *voice, register float *startbuf)
 	if (((jmods *) baudio->mixlocals)->envtype)
 	{
 		register int i, samplecount = audiomain->samplecount;
-		register float *buf = adsrbuf, value, diff;
+		register float *buf = adsrbuf, value, diff = 0;
 
 		value = voice->velocity * 12 * ((jmods *) baudio->mixlocals)->level;
 
@@ -591,6 +590,7 @@ int
 static bristolSamplerDestroy(audioMain *audiomain, Baudio *baudio)
 {
 printf("removing one sampler\n");
+	return(0);
 	bristolfree(freqbuf);
 	bristolfree(sbuf);
 	bristolfree(pmodbuf);
@@ -643,13 +643,20 @@ printf("initialising one sampler\n");
 	 */
 	initSoundAlgo(12, 0, baudio, audiomain, baudio->effect);
 
-	freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	sbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	pmodbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (freqbuf == NULL)
+		freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (sbuf == NULL)
+		sbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (pmodbuf == NULL)
+		pmodbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (adsrbuf == NULL)
+		adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (filtbuf == NULL)
+		filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (oscbbuf == NULL)
+		oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	if (oscabuf == NULL)
+		oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
 
 	baudio->mixlocals = (float *) bristolmalloc0(sizeof(jmods));
 	return(0);
