@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -915,118 +914,6 @@ sid2Poly1NoteLogic(Baudio *baudio, sid2mods *smods)
 	}
 }
 
-/*
- * Assign highest two notes to whatever voices are avilable, the lowest three
- * notes (that do not conflict) to the arpeggiator. Only arpeggiate if there
- * are three notes available.
- */
-static void
-sid2Poly_3NoteLogic(Baudio *baudio, sid2mods *smods)
-{
-	int i, ac = 0;
-	//int occ = 0;
-
-	bzero(smods->arpegtable, B_SID2_ARPEG_MAX * sizeof(unsigned short));
-
-	/*
-	 * This will be poly. We cannot use the KEYON flags since we are in 
-	 * mono mode but probably also want to actually see other notes being
-	 * pressed, our monophonic note logic supresses events for notes outsid2e
-	 * of the preference however they are still kept in the note mapping
-	 * table, we need to look at that.
-	 *
-	 * The matching is pretty trivial seeing as we only have three voices
-	 * to track so it is not a great overhead: for each key see if it was 
-	 * already known. If not, apply the 'middle' voice to it.
-	 */
-	for (i = 1; i < 128; i++)
-	{
-		/*
-		 * There are 3 cases:
-		 *
-		 * 	a. We do not know this key:
-		 * 		Assign to Voice-1 if lower than current
-		 * 		Assign to Voice-3 if high than current
-		 * 		Otherwise add it to arpeggiation list
-		 * 	b. We do know this key, its assigned - do nothing.
-		 * 	c. We do know this key, its gone:
-		 * 		Free voice if it was Voice-1 or Voice-3
-		 * 		Remove from arpeggiation list
-		 */
-		if (baudio->notemap.key[i] > 0)
-		{
-			if (smods->localmap[0][i] != i)
-			{
-				/* Not a known note - new , Check V-1 and V3 */
-				if ((smods->ckey[B_SID_VOICE_1] == 0)
-					|| (smods->ckey[B_SID_VOICE_1] > i))
-				{
-					smods->ckey[B_SID_VOICE_1] = i;
-					smods->lvel = smods->velocity[B_SID_VOICE_1]
-						= baudio->notemap.velocity[i] / 127.0;
-					sid2AssignVoice(smods, B_SID_VOICE_1, B_SID2_TRIG);
-					smods->ccount++;
-					smods->localmap[0][i] = i;
-					//occ++;
-				} else if ((smods->ckey[B_SID_VOICE_3] == 0)
-					|| (smods->ckey[B_SID_VOICE_3] < i))
-				{
-					smods->ckey[B_SID_VOICE_3] = i;
-					smods->lvel = smods->velocity[B_SID_VOICE_3]
-							= baudio->notemap.velocity[i] / 127.0;
-					sid2AssignVoice(smods, B_SID_VOICE_3, B_SID2_TRIG);
-					smods->ccount++;
-					smods->localmap[0][i] = i;
-					//occ++;
-				}
-			}
-
-			if ((smods->ckey[B_SID_VOICE_1] != i)
-				&& (smods->ckey[B_SID_VOICE_3] != i)
-				&& (ac < B_SID2_ARPEG_MAX))
-			{
-				//occ++;
-				smods->arpegtable[ac++] = i;
-			}
-
-			smods->localmap[0][i] = i;
-		} else if ((baudio->notemap.key[i] <= 0)
-				&& (smods->localmap[0][i] == i))
-		{
-
-			/* Was a known key? note off if allocated */
-			if (smods->ckey[B_SID_VOICE_1] == i)
-			{
-				sid2AssignVoice(smods, B_SID_VOICE_1, B_SID2_REL);
-				smods->ckey[B_SID_VOICE_1] = 0;
-				smods->ccount--;
-				//occ++;
-			} else if (smods->ckey[B_SID_VOICE_3] == i) {
-				sid2AssignVoice(smods, B_SID_VOICE_3, B_SID2_REL);
-				smods->ckey[B_SID_VOICE_3] = 0;
-				smods->ccount--;
-				//occ++;
-			}
-
-			smods->localmap[0][i] = 0;
-
-			if (smods->ccount < 0)
-				smods->ccount = 0;
-		}
-	}
-
-/*
-	if (occ)
-	{
-		printf("low %i, high %i\narpeg:",
-			smods->ckey[B_SID_VOICE_1], smods->ckey[B_SID_VOICE_3]);
-		for (ac = 0; smods->arpegtable[ac] != 0; ac++)
-			printf(" %i,", smods->arpegtable[ac]);
-		printf("\n");
-	}
-*/
-}
-
 /*
  * Take a fixed split point, below this point stuff the arpeggiation table, 
  * above this point use duophonic note preference - we should consid2er always
