@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -745,26 +744,18 @@ destroyOneSonic6Voice(audioMain *audiomain, Baudio *baudio)
 {
 	printf("destroying Sonic-6\n");
 
-	/*
-	 * We need to leave these, we may have multiple invocations running
-	 */
-	return(0);
-
-
-	bristolfree(SONICLOCAL->freqbuf);
+	bristolfree(SONICLOCAL->oscabuf);
+	bristolfree(SONICLOCAL->oscbbuf);
 	bristolfree(SONICLOCAL->noisebuf);
-
+	bristolfree(SONICLOCAL->freqbuf);
+	bristolfree(SONICLOCAL->filtbuf);
+	bristolfree(SONICLOCAL->rmbuf);
 	bristolfree(SONICLOCAL->lfoxbuf);
 	bristolfree(SONICLOCAL->lfoybuf);
 	bristolfree(SONICLOCAL->lfoxybuf);
-
-	bristolfree(SONICLOCAL->oscabuf);
-	bristolfree(SONICLOCAL->oscbbuf);
 	bristolfree(SONICLOCAL->adsrbuf);
-	bristolfree(SONICLOCAL->zerobuf);
 	bristolfree(SONICLOCAL->outbuf);
-	bristolfree(SONICLOCAL->filtbuf);
-	bristolfree(SONICLOCAL->rmbuf);
+	bristolfree(SONICLOCAL->zerobuf);
 	bristolfree(SONICLOCAL->scratch);
 
 	return(0);
