@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,9 +28,13 @@
 
 extern void inthandler();
 extern void initBristolAudio();
+#ifdef _BRISTOL_JACK
 extern int bristolJackInterface();
+#endif
 extern void allNotesOff();
 
+static int initCount = 0;
+
 int
 bristolgetsid(Baudio *baudio, int sid)
 {
@@ -55,14 +58,16 @@ bristolgetsid(Baudio *baudio, int sid)
 int
 bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 {
-	int result = 0;
+	int result = 0, hclose = -1;
 
 	/*
-	printf("bristolSystem(%i, %i) %x\n",
-		msg->params.bristol.operator,
-		msg->params.bristol.controller,
-		(msg->params.bristol.valueMSB << 7) + msg->params.bristol.valueLSB);
 	*/
+	if (audiomain->debuglevel > 1)
+		printf("bristolSystem(%i, %i, %i) %x\n",
+			msg->params.bristol.operator,
+			msg->params.bristol.controller,
+			msg->params.bristol.from,
+			(msg->params.bristol.valueMSB << 7) + msg->params.bristol.valueLSB);
 
 	/*
 	 * We are looking for a "hello" message from the GUI, which should include
@@ -85,8 +90,19 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 		 */
 		if (flags & BRISTOL_HELLO)
 		{
+			/*
+			 * When we get a hello we need to suppress midi processing for a
+			 * period of time whilst the new emulator is activated. It will be
+			 * reactivated when the algorithm has initted later.
+			 */
+			bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 0);
+
 			Baudio *baudio = (Baudio *) bristolmalloc0(sizeof(Baudio));
 
+			baudio->mixflags = (BRISTOL_HOLDDOWN|BRISTOL_SENSE);
+			/* Turn on sensing with a long timer, GUI can disable it later */
+			baudio->sensecount = 15 * audiomain->samplerate;
+
 			if (audiomain->atStatus == BRISTOL_EXIT)
 				audiomain->midiflags |= BRISTOL_HELLO;
 
@@ -97,7 +113,6 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 			 * to the source.
 			 */
 			baudio->midichannel = msg->params.bristol.channel;
-			baudio->mixflags = BRISTOL_HOLDDOWN;
 			baudio->samplerate = audiomain->samplerate;
 			baudio->samplecount = audiomain->samplecount;
 
@@ -110,6 +125,8 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 
 			if (baudio->sid == -1)
 				printf("SID failure....\n");
+			else if (audiomain->debuglevel > 2)
+				printf("	alloc SID %i\n", baudio->sid);
 
 			baudio->voicecount = (BRISTOL_PARAMMASK & flags);
 			if (audiomain->voiceCount <= 0)
@@ -123,8 +140,9 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 			if (baudio->voicecount > BRISTOL_MAXVOICECOUNT)
 				baudio->voicecount = BRISTOL_MAXVOICECOUNT;
 
-			printf("created %i voices: allocated %i to synth\n",
-				audiomain->voiceCount, baudio->voicecount);
+			if (audiomain->debuglevel)
+				printf("created %i voices: allocated %i to synth\n",
+					audiomain->voiceCount, baudio->voicecount);
 
 			if (audiomain->voiceCount <= 1)
 				audiomain->voiceCount = 1;
@@ -135,10 +153,19 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 			if (audiomain->audiolist != NULL)
 				audiomain->audiolist->last = baudio;
 			audiomain->audiolist = baudio;
+#ifndef BRISTOL_SEMAPHORE
+			jack_ringbuffer_go(audiomain->rb);
+#endif
 		}
 
 		if ((audiomain->midiflags & BRISTOL_HELLO) == 0)
+		{
+			printf("no hello, returning\n");
 			return(0);
+		}
+
+		if (audiomain->debuglevel > 2)
+			printf("commask %x\n", flags & BRISTOL_COMMASK);
 
 		/*
 		 * See what command we have, if any.
@@ -194,7 +221,7 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				if (baudio != NULL)
 				{
 					baudio->voicecount = (BRISTOL_PARAMMASK & flags);
-					if (audiomain->debuglevel >= 0)
+					if (audiomain->debuglevel)
 						printf("configured %i channel voices\n",
 							baudio->voicecount);
 				}
@@ -208,7 +235,11 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				if (audiomain->atStatus != BRISTOL_OK)
 					break;
 
+				if (audiomain->debuglevel)
+					printf("init request nr %i\n", initCount++);
+
 				baudio = audiomain->audiolist;
+				allNotesOff(audiomain, msg->params.bristol.channel);
 
 				while (baudio != NULL)
 				{
@@ -222,6 +253,7 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				audiomain->midiflags |= (BRISTOL_PARAMMASK & flags);
 
 				initBristolAudio(audiomain, baudio);
+				//bristolMidiOption(0, BRISTOL_NRP_MIDI_STOP, 0);
 
 				break;
 			}
@@ -233,6 +265,8 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				if ((baudio == NULL) || (baudio->mixflags == 0))
 					return(0);
 
+				bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 0);
+
 				/*
 				printf("exit algo: %i, %x, %x\n",
 					msg->params.bristol.channel,
@@ -288,12 +322,31 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 					if (audiomain->flags & BRISTOL_JACK)
 					{
 						audiomain->atReq = BRISTOL_REQSTOP;
+#ifdef BRISTOL_JACK_MULTI_CLOSE
+#ifdef _BRISTOL_JACK // The bristol idle thread should do this.
 						bristolJackInterface(NULL);
+#endif
+#endif
 					}
 
+					bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, 1);
+					audiomain->atStatus = BRISTOL_TERM;
+
 					inthandler();
-				} else
-					sleep(1);
+
+					return(0);
+				} else {
+					int i = 20;
+
+					while ((baudio->mixflags & BRISTOL_REMOVE) && (--i > 0))
+						usleep(100000);
+				}
+
+				hclose = msg->params.bristol.from;
+
+				bristolfree(baudio);
+
+				bristolMidiOption(0, BRISTOL_NRP_MIDI_GO, 1);
 
 				break;
 			}
@@ -333,9 +386,8 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 					break;
 
 				if (BRISTOL_PARAMMASK & flags)
-				{
 					baudio->mixflags |= BRISTOL_KEYHOLD;
-				} else {
+				else {
 					baudio->mixflags &= ~BRISTOL_KEYHOLD;
 					allNotesOff(audiomain, baudio->midichannel);
 				}
@@ -363,7 +415,6 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 					break;
 
 				baudio->highkey = (BRISTOL_PARAMMASK & flags);
-
 				break;
 			}
 			case BRISTOL_UNISON:
@@ -378,10 +429,8 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 				break;
 			}
 			case BRISTOL_ALL_NOTES_OFF:
-			{
 				allNotesOff(audiomain, -1);
 				break;
-			}
 			case BRISTOL_MIDICHANNEL:
 			{
 				Baudio *baudio;
@@ -406,18 +455,40 @@ bristolSystem(audioMain *audiomain, bristolMidiMsg *msg)
 
 				break;
 			}
+			case BRISTOL_REQ_FORWARD:
+				bristolMidiOption(msg->params.bristol.from,
+					BRISTOL_NRP_REQ_FORWARD, BRISTOL_PARAMMASK & flags);
+				break;
+			case BRISTOL_REQ_DEBUG:
+				//printf("SETUP DEBUG LEVELS: %i\n", BRISTOL_PARAMMASK & flags);
+				audiomain->debuglevel = BRISTOL_PARAMMASK & flags;
+				if (audiomain->debuglevel >= 2)
+					bristolMidiOption(0, BRISTOL_NRP_DEBUG,
+						audiomain->debuglevel - 2);
+				break;
 			default:
 				break;
 		}
 
+		if (audiomain->debuglevel > 2)
+			printf("cont 2: %i: res %i\n", msg->params.bristol.from, result);
+
 		/*
 		 * return a message, with the controlid assumably at the head
 		 * of the playlist - we have just created it.
-		 */
 		msg->params.bristol.valueLSB = result;
-		bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, result);;
+		bristolMidiPrint(msg);
+		 */
+		bristolMidiSendMsg(msg->params.bristol.from, 0, 127, 0, result);
+
 /*			audiomain->audiolist->controlid); */
+		if (hclose > 0)
+		{
+			usleep(100000);
+			bristolMidiClose(hclose);
+		}
 	}
+
 	return(0);
 }
 
