@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -83,10 +82,10 @@ u_char operator, float value)
 			TRILOGYLCL->spacestring = value;
 			break;
 		case 8:
-			TRILOGYLCL->mixstring = value * 0.2;
+			TRILOGYLCL->mixstring = value * 2.8f;
 			break;
 		case 9:
-			TRILOGYLCL->mixsynth = value * 0.25;
+			TRILOGYLCL->mixsynth = value * 1.5f;
 			break;
 		case 10:
 			TRILOGYLCL->env2filter = (value - 0.5) * 4;
@@ -958,9 +957,7 @@ destroyOneTrilogyVoice(audioMain *audiomain, Baudio *baudio)
 	printf("destroying trilogy synth\n");
 
 	bristolfree(TRILOGYLCL->freqbuf);
-
 	bristolfree(TRILOGYLCL->lfobuf);
-
 	bristolfree(TRILOGYLCL->oscabuf);
 	bristolfree(TRILOGYLCL->oscbbuf);
 	bristolfree(TRILOGYLCL->adsrbuf);
@@ -1033,28 +1030,17 @@ bristolTrilogyODCInit(audioMain *audiomain, Baudio *baudio)
 	initSoundAlgo(22, 0, baudio, audiomain, baudio->effect);
 	 */
 
-	if (TRILOGYLCL->oscabuf == (float *) NULL)
-		TRILOGYLCL->oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->oscbbuf == (float *) NULL)
-		TRILOGYLCL->oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->freqbuf == (float *) NULL)
-		TRILOGYLCL->freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->filtbuf == (float *) NULL)
-		TRILOGYLCL->filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->lfobuf == (float *) NULL)
-		TRILOGYLCL->lfobuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->adsrbuf == (float *) NULL)
-		TRILOGYLCL->adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->outbuf1 == (float *) NULL)
-		TRILOGYLCL->outbuf1 = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->outbuf2 == (float *) NULL)
-		TRILOGYLCL->outbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->zerobuf == (float *) NULL)
-		TRILOGYLCL->zerobuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->scratch == (float *) NULL)
-		TRILOGYLCL->scratch = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->syncbuf == (float *) NULL)
-		TRILOGYLCL->syncbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->lfobuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->outbuf1 = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->outbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->zerobuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->scratch = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->syncbuf = (float *) bristolmalloc0(audiomain->segmentsize);
 
 	baudio->mixflags |= BRISTOL_STEREO;
 
@@ -1123,28 +1109,17 @@ bristolTrilogyInit(audioMain *audiomain, Baudio *baudio)
 	TRILOGYLCL->mixstring = 1.0;
 	TRILOGYLCL->panstring = 0.5;
 
-	if (TRILOGYLCL->oscabuf == (float *) NULL)
-		TRILOGYLCL->oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->oscbbuf == (float *) NULL)
-		TRILOGYLCL->oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->freqbuf == (float *) NULL)
-		TRILOGYLCL->freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->filtbuf == (float *) NULL)
-		TRILOGYLCL->filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->lfobuf == (float *) NULL)
-		TRILOGYLCL->lfobuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->adsrbuf == (float *) NULL)
-		TRILOGYLCL->adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->outbuf1 == (float *) NULL)
-		TRILOGYLCL->outbuf1 = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->outbuf2 == (float *) NULL)
-		TRILOGYLCL->outbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->zerobuf == (float *) NULL)
-		TRILOGYLCL->zerobuf = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->scratch == (float *) NULL)
-		TRILOGYLCL->scratch = (float *) bristolmalloc0(audiomain->segmentsize);
-	if (TRILOGYLCL->syncbuf == (float *) NULL)
-		TRILOGYLCL->syncbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->oscabuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->oscbbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->freqbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->filtbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->lfobuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->adsrbuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->outbuf1 = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->outbuf2 = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->zerobuf = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->scratch = (float *) bristolmalloc0(audiomain->segmentsize);
+	TRILOGYLCL->syncbuf = (float *) bristolmalloc0(audiomain->segmentsize);
 
 	baudio->mixflags |= BRISTOL_STEREO;
 
