@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -90,6 +89,7 @@ baudio->effect[0]->param->param[5].float_val);
 }
 
 int vibraDone = 0;
+
 /*
  * This will only do the envelopes for the Dual manual. It will cover the
  * bass sustain envelope and the percussive envelope, both of which are global
@@ -183,6 +183,9 @@ bristolVoice *voice, register float *startbuf)
 	 */
 	fillFreqTable(baudio, voice, tmpbuf1, samplecount, 0);
 
+//	if (voice->locals[voice->index][0] == NULL);
+//		return(0);;
+
 	if (voice->key.key >= 36)
 	{
 		/*
@@ -238,7 +241,7 @@ bristolVoice *voice, register float *startbuf)
 			bufmerge(vbuf, 1.0, outbuf, 1.0, audiomain->samplecount);
 	} else {
 		/*
-		 * Run the bass oscillator. We should consider changing this to just
+		 * Ryn yhe bass oscillator. We should consider changing this to just
 		 * run in the postops, picking up the first available voice below midi
 		 * note 48, that way it would be monophonic and we could apply the
 		 * bass envelope quite easily.
@@ -461,10 +464,12 @@ bristolVoice *voice, register float *startbuf)
 int
 destroyOneVoxVoice(audioMain *audiomain, Baudio *baudio)
 {
+	printf("removing one vox\n");
 
 	/*
 	 * We need to leave these, we may have multiple invocations running
 	 */
+	baudio->mixlocals = NULL;
 	return(0);
 
 	if (tmpbuf1 != NULL)
@@ -489,8 +494,6 @@ destroyOneVoxVoice(audioMain *audiomain, Baudio *baudio)
 		bristolfree(bassbuf);
 	bassbuf = NULL;
 
-	baudio->mixlocals = NULL;
-
 	return(0);
 }
 
