@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -98,6 +97,9 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("dcoreset(%x)\n", operator);
 #endif
 
+	if (param->param[0].mem != NULL)
+		bristolfree(param->param[0].mem);
+	param->param[0].mem = bristolmalloc0(sizeof(float) * DCO_WAVE_SZE);
 	param->param[0].int_val = 0;
 	param->param[1].float_val = 0.5;
 	param->param[2].float_val = 1.0;
@@ -226,12 +228,53 @@ static int operate(bristolOP *operator,
 	count = specs->spec.io[DCO_OUT_IND].samplecount;
 	ib = specs->spec.io[DCO_IN_IND].buf;
 	ob = specs->spec.io[DCO_OUT_IND].buf;
-	wt = specs->wave[param->param[0].int_val];
 	wtp = local->wtp;
 	gain = param->param[3].float_val;
 	transp = param->param[1].float_val * param->param[2].float_val;
 	wtp = local->wtp;
 
+ 	if (bristolBLOcheck(voice->cFreq*transp)) {
+		wt = param->param[0].mem;
+		memset(wt, 0, DCO_WAVE_SZE * sizeof(float));
+		switch (param->param[0].int_val) {
+			case 0:
+				/* Sine, cannot BWL this */
+				wt = specs->wave[param->param[0].int_val];
+				break;
+			case 1:
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_SQUARE);
+				break;
+			case 2:
+				/* This will be pulse */
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_PULSE);
+				break;
+			case 3:
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_SAW);
+				break;
+			case 4:
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_TRI);
+				break;
+			case 5:
+			{
+				float ws[1024];
+				memset(ws, 0, DCO_WAVE_SZE * sizeof(float));
+				generateBLOwaveformF(voice->cFreq*transp, &ws[0], BLO_SAW);
+
+				for (obp = 0; obp < count; obp++)
+					wt[obp] = (ws[obp] + ws[(obp * 2) & 1023]) * 0.5;
+
+				break;
+			}
+			case 6:
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_RAMP);
+				break;
+			case 7:
+				generateBLOwaveformF(voice->cFreq*transp, wt, BLO_RAMP);
+				break;
+		}
+	} else
+		wt = specs->wave[param->param[0].int_val];
+
 /*printf("%i, %f, %i: %x %x %x\n", count, gain, param->param[0].int_val, wt, ib, ob); */
 	/*
 	 * Go jumping through the wavetable, with each jump defined by the value
@@ -413,7 +456,7 @@ fillWave(float *mem, int count, int type)
 			/* 
 			 * This is a square wave.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -434,6 +477,12 @@ fillWave(float *mem, int count, int type)
 			 * build this out of a subtracted offset ramp wave if BLO is 
 			 * enabled.
 			 */
+			if (blo.flags & BRISTOL_BLO)
+			{
+				for (i = 0;i < count; i++)
+					mem[i] = blopulse[i];
+				return;
+			}
 			for (i = 0;i < count / 5; i++)
 				mem[i] = BRISTOL_VPO * 2 / 3;
 			for (;i < count; i++)
@@ -449,7 +498,7 @@ fillWave(float *mem, int count, int type)
 			 * multiply by the range. We go from rear to front to table to make
 			 * the ramp wave have a positive leading edge.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = bloramp[i];
@@ -466,7 +515,7 @@ fillWave(float *mem, int count, int type)
 			 * Triangular wave. From MIN point, ramp up at twice the rate of
 			 * the ramp wave, then ramp down at same rate.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blotriangle[i];
@@ -485,19 +534,11 @@ fillWave(float *mem, int count, int type)
 			 * interesting strings sounds. If we have BLO enabled then we are
 			 * just going to layer the wave with the second harmonic of itself.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
-				int k = 0;
+				for (i = 0; i < count; i++)
+					mem[i] = (blosaw[i] + blosaw[(i * 2) & 1023]) * 0.5;
 
-				for (i = 0;i < count; i++)
-					mem[i] = blotriangle[i];
-
-				for (i = 0;i < count; i++)
-				{
-					mem[i] += blotriangle[k];
-					if ((k += 2) >= BRISTOL_BLO_SIZE)
-						k -= BRISTOL_BLO_SIZE;
-				}
 				return;
 			}
 			{
