@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -102,7 +101,7 @@ static int param(bristolOP *operator, bristolOPParams *param,
 		offset = 0;
 
 #ifdef BRISTOL_DBG
-	printf("param(%i, %f)\n", index, value);
+	printf("env param(%i, %f)\n", index, value);
 #endif
 
 	switch (index) {
@@ -245,6 +244,13 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	cgain = local->cgain;
 	ob = specs->spec.io[ENV_OUT_IND].buf;
 
+	/*
+	 * The offsets are only provided once so if we have it then it's function is
+	 * actually similar to note_off. Anyway.
+	if ((voice->offset >= 0) && (voice->flags & BRISTOL_KEYOFF))
+		printf("%i: env off offset %i\n", voice->key.key, voice->offset);
+	 */
+
 	if (voice->flags & BRISTOL_KEYOFF)
 	{
 		voice->flags |= BRISTOL_KEYOFFING;
@@ -264,14 +270,52 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	if (voice->flags & BRISTOL_KEYON)
 	{
 		/* There is some issue here */
-		if ((voice->baudio->voicecount > 1) || (param->param[6].float_val != 0))
+		if ((voice->baudio->voicecount > 1) || (param->param[6].float_val != 0)
+			|| (cgain == 0.0f))
 			cgain = 1.0;
+
 		local->cstate = STATE_ATTACK;
+		/*
+		 * This is for Jack Sample Accurate support. It is a compromise as I am
+		 * not a great fan of the feature: the envelope will delay its attack
+		 * event by the number of frames indicated in the voice offset, that
+		 * offset is taken from the Jack event (it is zero for all other bristol
+		 * MIDI interface drivers).
+		 *
+		 * State changes will not be sample accurate in the first version - they
+		 * are more work as the current operation has to continue until the time
+		 * of the event. Note off is not such a big deal as having a bit of 
+		 * decay will mask out the exact point of the note. One consideration is
+		 * application to REON event but a REON is going to be masked by the
+		 * current output anyway. Since all these events are related to Jack
+		 * then NOTE_OFF and REON are still accurate to the period, ie, they 
+		 * will behave consistently across different plays and when mastering
+		 * at max speed.
+		 *
+		 * Features other than envelope sync (and the existing LFO sync) are 
+		 * not likely to get considered - this is an analogue emulator and I
+		 * don't really want things like general oscillator sync.
+		 */
+		if ((~voice->flags & BRISTOL_KEYREON) &&
+			(voice->offset > 0) && (voice->offset < count))
+		{
+			if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
+				printf("envelope trigger offset %i frames\n", voice->offset);
+
+			memset(ob, 0.0f, voice->offset * sizeof(float));
+			ob += voice->offset;
+			count -= voice->offset;
+		}
 	} else if (voice->flags & BRISTOL_KEYREON)
 		local->cstate = STATE_ATTACK;
 
 	if (voice->flags & (BRISTOL_KEYON|BRISTOL_KEYREON))
 	{
+		/*
+		 * This accepts note on offsets from Jack. Note off offsets will be
+		 * based on period size for now.
+		printf("%i: env on  offset %i\n", voice->key.key, voice->offset);
+		 */
 		/* Rezero gain */
 		if (param->param[6].float_val != 0)
 			cgain = 1.0;
@@ -431,8 +475,10 @@ printf("unknown state: %i, %i\n", voice->key.key, local->cstate);
 				break;
 		}
 	}
+
 	local->cgain = cgain;
 	local->egain = egain;
+
 	return(0);
 }
 
