@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -104,6 +103,17 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("dcoreset(%x)\n", operator);
 #endif
 
+	/* The use of 1, 3 and 4 may make things clear later */
+	if (param->param[1].mem != NULL)
+		bristolfree(param->param[1].mem);
+	param->param[1].mem = bristolmalloc0(sizeof(float) * EXPDCO_WAVE_SZE);
+	if (param->param[3].mem != NULL)
+		bristolfree(param->param[3].mem);
+	param->param[3].mem = bristolmalloc0(sizeof(float) * EXPDCO_WAVE_SZE);
+	if (param->param[4].mem != NULL)
+		bristolfree(param->param[4].mem);
+	param->param[4].mem = bristolmalloc0(sizeof(float) * EXPDCO_WAVE_SZE);
+
 	param->param[0].float_val = 0;
 	param->param[1].float_val = 0.5;
 	param->param[2].float_val = 1.0;
@@ -282,6 +292,26 @@ static int operate(bristolOP *operator,
 	if ((sob = specs->spec.io[DCO_SYNC_OUT].buf) == NULL)
 		sob = specs->null;
 
+ 	if (bristolBLOcheck(voice->dFreq*transp)) {
+		memset(param->param[1].mem, 0, EXPDCO_WAVE_SZE * sizeof(float));
+		memset(param->param[3].mem, 0, EXPDCO_WAVE_SZE * sizeof(float));
+		memset(param->param[4].mem, 0, EXPDCO_WAVE_SZE * sizeof(float));
+
+		generateBLOwaveformF(voice->cFreq*transp,
+			param->param[1].mem, BLO_SQUARE);
+		generateBLOwaveformF(voice->cFreq*transp,
+			param->param[3].mem, BLO_RAMP);
+		generateBLOwaveformF(voice->cFreq*transp,
+			param->param[4].mem, BLO_TRI);
+	} else {
+		bcopy(specs->wave[1], param->param[1].mem,
+			EXPDCO_WAVE_SZE * sizeof(float));
+		bcopy(specs->wave[3], param->param[3].mem,
+			EXPDCO_WAVE_SZE * sizeof(float));
+		bcopy(specs->wave[4], param->param[4].mem,
+			EXPDCO_WAVE_SZE * sizeof(float));
+	}
+
 	/*
 	 * Go jumping through the wavetable, with each jump defined by the value
 	 * given on our input line, making sure we fill one output buffer.
@@ -323,8 +353,8 @@ static int operate(bristolOP *operator,
 			/*
 			 * Crossfade tri into ramp
 			 */
-			wt1 = specs->wave[4]; /* Triangular */
-			wt2 = specs->wave[3]; /* Ramp */
+			wt1 = param->param[4].mem; /* Triangular */
+			wt2 = param->param[3].mem; /* Ramp */
 			gain2 = wform * 3;
 			gain1 = (1.0 - gain2) * gain;
 			gain2 *= gain;
@@ -347,8 +377,8 @@ static int operate(bristolOP *operator,
 			 * Crossrade ramp into square, however I want the square to be
 			 * a difference of two ramps.....
 			 */
-			wt1 = specs->wave[3]; /* Triangular */
-			wt2 = specs->wave[1]; /* Square */
+			wt1 = param->param[3].mem; /* Triangular */
+			wt2 = param->param[1].mem; /* Square */
 			gain2 = (wform - 0.33) * 3;
 			gain1 = (1.0 - gain2) * gain;
 			gain2 *= ssg;
@@ -626,7 +656,7 @@ fillWave(float *mem, int count, int type)
 			 * This is a square wave although it should not be in use (we
 			 * generate PW).
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -662,7 +692,7 @@ fillWave(float *mem, int count, int type)
 			mem[0] = 0;
 			mem[1] = mem[count - 1] = mem[1] / 2;
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = bloramp[i];
@@ -677,7 +707,7 @@ fillWave(float *mem, int count, int type)
 			 * Triangular wave. From MIN point, ramp up at twice the rate of
 			 * the ramp wave, then ramp down at same rate.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blotriangle[i];
