@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -64,10 +63,49 @@
 
 static float srate;
 
-#define TANHF tanhf
+#define getcoff(c, k) ((c*c)*(1.0f-k)*20000 + k*4*voice->cFreq) / srate
+
+//#define TANHF(x) btanhf(voice->baudio->mixflags, x)
+#define TANHF(x) btanhf(mode, x)
+//#define TANHF(v) (v + 1e-10f) / sqrtf(1 + v * v)
+//#define TANHF(x) mode == 0? tanhf(x):0
+#define TANHFEED(x) btanhfeed(mode, x)
+//#define TANHFEED tanhf
+//#define TANHFEED(v) (v + 1e-10f) / sqrtf(1 + v * v)
+
 #define V2 40000.0
 #define OV2 0.000025
 
+static float
+btanhf(int mode, float v)
+{
+	/*
+	 * This should be 4 modes as we have two bit flags. Mode 1 are the real
+	 * lightweight chamberlain. That is more work on flag checking.
+	 */
+	switch (mode) {
+		case 0: return(v);
+		default:
+		case 2: return((v + 1e-10f) / sqrtf(1 + v * v));
+		case 3: return(tanhf(v));
+	}
+}
+
+static float
+btanhfeed(int mode, float v)
+{
+	/*
+	 * This should be 4 modes as we have two bit flags. Mode 1 are the real
+	 * lightweight chamberlain. That is more work on flag checking.
+	 */
+	switch (mode) {
+		default:
+		case 0:
+		case 2: return((v + 1e-10f) / sqrtf(1 + v * v));
+		case 3: return(tanhf(v));
+	}
+}
+
 /*
  * Reset any local memory information.
  */
@@ -206,8 +244,8 @@ param(bristolOP *operator, bristolOPParams *param,
 //printf("KBD: %f\n", value);
 			break;
 		case 4:
-			/* See if we are trying to avoid Houvilainen filters */
-			if ((intvalue == 4) && (bloflags & BRISTOL_LWF))
+			/* See if we are trying to avoid Huovilainen filters */
+			if ((intvalue == 4) && (blo.flags & BRISTOL_LWF))
 				intvalue = 0;
 			if (intvalue > 1) {
 				param->param[index].int_val = intvalue;
@@ -276,7 +314,7 @@ register float gain, register int count) /* Filter loop */
  * constant signal. The noise algorithm will work over multiple voices with no
  * detrimental effect.
  */
-static float scale = 0.00000000001;
+static float scale = 0.00000000001f;
 static int dngx1 = 0x67452301;
 static int dngx2 = 0xefcdab89;
 
@@ -288,7 +326,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	void *lcl)
 {
 	bristolFILTERlocal *local = lcl;
-	register int count;
+	register int count, mode;
 	register float *ib, *ob, *mb;
 	bristolFILTER *specs;
 	register float BLim, Res, Mod, gain;
@@ -297,6 +335,8 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 
 	register float velocity, output, ccut, cvdelta, cutoff;
 
+	mode = bfiltertype(voice->baudio->mixflags);
+
 	/*
 	 * Every operator accesses these variables, the count, and a pointer to
 	 * each buffer. We should consider passing them as readymade parameters?
@@ -317,14 +357,18 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 /*if (mb == 0) */
 /*return;
 
-	if (bloflags & BRISTOL_LWF)
+	if (blo.flags & BRISTOL_LWF)
 		voice->baudio->mixflags |= BRISTOL_LW_FILTER;
  */
 
-	/* See if we are limited to lightweight filters */
-	if ((param->param[4].int_val > 3) &&
-		(voice->baudio->mixflags & BRISTOL_LW_FILTER))
-		param->param[4].int_val = 0;
+	/* See if we  are limited to lightweight filters */
+	if (param->param[4].int_val != 3)
+	{
+		if ((blo.flags & BRISTOL_LWF) || (mode == 1))
+			param->param[4].int_val = 0;
+		else
+			param->param[4].int_val = 4;
+	}
 
 	if (param->param[4].int_val == 1) {
 		/*
@@ -489,7 +533,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		local->Bout = Bout;
 	} else if (param->param[4].int_val == 4) {
 		/*
-		 * This is an implementation of Antti Houvilainen's non-linear Moog
+		 * This is an implementation of Antti Huovilainen's non-linear Moog
 		 * emulation, tweaked just slightly to align with the sometimes rather
 		 * dubious bristol signal levels (its not that they are bad, just that
 		 * historically they have used 1.0 per octave rather than +/-1.0).
@@ -513,8 +557,6 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		float k2vg;
 		float coff;
 
-		float sr = srate;
-
 		float resonance = param->param[1].float_val;
 		Mod = param->param[2].float_val * param->param[2].float_val * 0.02;
 
@@ -533,22 +575,27 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		 *
 		 * We want to position param[3] such that it tunes at 0.5 and can then
 		 * be notched in the GUI, hence srate/4 rathern than /2.
-		coff = param->param[0].float_val * param->param[0].float_val * 0.25
-			+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-		 	* param->param[3].float_val * 4;
 		 *
-		 * This still needs some work.
+		 * Needed some changes to key tracking, it was a bit bipolar.
+		coff = (param->param[0].float_val * param->param[0].float_val)
+			* (1.0f - param->param[3].float_val)
+			* 20000 / srate
+				+
+			param->param[3].float_val * 4 * voice->cfreq / srate;
 		 */
-		coff = param->param[0].float_val * param->param[0].float_val
-			* 20000 / (sr * 2) /* Max filter at 20kHz with 2x oversampling */
-			+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-				* (12650.22 - 8) * 2)
-				/ (127.0 * srate);
-/*
-//			+ ((((float) voice->key.key) * 12658.22) / (127 * srate))
-			+ ((float) (voice->key.key - 36)) * 0.007812
-				* param->param[3].float_val * param->param[3].float_val;
-*/
+		coff = getcoff(param->param[0].float_val, param->param[3].float_val);
+
+		 /*
+		if (param->param[3].int_val == 0)
+			coff = param->param[0].float_val * param->param[0].float_val
+				* 20000 / srate;
+		else // Fraction of the current frequency by tracking by cutoff.
+			coff = param->param[3].float_val * param->param[0].float_val * 4 *
+				voice->cfreq / srate;
+		 */
+
+//			* voice->cfreq * param->param[3].float_val / srate;
+//printf("%f %f %f\n", voice->cfreq, coff, param->param[3].float_val);
 
 		for (; count > 0; count--)
 		{
@@ -560,10 +607,10 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 			 * We should really interpret coff (the configured frequency) as
 			 * a function up to about 20KHz whatever the resampling rate.
 			 */
-			if ((kfc = coff + *mb++ * Mod) > 0.83)
-				kfc = 0.83;
-			else if (kfc < 0)
-				kfc = 0;
+			if ((kfc = coff + *mb++ * Mod) > 0.5)
+				kfc = 0.5;
+			else if (kfc < 1e-10f)
+				kfc = 1e-10f;
 
 			// frequency & amplitude correction
 			kfcr = kfc * ( kfc * (1.8730 * kfc + 0.4955) - 0.6490) + 0.9988;
@@ -573,7 +620,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 			k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc * 0.5));
 
 			// cascade of 4 1st order sections
-			ay1  = az1 + k2vg * (TANHF((*ib*OV2 - 4*resonance*amf*kacr))
+			ay1  = az1 + k2vg * (TANHFEED((*ib*OV2 - 4*resonance*amf*kacr))
 				- TANHF(az1));
 			az1  = ay1;
 
@@ -592,7 +639,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 
 			// oversampling (repeat same block) and inject noise for denormals
 			dngx1 ^= dngx2;
-			ay1  = az1 + k2vg * (TANHF((*ib +dngx2 * scale) * OV2
+			ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * scale) * OV2
 				- 4*resonance*amf*kacr) - TANHF(az1));
 			dngx2 += dngx1;
 			az1  = ay1;
