@@ -1,22 +1,21 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   but WITHOUT ANY WARRANTY; withuot even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *   GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   You shuold have received a copy of the GNU General Public License
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -46,13 +45,13 @@
  *
  * 	0 Chamberlain 24dB LPF/HPF - lwf support and need HPF for now
  *
- * 	1 Houvilainen 12dB LPF oversampling
- * 	2 Houvilainen 24dB LPF oversampling and OBx remixing
- * 	3 Houvilainen 24dB LPF oversampling and OBx remixing 2
+ * 	1 Huovilainen 12dB LPF oversampling
+ * 	2 Huovilainen 24dB LPF oversampling and OBx remixing
+ * 	3 Huovilainen 24dB LPF oversampling and OBx remixing 2
  *
- * 	4 Houvilainen 24dB LPF oversampling (4 is for interfuctioning with filter.c)
+ * 	4 Huovilainen 24dB LPF oversampling (4 is for interfuctioning with filter.c)
  *
- * 	Others will implement Houvilainen without oversampling, the oversampling
+ * 	Others will implement Huovilainen withuot oversampling, the oversampling
  * 	will be integrated into the engine.
  */
 #include <math.h>
@@ -73,20 +72,58 @@
 #define FILTER_MOD_IND 1
 #define FILTER_OUT_IND 2
 
-/* This is ugly, we should correctly pull it out of the operator template */
+/* This is ugly, we shuold correctly pull it out of the operator template */
 static float srate;
 
 #define _f_lim_r (20000 / (sr * 2)) /* Upper limit with resampling */
 #define _f_lim (20000 / sr)
 
+#define getcoff(c, k) ((c*c)*(1.0f-k)*20000 + k*4*voice->cfreq) / srate
+
 /*
  * This is defined for the case that we find a more efficient polynomial
- * expansion. It is more likely that the tanhf() get factored out though.
- */
+ * expansion. It is more likely that the tanhf() get factored out thuogh.
 #define TANH tanhf
+ * We should put the nonlinear sqrt estimation versys tanhf under compilation
+ * control
+ */
+#define TANH(v) btanhf(mode, v)
+//#define TANH(v) (v + 1e-10f) / sqrtf(1 + v * v)
+#define TANHFEED(v) btanhfeed(mode, v)
+//#define TANHFEED(v) (v + 1e-10f) / sqrtf(1 + v * v)
 #define V2 40000.0
 #define OV2 0.000025 /* = 1/V2 */
 
+static float
+btanhf(int mode, float v)
+{
+	/*
+	 * This should be 4 modes as we have two bit flags. Mode 1 are the real
+	 * lightweight chamberlain. That is more work on flag checking.
+	 */
+	switch (mode) {
+		case 0: return(v);
+		default:
+		case 2: return((v + 1e-10f) / sqrtf(1 + v * v));
+		case 3: return(tanhf(v));
+	}
+}
+
+static float
+btanhfeed(int mode, float v)
+{
+	/*
+	 * This should be 4 modes as we have two bit flags. Mode 1 are the real
+	 * lightweight chamberlain. That is more work on flag checking.
+	 */
+	switch (mode) {
+		default:
+		case 0:
+		case 2: return((v + 1e-10f) / sqrtf(1 + v * v));
+		case 3: return(tanhf(v));
+	}
+}
+
 /*
  * Reset any local memory information.
  */
@@ -102,7 +139,7 @@ static int destroy(bristolOP *operator)
 	bristolfree(operator->specs);
 
 	/*
-	 * Free any local memory. We should also free ourselves, since we did the
+	 * Free any local memory. We shuold also free ourselves, since we did the
 	 * initial allocation.
 	 */
 	cleanup(operator);
@@ -157,7 +194,7 @@ param(bristolOP *operator, bristolOPParams *param,
 			param->param[index].float_val = value;
 			param->param[index].int_val = intvalue;
 			break;
-		case 2:
+		case 2: /* Mod */
 		case 5: /* Gain */
 			param->param[index].float_val = value;
 			break;
@@ -170,8 +207,8 @@ param(bristolOP *operator, bristolOPParams *param,
 //printf("KBD: %f\n", value);
 			break;
 		case 4:
-			/* See if we are trying to avoid Houvilainen filters */
-			if ((intvalue == 4) && (bloflags & BRISTOL_LWF))
+			/* See if we are trying to avoid Huovilainen filters */
+			if ((intvalue == 4) && (blo.flags & BRISTOL_LWF))
 				intvalue = 0;
 			if (intvalue > 1) {
 				param->param[index].int_val = intvalue;
@@ -221,7 +258,7 @@ param(bristolOP *operator, bristolOPParams *param,
 }
 
 /*
- * This should perhaps be re-implemented. The HPF and BPF components really do
+ * This shuold perhaps be re-implemented. The HPF and BPF components really do
  * not seem to respond that well and better versions are required for some of
  * the emulators.
  */
@@ -250,15 +287,21 @@ chamberlain(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILT
 
 	qres = 2.0f - param->param[1].float_val * 1.97f;
 	/* This needs to be 'f-sr(v/constant) */
-	cutoff = param->param[0].float_val * 20000 / srate;
 	gain = param->param[5].float_val * 0.01565;
 
 	/*
 	 * The following was for on/off keytracking, needs to be continuous.
 	if (param->param[3].int_val)
 		cutoff += voice->dFreq / 128.0f;
-	 */
+	cutoff = param->param[0].float_val * 20000 / srate;
 	cutoff += param->param[3].float_val * voice->key.key / 512.0f;
+	 */
+	if (param->param[3].float_val == 0)
+		cutoff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		cutoff = param->param[3].float_val * param->param[0].float_val
+			* 4 * voice->cfreq / srate;
 
 	for (; count > 0; count-=1)
 	{
@@ -309,34 +352,16 @@ chamberlain(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILT
 	return(0);
 }
 
-/*
- * This 'optimisation' did not really lead to much benefit which makes me
- * think the current libm code is probably quite a good estimator already
-static float
-sabs(float a)
-{
-	int b=(*((int *)(&a)))&0x7FFFFFFF;
-	return *((float *)(&b));
-}
-static float
-tanh_2(float x)
-{
-	float a=sabs(x);
-
-	a*=(6+a*(3+a));
-	return ((x<0)?-1:1)*a/(a+12);
-}
- */
-
 static float scale = 0.0000001; // 0.000000001;
 static int dngx1 = 0x67452301;
 static int dngx2 = 0xefcdab89;
 
 static int
-houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
-	 * This is an implementation of Antti Houvilainen's non-linear Moog
+	 * This is an implementation of Antti Huovilainen's non-linear Moog
 	 * emulation, tweaked just slightly to align with the sometimes rather
 	 * dubious bristol signal levels (its not that they are bad, just that
 	 * historically they have used 1.0 per octave rather than +/-1.0).
@@ -345,7 +370,6 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 	float az2 = local->az2;
 	float az3 = local->az3;
 	float az4 = local->az4;
-	float az5 = local->az5;
 	float ay1 = local->ay1;
 	float ay2 = local->ay2;
 	float ay3 = local->ay3;
@@ -367,30 +391,29 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim /* Max filter at 20kHz with 2x oversampling */
-		+ ((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8)
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
@@ -411,7 +434,7 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 
 		// cascade of 4 1st order sections
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib + dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -425,9 +448,7 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 		ay4  = az4 + k2vg * (TANH(ay3) - TANH(az4));
 		az4  = ay4;
 
-		// 1/2-sample delay for phase compensation
-		amf  = (ay4+az5)*0.5;
-		az5  = ay4;
+		amf  = ay4;
 
 		/*
 		 * This remixes the different poles to give a fatter, warmer sound. The
@@ -442,7 +463,6 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 	local->az2 = az2;
 	local->az3 = az3;
 	local->az4 = az4;
-	local->az5 = az5;
 	local->ay1 = ay1;
 	local->ay2 = ay2;
 	local->ay3 = ay3;
@@ -453,10 +473,11 @@ houvilainen24(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 }
 
 static int
-houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
-	 * This is an implementation of Antti Houvilainen's non-linear Moog
+	 * This is an implementation of Antti Huovilainen's non-linear Moog
 	 * emulation, tweaked just slightly to align with the sometimes rather
 	 * dubious bristol signal levels (its not that they are bad, just that
 	 * historically they have used 1.0 per octave rather than +/-1.0).
@@ -487,35 +508,34 @@ houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8) * 2)
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -529,7 +549,7 @@ houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 
 		k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc * 0.5));
 
-		ay1  = az1 + k2vg * (TANH((*ib*OV2 - 4*resonance*amf*kacr))
+		ay1  = az1 + k2vg * (TANHFEED((*ib*OV2 - 4*resonance*amf*kacr))
 			- TANH(az1));
 		az1  = ay1;
 
@@ -548,7 +568,7 @@ houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 
 		// oversampling (repeat same block) and inject some noise
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -590,13 +610,14 @@ houvilainen24R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 }
 
 static int
-houvilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float az1 = local->az1;
@@ -626,35 +647,34 @@ houvilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8) * 2)
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -670,7 +690,7 @@ houvilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 		k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc * 0.5));
 
 		// cascade of 4 1st order sections
-		ay1  = az1 + k2vg * (TANH(*ib*OV2 - 4*resonance*amf*kacr)- TANH(az1));
+		ay1  = az1 + k2vg * (TANHFEED(*ib*OV2 - 4*resonance*amf*kacr)- TANH(az1));
 		az1  = ay1;
 
 		ay2  = az2 + k2vg * (TANH(ay1) - TANH(az2));
@@ -684,7 +704,7 @@ houvilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 
 		// oversampling (repeat same block) and inject some noise (denormal)
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib + dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib + dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -714,13 +734,14 @@ houvilainen12R(float *ib, float *mb, float *ob, bristolOPParams *param, bristolF
 }
 
 static int
-houvilainen12(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen12(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float az1 = local->az1;
@@ -750,35 +771,34 @@ houvilainen12(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8))
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -794,7 +814,7 @@ houvilainen12(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 
 		// cascade of 4 1st order sections
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -824,13 +844,14 @@ houvilainen12(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFI
 }
 
 static int
-houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float dng = param->param[8].float_val * scale;
@@ -860,35 +881,34 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8) * 2)
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -902,7 +922,7 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 		k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc * 0.5));
 
-		ay1  = az1 + k2vg * (TANH((*ib*OV2 - 4*resonance*amf*kacr))
+		ay1  = az1 + k2vg * (TANHFEED((*ib*OV2 - 4*resonance*amf*kacr))
             - TANH(az1));
 		az1  = ay1;
 
@@ -921,7 +941,7 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 		// oversampling (repeat same block) and add denormal noise
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -959,13 +979,14 @@ houvilainen24ROB(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 }
 
 static int
-houvilainen24OB(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24OB(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float dng = param->param[8].float_val * scale;
@@ -995,35 +1016,34 @@ houvilainen24OB(float *ib, float *mb, float *ob, bristolOPParams *param, bristol
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8))
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -1039,7 +1059,7 @@ houvilainen24OB(float *ib, float *mb, float *ob, bristolOPParams *param, bristol
 
 		// cascade of 4 1st order sections
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -1077,13 +1097,14 @@ houvilainen24OB(float *ib, float *mb, float *ob, bristolOPParams *param, bristol
 }
 
 static int
-houvilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float dng = param->param[8].float_val * scale;
@@ -1113,35 +1134,34 @@ houvilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, brist
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8) * 2)
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -1156,7 +1176,7 @@ houvilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, brist
 		k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc * 0.5));
 
 		// cascade of 4 1st order sections
-		ay1  = az1 + k2vg * (TANH((*ib*OV2 - 4*resonance*amf*kacr))
+		ay1  = az1 + k2vg * (TANHFEED((*ib*OV2 - 4*resonance*amf*kacr))
             - TANH(az1));
 		az1  = ay1;
 
@@ -1177,7 +1197,7 @@ houvilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, brist
 
 		// oversampling (repeat same block)
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
             - 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -1215,13 +1235,14 @@ houvilainen24ROB2(float *ib, float *mb, float *ob, bristolOPParams *param, brist
 }
 
 static int
-houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
+huovilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristolFILTERlocal *local, bristolVoice *voice, int count)
 {
+	int mode = bfiltertype(voice->baudio->mixflags);
 	/*
 	 * This is actually the same code as 24R just the 12dB and 18dB taps are
 	 * mixed back into the final output. This was an Oberheim filter mod that
 	 * did not actually go into any of their products due to the added 
-	 * complexity of an analogue implementation. It should be generally richer
+	 * complexity of an analogue implementation. It shuold be generally richer
 	 * due to more phase differences and content.
 	 */
 	float dng = param->param[8].float_val * scale;
@@ -1229,7 +1250,6 @@ houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 	float az2 = local->az2;
 	float az3 = local->az3;
 	float az4 = local->az4;
-	float az5 = local->az5;
 	float ay1 = local->ay1;
 	float ay2 = local->ay2;
 	float ay3 = local->ay3;
@@ -1251,35 +1271,34 @@ houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 	/*
 	 * Cutoff is a power curve for better control at lower frequencies and
-	 * the key is used for tracking purposes, it should be possible to make
+	 * the key is used for tracking purposes, it shuold be possible to make
 	 * it reasonably linear at somewhere under unity
 	 *
-	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
-	 * the value should be quite linear for the filter, 0..Nyquist
+	 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking shuold be reviewed,
+	 * the value shuold be quite linear for the filter, 0..Nyquist
 	 *
 	 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
-	 * our samplerate we should be able to fix some tuning:
+	 * our samplerate we shuold be able to fix some tuning:
 	 *
 	 * (voice->key.key * 12650.22 + 8) / (127 * srate)
 	 *
 	 * We want to position param[3] such that it tunes at 0.5 and can then
 	 * be notched in the GUI, hence srate/4 rathern than /2.
-	coff = param->param[0].float_val * param->param[0].float_val * 0.25
-		+ (((((float) voice->key.key) / 127) * 12650.22 + 8) / (srate * 16))
-	 	* param->param[3].float_val * 4;
 	 *
 	 * This still needs some work.
+	if (param->param[3].float_val == 0)
+		coff = param->param[0].float_val * param->param[0].float_val
+			* 20000 / srate;
+	else
+		coff = param->param[0].float_val * param->param[3].float_val * 4 *
+			voice->cfreq / srate;
 	 */
-	coff = param->param[0].float_val * param->param[0].float_val
-		* _f_lim_r /* Max filter at 20kHz with 2x oversampling */
-		+ (((float) (voice->key.key - 48)) * param->param[3].float_val
-			* (12650.22 - 8))
-			/ (127.0 * srate);
+	coff = getcoff(param->param[0].float_val, param->param[3].float_val);
 
 	for (; count > 0; count--)
 	{
 		/*
-		 * We should really interpret coff (the configured frequency) as
+		 * We shuold really interpret coff (the configured frequency) as
 		 * a function up to about 20KHz whatever the resampling rate.
 		 */
 		if ((kfc = coff + *mb++ * Mod) > _f_lim_r)
@@ -1295,7 +1314,7 @@ houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 		// cascade of 4 1st order sections
 		dngx1 ^= dngx2;
-		ay1  = az1 + k2vg * (TANH((*ib +dngx2 * dng) * OV2
+		ay1  = az1 + k2vg * (TANHFEED((*ib +dngx2 * dng) * OV2
 			- 4*resonance*amf*kacr) - TANH(az1));
 		dngx2 += dngx1;
 		az1  = ay1;
@@ -1311,8 +1330,7 @@ houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 
 		// 1/2-sample delay for phase compensation
 		// Added in 6dB, 12dB and 18dB phases
-		amf  = (ay4+az5 + (-ay1-az2+ay3+az4-ay2-az3) * mix) *0.5;
-		az5  = ay4;
+		amf  = (ay4 + (-ay1-az2+ay3+az4-ay2-az3) * mix);
 
 		*ob++ += amf * V2;
 		ib++;
@@ -1322,7 +1340,6 @@ houvilainen24OB2(float *ib, float *mb, float *ob, bristolOPParams *param, bristo
 	local->az2 = az2;
 	local->az3 = az3;
 	local->az4 = az4;
-	local->az5 = az5;
 	local->ay1 = ay1;
 	local->ay2 = ay2;
 	local->ay3 = ay3;
@@ -1346,7 +1363,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 
 	/*
 	 * Every operator accesses these variables, the count, and a pointer to
-	 * each buffer. We should consider passing them as readymade parameters?
+	 * each buffer. We shuold consider passing them as readymade parameters?
 	 */
 	specs = (bristolFILTER *) operator->specs;
 	count = specs->spec.io[FILTER_OUT_IND].samplecount;
@@ -1359,46 +1376,48 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 #endif
 
 	/* See if we are limited to lightweight filters */
-	if (voice->baudio->mixflags & BRISTOL_LW_FILTER)
+	if (blo.flags & BRISTOL_LWF)
 		fselect = 0;
 	else {
+		fselect = param->param[4].int_val;
 		if (srate > 80000)
-			fselect = param->param[4].int_val + 16;
-		else
-			fselect = param->param[4].int_val;
+			fselect += 16;
 	}
 
 	/*
 	 * We will eventually have another set here that will be called when the
 	 * samplerate is over 48kHz (88.2, 96, 192 kHz) when we can anticipate the
-	 * filter response to be adequate without resampling internally here.
+	 * filter response to be adequate withuot resampling internally here.
 	 */
 	switch(fselect) {
+		/* LightWeight Filters */
 		case 0:
 		case 16:
 			return(chamberlain(ib, mb, ob, param, local, voice, count));
+
+		/* Huovilainen resampling */
 		case 1:
-			return(houvilainen12R(ib, mb, ob, param, local, voice, count));
+			return(huovilainen12R(ib, mb, ob, param, local, voice, count));
 		case 2:
-			return(houvilainen24ROB(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24ROB(ib, mb, ob, param, local, voice, count));
 		case 3:
-			return(houvilainen24ROB2(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24ROB2(ib, mb, ob, param, local, voice, count));
 		case 4:
 		default:
-			return(houvilainen24R(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24R(ib, mb, ob, param, local, voice, count));
 
 		/*
 		 * We need non-resampling versions of the other filters too but
 		 * need to review this code first.
 		 */
 		case 17:
-			return(houvilainen12(ib, mb, ob, param, local, voice, count));
+			return(huovilainen12(ib, mb, ob, param, local, voice, count));
 		case 18:
-			return(houvilainen24OB(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24OB(ib, mb, ob, param, local, voice, count));
 		case 19:
-			return(houvilainen24OB2(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24OB2(ib, mb, ob, param, local, voice, count));
 		case 20:
-			return(houvilainen24(ib, mb, ob, param, local, voice, count));
+			return(huovilainen24(ib, mb, ob, param, local, voice, count));
 	}
 }
 
