@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -101,7 +100,19 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("junoreset(%x)\n", operator);
 #endif
 
+	if (param->param[0].mem != NULL)
+		bristolfree(param->param[0].mem);
 	param->param[0].mem = bristolmalloc0(sizeof(float) * JUNODCO_WAVE_SZE);
+	if (param->param[1].mem != NULL)
+		bristolfree(param->param[1].mem);
+	param->param[1].mem = bristolmalloc0(sizeof(float) * JUNODCO_WAVE_SZE);
+	if (param->param[2].mem != NULL)
+		bristolfree(param->param[2].mem);
+	param->param[2].mem = bristolmalloc0(sizeof(float) * JUNODCO_WAVE_SZE);
+	if (param->param[3].mem != NULL)
+		bristolfree(param->param[3].mem);
+	param->param[3].mem = bristolmalloc0(sizeof(float) * JUNODCO_WAVE_SZE);
+
 	param->param[0].float_val = 512;
 	param->param[1].float_val = 1.0;
 	param->param[2].float_val = 1.0;
@@ -287,6 +298,22 @@ static int operate(bristolOP *operator,
 	subw = local->subw;
 	cpwm = local->cpwm;
 
+	if (bristolBLOcheck(voice->dFreq*transp))
+	{
+		wt = (float *) param->param[1].mem;
+
+		memset(param->param[2].mem, 0, JUNODCO_WAVE_SZE * sizeof(float));
+		memset(param->param[3].mem, 0, JUNODCO_WAVE_SZE * sizeof(float));
+
+		generateBLOwaveformF(voice->cFreq*transp, param->param[2].mem,BLO_RAMP);
+		generateBLOwaveformF(voice->cFreq*transp, param->param[3].mem, BLO_TRI);
+
+		buildJunoWave(param->param[1].mem,
+			param->param[4].float_val, param->param[2].mem,
+			param->param[5].float_val, param->param[3].mem);
+	} else
+		wt = (float *) param->param[0].mem;
+
 	if (voice->flags & BRISTOL_KEYON)
 		wtp = 0;
 
@@ -533,7 +560,7 @@ fillWave(float *mem, int count, int type)
 			/* 
 			 * This is a square wave.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -563,7 +590,7 @@ fillWave(float *mem, int count, int type)
 			for (i = count - 1;i >= 0; i--)
 				mem[i] = (((float) i / count) - 0.5) * BRISTOL_VPO * 2.0;
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = bloramp[i];
@@ -580,7 +607,7 @@ fillWave(float *mem, int count, int type)
 			 * Triangular wave. From MIN point, ramp up at twice the rate of
 			 * the ramp wave, then ramp down at same rate.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blotriangle[i];
@@ -598,7 +625,7 @@ fillWave(float *mem, int count, int type)
 			 * Would like to put in a jagged edged ramp wave. Should make some
 			 * interesting strings sounds.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				int k = 0;
 
