@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -63,7 +62,7 @@ float note_diff;
 #define DCO_WAVE_COUNT 6
 
 static void fillWave();
-static void buildProphetWave();
+static void buildProphetWave(float *, float , float *, float , float *, float , float *);
 
 /*
  * We should not be doing this however I need a quick resolution to some issues
@@ -111,10 +110,18 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 	printf("prophetreset(%x)\n", operator);
 #endif
 
+	if (param->param[0].mem != NULL)
+		bristolfree(param->param[0].mem);
+	if (param->param[1].mem != NULL)
+		bristolfree(param->param[1].mem);
+	if (param->param[2].mem != NULL)
+		bristolfree(param->param[2].mem);
 	param->param[0].mem = bristolmalloc0(sizeof(float) * PROPHETDCO_WAVE_SZE);
 	param->param[0].int_val = PROPHETDCO_WAVE_SZE / 2;
+	param->param[1].mem = bristolmalloc0(sizeof(float) * PROPHETDCO_WAVE_SZE);
 	param->param[1].float_val = 1.0;
 	param->param[2].float_val = 1.0;
+	param->param[2].mem = bristolmalloc0(sizeof(float) * PROPHETDCO_WAVE_SZE);
 	param->param[3].float_val = 1.0;
 	param->param[7].int_val = 0;
 	param->param[9].float_val = 1.0;
@@ -127,7 +134,7 @@ static int reset(bristolOP *operator, bristolOPParams *param)
 /*
  * Alter an internal parameter of an operator.
  */
-static int param(bristolOP *operator, bristolOPParams *param,
+static int pdcoparam(bristolOP *operator, bristolOPParams *param,
 	unsigned char index, float value)
 {
 	bristolPROPHETDCO *specs = (bristolPROPHETDCO *) operator->specs;
@@ -307,9 +314,9 @@ static int param(bristolOP *operator, bristolOPParams *param,
 
 static void
 buildProphetWave(register float *wave,
-	register int sine, register float *sinew,
-	register int ramp, register float *rampw,
-	register int tri, register float *triwave)
+	register float sine, register float *sinew,
+	register float ramp, register float *rampw,
+	register float tri, register float *triwave)
 {
 	register int i;
 
@@ -318,15 +325,12 @@ buildProphetWave(register float *wave,
 
 	bristolbzero(wave, sizeof(float) * PROPHETDCO_WAVE_SZE);
 
-	for (i = 0; i < PROPHETDCO_WAVE_SZE; i++)
+	for (i = 0; i < PROPHETDCO_WAVE_SZE; i+=4)
 	{
-		if (ramp)
-			*wave = *rampw++;
-		if (tri)
-			*wave += *triwave++;
-		if (sine)
-			*wave += *sinew++;
-		wave++;
+		*wave++ = *rampw++ * ramp + *triwave++ * tri + *sinew++ * sine;
+		*wave++ = *rampw++ * ramp + *triwave++ * tri + *sinew++ * sine;
+		*wave++ = *rampw++ * ramp + *triwave++ * tri + *sinew++ * sine;
+		*wave++ = *rampw++ * ramp + *triwave++ * tri + *sinew++ * sine;
 	}
 }
 
@@ -401,6 +405,13 @@ static int operate(bristolOP *operator,
 	else
 		sb = 0;
 
+	/*
+	buildProphetWave(param->param[0].mem,
+		param->param[8].int_val, specs->wave[7],
+		param->param[4].int_val, specs->wave[3],
+		param->param[5].int_val, specs->wave[4]);
+	*/
+
 	wt = (float *) param->param[0].mem;
 	width = param->param[0].float_val; /* Pulse width */
 	dosquare = param->param[6].int_val;
@@ -420,6 +431,45 @@ static int operate(bristolOP *operator,
 	wtp = local->wtp;
 	ramp = specs->wave[3];
 
+	/*
+	 * For real BLO we need to look at the key being generated and the transpose
+	 * from there decide whether to generate a new waveform. If the resulting
+	 * transpose is less that a limit then we use the default BLO wave. If it is
+	 * higher then we will regenerate that wave with fewer harmonics, probably
+	 * with a hard tailoff since the worst of the noise occurs at the highest 
+	 * octaves and needs some very heavy reductions to clean it up. It will
+	 * require some empirical values. This can eventually be buried into blo.c
+	 * for general usage.
+	 *
+	 * This is all for 0.50.1 though
+printf("prophetdco(key %i, t %f, h %i): %i\n", voice->key.key, transp,
+blo.harmonics, voice->key.key * transp < 95?
+	(int)  blo.harmonics:
+		 blo.harmonics - (voice->key.key - 95) * transp * 3 < 5? 5:
+			(int) ( blo.harmonics - (voice->key.key - 95) * transp * 3));
+	 */
+
+	if (bristolBLOcheck(voice->dFreq*transp))
+	{
+		memset(param->param[1].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+		memset(param->param[2].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+		wt = (float *) param->param[1].mem;
+		ramp = (float *) param->param[2].mem;
+		if (param->param[4].int_val) {
+			/* Put in BLO ramp wave, copy to PWM wave */
+			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_RAMP);
+			bcopy(wt, ramp, PROPHETDCO_WAVE_SZE * sizeof(float));
+		} else {
+			/* Put in ramp wave for PWM */
+			if (param->param[6].int_val)
+				generateBLOwaveformF(voice->cFreq*transp, ramp, BLO_RAMP);
+		}
+		if (param->param[5].int_val)
+			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_TRI);
+		if (param->param[8].int_val)
+			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_SINE);
+	}
+
 	/*
 	 * Go jumping through the wavetable, with each jump defined by the value
 	 * given on our input line, making sure we fill one output buffer.
@@ -469,7 +519,7 @@ static int operate(bristolOP *operator,
 
 			gdelta = wtpSqr - ((float) ((int) wtpSqr));
 
-			if (wtpSqr >= PROPHETDCO_WAVE_SZE_M)
+			if (wtpSqr == PROPHETDCO_WAVE_SZE_M)
 				S2 -= (ramp[(int) wtpSqr] + (ramp[0] - ramp[(int) wtpSqr])
 					* gdelta) * ssg;
 			else
@@ -569,7 +619,7 @@ prophetdcoinit(bristolOP **operator, int index, int samplerate, int samplecount)
 	(*operator)->operate = operate;
 	(*operator)->destroy = destroy;
 	(*operator)->reset = reset;
-	(*operator)->param= param;
+	(*operator)->param = pdcoparam;
 
 	specs = (bristolPROPHETDCO *) bristolmalloc0(sizeof(bristolPROPHETDCO));
 	(*operator)->specs = (bristolOPSpec *) specs;
@@ -778,7 +828,7 @@ fillWave(float *mem, int count, int type)
 			/* 
 			 * This is a square wave, with decaying plateaus.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -816,7 +866,7 @@ fillWave(float *mem, int count, int type)
 			mem[count - 1] = mem[1]
 				= BRISTOL_VPO;
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = bloramp[i];
@@ -829,7 +879,7 @@ fillWave(float *mem, int count, int type)
 			 * Triangular wave. From MIN point, ramp up at twice the rate of
 			 * the ramp wave, then ramp down at same rate.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blotriangle[i];
