@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -116,7 +115,7 @@ static int operate(bristolOP *operator,
 {
 	bristolVREVERB *specs;
 	register bristolVREVERBlocal *local = (bristolVREVERBlocal *) lcl;
-	register float *source, *dest, *dest2, *history;
+	register float *source, *dest, *dest2, *dest2_1, *dest_1, *history;
 	register int count, i, j;
 	register float wetdry, crossover, feedback, lgain, rgain, wet, linedata;
 
@@ -163,8 +162,8 @@ local->end[4]);
 
 	count = specs->spec.io[VREVERB_IN].samplecount;
 	source = specs->spec.io[VREVERB_IN].buf;
-	dest = specs->spec.io[VREVERB_OUT].buf;
-	dest2 = specs->spec.io[VREVERB_OUT2].buf;
+	dest_1 = dest = specs->spec.io[VREVERB_OUT].buf;
+	dest2_1 = dest2 = specs->spec.io[VREVERB_OUT2].buf;
 
 	feedback = param->param[FEEDBACK].float_val * 0.5;
 	crossover = param->param[CROSSOVER].float_val * 0.5;
@@ -203,9 +202,9 @@ local->end[4]);
 		 * And finally feed the output from the delay lines into the last line
 		 * and mix the wet/dry signal.
 		 */
-		*dest2++ = *dest2 * 0.5
+		*dest2++ = *dest2_1++ * 0.5
 			+ (history[local->lptr[5]] - linedata * 0.25) * lgain;
-		*dest++ = *dest * 0.5
+		*dest++ = *dest_1++* 0.5
 			+ (history[local->lptr[4]] - linedata * 0.25) * rgain;
 
 		history[local->lptr[5]] = linedata * 0.25 - wet * crossover;
