@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -539,7 +538,7 @@ nSpeed, Speed, inertia, param->param[3].int_val, local->tinc);
 static void
 newValvify(register float *busData, register int count, register float valve)
 {
-	register float div;
+	register float div, *busData2 = busData;
 
 	if ((valve == 0) || (valve == 1))
 		return;
@@ -561,9 +560,9 @@ newValvify(register float *busData, register int count, register float valve)
 		 * function.
 		 */
 		if (*busData > 0)
-			*busData = *busData++ * valve;
+			*busData++ = *busData2++ * valve;
 		else
-			*busData = *busData++ * div;
+			*busData++ = *busData2++ * div;
 	}
 }
 
