@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 #include "bristol.h"
@@ -70,9 +69,9 @@ printSoundList(Baudio *baudio, bristolSound *sound[])
 
 		printf("	name	%s\n", sound[i]->name);
 		printf("	index	%i\n", sound[i]->index);
-		printf("	op	%x\n", (size_t) sound[i]->operate);
-		printf("	param	%x\n", (size_t) sound[i]->param);
-		printf("	next	%x\n", sound[i]->next);
+		printf("	op	%p\n", sound[i]->operate);
+		printf("	param	%p\n", sound[i]->param);
+		printf("	next	%i\n", sound[i]->next);
 		printf("	flags	%x\n", sound[i]->flags);
 	}
 }
@@ -133,6 +132,7 @@ freeSoundAlgo(Baudio *baudio, int algo, bristolSound *sound[])
 		{
 			if (sound[algo]->param->param[j].mem != NULL)
 				bristolfree(sound[algo]->param->param[j].mem);
+			sound[algo]->param->param[j].mem = NULL;
 		}
 
 		bristolfree(sound[algo]->param);
