@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*#define BRISTOL_DBG */
@@ -265,7 +264,7 @@ register int count, register float inv)
 				+ wt1[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))))
 					* gain;
 		 */
-		if ((wtp >= TRILOGY_WAVE_SZE_M)
+		if ((((int) wtp) == TRILOGY_WAVE_SZE_M)
 			|| ((lsv < 0) && (sb[obp] >= 0)))
 			ob[obp] += (wt1[0] * (wtp - ((float) ((int) wtp)))
 				+ wt1[(int) wtp] * (1.0 - (wtp - ((float) ((int) wtp)))))
@@ -323,7 +322,7 @@ static int operate(bristolOP *operator,
 {
 	register bristolTRILOGYlocal *local = lcl;
 	register int count;
-	register float *ib, *ob, *sb, *pwb, *wt1, *wt2, wtp, lsv, transp;
+	register float *ib, *ob, *sb, *pwb, lsv, transp;
 	bristolTRILOGY *specs;
 
 	specs = (bristolTRILOGY *) operator->specs;
@@ -337,16 +336,10 @@ static int operate(bristolOP *operator,
 	ob = specs->spec.io[TRILOGY_OUT1_IND].buf;
 	sb = specs->spec.io[TRILOGY_SYNC_IND].buf;
 	pwb = specs->spec.io[TRILOGY_PWM_IND].buf;
-	wt1 = (float *) param->param[0].mem;
-	wt2 = specs->wave[1];
-	wtp = local->wtp;
 	lsv = local->lsv;
 
 	transp = param->param[8].float_val * param->param[9].float_val;
 
-	if (voice->flags & BRISTOL_KEYON)
-		local->wtp = wtp = 0;
-
 	/*
 	 * Here we should put in the pulse wave. The nice thing about making it a
 	 * routine is that we can then consider calling it multiple times for the
@@ -413,7 +406,6 @@ static int operate(bristolOP *operator,
 		sweeps[4] * transp,
 		local->wtppw15, lsv, count, 0.0);
 
-	local->wtp = wtp;
 	local->lsv = sb[count - 1];
 
 	return(0);
@@ -683,7 +675,7 @@ fillWave(register float *mem, register int count, int type)
 			/*
 			 * Ramp wave
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				int i;
 
@@ -701,7 +693,7 @@ fillWave(register float *mem, register int count, int type)
 			/* 
 			 * This is a square wave, with decaying plateaus.
 			 */
-			if (bloflags & BRISTOL_BLO)
+			if (blo.flags & BRISTOL_BLO)
 			{
 				for (i = 0;i < count; i++)
 					mem[i] = blosquare[i];
@@ -718,7 +710,7 @@ fillWave(register float *mem, register int count, int type)
 			{
 				int i;
 
-				if (bloflags & BRISTOL_BLO)
+				if (blo.flags & BRISTOL_BLO)
 				{
 					for (i = 0;i < count; i++)
 						mem[i] = blotriangle[i];
@@ -800,6 +792,16 @@ fillPDcos(float *mem, int count, float compress)
 	}
 }
 
+/*
+ * Resample the sine wave as per casio phase distortion algorithms.
+ *
+ * We have to get to M_PI/2 in compress steps, hence
+ *
+ *	 inc1 = M_PI/(2 * compress)
+ *
+ * Then we have to scan M_PI in (count - 2 * compress), hence:
+ *
+ *	 inc2 = M_PI/(count - 2 * compress)
 static void
 fillPDsine(float *mem, int count, float compress)
 {
@@ -808,18 +810,6 @@ fillPDsine(float *mem, int count, float compress)
 
 	if (comp < 1)
 		comp = 1;
-
-	/*
-	 * Resample the sine wave as per casio phase distortion algorithms.
-	 *
-	 * We have to get to M_PI/2 in compress steps, hence
-	 *
-	 *	 inc1 = M_PI/(2 * compress)
-	 *
-	 * Then we have to scan M_PI in (count - 2 * compress), hence:
-	 *
-	 *	 inc2 = M_PI/(count - 2 * compress)
-	 */
 	inc1 = ((float) M_PI) / (((float) 2) * ((float) comp));
 	inc2 = ((float) M_PI) / ((float) (count - 2 * comp));
 
@@ -835,6 +825,7 @@ fillPDsine(float *mem, int count, float compress)
 			j += inc2;
 	}
 }
+ */
 
 static void
 buildTrilogySound(bristolOP *operator, bristolOPParams *param)
