@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,10 +15,11 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
+#ifndef B_X11_H
+#define B_X11_H
 #include "brightonX11internals.h"
 
 extern void *brightonX11malloc(size_t);
@@ -29,6 +30,9 @@ extern int BCloseDisplay(bdisplay *);
 
 extern int BOpenWindow(brightonDisplay *, brightonWindow *, char *);
 extern int BCloseWindow(brightonDisplay *, brightonWindow *);
+extern int BCloseWindow(brightonDisplay *, brightonWindow *);
+extern void BRaiseWindow(brightonDisplay *, brightonWindow *);
+extern void BLowerWindow(brightonDisplay *, brightonWindow *);
 
 extern int BDrawArea(brightonDisplay *, brightonBitmap *, int, int, int, int, int, int);
 extern int BCopyArea(brightonDisplay *, int, int, int, int, int, int);
@@ -46,4 +50,5 @@ extern int BResizeWindow(brightonDisplay *, brightonWindow *, int, int);
 
 extern int BNextEvent(brightonDisplay *, brightonEvent *);
 
+#endif
 
