@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,15 +28,17 @@
  *
  * For rendering
  */
-#define BRIGHTON_DIRECTION_MASK	0x0003
-#define BRIGHTON_REVERSE		0x0001
-#define BRIGHTON_VERTICAL		0x0002
-#define BRIGHTON_NOSHADOW		0x0008
-#define BRIGHTON_WITHDRAWN		0x0020
-#define BRIGHTON_ANTIALIAS		0x0040
-#define BRIGHTON_HALFSHADOW		0x0080
-#define BRIGHTON_REDRAW			0x0100
-#define BRIGHTON_STRETCH		0x010000
+#define BRIGHTON_DIRECTION_MASK	0x00000003
+#define BRIGHTON_REVERSE		0x00000001
+#define BRIGHTON_VERTICAL		0x00000002
+#define BRIGHTON_NOSHADOW		0x00000008
+#define BRIGHTON_WITHDRAWN		0x00000020
+#define BRIGHTON_ANTIALIAS		0x00000040
+#define BRIGHTON_HALFSHADOW		0x00000080
+#define BRIGHTON_REDRAW			0x00000100
+#define BRIGHTON_STRETCH		0x00010000
+#define BRIGHTON_POST_WINDOW	0x00020000
+#define BRIGHTON_WINDOW			0x00040000
 
 /* #define BRIGHTON_CHECKBUTTON 0x0010 */
 #define BRIGHTON_SHIFTKEY		0x0020
