@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -127,12 +126,15 @@ typedef struct BrightonPanel {
 
 #define BRIGHTON_DEFAULT_DISPLAY ":0.0"
 #define BRIGHTON_NAMESIZE 64
-#define BRIGHTON_BIMAGE 0x020000
-#define BRIGHTON_ANTIALIAS_1 0x040000
-#define BRIGHTON_ANTIALIAS_2 0x080000
-#define BRIGHTON_ANTIALIAS_3 0x100000
-#define BRIGHTON_ANTIALIAS_4 0x200000
-#define BRIGHTON_ANTIALIAS_5 0x400000
+#define BRIGHTON_LIB_DEBUG		0x00010000
+#define BRIGHTON_BIMAGE			0x00020000
+#define BRIGHTON_ANTIALIAS_1	0x00040000
+#define BRIGHTON_ANTIALIAS_2	0x00080000
+#define BRIGHTON_ANTIALIAS_3	0x00100000
+#define BRIGHTON_ANTIALIAS_4	0x00200000
+#define BRIGHTON_ANTIALIAS_5	0x00400000
+#define _BRIGHTON_WINDOW		0x00800000
+#define BRIGHTON_BVIMAGE		0x01000000
 typedef struct BrightonDisplay {
 	unsigned int flags;
 	struct BrightonDisplay *next, *last;
@@ -147,13 +149,21 @@ typedef struct BrightonDisplay {
 /*
  * Flags for brightonWindow.flags
  */
-#define BRIGHTON_NO_DRAW 0x01
-#define BRIGHTON_BUSY 0x20
-#define BRIGHTON_NO_LOGO 0x40
+#define BRIGHTON_NO_DRAW		0x00000001
 /* SET_SIZE, DEV_ACTIVE are in here too */
-#define BRIGHTON_NO_ASPECT 0x80
+#define BRIGHTON_BUSY			0x00000020
+#define BRIGHTON_NO_LOGO		0x00000040
+#define BRIGHTON_NO_ASPECT		0x00000080
+#define _BRIGHTON_POST			0x00000100
+#define BRIGHTON_DEBUG			0x00000200
+#define BRIGHTON_EXITING		0x00000400
+#define BRIGHTON_AUTOZOOM		0x00000800
+#define BRIGHTON_SET_RAISE		0x00001000
+#define BRIGHTON_SET_LOWER		0x00002000
+#define _BRIGHTON_SET_HEIGHT	0x00004000
+#define BRIGHTON_ROTARY_UD		0x00008000
 
-#define BRIGHTON_ITEM_COUNT 512 /* max devices per panel */
+#define BRIGHTON_ITEM_COUNT		512 /* max devices per panel */
 /*
  * These are used to hold items that have been placed on the top layer. We
  * need this structure since these items may be inserted and removed.
@@ -171,6 +181,12 @@ typedef struct BrightonLayerItem {
 	int scale;
 } brightonLayerItem;
 
+typedef struct BrightonNRPcontrol {
+	int nrp;
+	struct brightonDevice *device;
+} brightonNRPcontrol;
+
+#define BRIGHTON_NRP_COUNT 128
 #define BRIGHTON_GANG_COUNT 8
 
 typedef struct BrightonWindow {
@@ -197,6 +213,8 @@ typedef struct BrightonWindow {
 	void *gc;
 	int x, y;
 	unsigned int width, height, depth, border;
+	unsigned int minw, minh;
+	unsigned int maxw, maxh;
 	float aspect;
 	int cmap_size;
 	int id;
@@ -211,10 +229,19 @@ typedef struct BrightonWindow {
 	int parentwin;
 	/* These should not really be in this structure, but in the synth. */
 	int midimap[128]; /* This could be u_char */
+	/*
+	 * Only build support for 128 controls. Supporting 16384 of them would be
+	 * rather large on memory however a bigger value could be useful - FFS.
+	 */
 	int GM2values[128];
+	/* These are for CC tracking */
 	struct brightonDevice *midicontrol[128][BRIGHTON_GANG_COUNT];
 	float midicontrolval[128][BRIGHTON_GANG_COUNT];
 	float midicontrolscaler[128];
+	/* These are for NRP tracking */
+	int nrpcount;
+	brightonNRPcontrol *nrpcontrol; //[BRIGHTON_NRP_COUNT];
+	/* CC value mapping tables */
 	u_char valuemap[128][128];
 	int kbdmap[256][2];
 	int dcTimeout;
@@ -283,7 +310,6 @@ extern void *brightonmalloc(size_t);
 extern FILE *brightonfopen(char *, char *);
 extern char *brightonfgets(char *, int, FILE *);
 extern int brightonfclose(FILE *);
-extern void brightonfree(void *);
 extern brightonPalette *brightonInitColormap(brightonWindow *, int);
 extern int brightonFindColor(brightonPalette *, int, unsigned short, unsigned short, unsigned short, int);
 extern int brightonFindFreeColor(brightonPalette *, int);
@@ -307,6 +333,8 @@ extern int brightonForwardEvent(brightonWindow *);
 extern int brightonLocation(brightonWindow *, int, int, int, int, int);
 extern int brightonXpmWrite(brightonWindow *, char *);
 
+extern int brightonRequestResize(brightonWindow *, int, int);
+
 extern brightonILocations *brightonLocator(brightonWindow *, int, int);
 extern brightonIResource *brightonPanelLocator(brightonWindow *, int, int);
 extern brightonILocations *brightonDeviceLocator(brightonIResource *, int, int);
@@ -330,8 +358,13 @@ extern void brightonAlphaLayer(brightonWindow *, brightonBitmap *, brightonBitma
 extern int brightonInitBitmap(brightonBitmap *, int);
 extern int brightonDoFinalRender(brightonWindow *, int, int, int, int);
 
+extern int BAutoRepeat(brightonDisplay *, int);
+
 extern int brightonGetDCTimer();
 extern int brightonDoubleClick();
+extern void brightonfree(void *);
+
+extern void brightonRegisterController(brightonDevice *);
 
 #endif /* BRIGHTONINTERNALS_H */
 
