@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -37,12 +36,13 @@
 
 /*#define BRISTOL_DBG */
 
-#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
 #include <unistd.h>
+#ifdef BRISTOL_SEMAPHORE
 #include <semaphore.h>
+#endif
 #include <time.h>
 #include <pthread.h>
 
@@ -50,6 +50,7 @@
 
 /* #include "bristolvers.h" */
 #include "bristoldebug.h"
+#include "ringbuffer.h"
 
 #include "bristolaudio.h"
 #include "bristolmidiapi.h"
@@ -75,11 +76,23 @@
 
 #define BRISTOL_ENGINE "bristolengine"
 
+#define BRISTOL_JACK_MULTI		16 /* 2 legacy plus 16 multiIO */
+#define BRISTOL_JACK_UUID_SIZE	32
+
 /*
  * The following are channels and flags. They are applied to bristol system
  * messages, and to the baudio.midiflags
+ *
+ * We should define a controller map for bristol:
+ *	0..63	Emulator operators (osc/env/filt, etc)
+ *	64-95	Engine processes:
+ *		64	Arpeggiator
+ *		65	LADI
+ *	96-126	Emulator effects except:
+ *	127		SYSTEM commands: start/stop/init, etc.
  */
-#define BRISTOL_SYSTEM			127
+#define BRISTOL_LADI			65
+#define BRISTOL_SYSTEM			127 /* 127/0 */
 #define BRISTOL_HELLO			0x00002000
 #define BRISTOL_COMMASK			0x00001f00
 #define BRISTOL_PARAMMASK		0x000000ff
@@ -100,12 +113,17 @@
 #define BRISTOL_EVENT_PITCH		0x00000f00 /* NOT SENT AS BRISTOL_SYSTEM */
 /*#define BRISTOL_FREE			0x00000f00 */
 #define BRISTOL_EXIT_ALGO		0x00001000
-#define BRISTOL_UNISON			0x00002000
-#define BRISTOL_REQON			0x00004000
+#define BRISTOL_UNISON			0x00001100
+#define BRISTOL_REQ_FORWARD		0x00001200 /* Disable event forwarding on chan*/
+#define BRISTOL_REQ_DEBUG		0x00001300 /* Debug level */
 #define BRISTOL_MIDI_DEBUG1		0x00008000 /* Just messages */
 #define BRISTOL_MIDI_DEBUG2		0x00010000 /* And internal functions() */
 #define BRISTOL_MIDI_NRP_ENABLE	0x00020000
 
+/* LADI are BRISTOL_SYSTEM 1278/1 */
+#define BRISTOL_LADI_SAVE_REQ	0x00000001
+#define BRISTOL_LADI_LOAD_REQ	0x00000002
+
 #define BRISTOL_CC_HOLD1		64
 
 #define SYSTEM_CHANNEL			0x07f /* used for initialisation requests. */
@@ -205,18 +223,20 @@
 #define B_NRO			35
 
 /*
- * Audio interface types. These are for audiomain.flags:
+ * Audio interface types. The loest bytes is resevered.
+ *
+ * These are for audiomain.flags:
  */
 #define BRISTOL_AUDIOMASK		0xff000000
 #define BRISTOL_ALSA			0x01000000
 #define BRISTOL_OSS				0x02000000
-#define BRISTOL_SLAB			0x04000000
+#define BRISTOL_SLAB			0x04000000 /* should deprecte this */
 #define BRISTOL_JACK			0x08000000
 #define BRISTOL_DSSI			0x10000000 /* Unlikely to be supported - LV2? */
-#define BRISTOL_OSC				0x20000000
-#define BRISTOL_DUMMY			0x40000000
-
-#define BRISTOL_AUDIOWAIT		0x01000000
+#define BRISTOL_DUMMY			0x20000000
+#define BRISTOL_GSTREAMER		0x40000000
+/* This appears to collide */
+#define BRISTOL_AUDIOWAIT		0x00002000
 
 /*
  * MIDI interface types, may interact with audio drivers.
@@ -224,14 +244,18 @@
 #define BRISTOL_MIDIMASK		0x007f0000
 #define BRISTOL_MIDI_ALSA		0x00010000
 #define BRISTOL_MIDI_OSS		0x00020000
-#define BRISTOL_MIDI_SLAB		0x00040000
+#define BRISTOL_MIDI_SLAB		0x00040000 /* this should be deprecated */
 #define BRISTOL_MIDI_JACK		0x00080000
 #define BRISTOL_MIDI_LADSP		0x00100000
 #define BRISTOL_MIDI_SEQ		0x00200000
+#define BRISTOL_MIDI_OSC		0x00400000
 
 #define BRISTOL_MIDI_WAIT		0x00800000
 
+/* Audio connection to Jack ports */
 #define BRISTOL_AUTO_CONN		0x00008000
+/* Separate registration for audio and midi */
+#define BRISTOL_JACK_DUAL		0x00004000
 
 #define BRISTOL_TERM -3
 #define BRISTOL_FAIL -2
@@ -345,6 +369,7 @@ typedef struct BristolVoice {
 	struct BAudio *baudio;
 	int index;
 	unsigned int flags;
+	int offset;
 	char ***locals;
 	/*
 	 * We need an event structure for each possible poly event. These will be
@@ -487,7 +512,10 @@ extern bristolIO *bristolIOinit();
  * a poly algorithm can use the rest.
 #define BRISTOL_SUSTAIN		0x0040000000000000ULL
  */
-#define BRISTOL_LW_FILTER	0x0002000000000000ULL
+#define bfiltertype(x)	((x&(BRISTOL_LW_FILTER|BRISTOL_LW_FILTER2)) >> 48)
+#define BRISTOL_LW_FILTER	0x0001000000000000ULL
+#define BRISTOL_LW_FILTER2	0x0002000000000000ULL
+#define BRISTOL_LW_FILTERS	(BRISTOL_LW_FILTER|BRISTOL_LW_FILTER2)
 #define BRISTOL_GLIDE_AUTO	0x0004000000000000ULL
 #define BRISTOL_SENSE		0x0008000000000000ULL
 #define BRISTOL_SEQUENCE	0x0010000000000000ULL
@@ -584,6 +612,9 @@ typedef struct BAudio {
 		int velocity[128];
 		int count;
 		int flags;
+		int low;
+		int high;
+		int extreme;
 	} notemap;
 } Baudio;
 
@@ -597,8 +628,13 @@ typedef struct AudioMain {
 	bristolVoice *newlast;
 	bristolOP **palette; /* operator templates */
 	bristolOP **effects; /* operator templates */
+#ifdef BRISTOL_SEMAPHORE
 	sem_t *sem_long;
 	sem_t *sem_short;
+#else
+	void *unused1;
+	void *unused2;
+#endif
 	struct timespec abstime;
 	int voiceCount;
 	int opCount;
@@ -626,6 +662,18 @@ typedef struct AudioMain {
 	float ingain;
 	float outgain;
 	char *microTonalMappingFile;
+	unsigned int SysID;
+	jack_ringbuffer_t *rb; /* MIDI thread to audio threadd */
+	jack_ringbuffer_t *rbfp; /* MIDI/Audio thread to forwarding thread */
+	char jackUUID[BRISTOL_JACK_UUID_SIZE];
+	int iocount;
+	float *io_i[BRISTOL_JACK_MULTI];
+	float *io_o[BRISTOL_JACK_MULTI];
+	float m_io_ogc;
+	float m_io_igc;
+	char *cmdline;
+	char *sessionfile;
+	char *controldev;
 } audioMain;
 
 extern int cleanup();
