@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -47,6 +46,16 @@
 #define BRISTOL_PERMMASK 0x00f
 #define BRISTOL_CONNMASK 0xff0
 
+/*
+ * Relogging flags
+ */
+#define BRISTOL_LOG_TERMINATE -1
+#define BRISTOL_LOG_BRISTOL 0
+#define BRISTOL_LOG_BRIGHTON 1
+#define BRISTOL_LOG_DAEMON 2 
+#define BRISTOL_LOG_SYSLOG 3
+#define BRISTOL_LOG_CONSOLE 4
+
 /*
  * Global limits
  */
@@ -62,8 +71,12 @@
 #define BRISTOL_ALSA_SEQ 3
  */
 
-#define BRISTOL_MIDI_TERMINATE 0x80000000
-#define BRISTOL_MIDI_INITTED 0x40000000
+#define BRISTOL_MIDI_TERMINATE	0x80000000
+#define BRISTOL_MIDI_INITTED	0x40000000
+#define BRISTOL_MIDI_FORWARD	0x20000000
+#define BRISTOL_MIDI_FHOLD		0x10000000
+#define BRISTOL_MIDI_GO			0x08000000
+#define BRISTOL_BMIDI_DEBUG		0x04000000
 
 /*
  * Channel state flags
@@ -95,6 +108,7 @@ typedef struct BristolMidiHandle {
 #if (BRISTOL_HAS_ALSA == 1)
 typedef struct BristolALSADev {
 	snd_rawmidi_t *handle; /* ALSA driver handle */
+//	snd_seq_t *seq_handle;
 } bristolALSADev;
 
 typedef struct BristolSeqDev {
@@ -125,7 +139,7 @@ typedef struct BristolMidiDev {
 		 * And descriptor types for any other dev libraries.
 		 */
 	} driver;
-	unsigned char buffer[BRISTOL_MIDI_BUFSIZE];
+	unsigned char buffer[BRISTOL_MIDI_BUFSIZE * 2];
 	int bufcount;
 	int bufindex;
 	bristolMidiChannel I_channel[BRISTOL_MIDI_CHANNELS];
@@ -135,21 +149,28 @@ typedef struct BristolMidiDev {
 
 typedef struct BristolMidiMain {
 	unsigned int flags;
+	unsigned int SysID;
 	bristolMidiDev dev[BRISTOL_MIDI_DEVCOUNT];
 	bristolMidiHandle handle[BRISTOL_MIDI_HANDLES];
+	int (*msgforwarder)();
 /*	int GM2values[128][16]; // Values all controllers by channel */
 /*	int mapping[128][16]; // default midi conntroller mapping table. */
 } bristolMidiMain;
 
 extern int bristolGetMidiFD(int);
-extern int bristolMidiDupFD();
 extern int bristolMidiDevRead(int, bristolMidiMsg *);
+extern int bristolMidiTCPRead(bristolMidiMsg *);
+extern void bristolMidiPost(bristolMidiMsg *);
 extern int bristolMidiSendNRP(int, int, int, int);
 extern int bristolMidiSendRP(int, int, int, int);
 extern int bristolMidiSendMsg(int, int, int, int, int);
 extern int bristolMidiSendKeyMsg(int, int, int, int, int);
 
+extern int bristolMidiControl(int, int, int, int, int);
+
 extern int bristolMidiOption(int, int, int);
+extern void bristolMidiRegisterForwarder(int (*)());
+
 
 extern char *getBristolCache(char *);
 #define NO_INTERPOLATE 0x01
@@ -158,6 +179,8 @@ extern int bristolGetFreqMap(char *, char *, fTab *, int, int, int);
 
 extern int bristolParseScala(char *, float *);
 
+extern pthread_t bristolOpenStdio(int);
+
 extern void bristolMidiValueMappingTable(u_char [128][128], int [128], char *);
 extern void bristolMidiToGM2(int [128], int [128], u_char [128][128], bristolMidiMsg *);
 extern int bristolMidiRawToMsg(unsigned char *, int, int, int, bristolMidiMsg *);
