@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -40,6 +39,7 @@ extern int bristolMidiRawWrite();
 extern int bristolMidiWrite();
 
 extern void bristolMidiPrint();
+extern void bristolMsgPrint();
 extern void bristolMidiPrintGM2();
 
 #define MIDI_CONTROLLER_COUNT 128
@@ -57,17 +57,19 @@ extern void bristolMidiPrintGM2();
 #define BRISTOL_MIDI_CHANNEL -5
 
 /* Conns are masked with 0x0ff0 */
-#define BRISTOL_CONN_UNIX		0x010
-#define BRISTOL_CONN_TCP		0x020
-#define BRISTOL_CONN_MIDI		0x040
-#define BRISTOL_CONN_OSSMIDI	0x080 /* As opposed to ALSA */
-#define BRISTOL_CONN_SEQ		0x100 /* ALSA Sequencer interface */
-#define BRISTOL_CONN_JACK		0x200 /* Jack Midi interface */
+#define BRISTOL_CONN_UNIX		0x00000010
+#define BRISTOL_CONN_TCP		0x00000020
+#define BRISTOL_CONN_MIDI		0x00000040
+#define BRISTOL_CONN_OSSMIDI	0x00000080 /* As opposed to ALSA */
+#define BRISTOL_CONN_SEQ		0x00000100 /* ALSA Sequencer interface */
+#define BRISTOL_CONN_JACK		0x00000200 /* Jack Midi interface */
 
-#define BRISTOL_CONN_PASSIVE 0x1000 /* listen socket. */
-#define BRISTOL_CONN_FORCE 0x2000
+#define BRISTOL_CONN_PASSIVE	0x00001000 /* listen socket. */
+#define BRISTOL_CONN_FORCE		0x00002000
 
-#define BRISTOL_CONN_NBLOCK 0x4000
+#define BRISTOL_CONN_NBLOCK		0x00004000
+#define BRISTOL_CONN_SYSEX		0x00008000
+#define BRISTOL_CONN_FORWARD	0x00010000
 
 /*
  * API flags
@@ -76,9 +78,9 @@ extern void bristolMidiPrintGM2();
 #define BRISTOL_WRONLY 2
 #define BRISTOL_DUPLEX 3
 
-#define BRISTOL_ACCEPT_SOCKET 0x80000000
-#define BRISTOL_CONTROL_SOCKET 0x40000000
-#define _BRISTOL_MIDI_DEBUG 0x20000000
+#define BRISTOL_ACCEPT_SOCKET	0x80000000
+#define BRISTOL_CONTROL_SOCKET	0x40000000
+#define _BRISTOL_MIDI_DEBUG		0x20000000
 
 /*
  * Midi command codes
@@ -134,6 +136,13 @@ extern void bristolMidiPrintGM2();
 #define BRISTOL_NRP_MNL_PREF	16375 /* = NRP 0x7f-77 = 0x3ff7 */
 #define BRISTOL_NRP_MNL_TRIG	16374 /* = NRP 0x7f-76 = 0x3ff6 */
 #define BRISTOL_NRP_MNL_VELOC	16373 /* = NRP 0x7f-75 = 0x3ff5 */
+#define BRISTOL_NRP_FORWARD		16372 /* = NRP 0x7f-74 = 0x3ff4 */
+#define BRISTOL_NRP_MIDI_GO		16371 /* = NRP 0x7f-73 = 0x3ff3 */
+#define BRISTOL_NRP_SYSID_H		16370 /* = NRP 0x7f-72 = 0x3ff2 */
+#define BRISTOL_NRP_SYSID_L		16369 /* = NRP 0x7f-71 = 0x3ff1 */
+#define BRISTOL_NRP_REQ_SYSEX	16368 /* = NRP 0x7f-70 = 0x3ff0 */
+#define BRISTOL_NRP_REQ_FORWARD	16367 /* = NRP 0x7f-6f = 0x3fef */
+#define BRISTOL_NRP_REQ_DEBUG	16366 /* = NRP 0x7f-6e = 0x3fee */
 
 #define MIDI_SYSTEM		0x0f0
 #define MIDI_SYSEX		0x0f8
@@ -151,13 +160,21 @@ extern void bristolMidiPrintGM2();
 #define BRISTOL_REQ_ALL		0x0ff
 #define BRISTOL_REQ_NSX		0x07f /* not sysex messages....... */
 
-#define MIDI_GM_DATAENTRY	38
+#define MIDI_GM_DATAENTRY	6
 #define MIDI_GM_NRP			99
 #define MIDI_GM_RP			101
+#define MIDI_GM_DATAENTRY_F	38
+#define MIDI_GM_NRP_F		98
+#define MIDI_GM_RP_F		100
+
+#define BRISTOL_KF_RAW		0
+#define BRISTOL_KF_TCP		1
+#define BRISTOL_KF_JACK		2
 
 typedef struct KeyMsg {
 	unsigned char key;
 	unsigned char velocity;
+	unsigned char flags;
 } keyMsg;
 
 typedef struct ChanPressMsg {
@@ -194,6 +211,7 @@ typedef struct BristolMidiMsg {
 	unsigned char mychannel;
 	unsigned char command;
 	struct timeval timestamp;
+	int offset;
 	unsigned int sequence;
 	union {
 		keyMsg key;
@@ -203,6 +221,7 @@ typedef struct BristolMidiMsg {
 		controlMsg controller;
 		programMsg program;
 		bristolMsg bristol;
+		bristolMsgType2 bristolt2;
 		sysexMsg *sysex;
 	} params;
 	/*
