@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -54,20 +53,21 @@
 #define SLAB_FULL_DUPLEX	13 /* 0xD */
 #define SLAB_HALF_DUPLEX	14 /* 0xE */
 #define SLAB_HOLD			15 /* 0xF */
-#define SLAB_FLAG_MASK		0x0f
+#define SLAB_FLAG_MASK		0x000f
 
-#define SLAB_NO_CONTROLS	0x10 /* prevent /dev/audio or /dev/audioctl ioctl */
+#define SLAB_NO_CONTROLS	0x0010 /* prevent /dev/audio or /dev/audioctl ioctl */
 /* SB_DUPLEX has taken the 0x20 flag position */
-#define SLAB_8_BIT_IN		0x40 /* For general convergence options */
-#define SLAB_8_BIT_OUT		0x80 /* For general convergence options */
+#define SLAB_8_BIT_IN		0x0040 /* For general convergence options */
+#define SLAB_8_BIT_OUT		0x0080 /* For general convergence options */
 #ifdef SUBFRAGMENT
-#define SLAB_SUBFRAGMENT	0x100 /* decouple blockSampleSize from fragemnts */
+#define SLAB_SUBFRAGMENT	0x0100 /* decouple blockSampleSize from fragemnts */
 #define SLAB_SUBF_IOCTL		0x2000 /* Allow subfragment to use old ioctl()s */
 #endif
 #ifdef IOCTL_DBG
-#define SLAB_AUDIODBG	0x200 /* For ioctl() debug info */
+#define SLAB_AUDIODBG	0x0200 /* For ioctl() debug info */
 #endif
-#define SLAB_FDUP		0x400 /* For ioctl() debug info */
+#define SLAB_FDUP		0x0400
+#define SLAB_AUDIODBG2	0x80000000 /* For verbose debug info */
 
 #ifdef SB_ONR
 #define SLAB_NO_ONR	1
