@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -25,7 +24,6 @@
 /*
  * This should be set up according to system type.
  */
-#if (MACHINE == FreeBSD || MACHINE == linux)
 #ifndef INC_SOURCE
 #define SLAB_NULL			-1
 #define SLAB_TREBLE			SOUND_MIXER_TREBLE
@@ -94,7 +92,6 @@
 #define SLAB_VOL_PCM_MASK		1<<SLAB_VOL_PCM
 
 #define SLAB_REC_SRC	0xff
-#endif
 
 /*
  * These allow the same parameters to be used for either 0-100 ranges (this
