@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -75,20 +74,6 @@ typedef int sig_type;
 #define	O_RDWR		2
 #endif
 
-/* define types of library functions */
-
-/* char 	*malloc(), *calloc(); */
-char 	*getenv(), *ctime();
-#if (MACHINE == FreeBSD)
-long 	ftell();
-#else
-off_t ftell(), tell();
-#endif
-off_t 	lseek();
-time_t 	time();
-int	atoi();
-long	atol();
-
 /*
  * The following are for the audio net tap
  */
@@ -96,7 +81,6 @@ long	atol();
 #define SLAB_OUTPUT_TAP 2
 
 /* define types of own functions */
-
 char *mk_file_name(), *home_relative();
 char *date_time(), *user_name();
 char *copy_str();
