@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -458,7 +457,7 @@ cacheFindColor(unsigned short r, unsigned short g, unsigned short b, int cm)
 	c_row *row;
 	cc_entry *entry;
 	int j = 0;
-	float d = 0;
+	int d = 0;
 	unsigned short mask = c_table.mask; // 0xffff << (16 - cm);
 
 	if ((r == 0) && (g == 0) && (b == 0xff00))
@@ -495,7 +494,8 @@ cacheFindColor(unsigned short r, unsigned short g, unsigned short b, int cm)
 	 */
 	for (j = row->start; j >= 0; j = entry[j].next)
 	{
-		d+=1.0;
+		if ((d++) > 10000)
+			return(-1);
 
 		if (entry[j].SCAN_G == SCAN_G)
 			break;
@@ -526,7 +526,9 @@ cacheFindColor(unsigned short r, unsigned short g, unsigned short b, int cm)
 	 */
 	for (; j >= 0; j = entry[j].next)
 	{
-		d+=1.0;
+		if ((d++) > 10000)
+			return(-1);
+
 		/* Check green still matches */
 		if (entry[j].SCAN_G != SCAN_G)
 		{
