@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -91,7 +90,7 @@ static void
 brightonDestroyDevices(brightonDevice *device)
 {
 #warning we return early here, that may have to be resolved
-printf("destroying %x\n", (size_t) device);
+printf("destroying %p\n", device);
 return;
 	if ((device == 0) || (device->device == -1))
 		return;
@@ -171,23 +170,25 @@ brightonEvent *event)
 {
 	char *image = "bitmaps/images/cable.xpm";
 
-	if ((index < 0) && (event->type != BRIGHTON_EXPOSE))
-		return(-1);
-	if (panel < 0)
-		return(-1);
-	if (bwin == NULL)
+	if (((index < 0) && (event->type != BRIGHTON_EXPOSE))
+		|| (panel < 0)
+		|| (bwin == NULL))
 		return(-1);
 
 	if (panel >= bwin->app->nresources)
 	{
-		printf("panel count %i over %i\n", panel, bwin->app->nresources);
+		if (bwin->flags & BRIGHTON_DEBUG)
+			printf("panel count %i over %i\n", panel, bwin->app->nresources);
 		return(-1);
 	}
 
 	if (index >= bwin->app->resources[panel].ndevices)
 	{
+		/*
+		 * Drop this debug
 		printf("index %i over panel count %i (transposed keyboard?)\n",
 			index, bwin->app->resources[panel].ndevices);
+		 */
 		return(-1);
 	}
 
