@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -37,9 +36,22 @@ int gs)
 	/*
 	 * Connect to the display.
 	 */
-	if ((display = brightonOpenDisplay(NULL)) == 0)
+#ifdef BRIGHTON_HAS_X11
+	if (app->flags & BRIGHTON_WINDOW)
+	{
+		printf("brightonInterface(cli)\n");
+		display = brightonOpenDisplay("cli");
+		display->flags |= _BRIGHTON_WINDOW;
+	} else if ((display = brightonOpenDisplay(NULL)) == 0) {
+		printf("brightonInterface() failed\n");
 		return(0);
-
+	}
+#else
+	app->flags |= BRIGHTON_WINDOW;
+	printf("brightonInterface(cli)\n");
+	display = brightonOpenDisplay("cli");
+	display->flags |= _BRIGHTON_WINDOW;
+#endif
 	/*
 	 * Link it into our list.
 	 */
@@ -49,8 +61,17 @@ int gs)
 	dlist = display;
 
 	/* This should be made into an option and needs to be stuffed early */
-	if (library != 0)
+	if ((library != 0) && (display->flags & BRIGHTON_BIMAGE))
 		display->flags |= BRIGHTON_BIMAGE;
+	else
+		display->flags &= ~BRIGHTON_BIMAGE;
+
+	if (aa & BRIGHTON_LIB_DEBUG)
+	{
+		printf("libbrighton debuging enabled\n");
+		display->flags |= BRIGHTON_LIB_DEBUG;
+	}
+	aa &= ~BRIGHTON_LIB_DEBUG;
 
 	switch (aa) {
 		case 1:
@@ -79,6 +100,9 @@ int gs)
 		BRIGHTON_DEFAULT_ICON,
 		quality, gs)))
 	{
+		if (display->flags & BRIGHTON_LIB_DEBUG)
+			display->bwin->flags |= BRIGHTON_DEBUG;
+
 		if ((display->bwin->quality = quality) < 2)
 			display->bwin->quality = 2;
 		else if (display->bwin->quality > 8)
@@ -115,7 +139,9 @@ brightonRemoveInterface(brightonWindow *bwin)
 {
 	brightonDisplay *d;
 
-printf("brightonRemoveInterface(%x)\n", (size_t) bwin);
+printf("brightonRemoveInterface(%p)\n", bwin);
+
+	bwin->flags |= BRIGHTON_EXITING;
 
 	if ((d = brightonFindDisplay(dlist, (brightonDisplay *) bwin->display))
 		== 0)
@@ -124,6 +150,7 @@ printf("brightonRemoveInterface(%x)\n", (size_t) bwin);
 	if (bwin->template->destroy != 0)
 		bwin->template->destroy(bwin);
 
+	BAutoRepeat(bwin->display, 1);
 	brightonDestroyInterface(bwin);
 
 	/*
