@@ -2,12 +2,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -16,8 +16,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -46,7 +45,7 @@ int a, int b, int c, int d)
 	float x1, y1, x2, y2;
 	float xstep, ystep, srcx;
 	float length, height, shearx;
-	int offset, cdown;
+	int offset, cdown, p;
 
 	/*
 	 * This will not be a simple rotation. The bitmap needs to be stretched 
@@ -107,10 +106,13 @@ int a, int b, int c, int d)
 				if (isblue(offset, bwin->display->palette, src->pixels))
 					continue;
 
-				dst->pixels[(int) ((y1 + ystep
+				if ((p = ((int) ((y1 + ystep
 					+ ((int) ((height * xstep)/length)))
-					* dst->width + x1 + xstep)]
-						= src->pixels[offset];
+					* dst->width + x1 + xstep)))
+					> dst->width * dst->height)
+					continue;
+
+				dst->pixels[p] = src->pixels[offset];
 			}
 		}
 	} else {
@@ -165,9 +167,11 @@ int a, int b, int c, int d)
 				if (isblue(offset, bwin->display->palette, src->pixels))
 					continue;
 
-				dst->pixels[(int) ((y1 + ystep) * dst->width
-					+ x1 + xstep + oy)]
-						= src->pixels[offset];
+				if ((p = ((int) ((y1 + ystep) * dst->width
+					+ x1 + xstep + oy))) >= dst->width * dst->height)
+					continue;
+
+				dst->pixels[p] = src->pixels[offset];
 			}
 		}
 	}
@@ -347,20 +351,23 @@ brightonRemove(brightonWindow *bwin, int id)
 		return(0);
 
 	bwin->items[id].id = 0;
+
+	a = bwin->items[id].x;
+	b = bwin->items[id].y;
+	c = bwin->items[id].w;
+	d = bwin->items[id].h;
+
 	brightonInitBitmap(bwin->tlayer, -1);
 
 	for (i = 0; i < BRIGHTON_ITEM_COUNT; i++)
+	{
 		if (bwin->items[i].id > 0)
 			brightonSRotate(bwin, bwin->items[i].image, bwin->tlayer,
 				bwin->items[i].x, 
 				bwin->items[i].y, 
 				bwin->items[i].w, 
 				bwin->items[i].h);
-
-	a = bwin->items[id].x;
-	b = bwin->items[id].y;
-	c = bwin->items[id].w;
-	d = bwin->items[id].h;
+	}
 
 	if (c < a) {
 		a = c;
@@ -379,7 +386,7 @@ brightonRemove(brightonWindow *bwin, int id)
 	return(0);
 }
 
-static char tImage[256];
+static char tImage[1024];
 
 /*
  * This is called for world changes
@@ -394,28 +401,34 @@ brightonRePlace(brightonWindow *bwin)
 	{
 		if (bwin->items[id].id > 0)
 		{
-			scale = ((float) bwin->width) / ((float) bwin->items[id].scale);
 			flags = bwin->items[id].flags;
-			w = ((float) bwin->items[id].w) * scale;
-			h = ((float) bwin->items[id].h) * scale;
-			x = ((float) bwin->items[id].x) * scale;
-			y = ((float) bwin->items[id].y) * scale;
+			if (flags & BRIGHTON_LAYER_ALL) {
+				x = y = 0;
+				w = bwin->width;
+				h = bwin->height;
+			} else {
+				scale = ((float) bwin->width) / ((float) bwin->items[id].scale);
+				w = ((float) bwin->items[id].w) * scale;
+				h = ((float) bwin->items[id].h) * scale;
+				x = ((float) bwin->items[id].x) * scale;
+				y = ((float) bwin->items[id].y) * scale;
+			}
 
 			sprintf(tImage, "%s", bwin->items[id].image->name);
 
 			/* Remove this, it will result in the rest being repainted */
-			brightonRemove(bwin, id);
 
 			if (flags & BRIGHTON_LAYER_PLACE)
-				brightonPlace(bwin, &tImage[0], x, y, w, h);
-			else
-				brightonPut(bwin, &tImage[0], x, y, w, h);
+			{
+				brightonRemove(bwin, id);
+				brightonPlace(bwin, tImage, x, y, w, h);
+			} else {
+				bwin->items[id].id = 0;
+				brightonPut(bwin, tImage, x, y, w, h);
+			}
 		}
 	}
 
-	if (id == BRIGHTON_ITEM_COUNT)
-		return(0);
-
 	return(0);
 }
 
