@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -43,18 +42,20 @@ int
 brightonDevUndraw(brightonWindow *bwin, brightonBitmap *dest, int ix, int iy,
 int w, int h)
 {
-	int x, y, dy;
+	int x, y, z, dy, s = dest->width * dest->height;
 
 /*printf("undraw %x %x %x, %i %i %i %i\n", */
 /*bwin, dest, dest->pixels, ix, iy, w, h); */
 
-	for (y = iy ;y < (iy + h); y++)
+	for (y = iy; y <= (iy + h - 1); y++)
 	{
 		dy = y * dest->width;
 
-		for (x = ix;x < (ix + w); x++)
+		for (x = ix; x <= (ix + w - 1); x++)
 		{
-			dest->pixels[dy + x] = -1;
+			if (((z = dy + x) < 0) || (z > s))
+				continue;
+			dest->pixels[z] = -1;
 		}
 	}
 	return(0);
@@ -533,7 +534,7 @@ brightonDestroyInterface(brightonWindow *bwin)
 {
 	int i;
 
-printf("brightonDestroyInterface(%x): %i\n", (size_t) bwin, bwin->app->nresources);
+	printf("brightonDestroyInterface(%p): %i\n", bwin, bwin->app->nresources);
 
 	for (i = 0; i < bwin->app->nresources; i++)
 	{
