@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 /*
@@ -27,8 +26,6 @@
 
 #include "brightoninternals.h"
 
-extern int brightonRegisterController();
-
 int
 destroyScale(brightonDevice *dev)
 {
@@ -117,10 +114,10 @@ displayscale(brightonDevice *dev)
 			 */
 			brightonFinalRender(dev->bwin,
 				dev->x + dev->lastposition
-					+ dev->bwin->app->resources[dev->panel].sx - 5,
+					+ dev->bwin->app->resources[dev->panel].sx - 7,
 				dev->y + dev->bwin->app->resources[dev->panel].sy,
 				dev->position - dev->lastposition + (dev->width >> 2) + 5,
-				dev->height + 7);
+				dev->height + 9);
 		} else {
 			brightonFinalRender(dev->bwin,
 				dev->x + dev->position
@@ -135,8 +132,8 @@ displayscale(brightonDevice *dev)
 			brightonDevUndraw(dev->bwin, dev->bwin->dlayer,
 				dev->x + dev->bwin->app->resources[dev->panel].sx,
 				dev->y + ((int) dev->lastposition)
-					+ dev->bwin->app->resources[dev->panel].sy,
-				dev->width, dev->height / 4);
+					+ dev->bwin->app->resources[dev->panel].sy - 1,
+				dev->width, dev->height / 4 + 1);
 			brightonRenderShadow(dev, 1);
 		}
 
@@ -403,6 +400,35 @@ configure(brightonDevice *dev, brightonEvent *event)
 						dev->value -= ((float) 1) / 16384;
 				}
 				break;
+			case 0xff51:
+				if (dev->bwin->app->resources[dev->panel].devlocn[dev->index].flags & BRIGHTON_VERTICAL)
+				{
+					if (dev->flags & BRIGHTON_SHIFTKEY)
+						dev->value -= ((float) 2048) / 16384;
+					else
+						dev->value -= ((float) 32) / 16384;
+				} else {
+					if (dev->flags & BRIGHTON_SHIFTKEY)
+						dev->value += ((float) 2048) / 16384;
+					else
+						dev->value += ((float) 32) / 16384;
+				}
+				break;
+			case 0xff53:
+				/*if (dev->flags & BRIGHTON_VERTICAL) */
+				if (dev->bwin->app->resources[dev->panel].devlocn[dev->index].flags & BRIGHTON_VERTICAL)
+				{
+					if (dev->flags & BRIGHTON_SHIFTKEY)
+						dev->value += ((float) 2048) / 16384;
+					else
+						dev->value += ((float) 32) / 16384;
+				} else {
+					if (dev->flags & BRIGHTON_SHIFTKEY)
+						dev->value -= ((float) 2048) / 16384;
+					else
+						dev->value -= ((float) 32) / 16384;
+				}
+				break;
 		}
 
 		considercallback(dev);
