@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -72,8 +71,16 @@ int cmapsize, int flags, int quality, int gs)
 	}
 	bwin->aspect = ((float) bwin->width) / bwin->height;
 
+	if (app->flags & BRIGHTON_POST_WINDOW)
+		bwin->flags |= _BRIGHTON_POST;
+
 	if (BOpenWindow(display, bwin, app->name) == 0)
-		clearout();
+	{
+		brightonfree(bwin);
+		bwin = 0;
+		clearout(-1);
+		return(0);
+	}
 
 	bwin->flags |= BRIGHTON_ACTIVE;
 
@@ -97,7 +104,8 @@ brightonDestroyWindow(brightonWindow *bwin)
 {
 	brightonBitmap *bitmap;
 
-printf("brightonDestroyWindow()\n");
+	printf("brightonDestroyWindow()\n");
+
 	BFlush((brightonDisplay *) bwin->display);
 
 	BCloseWindow((brightonDisplay *) bwin->display, bwin);
