@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -27,10 +26,14 @@ BFreeColor(brightonDisplay *display, brightonPalette *color)
 {
 	bdisplay *bd = (bdisplay *) display->display;
 
-	XFreeColors(bd->display, bd->cm, (unsigned long *) &(color->pixel), 1, 0);
+	if (~display->flags & _BRIGHTON_WINDOW)
+	{
+		XFreeColors(bd->display, bd->cm,
+			(unsigned long *) &(color->pixel), 1, 0);
 	 
-	if (color->gc != NULL)
-		XFreeGC(bd->display, color->gc);
+		if (color->gc != NULL)
+			XFreeGC(bd->display, color->gc);
+	}
 
 	color->flags &= ~B_ALLOCATED;
 
@@ -56,7 +59,8 @@ BAllocColorByName(brightonDisplay *display, brightonPalette *color, char *name)
 
 	screen = (XColor *) brightonX11malloc(sizeof(XColor));
 
-	XAllocNamedColor(bd->display, bd->cm, name, screen, &exact);
+	if (~display->flags & _BRIGHTON_WINDOW)
+		XAllocNamedColor(bd->display, bd->cm, name, screen, &exact);
 
 	color->red = exact.red;
 	color->green = exact.green;
@@ -84,7 +88,7 @@ unsigned short r, unsigned short g, unsigned short b)
 	bdisplay *bd = (bdisplay *) display->display;
 	unsigned long valuemask = GCForeground;
 	XGCValues values;
-	XColor *screen;
+	XColor *screen = 0;
 
 	if (color->color == 0)
 	{
@@ -94,7 +98,8 @@ unsigned short r, unsigned short g, unsigned short b)
 		screen->blue = b;
 		screen->flags = DoRed|DoGreen|DoBlue;
 
-		XAllocColor(bd->display, bd->cm, screen);
+		if (~display->flags & _BRIGHTON_WINDOW)
+			XAllocColor(bd->display, bd->cm, screen);
 
 		color->color = screen;
 		color->pixel = screen->pixel;
@@ -103,8 +108,10 @@ unsigned short r, unsigned short g, unsigned short b)
 
 	values.foreground = screen->pixel;
 
-	color->gc = XCreateGC(bd->display,
-		(Window) ((brightonWindow *) display->bwin)->win, valuemask, &values);
+	if (~display->flags & _BRIGHTON_WINDOW)
+		color->gc = XCreateGC(bd->display,
+			(Window) ((brightonWindow *) display->bwin)->win,
+				valuemask, &values);
 
 	color->flags |= B_ALLOCATED;
 
@@ -171,7 +178,8 @@ unsigned short r, unsigned short g, unsigned short b)
 		screen->blue = b;
 		screen->flags = DoRed|DoGreen|DoBlue;
 
-		XAllocColor(bd->display, bd->cm, screen);
+		if (~display->flags & _BRIGHTON_WINDOW)
+			XAllocColor(bd->display, bd->cm, screen);
 
 		color->color = screen;
 		color->pixel = screen->pixel;
@@ -195,6 +203,9 @@ BInitColorMap(brightonDisplay *display)
 		return(0);
 	}
 
+	if (display->flags & _BRIGHTON_WINDOW)
+		return(display->palette);
+
 	bd->cm = DefaultColormap(bd->display, bd->screen_num);
 
 	if (!XMatchVisualInfo(bd->display, bd->screen_num, bd->depth,
