@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -52,22 +51,33 @@ bdisplay *
 BOpenDisplay(brightonDisplay *bd, char *displayname)
 {
 	bdisplay *display, *exists;
+	int ign, major, minor;
+	Bool pixmaps;
 
 	display = (bdisplay *) brightonX11malloc(sizeof(bdisplay));
 
-	if ((exists = bFindDisplayByName(displays, displayname)) != 0)
+	//printf("BOpenDisplay(%s)\n", displayname);
+
+	if (strcmp(displayname, "cli") == 0)
 	{
-		printf("reusing display %s\n", displayname);
-		exists->uses++;
-		display->uses++;
-		bcopy(exists, display, sizeof(bdisplay));
-	} else {
-		if ((display->display = XOpenDisplay(displayname)) == NULL)
-		{
-			printf("cannot connect to %s\n", XDisplayName(displayname));
-			return(0);
-		}
+		display->display = (void *) 0xdeadbeef;
+		display->flags |= _BRIGHTON_WINDOW;
 		display->uses = 1;
+	} else {
+		if ((exists = bFindDisplayByName(displays, displayname)) != 0)
+		{
+			printf("reusing display %s\n", displayname);
+			exists->uses++;
+			display->uses++;
+			bcopy(exists, display, sizeof(bdisplay));
+		} else {
+			if ((display->display = XOpenDisplay(displayname)) == NULL)
+			{
+				printf("cannot connect to %s\n", XDisplayName(displayname));
+				return(0);
+			}
+			display->uses = 1;
+		}
 	}
 
 	/*
@@ -78,19 +88,45 @@ BOpenDisplay(brightonDisplay *bd, char *displayname)
 		displays->last = display;
 	displays = display;
 
-	printf("connected to %s\n", XDisplayName(displayname));
-
 	sprintf(display->name, "%s", displayname);
 
-	display->width = DisplayWidth(display->display, display->screen_num);
-	display->height = DisplayHeight(display->display, display->screen_num);
-	display->screen_num = DefaultScreen(display->display);
-	display->screen_ptr = DefaultScreenOfDisplay(display->display);
+	if (~display->flags & _BRIGHTON_WINDOW)
+	{
+		printf("connected to %s\n", XDisplayName(displayname));
+
+		display->width = DisplayWidth(display->display, display->screen_num);
+		display->height = DisplayHeight(display->display, display->screen_num);
+		display->screen_num = DefaultScreen(display->display);
+		display->screen_ptr = DefaultScreenOfDisplay(display->display);
+	} else
+		printf("not connected to display: cli\n");
 
 	bd->width = display->width;
 	bd->height = display->height;
 	bd->depth = display->depth;
 
+#ifdef BRIGHTON_XIMAGE
+#ifdef BRIGHTON_SHMIMAGE
+   	/*
+	 * Check for the XShm extension - mark that we want to use them
+	 */
+	bd->flags |= BRIGHTON_BIMAGE;
+   	if (XQueryExtension(display->display, "MIT-SHM", &ign, &ign, &ign))
+	{
+   		if (XShmQueryVersion(display->display, &major, &minor, &pixmaps)
+			== True)
+		{
+   			printf("XShm extention version %d.%d %s shared pixmaps\n",
+	   			major, minor, (pixmaps==True) ? "with" : "without");
+
+//			if (pixmaps != True)
+//				display->flags &= ~BRIGHTON_BIMAGE;
+   		} else
+			bd->flags &= ~BRIGHTON_BIMAGE;
+   	}
+#endif
+#endif
+
 	return(display);
 }
 
@@ -110,7 +146,8 @@ BCloseDisplay(bdisplay *display)
 		/*
 		 * Free the display, disconnect, etc.
 		 */
-		XCloseDisplay(d->display);
+		if (~display->flags & _BRIGHTON_WINDOW)
+			XCloseDisplay(d->display);
 	}
 
 	/*
