@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -33,8 +32,8 @@ int command[BLASTEvent] = {
     BRIGHTON_MOTION,
     BRIGHTON_ENTER,
     BRIGHTON_LEAVE,
-    BRIGHTON_NONE,
-    BRIGHTON_NONE,
+    BRIGHTON_FOCUS_IN, /* FocusIn */
+    BRIGHTON_FOCUS_OUT, /* FocusOut */
     BRIGHTON_NONE,
     BRIGHTON_EXPOSE,
     BRIGHTON_GEXPOSE,
@@ -67,7 +66,7 @@ BSendEvent(brightonDisplay *display, brightonWindow *bwin, brightonEvent *event)
 	bdisplay *bd = (bdisplay *) display->display;
 	XEvent xevent;
 
-	if (bwin->flags & BRIGHTON_BUSY)
+	if ((bwin->flags & BRIGHTON_BUSY) || (display->flags & _BRIGHTON_WINDOW))
 		return(0);
 
 	xevent.xany.type = KeyPress;
@@ -85,8 +84,6 @@ BSendEvent(brightonDisplay *display, brightonWindow *bwin, brightonEvent *event)
 	return(0);
 }
 
-int print = 0;
-
 int
 OldBNextEvent(brightonDisplay *display, brightonEvent *event)
 {
@@ -100,19 +97,6 @@ OldBNextEvent(brightonDisplay *display, brightonEvent *event)
 	event->command = command[xevent.xany.type];
 
 printf("event %i: %x\n", xevent.xany.type, event->wid);
-/*printf("Event %i: %x\n", event->type, xevent.xany.window); */
-
-/*if (event->command == BRIGHTON_DESTROY) */
-/*print = 1; */
-
-/*if (print) */
-/*{ */
-/*printf("	%i: send: %x, me: %x, disp: %x\n", */
-/*xevent.xany.type, */
-/*xevent.xany.send_event, xevent.xany.window, xevent.xany.display); */
-/*if (event->command == BRIGHTON_DESTROY) */
-/*printf("	destroyed: %x\n", xevent.xdestroywindow.event); */
-/*} */
 
 	switch (xevent.xany.type) {
 		case KeyPress:
@@ -167,6 +151,9 @@ BNextEvent(brightonDisplay *display, brightonEvent *event)
 	bdisplay *bd = (bdisplay *) display->display; 
 	XEvent xevent;
 
+	if (display->flags & _BRIGHTON_WINDOW)
+		return(0);
+
 	/*
 	 * This is a bit of overkill, I want to just use NextEvent however I need
 	 * to also get some notifications from the midi control channel into this
@@ -183,27 +170,13 @@ BNextEvent(brightonDisplay *display, brightonEvent *event)
 		event->wid = xevent.xany.window;
 		event->command = command[xevent.xany.type];
 
-/*printf("event %i: %x\n", xevent.xany.type, event->wid); */
-/*printf("Event %i: %x\n", event->type, xevent.xany.window); */
-
-/*if (event->command == BRIGHTON_DESTROY) */
-/*print = 1; */
-
-/*if (print) */
-/*{ */
-/*printf("	%i: send: %x, me: %x, disp: %x\n", */
-/*xevent.xany.type, */
-/*xevent.xany.send_event, xevent.xany.window, xevent.xany.display); */
-/*if (event->command == BRIGHTON_DESTROY) */
-/*printf("	destroyed: %x\n", xevent.xdestroywindow.event); */
-/*} */
-
 		switch (xevent.xany.type) {
 			case KeyPress:
 			case KeyRelease:
 				event->x = xevent.xkey.x;
 				event->y = xevent.xkey.y;
 				event->key = XLookupKeysym(&xevent.xkey, 0);
+				event->flags = xevent.xkey.state;
 				/*
 				 * Left shift for right shift
 				 */
@@ -235,6 +208,9 @@ BNextEvent(brightonDisplay *display, brightonEvent *event)
 				event->x = xevent.xmotion.x;
 				event->y = xevent.xmotion.y;
 				break;
+			case FocusIn:
+			case FocusOut:
+				break;
 			case ConfigureNotify:
 				event->x = xevent.xconfigure.x;
 				event->y = xevent.xconfigure.y;
