@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,13 +15,11 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "brightonX11internals.h"
-#include "brightoninternals.h"
 #include "icon_bitmap.xbm"
 
 /* We should bury this somewhere in the window */
@@ -34,6 +32,12 @@ BOpenWindow(brightonDisplay *display, brightonWindow *bwin, char *programme)
 {
 	bdisplay *bd = (bdisplay *) display->display;
 
+	if (display->flags & _BRIGHTON_WINDOW)
+	{
+		bd->flags |= _BRIGHTON_WINDOW;
+		return(bwin->win = (Window) 0xdeadbeef);
+	}
+
 	bwin->win = XCreateSimpleWindow(bd->display,
 		RootWindow(bd->display,
 		bd->screen_num),
@@ -55,8 +59,8 @@ BOpenWindow(brightonDisplay *display, brightonWindow *bwin, char *programme)
 		(unsigned int) icon_bitmap_height);
 
 	bd->size_hints.flags = PPosition|PSize|PMinSize;
-	bd->size_hints.min_width = 150;
-	bd->size_hints.min_height = 100;
+	bd->size_hints.min_width = 100;
+	bd->size_hints.min_height = 10;
 
 	bd->icon_name = programme;
 	bwin->window_name = programme;
@@ -65,7 +69,6 @@ BOpenWindow(brightonDisplay *display, brightonWindow *bwin, char *programme)
 	{
 		printf("%s: allocation error for icon failed\n", programme);
 		display->bwin = 0;
-		brightonfree(bwin);
 		return(0);
 	}
 	if (XStringListToTextProperty(&bwin->window_name, 1, &bd->windowName)
@@ -73,7 +76,6 @@ BOpenWindow(brightonDisplay *display, brightonWindow *bwin, char *programme)
 	{
 		printf("%s: allocation error for window failed\n", programme);
 		display->bwin = 0;
-		brightonfree(bwin);
 		return(0);
 	}
 
@@ -113,6 +115,9 @@ BOpenWindow(brightonDisplay *display, brightonWindow *bwin, char *programme)
 
 	XMapWindow(bd->display, bwin->win);
 
+	if (~bwin->flags & _BRIGHTON_POST)
+		XIconifyWindow(bd->display, bwin->win, bd->screen_num);
+
 	/* Incorrect location? Should maybe be in win structure? */
 	bwin->gc = DefaultGC(bd->display, bd->screen_num);
 
@@ -124,27 +129,53 @@ BCloseWindow(brightonDisplay *display, brightonWindow *bwin)
 {
 	bdisplay *bd = display->display;
 
-	XDestroyWindow(bd->display, bwin->win);
+	if (~display->flags & _BRIGHTON_WINDOW)
+		XDestroyWindow(bd->display, bwin->win);
 
 	return(0);
 }
 
+void
+BRaiseWindow(brightonDisplay *display, brightonWindow *bwin)
+{
+	bdisplay *bd = display->display;
+
+	if (~display->flags & _BRIGHTON_WINDOW)
+		XRaiseWindow(bd->display, bwin->win);
+}
+
+void
+BLowerWindow(brightonDisplay *display, brightonWindow *bwin)
+{
+	bdisplay *bd = display->display;
+
+	if (~display->flags & _BRIGHTON_WINDOW)
+		XLowerWindow(bd->display, bwin->win);
+}
+
 int
 BGetGeometry(brightonDisplay *display, brightonWindow *bwin)
 {
 	bdisplay *bd = display->display;
 
-	if (XGetGeometry(bd->display, RootWindow(bd->display, bd->screen_num),
-		&bd->root, (int *) (&bd->x), (int *) (&bd->y), &bd->width, &bd->height,
-		&bd->border, &bd->depth) < 0)
-		printf("cannot get root window geometry\n");
+	if (~bd->flags & _BRIGHTON_WINDOW)
+	{
+		if (XGetGeometry(bd->display, RootWindow(bd->display, bd->screen_num),
+			&bd->root, (int *) (&bd->x), (int *) (&bd->y), &bd->width,
+			&bd->height, &bd->border, &bd->depth) < 0)
+			printf("cannot get root window geometry\n");
 
-	/*
-	 * I would prefer for these to come from the bitmap itself.
-	 */
-	bwin->width = bd->width;
-	bwin->height = bd->height;
-	bwin->depth = bd->depth;
+		/*
+		 * I would prefer for these to come from the bitmap itself.
+		 */
+		bwin->width = bd->width;
+		bwin->height = bd->height;
+		bwin->depth = bd->depth;
+	} else {
+		bwin->width = 1024;
+		bwin->height = 768;
+		bwin->depth = 24;
+	}
 
 	return(0);
 }
@@ -154,7 +185,8 @@ BFlush(brightonDisplay *display)
 {
 	bdisplay *bd = (bdisplay *) display->display;
 
-	XFlush(bd->display);
+	if (~display->flags & _BRIGHTON_WINDOW)
+		XFlush(bd->display);
 
 	return(0);
 }
@@ -164,6 +196,9 @@ BAutoRepeat(brightonDisplay *display, int onoff)
 {
 	bdisplay *bd = display->display;
 
+	if (display->flags & _BRIGHTON_WINDOW)
+		return(0);
+
 	if (onoff) {
 		XAutoRepeatOn(bd->display);
 	} else {
