@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 #define DUPLICATE
@@ -82,8 +81,10 @@ bristolAudioOpen(char *device, int rate, int count, int flags)
 	/*
 	 * need to keep the as well, and need to reference the format to make sure
 	 * we get the correct buffer size. For now only shorts on output.
-	 */
+	 *
+	 * Can this warning, multichannel will be handled with Jack
 #warning Fix buf calc for formats and channels
+	 */
 	audioDev.fragSize = count * sizeof(short) * audioDev.channels;
 	audioDev.fragBuf = 0;
 	audioDev.OSegmentSize = audioDev.fragSize;
@@ -111,8 +112,8 @@ bristolAudioOpen(char *device, int rate, int count, int flags)
 		return(-1);
 
 	printf(
-		"opened audio device with a fragment size of %i, buffer %x, fd %i/%i\n",
-		audioDev.fragSize, (size_t) audioDev.fragBuf, audioDev.fd, audioDev.fd2);
+		"opened audio device with a fragment size of %i, buffer %p, fd %i/%i\n",
+		audioDev.fragSize, audioDev.fragBuf, audioDev.fd, audioDev.fd2);
 
 	return(audioDev.fragSize);
 }
@@ -131,9 +132,9 @@ bristolAudioWrite(register float *buf, register int count)
 	register short *audioBuf;
 	register int clipped = 0, result, Count = count;
 
-#ifdef DEBUG
-	printf("bristolAudioWrite(%x, %i), %i\n", buf, count, audioDev.samplecount);
-#endif
+	if (audioDev.flags & SLAB_AUDIODBG2)
+		printf("bristolAudioWrite(%p, %i), %i\n",
+			buf, count, audioDev.samplecount);
 
 	/*
 	 * Based on the assumption that we are always going to be working with
@@ -217,10 +218,10 @@ bristolAudioRead(register float *buf, register int count)
 {
 	register short *audioBuf;
 	register int count2 = count;
+	register float norm = 12.0f/32768.0f;
 
-#ifdef DEBUG
-	printf("bristolAudioRead(%i), %i\n", count, audioDev.samplecount);
-#endif
+	if (audioDev.flags & SLAB_AUDIODBG2)
+		printf("bristolAudioRead(%i), %i\n", count, audioDev.samplecount);
 
 	/*
 	 * Based on the assumption that we are always going to be working with
@@ -238,7 +239,8 @@ bristolAudioRead(register float *buf, register int count)
 
 	if (audioRead(&audioDev, audioDev.fragBuf, audioDev.samplecount) < 0)
 	{
-		printf("Read Failed: fs %i, %x\n", audioDev.fragSize, (size_t) audioDev.fragBuf);
+		printf("Read Failed: fs %i, %p\n",
+			audioDev.fragSize, audioDev.fragBuf);
 		return(-6);
 	}
 
@@ -246,32 +248,53 @@ bristolAudioRead(register float *buf, register int count)
 	 * Demultiplex channels.
 	 *
 	 * The count is samples, but data is in frames, ie, two samples for the
-	 * time being.
+	 * time being. I really want this data to be in a format that is +/-12,
+	 * that means we have to normalise it and we should do it here.
 	 */
 	for (; count > 0; count-=8)
 	{
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
 	}
 
 	audioBuf = ((short *) audioDev.fragBuf) - 1;
 
 	for (; count2 > 0; count2-=8)
 	{
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
-		*buf++ = (float) *(audioBuf+=2);
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+		*buf++ = ((float) *(audioBuf+=2)) * norm;
+	}
+
+	return(0);
+}
+
+/*
+ * This was added for enabling audio debug
+ */
+int
+bristolAudioOption(int option, int value)
+{
+	switch (option) {
+		case BRISTOL_NRP_REQ_DEBUG:
+			if (value == 0)
+				audioDev.cflags &= ~SLAB_AUDIODBG;
+			else if (value == 1)
+				audioDev.cflags |= SLAB_AUDIODBG;
+			else
+				audioDev.cflags |= SLAB_AUDIODBG|SLAB_AUDIODBG2;
+			break;
 	}
 
 	return(0);
