@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -54,7 +53,7 @@ int bristolIOprint(bristolOPIO *io)
 	printf("		rate	%i\n", io->samplerate);
 	printf("		count	%i\n", io->samplecount);
 	printf("		flags	%x\n", io->flags);
-	printf("		buf	%i\n", (size_t) io->buf);
+	printf("		buf	%p\n", io->buf);
 
 	return(0);
 }
@@ -70,9 +69,9 @@ int bristolSpecPrint(bristolOPSpec *specs)
 	printf("		name	%s\n", specs->opname);
 	printf("		desc	%s\n", specs->description);
 	printf("		pcount	%i\n", specs->pcount);
-	printf("		param	%x\n", (size_t) specs->param);
+	printf("		param	%p\n", specs->param);
 	printf("		iocount	%i\n", specs->iocount);
-	printf("		io	%x\n", (size_t) specs->io);
+	printf("		io	%p\n", specs->io);
 	printf("		lclsize	%i\n", specs->localsize);
 
 	for (i = 0; i < specs->iocount; i++)
@@ -93,15 +92,15 @@ bristolOPprint(bristolOP *operator)
 
 	printf("	index	%i\n", operator->index);
 	printf("	flags	%i\n", operator->flags);
-	printf("	next	%x\n", (size_t) operator->next);
-	printf("	last	%x\n", (size_t) operator->last);
-	printf("	spec	%x\n", (size_t) operator->specs);
+	printf("	next	%p\n", operator->next);
+	printf("	last	%p\n", operator->last);
+	printf("	spec	%p\n", operator->specs);
 	printf("	size	%i\n", operator->size);
-	printf("	init	%x\n", (size_t) operator->init);
-	printf("	dest	%x\n", (size_t) operator->destroy);
-	printf("	reset	%x\n", (size_t) operator->reset);
-	printf("	param	%x\n", (size_t) operator->param);
-	printf("	operate	%x\n", (size_t) operator->operate);
+	printf("	init	%p\n", operator->init);
+	printf("	dest	%p\n", operator->destroy);
+	printf("	reset	%p\n", operator->reset);
+	printf("	param	%p\n", operator->param);
+	printf("	operate	%p\n", operator->operate);
 
 	bristolSpecPrint(operator->specs);
 
