@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -42,6 +41,7 @@
 #include "slabrevisions.h"
 #include "slabaudiodev.h"
 #include "slabcdefs.h"
+#include "bristol.h"
 
 #if (BRISTOL_HAS_ALSA == 1)
 #include "slabalsadev.h"
@@ -58,21 +58,12 @@ extern int alsaDevAudioStart(duplexDev *);
 #include <unistd.h>
 #include <fcntl.h>
 #ifdef SUBFRAGMENT
-#if (MACHINE == FreeBSD)
 #include <stdlib.h>
-#else
 #include <malloc.h>
 #endif
-#endif
 
 #include <sys/ioctl.h>
 
-/*
- * This gets picked up on MacOS-X
-#if (MACHINE == FreeBSD)
-#include <sys/soundcard.h>
-#endif
-*/
 #if (BRISTOL_HAS_OSS == 1)
 /*
  * This may want to be /usr/lib/oss/include/sys/soundcard.h if someone has
@@ -223,7 +214,7 @@ int
 audioClose(duplexDev *audioDev)
 {
 	if (audioDev->cflags & SLAB_AUDIODBG)
-		printf("audioClose(%08x, %i, %s)\n", (int) audioDev, audioDev->devID,
+		printf("audioClose(%p, %i, %s)\n", audioDev, audioDev->devID,
 			audioDev->devName);
 
 	if (audioDev->flags & AUDIO_DUMMY) {
@@ -250,6 +241,9 @@ audioClose(duplexDev *audioDev)
 		audioDev->fd2 = -1;
 	}
 
+	bristolfree(audioDev->fragBuf);
+	audioDev->fragBuf = NULL;
+
 	return(0);
 }
 
@@ -257,14 +251,14 @@ int
 audioOpen(duplexDev *audioDev, int device, int flags)
 {
 	if (audioDev->cflags & SLAB_AUDIODBG)
-		printf("audioOpen(%08x, %i, %i): %s\n", (int) audioDev, device, flags,
+		printf("audioOpen(%p, %i, %i): %s\n", audioDev, device, flags,
 			audioDev->devName);
 
 	if (audioDev->flags & AUDIO_DUMMY) {
 		printf("using AUDIO_DUMMY interface\n");
 
 		if (audioDev->fragBuf)
-			free(audioDev->fragBuf);
+			bristolfree(audioDev->fragBuf);
 
 		if (audioDev->channels == 0)
 			audioDev->channels = 2;
@@ -277,7 +271,7 @@ audioOpen(duplexDev *audioDev, int device, int flags)
 		if (audioDev->readSampleRate == 0)
 			audioDev->readSampleRate = 44100;
 
-		audioDev->fragBuf = (char *) malloc(audioDev->fragSize);
+		audioDev->fragBuf = (char *) bristolmalloc(audioDev->fragSize);
 
 		return(10);
 	}
