@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -56,11 +55,14 @@
 #include "slabrevisions.h"
 #include "slabcdefs.h"
 #include "slabaudiodev.h"
+#include "bristol.h"
 
 #if (BRISTOL_HAS_ALSA == 1)
 
 #include "slabalsadev.h"
 
+#include <alsa/pcm.h>
+
 #include <sys/poll.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -80,6 +82,9 @@
 
 #include <time.h>
 
+static int adInit = 1;
+static struct adev alsaDev[MAX_DEVICES + 1];
+
 void setscheduler(audioDev)
 duplexDev *audioDev;
 {
@@ -105,6 +110,11 @@ duplexDev *audioDev;
 #endif
 }
 
+#define SND_PCM_OPEN_PLAYBACK SND_PCM_STREAM_PLAYBACK
+#define SND_PCM_OPEN_CAPTURE SND_PCM_STREAM_CAPTURE
+
+snd_output_t *output = NULL;
+
 /*
  * This assumes the API will manage the memory blocks allocated to the handles,
  * so will only reset them to NULL, not free them.
@@ -145,14 +155,12 @@ printf("	alsaDevClose(%08x): %08x, %08x\n",
 
 	audioDev->fd = audioDev->fd2 = -1;
 
+	bristolfree(output);
+	output = NULL;
+
 	return(0);
 }
 
-#define SND_PCM_OPEN_PLAYBACK SND_PCM_STREAM_PLAYBACK
-#define SND_PCM_OPEN_CAPTURE SND_PCM_STREAM_CAPTURE
-
-snd_output_t *output = NULL;
-
 int
 alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 	snd_pcm_hw_params_t **h_params,
@@ -167,7 +175,9 @@ alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 	struct pollfd *pfds;
 
 	snd_pcm_hw_params_alloca(h_params);
+	memset(*h_params, 0, sizeof(snd_pcm_hw_params));
 	snd_pcm_sw_params_alloca(s_params);
+	memset(*s_params, 0, sizeof(snd_pcm_hw_params));
 
 	if (audioDev->fragSize == 0)
 		audioDev->fragSize = DEF_FRAGSIZE;
@@ -180,14 +190,20 @@ alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 		stream = SND_PCM_STREAM_PLAYBACK;
 
 	/*
-	 * Open the device.
+	 * Open the device. This use of handle may cause a small memory leak when
+	 * the audio interface is restarted, I don't know if I own the memory or
+	 * if the library does.
 	 */
+	bristolfree(*handle);
 	if (snd_pcm_open(handle, devicename, stream, 0) < 0)
 	{
 		fprintf(stderr, "Error opening PCM device %s\n", devicename);
+		bristolfree(devicename);
 		return(-1);
 	}
 
+	bristolfree(devicename);
+
 	if (snd_pcm_hw_params_any(*handle, *h_params) < 0)
 	{
 		printf("Cound not get %s any params\n", dir);
@@ -225,8 +241,10 @@ alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 	 * study anyway
 	 *
 	 * The following only works for 16bit stereo samples.
-	 */
+	 *
+	 * Can this warning, we are going to use Jack for multichannel stuff
 #warning Not compatible with alternative sample sizes or channel counts
+	 */
 	count = audioDev->fragSize >> 2;
 	fragsize = count * audioDev->preLoad;
 	if (snd_pcm_hw_params_set_period_size(*handle, *h_params, count, 0) < 0)
@@ -301,7 +319,7 @@ alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 	snd_pcm_dump(*handle, output);
 
 	nfds = snd_pcm_poll_descriptors_count(*handle);
-	pfds = (struct pollfd *) malloc(sizeof (struct pollfd) * nfds);
+	pfds = (struct pollfd *) bristolmalloc(sizeof (struct pollfd) * nfds);
 	snd_pcm_poll_descriptors(*handle, pfds, nfds);
 
 	if (strcmp(dir, "capture") == 0)
@@ -315,7 +333,7 @@ alsaChannelConfigure(duplexDev *audioDev, snd_pcm_t **handle,
 			printf("prepare failure on playback channel\n");
 	}
 
-	free(pfds);
+	bristolfree(pfds);
 
 	return(0);
 }
@@ -334,8 +352,16 @@ alsaDevOpen(duplexDev *audioDev, int devID, int flags, int fragSize)
 	 * rather than process priority scheduling.
 	 */
 
-if (audioDev->cflags & SLAB_AUDIODBG)
-printf("opening device %s, flags %08x\n", audioDev->devName, audioDev->flags);
+	if (audioDev->cflags & SLAB_AUDIODBG)
+		printf("opening device %s, flags %08x\n",
+			audioDev->devName, audioDev->flags);
+
+	if (adInit)
+	{
+		printf("init %i bytes of device structure\n", (int) sizeof(alsaDev));
+		bzero(alsaDev, sizeof(alsaDev));
+		adInit = 0;
+	}
 
 	if (audioDev->channels == 0)
 		audioDev->channels = 2;
@@ -352,6 +378,8 @@ printf("opening device %s, flags %08x\n", audioDev->devName, audioDev->flags);
 	else if (flags == SLAB_ORDWR)
 		flags = SLAB_OWRONLY|SLAB_RDONLY;
 
+	bristolfree(output);
+	output = NULL;
 	err = snd_output_stdio_attach(&output, stdout, 0);
 
 	/*
@@ -388,43 +416,68 @@ printf("opening device %s, flags %08x\n", audioDev->devName, audioDev->flags);
 			return(-1);
 	}
 
-	if (audioDev->fragBuf)
-		free(audioDev->fragBuf);
+	bristolfree(audioDev->fragBuf);
 
-	audioDev->fragBuf = (char *) malloc(audioDev->fragSize);
+	audioDev->fragBuf = (char *) bristolmalloc(audioDev->fragSize);
 
 /*printf("	alsaDevOpen(%08x): %08x, %08x\n", */
 /*			audioDev, */
 /*			&alsaDev[audioDev->devID].chandle, */
 /*			&alsaDev[audioDev->devID].phandle); */
 
+//	snd_pcm_link(alsaDev[audioDev->devID].chandle,
+//		alsaDev[audioDev->devID].phandle);
+
 	return(0);
 }
 
 int
 alsaDevAudioStart(duplexDev *audioDev)
 {
-	int err;
+#ifdef _BRISTOL_DRAIN
+	int err, i;
 
-	if ((audioDev->flags == ADIOD_OUTPUT)
-		|| (audioDev->flags == ADIOD_DUPLEX))
+	printf("restart audio interface %i %i\n",
+		audioDev->samplecount, audioDev->preLoad);
+
+	snd_pcm_drop(alsaDev[audioDev->devID].phandle);
+	snd_pcm_prepare(alsaDev[audioDev->devID].phandle);
+
+	for (i = 0; i < audioDev->preLoad; i++)
+		snd_pcm_writei(alsaDev[audioDev->devID].phandle, audioDev->fragBuf,
+			audioDev->samplecount);
+
+	snd_pcm_drop(alsaDev[audioDev->devID].chandle);
+	snd_pcm_prepare(alsaDev[audioDev->devID].chandle);
+
+	if ((err = snd_pcm_start(alsaDev[audioDev->devID].phandle)) < 0) {
+		printf("Playback start error: %s\n", snd_strerror(err));
+		return(-1);
+	}
+
+	if ((err = snd_pcm_start(alsaDev[audioDev->devID].chandle)) < 0) {
+		printf("Record start error: %s\n", snd_strerror(err));
+		return(-1);
+	}
+#else
+	if ((audioDev->flags == ADIOD_OUTPUT) || (audioDev->flags == ADIOD_DUPLEX))
 	{
-printf("Start playback\n");
+		printf("Start playback %i\n", audioDev->preLoad);
 		if ((err = snd_pcm_start(alsaDev[audioDev->devID].phandle)) < 0) {
 			printf("Playback start error: %s\n", snd_strerror(err));
 			return(-1);
 		}
 	}
 
-	if ((audioDev->flags == ADIOD_INPUT)
-		|| (audioDev->flags == ADIOD_DUPLEX))
+	if ((audioDev->flags == ADIOD_INPUT) || (audioDev->flags == ADIOD_DUPLEX))
 	{
-printf("Start capture\n");
+		printf("Start capture\n");
 		if ((err = snd_pcm_start(alsaDev[audioDev->devID].chandle)) < 0) {
 			printf("Record start error: %s\n", snd_strerror(err));
 			return(-1);
 		}
 	}
+#endif
 
 	return(0);
 }
@@ -453,6 +506,9 @@ int count;
 	if (audioDev->flags & AUDIO_DUMMY)
 		return(count);
 
+	if (audioDev->flags & SLAB_AUDIODBG2)
+		printf("alsa audioRead(%i)\n", count);
+
 #if (BRISTOL_HAS_ALSA == 1)
 	/*
 	 * This if for ALSA 0.9 and 1.0
@@ -496,6 +552,9 @@ int count;
 {
 	int result;
 
+	if (audioDev->flags & SLAB_AUDIODBG2)
+		printf("alsa audioRead(%i)\n", count);
+
 	if (audioDev->flags & AUDIO_DUMMY)
 	{
 		usleep(100000);
