@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -35,32 +34,154 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #ifdef _BRISTOL_JACK
-#include "bristol.h"
+#if (BRISTOL_HAS_ALSA == 1)
+#include <alsa/iatomic.h>
+#endif
 
+/*
+ * Drop this atomic stuff, it comes from the ALSA library and it not present on
+ * a lot of systems. It is failing the Debian hurdles on some pretty wild host
+ * architectures admittedly. Now the benefits of a real atomic access here are
+ * small: there is one writer changing a value from 1 to zero. There is one
+ * reader waiting for it to be zero, even as a non-atomic operation the worst
+ * this can do is delay detection by a period.
+ */
+#ifdef DONT_USE_THIS //IATOMIC_DEFINED // Not very portable
+atomic_t closedown = ATOMIC_INIT(1);
+#else
+int closedown = 1;
+#endif
+
+#include "bristol.h"
 #include "bristoljack.h"
 
+#ifdef _BRISTOL_JACK_SESSION
+#include <jack/session.h>
+char sessionfile[1024];
+char commandline[1024];
+#endif
+
 extern int dupfd;
+static char defreg[8] = "bristol";
+static char *regname = defreg;
 
 static jackDev jackdev;
 static float *outbuf, *inbuf;
 
 static int fsize = sizeof(jack_default_audio_sample_t);
 
+#ifdef _BRISTOL_JACK_MIDI
+extern void bristolJackSetMidiHandle(jack_client_t *);
+extern int jackMidiRoutine(jack_nframes_t, void *);
+#endif
+
 static void
 jack_shutdown(void *jackdev)
 {
 	((jackDev *) jackdev)->audiomain->atReq = BRISTOL_REQSTOP;
 }
 
+#ifdef _BRISTOL_JACK_SESSION
+/*
+ * This is polled by the GUI to see if any session events need handling. It is
+ * called fromm the idle loop of the engine parent thread.
+ */
+int
+bristolJackSessionCheck(audioMain *audiomain)
+{
+	int rval = 0;
+
+	if ((jackdev.sEvent == NULL) || (jack_set_session_callback == NULL))
+		return(0);
+
+	snprintf(sessionfile, 1024, "%s%s", jackdev.sEvent->session_dir, regname);
+	audiomain->sessionfile = sessionfile;
+
+	snprintf(commandline, 1024, "%s -jsmfile \"${SESSION_DIR}%s\" -jsmuuid %s",
+		audiomain->cmdline, regname, jackdev.sEvent->client_uuid);
+
+	jackdev.sEvent->command_line = bristolmalloc(strlen(commandline) + 1);
+	sprintf(jackdev.sEvent->command_line, "%s", commandline);
+
+	/*
+	 * Need to flag the event for outside handling
+	 */
+	if (audiomain->debuglevel > 1)
+	{
+		if (jackdev.audiomain->jackUUID[0] != '\0')
+			printf("jack session callback: %s %s\n",
+				jackdev.sEvent->client_uuid,
+				jackdev.audiomain->jackUUID);
+		else
+			printf("jack session callback: %s\n", jackdev.sEvent->client_uuid);
+
+		printf("session file is %s\n", audiomain->sessionfile);
+
+		printf("command line is %s\n", jackdev.sEvent->command_line);
+	}
+
+	rval = jackdev.sEvent->type;
+
+	jack_session_reply(jackdev.handle, jackdev.sEvent);
+
+	/* This was not malloc'ed do does not belong to jack session manager */
+	jack_session_event_free(jackdev.sEvent);
+	jackdev.sEvent = NULL;
+
+	return(rval);
+}
+
+void
+jack_session_callback(jack_session_event_t *event, void *arg)
+{
+	if (jack_set_session_callback == NULL)
+		return;
+
+	if (jackdev.sEvent != NULL)
+		jack_session_event_free(jackdev.sEvent);
+
+	/* This is not a very good signaling method, it has race conditions */
+	jackdev.sEvent = event;
+}
+#endif
+
+static void
+bufstats(float *buf, int nframes, int stage)
+{
+	float max = 0.0f, min = 1.0f;
+
+	if (buf == NULL)
+		return;
+
+	for (; nframes > 0; nframes-= 8)
+	{
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+		max = *buf > max? *buf:max; min = *buf < min? *buf:min; buf++;
+	}
+	printf("%i %p: min %f, max %f\n", stage, buf, min, max);
+}
+
 static int
 audioShim(jack_nframes_t nframes, void *jd)
 {
 	jackDev *jackdev = (jackDev *) jd;
 	register float *outL, *outR, *toutbuf = outbuf, gain;
-	register int i = nframes, nint;
+	register int i, nint;
+
+#ifdef _BRISTOL_JACK_MIDI
+	if (~jackdev->audiomain->flags & BRISTOL_JACK_DUAL)
+		jackMidiRoutine(nframes, NULL);
+#endif
 
 	/*
 	 * We may need to consider jack changing its nframes on the fly. Whilst
@@ -70,8 +191,62 @@ audioShim(jack_nframes_t nframes, void *jd)
 	 * operation, or alternatively add in a shim to do some reframing.
 	 * #warning need to consider jackd changes to nframes?
 	 */
-	if (jackdev->jack_in[0])
-		jackdev->inbuf[0] = jack_port_get_buffer(jackdev->jack_in[0], nframes);
+	if (jackdev->jack_in[BRISTOL_JACK_STDINL])
+		jackdev->inbuf[BRISTOL_JACK_STDINL]
+			= jack_port_get_buffer(jackdev->jack_in[BRISTOL_JACK_STDINL],
+				nframes);
+	if (jackdev->jack_in[BRISTOL_JACK_STDINR])
+		jackdev->inbuf[BRISTOL_JACK_STDINR]
+			= jack_port_get_buffer(jackdev->jack_in[BRISTOL_JACK_STDINR],
+				nframes);
+
+	if (jackdev->iocount > 0)
+	{
+		int j;
+		float *buf;
+
+		if ((gain = jackdev->audiomain->m_io_igc) <= 0)
+			gain = 1.0;
+
+		for (i = 0; i < jackdev->iocount; i++)
+		{
+			jackdev->inbuf[i] = buf =
+				jackdev->audiomain->io_i[i] = 
+				jack_port_get_buffer(jackdev->jack_in[i], nframes);
+
+			for (j = 0; j < nframes; j+=8)
+			{
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+			}
+
+			jackdev->outbuf[i] =
+				jackdev->audiomain->io_o[i] = 
+				jack_port_get_buffer(jackdev->jack_out[i], nframes);
+
+			memset(jackdev->outbuf[i], 0.0f, nframes * sizeof(float));
+		}
+	}
+
+	toutbuf = inbuf;
+	gain = 12.0f;
+	for (i = 0; i < nframes; i+=8)
+	{
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+		*toutbuf++ = jackdev->inbuf[BRISTOL_JACK_STDINL][i] * gain;
+	}
 
 	/*
 	 * For now we are not going to be too picky about inbuf.
@@ -110,31 +285,83 @@ audioShim(jack_nframes_t nframes, void *jd)
 		gain = 1.0f;
 	gain /= 32768.0;
 
-	outL = (float *) jack_port_get_buffer(jackdev->jack_out[0], nframes);
-	outR = (float *) jack_port_get_buffer(jackdev->jack_out[1], nframes);
+	outL = (float *)
+		jack_port_get_buffer(jackdev->jack_out[BRISTOL_JACK_STDOUTL], nframes);
+	outR = (float *)
+		jack_port_get_buffer(jackdev->jack_out[BRISTOL_JACK_STDOUTR], nframes);
 
 	/*
 	 * Deinterleave our output through to the jack buffers.
 	 */
-	for (; i > 0; i--)
+	toutbuf = outbuf;
+	for (i = nframes; i > 0; i--)
 	{
 		*outL++ = *toutbuf++ * gain;
 		*outR++ = *toutbuf++ * gain;
 	}
 
+	if (jackdev->iocount > 0)
+	{
+		register int j;
+		register float *buf, gain;
+
+		if ((gain = jackdev->audiomain->m_io_ogc) <= 0)
+			gain = 1.0;
+
+		for (i = 0; i < jackdev->iocount; i++)
+		{
+			buf = jackdev->outbuf[i];
+
+			for (j = 0; j < nframes; j+=8)
+			{
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+				*buf++ *= gain;
+			}
+
+			memset(jackdev->inbuf[i], 0.0f, nframes * sizeof(float));
+		}
+	}
+
+	if (jackdev->audiomain->debuglevel > 9)
+	{
+		bufstats(jackdev->outbuf[0], nframes, 1);
+		bufstats(jackdev->outbuf[2], nframes, 2);
+	}
+
 	return(0);
 }
 
 static void
 bristolIntJackClose()
 {
+	int i;
+
+	--closedown;
+
 	printf("closedown on interrupt\n");
 
 	jack_deactivate(jackdev.handle);
 
-	jack_port_unregister(jackdev.handle, jackdev.jack_out[0]);
-	jack_port_unregister(jackdev.handle, jackdev.jack_out[1]);
-	jack_port_unregister(jackdev.handle, jackdev.jack_in[0]);
+	jack_port_unregister(jackdev.handle,
+		jackdev.jack_out[BRISTOL_JACK_STDOUTL]);
+	jack_port_unregister(jackdev.handle,
+		jackdev.jack_out[BRISTOL_JACK_STDOUTR]);
+	jack_port_unregister(jackdev.handle,
+		jackdev.jack_in[BRISTOL_JACK_STDINL]);
+	jack_port_unregister(jackdev.handle,
+		jackdev.jack_in[BRISTOL_JACK_STDINR]);
+
+	for (i = 0; i < jackdev.iocount; i++)
+	{
+		jack_port_unregister(jackdev.handle, jackdev.jack_in[i]);
+		jack_port_unregister(jackdev.handle, jackdev.jack_out[i]);
+	}
 
 	jack_client_close(jackdev.handle);
 
@@ -144,26 +371,61 @@ bristolIntJackClose()
 static int
 bristolJackClose(jackDev *jackdev)
 {
-	if (jackdev->handle == NULL)
+	int i;
+
+	if ((jackdev->handle == NULL) || (jackdev->jack_out[BRISTOL_JACK_STDOUTL] == NULL))
 		return(-1);
 
-	printf("unregistering jack interface\n");
+	/*
+	 * So, if we don't have the ALSA atomic headers then just run it on ints,
+	 * this bit of code is far from critical but collisions cause some gory
+	 * shutdown sequences. The code used to call this shutdown from two 
+	 * different threads, removed on of them instead to prevent the possibility
+	 * of them battering each other.
+	 */
+#ifdef DONT_USE_THIS //IATOMIC_DEFINED // Not very portable
+	if (atomic_dec_and_test(&closedown))
+#else
+	if (--closedown == 0)
+#endif
+		printf("unregistering jack interface: %p->%p\n",
+			jackdev, jackdev->handle);
+	else {
+		printf("interface unregistered\n");
+		return(0);
+	}
 
 	jack_deactivate(jackdev->handle);
 
-	jack_port_unregister(jackdev->handle, jackdev->jack_out[0]);
-	jack_port_unregister(jackdev->handle, jackdev->jack_out[1]);
-	jack_port_unregister(jackdev->handle, jackdev->jack_in[0]);
+	usleep(100000);
+
+	jack_port_unregister(jackdev->handle,
+		jackdev->jack_out[BRISTOL_JACK_STDOUTL]);
+	jack_port_unregister(jackdev->handle,
+		jackdev->jack_out[BRISTOL_JACK_STDOUTR]);
+	jack_port_unregister(jackdev->handle,
+		jackdev->jack_in[BRISTOL_JACK_STDINL]);
+	jack_port_unregister(jackdev->handle,
+		jackdev->jack_in[BRISTOL_JACK_STDINR]);
+
+	for (i = 0; i < jackdev->iocount; i++)
+	{
+		jack_port_unregister(jackdev->handle, jackdev->jack_in[i]);
+		jack_port_unregister(jackdev->handle, jackdev->jack_out[i]);
+	}
+
+	jackdev->jack_out[BRISTOL_JACK_STDOUTL] = NULL;
 
 	jack_client_close(jackdev->handle);
 
 	/*
 	 * The API does not seem to be clear on who owns this however since we
 	 * are exiting.....
-	 */
 	jackdev->handle = NULL;
+	 */
 
 	jackdev->audiomain->atReq |= BRISTOL_REQSTOP;
+	jackdev->audiomain->mtReq |= BRISTOL_REQSTOP;
 
 	return(0);
 }
@@ -181,7 +443,7 @@ jackPortStats(jackDev *jackdev, int dir)
 {
 	int i;
 
-	free(jackdev->ports);
+	bristolfree(jackdev->ports);
 
 	if ((jackdev->ports = jack_get_ports(jackdev->handle, NULL, NULL,
 		JackPortIsPhysical|dir)) == NULL)
@@ -194,9 +456,6 @@ jackPortStats(jackDev *jackdev, int dir)
 		printf("Found port %s\n", jackdev->ports[i]);
 }
 
-static char defreg[8] = "bristol";
-static char *regname = defreg;
-
 static int
 bristolJackOpen(jackDev *jackdev, audioMain *audiomain,
 JackProcessCallback shim)
@@ -206,9 +465,23 @@ JackProcessCallback shim)
 	if (audiomain->audiodev != NULL)
 		regname = audiomain->audiodev;
 
+#ifdef _BRISTOL_JACK_SESSION
+	if (audiomain->jackUUID[0] == '\0')
+	{
+		printf("registering jack interface: %s\n", regname);
+		jackdev->handle = jack_client_open(regname, JackNullOption, NULL);
+	} else {
+		printf("reregistering jack interface: %s, UUDI %s\n",
+			regname, &audiomain->jackUUID[0]);
+		jackdev->handle = jack_client_open(regname, JackSessionID, NULL,
+			&audiomain->jackUUID[0]);
+	}
+#else
 	printf("registering jack interface: %s\n", regname);
+	jackdev->handle = jack_client_open(regname, JackNullOption, NULL);
+#endif
 
-	if ((jackdev->handle = jack_client_open(regname, 0, NULL)) == 0)
+	if (jackdev->handle == 0)
 	{
 		char process_indexed[32];
 
@@ -219,7 +492,17 @@ JackProcessCallback shim)
 		 */
 		snprintf(process_indexed, 32, "%s_%i", regname, audiomain->port);
 
-		if ((jackdev->handle = jack_client_open(process_indexed, 0, NULL)) == 0)
+#ifdef _BRISTOL_JACK_SESSION
+		if (audiomain->jackUUID[0] == '\0')
+			jackdev->handle = jack_client_open(regname, JackNullOption, NULL);
+		else
+			jackdev->handle = jack_client_open(regname, JackSessionID, NULL,
+				&audiomain->jackUUID[0]);
+#else
+		jackdev->handle = jack_client_open(regname, JackNullOption, NULL);
+#endif
+
+		if (jackdev->handle == 0)
 		{
 			printf("Cannot connect to jack\n");
 			audiomain->atStatus = BRISTOL_REQSTOP;
@@ -227,6 +510,11 @@ JackProcessCallback shim)
 		}
 	}
 
+#ifdef _BRISTOL_JACK_MIDI
+	if (~audiomain->flags & BRISTOL_JACK_DUAL)
+		bristolJackSetMidiHandle(jackdev->handle);
+#endif
+
 	signal(SIGINT, bristolIntJackClose);
 	signal(SIGTERM, bristolIntJackClose);
 	signal(SIGHUP, bristolIntJackClose);
@@ -285,14 +573,20 @@ JackProcessCallback shim)
 
 	jack_on_shutdown(jackdev->handle, jack_shutdown, (void *) jackdev);
 
-	if ((jackdev->jack_out[0] = jack_port_register(jackdev->handle,
+#ifdef _BRISTOL_JACK_SESSION
+	if (jack_set_session_callback)
+		jack_set_session_callback(jackdev->handle, jack_session_callback,
+			(void *) jackdev);
+#endif
+
+	if ((jackdev->jack_out[BRISTOL_JACK_STDOUTL] = jack_port_register(jackdev->handle,
 		"out_left", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)) == NULL)
 	{
 		printf("Cannot register jack port\n");
 		audiomain->atStatus = BRISTOL_REQSTOP;
 		return(-1);
 	}
-	if ((jackdev->jack_out[1] = jack_port_register(jackdev->handle,
+	if ((jackdev->jack_out[BRISTOL_JACK_STDOUTR] = jack_port_register(jackdev->handle,
 		"out_right", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)) == NULL)
 	{
 		printf("Cannot register jack port\n");
@@ -306,8 +600,15 @@ JackProcessCallback shim)
 	 * - read/modify/write sequences. We could drop this registration, but it
 	 * can stay in for now, it will be used in the future.
 	 */
-	if ((jackdev->jack_in[0] = jack_port_register(jackdev->handle,
-		"in", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0)) == NULL)
+	if ((jackdev->jack_in[BRISTOL_JACK_STDINL] = jack_port_register(jackdev->handle,
+		"in_left", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0)) == NULL)
+	{
+		printf("Cannot register jack port\n");
+		audiomain->atStatus = BRISTOL_REQSTOP;
+		return(-1);
+	}
+	if ((jackdev->jack_in[BRISTOL_JACK_STDINR] = jack_port_register(jackdev->handle,
+		"in_right", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0)) == NULL)
 	{
 		printf("Cannot register jack port\n");
 		audiomain->atStatus = BRISTOL_REQSTOP;
@@ -360,29 +661,29 @@ JackProcessCallback shim)
 				if (strcmp(lport, jackdev->ports[jport]) == 0)
 				{
 					if (jack_connect(jackdev->handle,
-						jack_port_name(jackdev->jack_out[0]),
+						jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 							jackdev->ports[jport]) != 0)
 					{
 						printf("Bristol Failed Conn: %s to %s failed\n",
-							jack_port_name(jackdev->jack_out[0]),
+							jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 							jackdev->ports[jport]);
 					} else
 						printf("Bristol Env AutoConn: %s to %s\n",
-							jack_port_name(jackdev->jack_out[0]),
+							jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 							jackdev->ports[jport]);
 				}
 				if (strcmp(rport, jackdev->ports[jport]) == 0)
 				{
 					if (jack_connect(jackdev->handle,
-						jack_port_name(jackdev->jack_out[1]),
+						jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTR]),
 							jackdev->ports[jport]) != 0)
 					{
 						printf("Bristol Failed Conn: %s to %s failed\n",
-							jack_port_name(jackdev->jack_out[1]),
+							jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTR]),
 							jackdev->ports[jport]);
 					} else
 						printf("Bristol Env AutoConn: %s to %s\n",
-							jack_port_name(jackdev->jack_out[0]),
+							jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 							jackdev->ports[jport]);
 				}
 			}
@@ -404,16 +705,16 @@ JackProcessCallback shim)
 			}
 
 			if (jack_connect(jackdev->handle,
-				jack_port_name(jackdev->jack_out[0]),
+				jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 					jackdev->ports[jport]) != 0)
 			{
 				printf("Bristol Defaulted Conn: %s to %s failed\n",
-					jack_port_name(jackdev->jack_out[0]),
+					jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 					jackdev->ports[jport]);
 				continue;
 			} else
 				printf("Bristol Defaulted Conn: %s to %s\n",
-					jack_port_name(jackdev->jack_out[0]),
+					jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTL]),
 					jackdev->ports[jport]);
 
 			for (jport++; jackdev->ports[jport] != 0; jport++)
@@ -425,16 +726,16 @@ JackProcessCallback shim)
 				}
 
 				if (jack_connect(jackdev->handle,
-					jack_port_name(jackdev->jack_out[1]),
+					jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTR]),
 						jackdev->ports[jport]) != 0)
 				{
 					printf("Bristol Defaulted Conn: %s to %s failed\n",
-						jack_port_name(jackdev->jack_out[1]),
+						jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTR]),
 						jackdev->ports[jport]);
 					continue;
 				} else
 					printf("Bristol Defaulted Conn: %s to %s\n",
-						jack_port_name(jackdev->jack_out[1]),
+						jack_port_name(jackdev->jack_out[BRISTOL_JACK_STDOUTR]),
 						jackdev->ports[jport]);
 				break;
 			}
@@ -466,7 +767,7 @@ JackProcessCallback shim)
 				if (strcmp(iport, jackdev->ports[jport]) == 0)
 				{
 					if (jack_connect(jackdev->handle, jackdev->ports[jport],
-						jack_port_name(jackdev->jack_in[0])) != 0)
+						jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL])) != 0)
 					{
 						/*
 						 * We don't actually need to fail any of these, it just
@@ -474,12 +775,12 @@ JackProcessCallback shim)
 						 */
 						printf("Bristol Env AutoConn: %s to %s failed\n",
 							jackdev->ports[jport],
-							jack_port_name(jackdev->jack_in[0]));
+							jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 						continue;
 					} else
 						printf("Bristol Env AutoConn: %s to %s\n",
 							jackdev->ports[jport],
-							jack_port_name(jackdev->jack_in[0]));
+							jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 				}
 			}
 		} else for (jport = 0; jackdev->ports[jport] != 0; jport++)
@@ -491,66 +792,96 @@ JackProcessCallback shim)
 			}
 
 			if (jack_connect(jackdev->handle, jackdev->ports[jport],
-				jack_port_name(jackdev->jack_in[0])) != 0)
+				jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL])) != 0)
 			{
 				/*
 				 * We don't actually need to fail any of these, it just means
 				 * we have no defaults.
 				 */
 				printf("Bristol Defaulted Conn: %s to %s failed\n",
-					jackdev->ports[jport], jack_port_name(jackdev->jack_in[0]));
+					jackdev->ports[jport], jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 				continue;
 			} else
 				printf("Bristol Defaulted Conn: %s to %s\n",
-					jackdev->ports[jport], jack_port_name(jackdev->jack_in[0]));
+					jackdev->ports[jport], jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 
 			break;
 		}
 	}
 
+	/*
+	 * So now see if we were requested to register more ports.
+	 */
+	if ((jackdev->iocount = audiomain->iocount) > 0)
+	{
+		int i;
+		char pn[256];
+
+		for (i = 0; i < jackdev->iocount; i++)
+		{
+			sprintf(pn, "out_%i", i + 1);
+			if ((jackdev->jack_out[i] = jack_port_register(jackdev->handle,
+				pn, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0)) == NULL)
+			{
+				printf("Cannot register jack port\n");
+				audiomain->atStatus = BRISTOL_REQSTOP;
+				return(-1);
+			}
+			sprintf(pn, "in_%i", i + 1);
+			if ((jackdev->jack_in[i] = jack_port_register(jackdev->handle,
+				pn, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0)) == NULL)
+			{
+				printf("Cannot register jack port\n");
+				audiomain->atStatus = BRISTOL_REQSTOP;
+				return(-1);
+			}
+		}
+	}
+
 	return(0);
 }
 
 /*
- * This was just to test dynamic registrations whilst the jack link is active
- */
+ * Test dynamic disconnect and reconnects. Eventually each synth will
+ * potentially register its own set of ports. Want to do this with a
+ * reasonably large number of iterations. Correctly speaking we should
+ * probably take a copy of the jack_in port, null the jackdev entry
+ * and then unregister it.
 static void
 jackPortBounce(jackDev *jackdev)
 {
-	/*
-	 * Test dynamic disconnect and reconnects. Eventually each synth will
-	 * potentially register its own set of ports. Want to do this with a
-	 * reasonably large number of iterations. Correctly speaking we should
-	 * probably take a copy of the jack_in port, null the jackdev entry
-	 * and then unregister it.
-	 */
 	jack_disconnect(jackdev->handle,
-		jackdev->ports[0], jack_port_name(jackdev->jack_in[0]));
+		jackdev->ports[0], jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 
-	if (jack_port_unregister(jackdev->handle, jackdev->jack_in[0]) == 0)
+	if (jack_port_unregister(jackdev->handle, jackdev->jack_in[BRISTOL_JACK_STDINL]) == 0)
 	{
 		printf("Unregistered port\n");
-		jackdev->jack_in[0] = 0;
+		jackdev->jack_in[BRISTOL_JACK_STDINL] = 0;
 
 		sleep(5);
 
-		if ((jackdev->jack_in[0] = jack_port_register(jackdev->handle,
+		if ((jackdev->jack_in[BRISTOL_JACK_STDINL] = jack_port_register(jackdev->handle,
 			"in", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0)) != NULL)
 			printf("Registered port\n");
 	} else
 		sleep(5);
 
 	jack_connect(jackdev->handle,
-		jackdev->ports[0], jack_port_name(jackdev->jack_in[0]));
+		jackdev->ports[0], jack_port_name(jackdev->jack_in[BRISTOL_JACK_STDINL]));
 }
 
-/*
- * This should go out as the first release with Jack. After that the interface
- * will change - at the moment Jack subsumes Bristol, and this is the wrong
- * way around. The audiomain structure is buried inside the jack structure,
- * but I would prefer the contrary. In addition, with it the contrary then
- * it would be easier to integrate alternative distribution drivers (DSSI).
+jack_client_t *
+bristolJackGetAudioHandle()
+{
+	if (jackdev.audiomain == NULL)
+		sleep(1);
+	if (jackdev.audiomain->flags & BRISTOL_JACK_DUAL)
+		return(NULL);
+	sleep(1);
+	return(jackdev.handle);
+}
  */
+
 int
 bristolJackInterface(audioMain *audiomain)
 {
@@ -572,8 +903,6 @@ bristolJackInterface(audioMain *audiomain)
 	while (audiomain->atReq != BRISTOL_REQSTOP)
 		sleep(1);
 
-	bristolJackClose(&jackdev);
-
 	return(0);
 }
 #endif /* _BRISTOL_JACK */
