@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -41,6 +40,7 @@
  */
 #include "slabrevisions.h"
 #include "slabaudiodev.h"
+#include "bristol.h"
 
 int newInitAudioDevice2(duplexDev *, int, int);
 static void checkAudioCaps2(duplexDev *, int, int);
@@ -63,7 +63,7 @@ int devID;
 	 */
 	if (audioDev->fragBuf != (char *) NULL)
 	{
-		free(audioDev->fragBuf);
+		bristolfree(audioDev->fragBuf);
 		audioDev->fragBuf = (char *) NULL;
 	}
 
@@ -163,9 +163,10 @@ int devID;
 		printf("ioctl(%i, SNDCTL_DSP_SETFMT, &%i)\n", audioDev->fd, data);
 
 	if ((results = ioctl(audioDev->fd, SNDCTL_DSP_SETFMT, &data)) == 0)
+	{
 		if (audioDev->cflags & SLAB_AUDIODBG)
 			printf("Set audio format: %i\n", data);
-	else
+	} else
 		printf("Set resolution failed: %i\n", results);
 
 	/*
@@ -224,11 +225,17 @@ int devID;
 		if (audioDev->cflags & SLAB_AUDIODBG)
 			printf("ioctl(%i, SNDCTL_DSP_GETBLKSIZE, &0): %i bytes\n",
 				audioDev->fd, audioDev->fragSize);
-		audioDev->fragBuf = (char *) malloc(audioDev->fragSize);
+		audioDev->fragBuf = (char *) bristolmalloc(audioDev->fragSize);
 	}
 	else
 		printf("Get frag size failed: %i\n", results);
 
+#ifdef _BRISTOL_DRAIN
+	for (data = 0; data < audioDev->preLoad; data++)
+		results = write(audioDev->fd, audioDev->fragBuf,
+			audioDev->samplecount * 2 * audioDev->channels);
+#endif
+
 #endif /* BRISTOL_HAS_OSS */
 
 	return(0);
