@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -79,8 +78,6 @@ extern int getAlsaCapByName();
 extern int getAlsaCapability();
 #endif
 
-static void checkAudioCaps2(duplexDev *, int, int);
-
 /*
  * Setup audio parameters with device support.
  */
@@ -116,30 +113,6 @@ short left, right;
 	setAudioOSSparam(audioDev, devID, param, left, right);
 }
 
-/*
- * Needs to be ALSAfied. This is a GUI operation only since ALSA was supported.
- */
-static void
-checkAudioCaps2(audioDev, devID, fd)
-duplexDev *audioDev;
-int devID, fd;
-{
-	if ((devID < 0) || (devID >= MAX_DEVICES))
-		return;
-	if (audioDev->cflags & SLAB_NO_CONTROLS)
-		return;
-
-#if (BRISTOL_HAS_ALSA == 1)
-	if ((audioDev->flags & AUDIO_ALSA) != 0)
-	{
-		checkAudioALSAcaps(audioDev, devID);
-		return;
-	}
-#endif
-
-	checkAudioOSScaps(audioDev, devID, fd);
-}
-
 void
 mixerClose(audioDev)
 duplexDev *audioDev;
@@ -262,7 +235,7 @@ setAudioValue(audioDev, controller, side, value)
 duplexDev *audioDev;
 {
 	if (audioDev->cflags & SLAB_AUDIODBG)
-		printf("setAudioValue(%08x, %i, %i, %i)\n", (int) audioDev, controller, side, value); 
+		printf("setAudioValue(%p, %i, %i, %i)\n", audioDev, controller, side, value); 
 	if (audioDev->cflags & SLAB_NO_CONTROLS)
 	{
 		return 0;
