@@ -1,12 +1,12 @@
 
 /*
  *  Diverse SLab audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -92,6 +91,8 @@ int pid = -1;
 int status;
 #endif
 
+static int d;
+
 #ifdef MASTERING
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -198,8 +199,10 @@ openMaster(duplexDev *audioDev, int type, char *fileName, int count)
 int
 writeMaster(duplexDev *audioDev, int type, int fd, void *buffer, int size)
 {
+	int d;
+
 	if (audioDev->cflags & SLAB_AUDIODBG)
-		printf("writeMaster(%i, %i, %i, %i)\n", type, fd, (int) buffer, size);
+		printf("writeMaster(%i, %i, %p, %i)\n", type, fd, buffer, size);
 
 	switch(type & MASTERING_MASK)
 	{
@@ -211,7 +214,7 @@ writeMaster(duplexDev *audioDev, int type, int fd, void *buffer, int size)
 			 */
 			cdrFormat(buffer, size >> 2);
 
-			write(fd, buffer, size);
+			d = write(fd, buffer, size);
 			break;
 #endif
 #ifdef MASTER_WAV
@@ -228,7 +231,7 @@ writeMaster(duplexDev *audioDev, int type, int fd, void *buffer, int size)
 			/*
 			 * We don't need to byteswap first.
 			 */
-			write(fd, buffer, size);
+			d = write(fd, buffer, size);
 			break;
 	}
 	return(0);
@@ -344,7 +347,7 @@ cdrPad(int fd, int size)
 #endif
 
 	while (outCount > 0) {
-		write(fd, extend, 1);
+		d = write(fd, extend, 1);
 		outCount-=1;
 	}
 }
@@ -384,7 +387,7 @@ writeWaveHdr(duplexDev *audioDev, int fd, int count)
 	header.data_chunk	= DATA;
 	header.data_length	= count / header.modus / header.byte_p_spl;
 
-	write(fd, &header, 44);
+	d = write(fd, &header, 44);
 }
 #endif
 
