@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -133,6 +132,10 @@
 #include <math.h>
 
 #include "bristolsid.h"
+#include "bristol.h"
+
+extern void * bristolmalloc();
+extern void bristolfree();
 
 #define B_SID_RAMP		0
 #define B_SID_TRI		1
@@ -314,7 +317,7 @@ unsigned short freqmap[128] = {
 #define FREQ_HI 1
 
 typedef unsigned char SIDPhase[2];
-static SIDPhase *phasemap = (SIDPhase *) freqmap;
+/* static SIDPhase *phasemap = (SIDPhase *) freqmap; */
 
 /*
  * Structure for the phase accumulating oscillators. There are 3 per voices
@@ -1161,6 +1164,8 @@ bSidDoEnv(sidVoice *voice, unsigned char vflags)
  * If no filter poles are selected then we pass input to output, this is not 
  * per the original, added in a check for selection otherwise zero output.
  */
+#define TANHF(x) x
+
 static void
 bSidDoFilter(bSid *s)
 {
@@ -1193,9 +1198,9 @@ bSidDoFilter(bSid *s)
 	highpass = s->filter.delay2 - s->filter.delay4 - qres * s->filter.delay3;
 	s->filter.delay3 = kfc * highpass + s->filter.delay3;
 
+	/* Houvilainen LPF-24 */
 	if (s->reg[B_SID_CONTROL] & B_SID_C_LPF)
 	{
-		/* Houvilainen */
 		float kfcr;
 		float kacr;
 		float k2vg;
@@ -1212,19 +1217,19 @@ bSidDoFilter(bSid *s)
 		// cascade of 4 1st order sections
 		s->filter.ay1 = s->filter.az1 + k2vg * (tanhf(s->sfMix * OV2
 			- 4*s->filter.resonance*s->filter.amf*kacr)
-			- tanhf(s->filter.az1));
+			- TANHF(s->filter.az1));
 		s->filter.az1 = s->filter.ay1;
 
 		s->filter.ay2 = s->filter.az2
-			+ k2vg * (tanhf(s->filter.ay1) - tanhf(s->filter.az2));
+			+ k2vg * (TANHF(s->filter.ay1) - TANHF(s->filter.az2));
 		s->filter.az2 = s->filter.ay2;
 
 		s->filter.ay3 = s->filter.az3
-			+ k2vg * (tanhf(s->filter.ay2) - tanhf(s->filter.az3));
+			+ k2vg * (TANHF(s->filter.ay2) - TANHF(s->filter.az3));
 		s->filter.az3 = s->filter.ay3;
 
 		s->filter.ay4 = s->filter.az4
-			+ k2vg * (tanhf(s->filter.ay3) - tanhf(s->filter.az4));
+			+ k2vg * (TANHF(s->filter.ay3) - TANHF(s->filter.az4));
 		s->filter.az4 = s->filter.ay4;
 
 		// 1/2-sample delay for phase compensation
@@ -1244,6 +1249,7 @@ bSidDoFilter(bSid *s)
 			- s->filter.ay1-s->filter.az2) * s->filter.fmix) * V2 * 0.5;
 	} else
 		s->sfMix = 0;
+
 	/* mix filter output into output buffer */
 	if (s->reg[B_SID_FILT_M_VOL] & B_SID_F_LP)
 		s->sfMix += s->filter.delay4;
@@ -1462,6 +1468,7 @@ bSidIOSNRatio(int id, float param)
 {
 	if (_B_SID_DEBUG_A)
 		printf("s %i: S/N ratio %f\n", id, param * param + 0.000000001);
+
 	return(SID[id]->snratio = 0.1 * param * param + 0.000000001);
 }
 
