@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -27,6 +26,7 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
+#include <string.h>
 #include <stdio.h>
 #include <fcntl.h>
 
@@ -38,6 +38,8 @@ extern int open_remote_socket(char *, int, int, int);
 extern int bristolMidiFindDev(char *);
 extern int bristolMidiFindFreeHandle();
 
+static char bristol_sockname[128];
+
 int
 bristolMidiTCPPassive(char *devname, int conntype, int chan, int msgs,
 int (*callback)(), void *param, int dev, int handle)
@@ -45,10 +47,13 @@ int (*callback)(), void *param, int dev, int handle)
 	struct sockaddr address;
 
 #ifdef DEBUG
-	printf("bristolMidiTCPPassive(%i, %i)\n", dev, handle);
+	printf("bristolMidiTCPPassive(%s, %i, %i)\n", devname, dev, handle);
 #endif
 
-	if (conntype & BRISTOL_CONN_UNIX)
+	if ((conntype & BRISTOL_CONN_UNIX) ||
+		((strncmp("unix", devname, 4) == 0)
+		&& (strlen(devname) > 4)
+		&& (devname[4] == ':')))
 		conntype = 0;
 	else
 		conntype = 1;
@@ -60,7 +65,7 @@ int (*callback)(), void *param, int dev, int handle)
 	{
 		int count = 0;
 
-		while ((bmidi.dev[dev].fd = open_remote_socket(devname, chan, 2, -1))
+		while ((bmidi.dev[dev].fd = open_remote_socket(devname, chan, 8, -1))
 			< 0)
 		{
 			printf("Could not open control socket, count %i\n", count);
@@ -81,7 +86,7 @@ int (*callback)(), void *param, int dev, int handle)
 		 */
 		unlink(devname);
 
-		if ((bmidi.dev[dev].fd = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
+		if ((bmidi.dev[dev].fd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0)
 		{
 			printf("Could not get control socket\n");
 			return(BRISTOL_MIDI_DEVICE);
@@ -95,18 +100,23 @@ int (*callback)(), void *param, int dev, int handle)
 
 		printf("Opened Unix named control socket\n");
 
-		address.sa_family = AF_UNIX;
-		sprintf(&address.sa_data[0], devname);
+		address.sa_family = AF_LOCAL;
+		if ((strlen(devname) > 5) && (devname[4] == ':'))
+			snprintf(bristol_sockname, 64, "/tmp/br.%s", &devname[5]);
+		else
+			sprintf(bristol_sockname, "%s", BRISTOL_SOCKNAME);
+
+		snprintf(&address.sa_data[0], 14, "%s", bristol_sockname);
 
 		if (bind(bmidi.dev[dev].fd, &address, sizeof(struct sockaddr))) 
-			printf("Could not bind name\n");
+			printf("Could not bind name: %s\n", bristol_sockname);
 		else
-			printf("Bound name to socket: %s\n", devname);
+			printf("Bound name to socket: %s\n", bristol_sockname);
 
-		if (listen(bmidi.dev[dev].fd, 2) < 0)
+		if (listen(bmidi.dev[dev].fd, 8) < 0)
 			printf("Could not configure listens\n");
 		else
-			printf("Activated two listens on socket\n");
+			printf("Activated listens on socket\n");
 
 		/*
 		 * Make it world read/writeable.
@@ -129,6 +139,7 @@ acceptConnection(int acceptdev)
 	int dev, handle, chandle;
 	socklen_t addrlen;
 	struct sockaddr address;
+	struct linger blinger;
 
 	/*
 	 * We have read on the control socket, attempt to accept the connection.
@@ -149,7 +160,7 @@ acceptConnection(int acceptdev)
 	if ((bmidi.dev[dev].fd =
 		accept(bmidi.dev[acceptdev].fd, &address, &addrlen)) < 0)
 	{
-		printf("No accepts waiting\n");
+		//printf("No accepts waiting\n");
 		return(-1);
 	}
 
@@ -175,12 +186,13 @@ acceptConnection(int acceptdev)
 	}
 	if (chandle == BRISTOL_MIDI_HANDLES)
 	{
-		printf("Did not find related acept socket\n");
+		printf("Did not find related accept socket\n");
 		close(bmidi.dev[dev].fd);
 		bmidi.dev[dev].fd = -1;
 		return(-1);
 	}
 
+	bmidi.handle[handle].handle = handle; /* That looks kind of wierd! */
 	bmidi.handle[handle].handle = handle; /* That looks kind of wierd! */
 	bmidi.handle[handle].state = BRISTOL_MIDI_OK;
 	bmidi.handle[handle].channel = bmidi.handle[chandle].channel;
@@ -190,6 +202,15 @@ acceptConnection(int acceptdev)
 	bmidi.handle[handle].callback = bmidi.handle[chandle].callback;
 	bmidi.handle[handle].param = bmidi.handle[chandle].param;
 
+	blinger.l_onoff = 1;
+	blinger.l_linger = 2;
+
+	if (setsockopt(bmidi.dev[dev].fd, SOL_SOCKET, SO_LINGER,
+		&blinger, sizeof(struct linger)) < 0)
+		printf("server linger failed\n");
+
+//printf("inetServer received: %i %i %i\n", handle, bmidi.handle[handle].dev, bmidi.dev[bmidi.handle[handle].dev].fd);
+
 	return(0);
 }
 
