@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,23 +15,14 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-/*
- * This code should open the midi device (working with ALSA raw midi only for
- * the moment (9/11/01)), and read data from it. Not sure how it will be read,
- * either buffers, events, or perhaps just raw data. At some point in the 
- * development this will become a separate thread in the synth code.
- */
-
 /*#define DEBUG */
 
 #include <unistd.h>
-
-#include "bristolmidi.h"
+#include <stdio.h>
 
 int
 bristolPhysWrite(int fd, unsigned char *message, int size)
@@ -42,6 +33,7 @@ bristolPhysWrite(int fd, unsigned char *message, int size)
 
 	if (write(fd, message, size) != size)
 	{
+		//perror("midi write error");
 		printf("midi write error, fd %i, size %i\n", fd, size);
 		return(1);
 	}
