@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,12 +29,19 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <fcntl.h>
+#include <time.h>
+#include <syslog.h>
+#include <pthread.h>
 
+#include "bristolmidi.h"
 #include "bristolmidiapi.h"
 //#include "bristolmidiapidata.h"
 #include "bristolmidieventnames.h"
@@ -43,6 +49,9 @@
 #define NO_INTERPOLATE 0x01
 
 static char *myHome = NULL;
+static char tmppath[BUFSZE];
+
+pthread_t lthread;
 
 /*
  * Search the file path given below to look for the desired file. Return the
@@ -60,7 +69,6 @@ getBristolCache(char *file)
 {
 	struct stat statbuf;
 	char *envcache;
-	char path[1024];
 
 	/*
 	 * Find the file specified in the request. Use the search order given
@@ -77,7 +85,9 @@ getBristolCache(char *file)
 	 *	$BRISTOL/memory/profiles - return $BRISTOL
 	 */
 	if (myHome == NULL)
-		myHome = (char *) calloc(1024, 1);
+		myHome = (char *) calloc(BUFSZE, 1);
+	else
+		return(myHome);
 
 	/*
 	 * See if we have an env configured:
@@ -85,41 +95,44 @@ getBristolCache(char *file)
 	 */
 	if ((envcache = getenv("BRISTOL_CACHE")) != NULL)
 	{
-		sprintf(path, "%s/memory/profiles/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/memory/profiles/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			/*
 			 * Path exists.
 			 */
 			sprintf(myHome, "%s", envcache);
-			/* printf("opened BRISTOL_CACHE: %s\n", path); */
+			/* printf("opened BRISTOL_CACHE: %s\n", tmppath); */
 			return(myHome);
 		} else {
 			/*
 			 * Ok, path was desired but not available, build the tree if we
 			 * can and then carry on searching.
 			 */
-			sprintf(path, "%s", envcache);
-			mkdir(path, 0755);
-			sprintf(path, "%s/memory", envcache);
-			mkdir(path, 0755);
-			sprintf(path, "%s/memory/profiles", envcache);
-			mkdir(path, 0755);
+			sprintf(tmppath, "%s", envcache);
+			mkdir(tmppath, 0755);
+			sprintf(tmppath, "%s/memory", envcache);
+			mkdir(tmppath, 0755);
+			sprintf(tmppath, "%s/memory/profiles", envcache);
+			mkdir(tmppath, 0755);
+			//printf("making %s/memory\n", envcache);
 		}
 
-		sprintf(path, "%s/memory/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/memory/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			/*
 			 * Path exists.
 			 */
 			sprintf(myHome, "%s", envcache);
-			/* printf("opened BRISTOL_CACHE: %s\n", path); */
+			/* printf("opened BRISTOL_CACHE: %s\n", tmppath); */
 			return(myHome);
 		}
 
-		printf("create cache %s\n", path);
-		mkdir(path, 0755);
+		//printf("1 create cache %s\n", tmppath);
+		mkdir(tmppath, 0755);
+
+		return(tmppath);
 	}
 
 	/*
@@ -128,55 +141,56 @@ getBristolCache(char *file)
 	 */
 	if ((envcache = getenv("HOME")) != NULL)
 	{
-		sprintf(path, "%s/.bristol/memory/profiles/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/.bristol/memory/profiles/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			/*
 			 * Path exists.
 			 */
 			sprintf(myHome, "%s/.bristol", envcache);
-			/* printf("opened private profile cache: %s\n", path); */
+			/* printf("opened private profile cache: %s\n", tmppath); */
 			return(myHome);
 		} else {
 			/*
 			 * Ok, path was desired but not available, build the tree if we
 			 * can and then carry on searching.
 			 */
-			sprintf(path, "%s", envcache);
-			mkdir(path, 0755);
-			sprintf(path, "%s/memory", envcache);
-			mkdir(path, 0755);
-			sprintf(path, "%s/memory/profiles", envcache);
-			mkdir(path, 0755);
+			sprintf(tmppath, "%s", envcache);
+			//printf("2 create cache %s\n", tmppath);
+			mkdir(tmppath, 0755);
+			sprintf(tmppath, "%s/memory", envcache);
+			mkdir(tmppath, 0755);
+			sprintf(tmppath, "%s/memory/profiles", envcache);
+			mkdir(tmppath, 0755);
 		}
 
-		sprintf(path, "%s/.bristol/memory/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/.bristol/memory/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			/*
 			 * Path exists.
 			 */
 			sprintf(myHome, "%s/.bristol", envcache);
-			/* printf("opened private memory cache: %s\n", path); */
+			/* printf("opened private memory cache: %s\n", tmppath); */
 			return(myHome);
 		}
 	}
 
 	if ((envcache = getenv("BRISTOL")) != NULL)
 	{
-		sprintf(path, "%s/memory/profiles/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/memory/profiles/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			sprintf(myHome, "%s", envcache);
-			/* printf("opened factory cache: %s\n", path); */
+			/* printf("opened factory cache: %s\n", tmppath); */
 			return(myHome);
 		}
 
-		sprintf(path, "%s/memory/%s", envcache, file);
-		if (stat(path, &statbuf) == 0)
+		sprintf(tmppath, "%s/memory/%s", envcache, file);
+		if (stat(tmppath, &statbuf) == 0)
 		{
 			sprintf(myHome, "%s", envcache);
-			/* printf("opened factory cache: %s\n", path); */
+			/* printf("opened factory cache: %s\n", tmppath); */
 			return(myHome);
 		}
 	}
@@ -190,24 +204,23 @@ bristolGetMap(char *file, char *match, float *points, int count, int flags)
 	int i, n = 0, mapped = 0;
 	FILE *fd;
 	float from, delta;
-	char path[256];
 
 	/*
 	 * Open and read configuration. Should consider seaching
 	 * $HOME/.bristol/memory and $BRISTOL_DB/memory.
 	 */
-	sprintf(path, "%s/memory/profiles/%s", getBristolCache("profiles"), file);
-//printf("		%s\n", path);
+	sprintf(tmppath, "%s/memory/profiles/%s", getBristolCache("profiles"), file);
+/*printf("		%s\n", tmppath);*/
 
 	/*
 	 * If we can open the file then clean the array and start again. If not
 	 * then see if we have a factory copy available.
 	 */
-	if ((fd = fopen(path, "r")) == NULL)
+	if ((fd = fopen(&tmppath[0], "r")) == NULL)
 	{
-		sprintf(path, "%s/memory/profiles/%s", getenv("BRISTOL"), file);
-//printf("attempt	%s\n", path);
-		fd = fopen(path, "r");
+		sprintf(tmppath, "%s/memory/profiles/%s", getenv("BRISTOL"), file);
+/*printf("attempt	%s\n", tmppath); */
+		fd = fopen(&tmppath[0], "r");
 	}
 
 	if (fd != NULL)
@@ -574,3 +587,262 @@ bristolMidiValueMappingTable(u_char valuemap[128][128], int midimap[128], char *
 	}
 }
 
+static int ztime = 0, uztime, std_out, f_out = -1, bsyslog = 0;
+FILE *logInput = NULL;
+
+/*
+ * This will close stdin, stdout and stderr, then open some files for debug
+ * output. First an attempt will be made to open files in /var/log/<process>.log
+ * then ~/.bristol/log/<process>.log (with O_TRUNC in this case) and creating
+ * the directory if necessary. Failing that /dev/null.
+ *
+ * We should look to pthread() a logging process that takes a pipe() to the main
+ * threads then prints every message that comes from the pipd()ed stdout/stderr
+ * to a date formatted message to the log file on new fd:
+ *
+ *	create pipe.
+ *	close 0, 1, 2.
+ *	createthread
+ *		parent
+ *			dup2() pipe write fd onto 1, 2
+ *			closes pipe write fd
+ *		child
+ *			opens logfile
+ *			dup2() pipe read fd onto 0
+ *			closes pipe read fd
+ *			reads from pipe - gets()? May just scan for \n and add date
+ *			formats to log fd
+ */
+void
+logthread(char *process)
+{
+	char filename[BUFSZE], *inputtext = filename, outputtext[BUFSZE], ttext[BUFSZE];
+	time_t timep;
+	struct tm *tm;
+	struct timeval tv;
+	int len;
+
+	sprintf(filename, "/var/log/%s.log", process);
+
+	/* See about /var/log */
+	if ((f_out = open(filename, O_CREAT|O_WRONLY|O_APPEND, 0644)) < 0)
+	{
+		/* see about ~/.bristol/log */
+		sprintf(filename, "%s/.bristol/log/%s.log", getenv("HOME"), process);
+
+		if ((f_out = open(filename, O_TRUNC|O_CREAT|O_WRONLY, 0644)) < 0)
+		{
+			sprintf(filename, "%s/.bristol/log", getenv("HOME"));
+			mkdir(filename, 0755);
+
+			sprintf(filename, "%s/.bristol/log/%s.log", getenv("HOME"),	
+				process);
+
+			if ((f_out = open(filename, O_TRUNC|O_CREAT|O_WRONLY, 0644)) < 0)
+				/* All failed, open /dev/null */
+				f_out = open("/dev/null", O_WRONLY);
+		}
+	}
+
+	gettimeofday(&tv, NULL);
+	/*
+	 * The use of ztime is just to pull the seconds back to when the app
+	 * was started.
+	 */
+	ztime = (int) tv.tv_sec;
+	uztime = (int) tv.tv_usec;
+
+	/*
+	 * At this point we should pretty print the current date into the
+	 * file so that we can correlated different invocations - bristol
+	 * and brighton now have separate log files and if in /var/log they
+	 * may keep track of several invocations.
+	 */
+//	while ((c = read(0, &inputtext[i], 1)) >= 0) {
+	/*
+	while (1) {
+		c = read(0, &inputtext[i], 1);
+		if (c == 0)
+		{
+			usleep(100000);
+			continue;
+		}
+		if (inputtext[i++] != '\n')
+			continue;
+		inputtext[i-1] = '\0';
+		i = 0;
+	*/
+
+	while (fgets(inputtext, BUFSZE, logInput) > 0)
+	{
+		if (((len = strlen(inputtext)) > 0)
+			&& (inputtext[len - 1] != '\n'))
+			sprintf(inputtext, "(suppressed excess message %i bytes)", len);
+		else
+			inputtext[len - 1] = '\0';
+
+		gettimeofday(&tv, NULL);
+
+		if (bsyslog)
+		{
+			if (std_out > 0)
+			{
+				//fsync(std_out);
+				close(std_out);
+				std_out = -1;
+			}
+			if (f_out > 0)
+			{
+				//fsync(f_out);
+				close(f_out);
+				f_out = -1;
+			}
+
+			sprintf(outputtext, "[%05.6f] %s\n",
+				(float) (((int) tv.tv_usec) < uztime?
+					(int) tv.tv_sec - ztime - 1:
+					(int) tv.tv_sec - ztime)
+					+ (float) (((int) tv.tv_usec) < uztime?
+						((int) tv.tv_usec) + (1000000 - uztime):
+						(((int) tv.tv_usec) - uztime)) / 1000000,
+				inputtext);
+			syslog(LOG_USER|LOG_INFO, "%s", outputtext);
+			continue;
+		}
+
+		time(&timep);
+		tm = localtime(&timep);
+		// day month year: %d %m %y
+		// %b %e
+		// The one taken is similar to syslog format:
+		strftime(ttext, BUFSZE, "%b %e %T", tm);
+
+		sprintf(outputtext, "%s %-8s [%05.6f] %s\n",
+			ttext, process,
+			(float) (((int) tv.tv_usec) < uztime?
+				(int) tv.tv_sec - ztime - 1:
+				(int) tv.tv_sec - ztime)
+				+ (float) (((int) tv.tv_usec) < uztime?
+					((int) tv.tv_usec) + (1000000 - uztime):
+					(((int) tv.tv_usec) - uztime)) / 1000000,
+			inputtext);
+
+		if (std_out < 0)
+			continue;
+
+		if (write(std_out, outputtext, strlen(outputtext)) < 0)
+			pthread_exit(0);
+		fsync(std_out);
+	}
+
+	/*
+	sprintf(outputtext, "%s %-8s [%05.6f] %s\n",
+		ttext, process,
+		(float) (((int) tv.tv_usec) < uztime?
+			(int) tv.tv_sec - ztime - 1:
+			(int) tv.tv_sec - ztime)
+			+ (float) (((int) tv.tv_usec) < uztime?
+				((int) tv.tv_usec) + (1000000 - uztime):
+				(((int) tv.tv_usec) - uztime)) / 1000000,
+		"terminating log thread");
+	write(std_out, outputtext, strlen(outputtext));
+
+	 * We are not going to redirect the input stream for now. If we end up
+	 * going for real daemon processing this needs to be cleared up in separate
+	 * code that detaches the controlling terminal, and a few other loose ends.
+	tmpfd = open("/dev/null", O_RDONLY);
+	dup2(tmpfd, STDIN_FILENO);
+	close(tmpfd);
+	 */
+
+	if (bsyslog)
+		closelog();
+	else {
+		close(std_out);
+	}
+
+	pthread_exit(0);
+}
+
+static char procname[64];
+static struct timeval tv;
+static int lcons = 0;
+
+pthread_t
+bristolOpenStdio(int mode)
+{
+	int pipefds[2];
+	int count = 40;
+
+	if (lcons)
+		return(0);
+
+	switch (mode) {
+		case BRISTOL_LOG_TERMINATE:
+			if (lthread != 0)
+				pthread_cancel(lthread);
+			return(0);
+		case BRISTOL_LOG_BRISTOL:
+			sprintf(procname, "%s", "bristol");
+			break;
+		case BRISTOL_LOG_BRIGHTON:
+			sprintf(procname, "%s", "brighton");
+			break;
+		case BRISTOL_LOG_DAEMON:
+			close(std_out);
+			std_out = f_out;
+			/*
+			 * We need this to log into the file rather than stdtio
+			 */
+			printf("\nstarting file logging [@%i.%i]\n",
+				(int) tv.tv_sec, (int) tv.tv_usec);
+
+			return(0);
+		case BRISTOL_LOG_SYSLOG:
+			openlog(procname, LOG_CONS|LOG_NDELAY|LOG_NOWAIT, LOG_USER);
+
+			bsyslog = 1;
+
+			return(0);
+		case BRISTOL_LOG_CONSOLE:
+			lcons = 1;
+			return(0);
+	}
+
+	gettimeofday(&tv, NULL);
+	printf("starting logging thread [@%i.%i]\n",
+		(int) tv.tv_sec, (int) tv.tv_usec);
+
+	if (pipe(pipefds) < 0)
+		return(0);
+
+	fcntl(pipefds[0], F_SETFL, O_RDONLY);
+	fcntl(pipefds[1], F_SETFL, O_WRONLY|O_NDELAY);
+
+	std_out = dup(STDOUT_FILENO);
+
+	dup2(pipefds[0], STDIN_FILENO);
+	dup2(pipefds[1], STDERR_FILENO);
+
+	if ((logInput = fdopen(STDIN_FILENO, "r")) == NULL)
+		printf("Could not fdopen() log fd\n");
+
+	dup2(pipefds[1], STDOUT_FILENO);
+	close(pipefds[0]);
+	close(pipefds[1]);
+
+	pthread_create(&lthread, NULL, (void *) logthread, &procname);
+
+	while (f_out < 0)
+	{
+		usleep(100000);
+		if (--count < 0)
+			break;
+	}
+
+	printf("starting console logging [@%i.%i]\n",
+		(int) tv.tv_sec, (int) tv.tv_usec);
+
+	return(lthread);
+}
+
