@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -35,43 +34,78 @@
 #include <jack/jack.h>
 #include <jack/midiport.h>
 
-/* We only have one of these for a single registration per engine. */
-static jack_client_t *client;
-
-static jack_port_t *input_port;
 extern bristolMidiMain bmidi;
 
-static int
+/* We only have one of these for a single registration per engine. */
+static jack_client_t *client = NULL;
+static int deviceIndex = -1;
+
+static jack_port_t *input_port = NULL;
+extern void checkcallbacks(bristolMidiMsg *);
+
+void
+jackMidiRegisterPort(jack_port_t *midi_input_port)
+{
+	input_port = midi_input_port;
+}
+
+int
 jackMidiRoutine(jack_nframes_t nframes, void *arg)
 {
 	int i;
-	void* port_buf = jack_port_get_buffer(input_port, nframes);
+	void* port_buf;
 	jack_midi_event_t in_event;
-	jack_nframes_t event_count = jack_midi_get_event_count(port_buf);
+	jack_nframes_t event_count;
 	bristolMidiMsg msg;
 
+	if (input_port == NULL)
+		return(0);
+
+	port_buf = jack_port_get_buffer(input_port, nframes);
+	event_count = jack_midi_get_event_count(port_buf);
+
+	bmidi.dev[deviceIndex].flags = BRISTOL_CONN_JACK|0x40000000;
+
 	if (event_count <= 0)
 		return(0);
 
-/* printf("jack midi routine: %i\n", event_count); */
+	/*
+	else
+		printf("received %i events\n", event_count);
+`	*/
 	for (i = 0; i < event_count; i++)
 	{
-		jack_midi_event_get(&in_event, port_buf, i);
-/*
-		printf("    event %d time is %d. 1st byte is 0x%x\n",
-			i, in_event.time, *(in_event.buffer));
-*/
-		
+		memset(&in_event, 0, sizeof(jack_midi_event_t));
+		memset(&msg, 0, sizeof(bristolMidiMsg));
+
+		if (jack_midi_event_get(&in_event, port_buf, i) != 0)
+			continue;
+
+		/*
+		printf("    event %d time is %d. %x/%i/%i (%i)\n", i, in_event.time,
+			*(in_event.buffer), *(in_event.buffer+1), *(in_event.buffer+2),
+			in_event.size);
+		*/
+
 		/*
 		 * Convert these events into our internal format and then have then
 		 * dispatched. We could probably call a quite early routine since we
 		 * appear to have reasonably raw midi.
+		 *
+		 * Jack always gives me a status byte. We need to parse this then parse
+		 * the actual midi message.
 		 */
-		if ((msg.params.bristol.msgLen =
-			bristolMidiRawToMsg(in_event.buffer, in_event.size,
-				0, (int) arg, &msg)) > 0)
+		bristolMidiRawToMsg(in_event.buffer, in_event.size, 0, deviceIndex,
+			&msg);
+		if (bristolMidiRawToMsg(in_event.buffer + 1, in_event.size - 1, 0,
+			deviceIndex, &msg) > 0)
+		{
+			msg.params.bristol.msgLen = in_event.size;
+			msg.params.bristol.from = deviceIndex;
+			msg.offset = in_event.time;
+
 			checkcallbacks(&msg);
-		else
+		} else if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
 			printf("unknown jack midi event\n");
 	}
 
@@ -110,29 +144,49 @@ int
 bristolMidiJackOpen(char *devname, int flags, int chan, int messages,
 int (*callback)(), void *param, int dev, int handle)
 {
-	sleep(1);
-	if ((client = jack_client_new(devname)) == 0)
+	printf("bristolMidiJackOpen(%s, %i, %x)\n", devname, dev, flags);
+
+	deviceIndex = dev;
+
+//	bmidi.flags &= ~BRISTOL_MIDI_FORWARD;
+//	bmidi.dev[dev].flags = BRISTOL_CONN_JACK|0x40000000;
+
+	if (client == 0)
 	{
-		fprintf(stderr, "jack server not running?\n");
-		return 1;
-	}
+		if ((client = jack_client_open(devname, 0, NULL)) == 0)
+		{
+			fprintf(stderr, "jack server not running?\n");
+			return 1;
+		}
+		printf("registered jack midi name %s\n", devname);
 	
-	jack_set_process_callback(client, jackMidiRoutine, (void *) dev);
+		jack_set_process_callback(client, jackMidiRoutine, (void *) dev);
 
-	jack_on_shutdown(client, jack_midi_shutdown, 0);
+		jack_on_shutdown(client, jack_midi_shutdown, 0);
 
-	input_port = jack_port_register(client, "midi_in",
-		JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
+		input_port = jack_port_register(client, "midi_in",
+			JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
 
-	if (jack_activate(client))
-	{
-		fprintf(stderr, "cannot activate client");
-		return 1;
+		if (jack_activate(client))
+		{
+			fprintf(stderr, "cannot activate client");
+			return 1;
+		}
+	} else {
+		input_port = jack_port_register(client, "midi_in",
+			JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
+		printf("reused jack registration\n");
 	}
 
 	return(handle);
 }
 
+void
+bristolJackSetMidiHandle(jack_client_t *jackHandle)
+{
+	client = jackHandle;
+}
+
 int
 bristolMidiJackClose(int handle)
 {
@@ -150,6 +204,7 @@ int (*callback)(), void *param, int dev, int handle)
 	printf("Jack MIDI requested but not linked into the library\n");
 	return(BRISTOL_MIDI_DRIVER);
 }
+
 int
 bristolMidiJackClose(int handle)
 {
