@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -79,6 +78,24 @@ bristolMidiDevSanity(int dev)
 	return(BRISTOL_MIDI_OK);
 }
 
+void
+bristolMidiPrintHandle(handle)
+{
+	printf("	Handle:	 %i\n", bmidi.handle[handle].handle);
+	printf("	State:   %i\n", bmidi.handle[handle].state);
+	printf("	Channel: %i\n", bmidi.handle[handle].channel);
+	printf("	Dev:     %i\n", bmidi.handle[handle].dev);
+	printf("	Flags:   %x\n", bmidi.handle[handle].flags);
+
+	if ((bmidi.handle[handle].dev < 0)
+		|| (bmidi.handle[handle].dev >= BRISTOL_MIDI_DEVCOUNT))
+		return;
+
+	printf("		devfd:  %i\n", bmidi.dev[bmidi.handle[handle].dev].fd);
+	printf("		state:  %i\n", bmidi.dev[bmidi.handle[handle].dev].state);
+	printf("		hcount: %i\n", bmidi.dev[bmidi.handle[handle].dev].handleCount);
+}
+
 int
 bristolMidiSanity(handle)
 {
@@ -89,6 +106,11 @@ bristolMidiSanity(handle)
 	if ((handle < 0) || (handle > BRISTOL_MIDI_HANDLES))
 		return(BRISTOL_MIDI_HANDLE);
 
+	/*
+	 * So we can at least print it
+	bristolMidiPrintHandle(handle);
+	 */
+
 	if (bmidi.handle[handle].state < 0)
 		return(BRISTOL_MIDI_HANDLE);
 
