@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,7 +30,7 @@
 
 #include "bristolmidi.h"
 
-//#define DEBUG
+#define DEBUG
 
 extern bristolMidiMain bmidi;
 extern int bristolFreeHandle();
@@ -46,12 +45,10 @@ bristolMidiSeqOpen(char *devname, int flags, int chan, int messages,
 int (*callback)(), void *param, int dev, int handle)
 {
 #if (BRISTOL_HAS_ALSA == 1)
-	int nfds, err;
-	int client, port, queue, v1, v2;
+	int nfds, err, caps = 0, polldir = 0;
+	int client, queue;
 	char linkname[256];
 	snd_seq_port_info_t *pinfo;
-	snd_seq_port_subscribe_t *sub;
-	snd_seq_addr_t addr;
   
 #ifdef DEBUG
 	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
@@ -59,12 +56,37 @@ int (*callback)(), void *param, int dev, int handle)
 #endif
 
 #if (SND_LIB_MAJOR == 1)
+	caps = SND_SEQ_PORT_TYPE_MIDI_GENERIC | SND_SEQ_PORT_TYPE_SYNTHESIZER | SND_SEQ_PORT_TYPE_SOFTWARE | SND_SEQ_PORT_TYPE_SYNTH | SND_SEQ_PORT_TYPE_APPLICATION;
+
 	/*
 	 * Well, we got here, so I could perhaps consider assuming that we have
 	 * ALSA 0.9 or greater SEQ support.
 	 */
-	bmidi.dev[dev].flags = SND_SEQ_OPEN_DUPLEX;
-	if (snd_seq_open(&bmidi.dev[dev].driver.seq.handle, "hw",
+	bmidi.dev[dev].flags = 0;
+
+/*
+	if (flags & BRISTOL_WRONLY)
+	{
+		bmidi.dev[dev].flags |= SND_SEQ_OPEN_OUTPUT;
+		caps |= SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_READ;
+	}
+*/
+
+	if (flags & BRISTOL_RDONLY)
+	{
+		polldir |= POLLIN;
+		bmidi.dev[dev].flags |= SND_SEQ_OPEN_INPUT;
+		caps |= SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE;
+	}
+
+	if (flags & BRISTOL_WRONLY)
+	{
+		polldir |= POLLOUT;
+		bmidi.dev[dev].flags |= SND_SEQ_OPEN_OUTPUT;
+		caps |= SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_READ;
+	}
+
+	if (snd_seq_open(&bmidi.dev[dev].driver.seq.handle, "default",
 		bmidi.dev[dev].flags, 0) != 0)
 	{
 		/* 
@@ -82,39 +104,52 @@ int (*callback)(), void *param, int dev, int handle)
 	if ((err = snd_seq_set_client_name(bmidi.dev[dev].driver.seq.handle,
 		devname)) < 0)
 	{
-		fprintf(stderr, "Set client info error: %s\n", snd_strerror(err));
+		printf("Set client info error: %s\n", snd_strerror(err));
 		return(BRISTOL_MIDI_DRIVER);
 	}
 
 	if ((client = snd_seq_client_id(bmidi.dev[dev].driver.seq.handle)) < 0) {
-		fprintf(stderr,
+		printf(
 			"Cannot determine client number: %s\n", snd_strerror(client));
 		return(BRISTOL_MIDI_DRIVER);
 	}
 	printf("Client ID = %i\n", client);
 	if ((queue = snd_seq_alloc_queue(bmidi.dev[dev].driver.seq.handle)) < 0) {
-		fprintf(stderr, "Cannot allocate queue: %s\n", snd_strerror(queue));
+		printf( "Cannot allocate queue: %s\n", snd_strerror(queue));
 		return(BRISTOL_MIDI_DRIVER);
 	}
 	printf("Queue ID = %i\n", queue);
 	if ((err = snd_seq_nonblock(bmidi.dev[dev].driver.seq.handle, 1)) < 0)
-		fprintf(stderr, "Cannot set nonblock mode: %s\n", snd_strerror(err));
+		printf( "Cannot set nonblock mode: %s\n", snd_strerror(err));
+
 	snd_seq_port_info_alloca(&pinfo);
-	sprintf(linkname, "%s input", devname);
+
+	sprintf(linkname, "%s io", devname);
+
+	if (~bmidi.dev[dev].flags & SND_SEQ_OPEN_INPUT)
+		sprintf(linkname, "%s output", devname);
+	else if (~bmidi.dev[dev].flags & SND_SEQ_OPEN_OUTPUT)
+		sprintf(linkname, "%s input", devname);
+
 	snd_seq_port_info_set_name(pinfo, linkname);
-	snd_seq_port_info_set_capability(pinfo, SND_SEQ_PORT_CAP_WRITE
-		| SND_SEQ_PORT_CAP_READ | SND_SEQ_PORT_CAP_SUBS_WRITE);
+	snd_seq_port_info_set_capability(pinfo, caps);
+    snd_seq_port_info_set_type (pinfo, caps);
+
 	if ((err = snd_seq_create_port(bmidi.dev[dev].driver.seq.handle, pinfo))
 		< 0)
 	{
-		fprintf(stderr, "Cannot create input port: %s\n", snd_strerror(err));
+		printf( "Cannot create input port: %s\n", snd_strerror(err));
 		return(BRISTOL_MIDI_DRIVER);
 	}
+
+/*
 	port = snd_seq_port_info_get_port(pinfo);
 
-	if ((err = snd_seq_start_queue(bmidi.dev[dev].driver.seq.handle,
-		queue, NULL)) < 0)
-	   fprintf(stderr, "Timer event output error: %s\n", snd_strerror(err));
+	if ((bmidi.dev[dev].flags & SND_SEQ_OPEN_INPUT)
+		&& ((err = snd_seq_start_queue(bmidi.dev[dev].driver.seq.handle,
+		queue, NULL)) < 0))
+	   printf( "Timer event output error: %s\n", snd_strerror(err));
+
 	while (snd_seq_drain_output(bmidi.dev[dev].driver.seq.handle) > 0)
 		sleep(1);
 
@@ -122,7 +157,7 @@ int (*callback)(), void *param, int dev, int handle)
 	addr.client = SND_SEQ_CLIENT_SYSTEM;
 	addr.port = SND_SEQ_PORT_SYSTEM_ANNOUNCE;
 
-	//printf("Registering %i %i\n", addr.client, addr.port);
+	printf("Registering %i %i\n", addr.client, addr.port);
 
 	snd_seq_port_subscribe_set_sender(sub, &addr);
 	addr.client = client;
@@ -137,8 +172,7 @@ int (*callback)(), void *param, int dev, int handle)
 	if ((err = snd_seq_subscribe_port(bmidi.dev[dev].driver.seq.handle, sub))
 		< 0)
 	{
-		fprintf(stderr,
-			"Cannot subscribe announce port: %s\n", snd_strerror(err));
+		printf("Cannot subscribe announce port: %s\n", snd_strerror(err));
 		return(BRISTOL_MIDI_DRIVER);
 	}
 
@@ -148,19 +182,23 @@ int (*callback)(), void *param, int dev, int handle)
 	if ((err = snd_seq_subscribe_port(bmidi.dev[dev].driver.seq.handle, sub))
 		< 0)
 	{
-		fprintf(stderr, "Cannot subscribe timer port: %s\n", snd_strerror(err));
+		printf( "Cannot subscribe timer port: %s\n", snd_strerror(err));
 		return(BRISTOL_MIDI_DRIVER);
 	}
 
 	snd_seq_port_subscribe_set_time_real(sub, 0);
+*/
+
 		/*
 		if (tolower(*ptr) == 'r') {
 			snd_seq_port_subscribe_set_time_real(sub, 1);
 			ptr++;
 		}
 		*/
+
+/*
 	if (sscanf(devname, "%i.%i", &v1, &v2) != 2) {
-		fprintf(stderr, "Device name \"%s\" did not parse, defaults 128.0\n",
+		printf( "Device name \"%s\" did not parse, defaults 128.0\n",
 			devname);
 		v1 = 128;
 		v2 = 0;
@@ -168,12 +206,11 @@ int (*callback)(), void *param, int dev, int handle)
 	addr.client = v1;
 	addr.port = v2;
 
-/*
 	snd_seq_port_subscribe_set_sender(sub, &addr);
 	if ((err = snd_seq_subscribe_port(bmidi.dev[dev].driver.seq.handle, sub))
 		< 0)
 	{
-		fprintf(stderr, "Cannot subscribe port %i from client %i: %s\n",
+		printf( "Cannot subscribe port %i from client %i: %s\n",
 			v2, v1, snd_strerror(err));
 	}
 */
@@ -182,16 +219,18 @@ int (*callback)(), void *param, int dev, int handle)
 	 * Now we need to find a file descriptor.
 	 */
 	if ((nfds = snd_seq_poll_descriptors_count(bmidi.dev[dev].driver.seq.handle,
-		POLLIN)) < 1)
+		polldir)) < 1)
 		printf("issue getting descriptors: %i\n", nfds);
 	else {
 		struct pollfd *pfds;
 
 		pfds = (struct pollfd *) malloc(sizeof (struct pollfd) * nfds);
 		snd_seq_poll_descriptors(bmidi.dev[dev].driver.seq.handle,
-			pfds, nfds, POLLIN);
+			pfds, nfds, polldir);
 
 		bmidi.dev[dev].fd = pfds[0].fd;
+
+		free(pfds);
 	}
 
 	bmidi.dev[dev].flags = BRISTOL_CONN_SEQ;
@@ -384,18 +423,19 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			break;
 
 		case SND_SEQ_EVENT_NOTEON:
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x/%02x ",
 					dev,
 					ev->data.note.channel,
 					ev->data.note.note,
 					ev->data.note.velocity);
-
+#endif
 			if (ev->data.note.velocity == 0) {
-				msg->command = MIDI_NOTE_OFF|ev->data.note.channel;
+				msg->command = MIDI_NOTE_OFF| ev->data.note.channel;
 				msg->params.key.velocity = 64;
 			} else {
-				msg->command = MIDI_NOTE_ON|ev->data.note.channel;
+				msg->command = MIDI_NOTE_ON| ev->data.note.channel;
 				msg->params.key.velocity = ev->data.note.velocity;
 			}
 			msg->channel = ev->data.note.channel;
@@ -405,14 +445,15 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			break;
 
 		case SND_SEQ_EVENT_NOTEOFF:
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x/%02x ",
 					dev,
 					ev->data.note.channel,
 					ev->data.note.note,
 					ev->data.note.velocity);
-
-			msg->command = MIDI_NOTE_OFF|ev->data.note.channel;
+#endif
+			msg->command = MIDI_NOTE_OFF| ev->data.note.channel;
 			msg->channel = ev->data.note.channel;
 			msg->params.key.key = ev->data.note.note,
 			msg->params.key.velocity = ev->data.note.velocity;
@@ -428,7 +469,7 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 					ev->data.note.note,
 					ev->data.note.velocity);
 #endif
-			msg->command = MIDI_POLY_PRESS|ev->data.note.channel;
+			msg->command = MIDI_POLY_PRESS| ev->data.note.channel;
 			msg->channel = ev->data.note.channel;
 			msg->params.pressure.key = ev->data.note.note,
 			msg->params.pressure.pressure = ev->data.note.velocity;
@@ -436,17 +477,19 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			break;
 
 		case SND_SEQ_EVENT_CONTROLLER:
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x/%02x ",
 					dev,
 					ev->data.control.channel,
 					ev->data.control.param,
 					ev->data.control.value);
+#endif
 
 			/*
 			 * Hm, we may need to recode these to 7 bit midi values?
 			 */
-			msg->command = MIDI_CONTROL|ev->data.note.channel;
+			msg->command = MIDI_CONTROL| ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			msg->params.controller.c_id = ev->data.control.param,
 			msg->params.controller.c_val = ev->data.control.value;
@@ -464,14 +507,15 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			/*
 			 * May need to have a registered callback here from GUI.
 			 */
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x ",
 					dev,
 					ev->data.control.channel,
 					ev->data.control.value);
+#endif
 
-
-			msg->command = MIDI_PROGRAM|ev->data.note.channel;
+			msg->command = MIDI_PROGRAM| ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			msg->params.program.p_id = ev->data.control.value;
 			msg->sequence = bmidi.dev[dev].sequence++;
@@ -480,13 +524,14 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			break;
 
 		case SND_SEQ_EVENT_CHANPRESS:
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x ",
 					dev,
 					ev->data.control.channel,
 					ev->data.control.value);
-
-			msg->command = MIDI_CONTROL|ev->data.note.channel;
+#endif
+			msg->command = MIDI_CHAN_PRESS;
 			msg->channel = ev->data.control.channel;
 			msg->params.channelpress.pressure = ev->data.control.value;
 			msg->sequence = bmidi.dev[dev].sequence++;
@@ -494,16 +539,18 @@ translate_event(snd_seq_event_t *ev, bristolMidiMsg *msg, int dev)
 			break;
 
 		case SND_SEQ_EVENT_PITCHBEND:
-			if (dev)
+#ifdef DEBUG
+			if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
 				printf("c%i-%02x/%02x ",
 					dev,
 					ev->data.control.channel,
 					ev->data.control.value);
+#endif
 
 			/*
 			 * Hm, we may need to recode these to 7 bit midi values?
 			 */
-			msg->command = MIDI_PITCHWHEEL|ev->data.note.channel;
+			msg->command = MIDI_PITCHWHEEL| ev->data.note.channel;
 			msg->channel = ev->data.control.channel;
 			ev->data.control.value += 8192;
 			msg->params.pitch.lsb = ev->data.control.value & 0x7f;
@@ -630,9 +677,8 @@ bristolMidiSeqRead(int dev, bristolMidiMsg *msg)
 	snd_seq_event_t *ev;
 	int res;
 
-#ifdef DEBUG
-/*	printf("bristolMidiSeqRead()\n"); */
-#endif
+	if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+		printf("bristolMidiSeqRead()\n");
 
 	/*
 	 * Get the message, convert its type into a bristol message. Then see if
@@ -644,6 +690,9 @@ bristolMidiSeqRead(int dev, bristolMidiMsg *msg)
 		translate_event(ev, msg,
 			bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG? dev: 0);
 
+		if (bmidi.flags & BRISTOL_BMIDI_DEBUG)
+			printf("msg->command = %02x\n", msg->command);
+
 		if (msg->command != 0xff)
 		{
 			msg->params.bristol.from = dev;
@@ -661,3 +710,172 @@ bristolMidiSeqRead(int dev, bristolMidiMsg *msg)
 	return(BRISTOL_MIDI_OK);
 }
 
+/*
+ * Note that we should not read on a handle - we should read on a device, and
+ * then forward to all the handles.
+ */
+int
+bristolMidiSeqKeyEvent(int dev, int op, int ch, int key, int velocity)
+{
+#if (BRISTOL_HAS_ALSA == 1)
+#if (SND_LIB_MAJOR == 1)
+	snd_seq_event_t ev;
+
+#ifdef DEBUG
+	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		printf("bristolMidiSeqKeyEvent(%i, %i, %i, %i)\n",
+			op, ch, key, velocity);
+#endif
+
+	/*
+	 * Build seq event, send it.
+	 */
+	snd_seq_ev_clear(&ev); 
+	if (op == BRISTOL_EVENT_KEYON)
+		ev.type = SND_SEQ_EVENT_NOTEON;
+	else
+		ev.type = SND_SEQ_EVENT_NOTEOFF;
+
+	ev.queue = SND_SEQ_QUEUE_DIRECT;
+
+	ev.dest.client = ch;
+	ev.dest.port = 0;
+
+	ev.data.note.channel = 0;
+	ev.data.note.note = key;
+	ev.data.note.velocity = velocity;
+
+	if (snd_seq_event_output_direct(bmidi.dev[dev].driver.seq.handle, &ev) < 0)
+	{
+		printf("SeqSend failed: %p\n", bmidi.dev[dev].driver.seq.handle);
+		return(BRISTOL_MIDI_DRIVER);
+	}
+#endif
+#endif
+
+	return(BRISTOL_MIDI_OK);
+}
+
+/*
+ * Note that we should not read on a handle - we should read on a device, and
+ * then forward to all the handles.
+ */
+int
+bristolMidiSeqPPressureEvent(int dev, int op, int ch, int k, int p)
+{
+#if (BRISTOL_HAS_ALSA == 1)
+#if (SND_LIB_MAJOR == 1)
+	snd_seq_event_t ev;
+
+#ifdef DEBUG
+	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		printf("bristolMidiPressureEvent(%i, %i, %i)\n",
+			op, ch, p);
+#endif
+	/*
+	 * Build seq event, send it.
+	 */
+	ev.type = SND_SEQ_EVENT_KEYPRESS;
+
+	ev.queue = SND_SEQ_QUEUE_DIRECT;
+
+	ev.dest.client = op;
+	ev.dest.port = 0;
+
+	ev.data.control.channel = ch;
+	ev.data.control.param = k;
+	ev.data.control.value = p;
+
+	if (snd_seq_event_output_direct(bmidi.dev[dev].driver.seq.handle, &ev) < 0)
+	{
+		printf("SeqSend failed: %p\n", bmidi.dev[dev].driver.seq.handle);
+		return(BRISTOL_MIDI_DRIVER);
+	}
+#endif
+#endif
+
+	return(BRISTOL_MIDI_OK);
+}
+
+/*
+ * Note that we should not read on a handle - we should read on a device, and
+ * then forward to all the handles.
+ */
+int
+bristolMidiSeqPressureEvent(int dev, int op, int ch, int p)
+{
+#if (BRISTOL_HAS_ALSA == 1)
+#if (SND_LIB_MAJOR == 1)
+	snd_seq_event_t ev;
+
+#ifdef DEBUG
+	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		printf("bristolMidiPressureEvent(%i, %i, %i)\n",
+			op, ch, p);
+#endif
+	/*
+	 * Build seq event, send it.
+	 */
+	ev.type = SND_SEQ_EVENT_CHANPRESS;
+
+	ev.queue = SND_SEQ_QUEUE_DIRECT;
+
+	ev.dest.client = op;
+	ev.dest.port = 0;
+
+	ev.data.control.channel = ch;
+	ev.data.control.value = p;
+
+	if (snd_seq_event_output_direct(bmidi.dev[dev].driver.seq.handle, &ev) < 0)
+	{
+		printf("SeqSend failed: %p\n", bmidi.dev[dev].driver.seq.handle);
+		return(BRISTOL_MIDI_DRIVER);
+	}
+#endif
+#endif
+
+	return(BRISTOL_MIDI_OK);
+}
+
+/*
+ * Note that we should not read on a handle - we should read on a device, and
+ * then forward to all the handles.
+ */
+int
+bristolMidiSeqCCEvent(int dev, int op, int ch, int CC, int CV)
+{
+#if (BRISTOL_HAS_ALSA == 1)
+#if (SND_LIB_MAJOR == 1)
+	snd_seq_event_t ev;
+
+#ifdef DEBUG
+	if (bmidi.dev[dev].flags & _BRISTOL_MIDI_DEBUG)
+		printf("bristolMidiCCKeyEvent(%i, %i, %i, %i)\n",
+			op, ch, CC, CV);
+#endif
+
+	/*
+	 * Build seq event, send it.
+	 */
+	ev.type = SND_SEQ_EVENT_CONTROLLER;
+
+	ev.queue = SND_SEQ_QUEUE_DIRECT;
+
+	ev.dest.client = ch;
+	ev.dest.port = 0;
+
+	ev.data.control.channel = 0;
+	ev.data.control.param = CC;
+	ev.data.control.value = CV;
+
+	if (snd_seq_event_output_direct(bmidi.dev[dev].driver.seq.handle, &ev) < 0)
+	{
+		printf("SeqSend failed: %p\n", bmidi.dev[dev].driver.seq.handle);
+		return(BRISTOL_MIDI_DRIVER);
+	}
+#endif
+#endif
+
+	return(BRISTOL_MIDI_OK);
+}
+
