@@ -1,12 +1,12 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2009
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
  *
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -105,7 +104,7 @@ int 		listens; /* Parameter for the number of connections accepted */
 	if (listen(remote_socket_descriptor, listens) < 0)
 		printf("Cannot listen to socket\n");
 
-	fcntl(remote_socket_descriptor, F_SETFL, O_NONBLOCK);
+	//fcntl(remote_socket_descriptor, F_SETFL, O_NONBLOCK);
 
 	return(remote_socket_descriptor);
 }
